@@ -285,6 +285,18 @@ HRESULT WINAPI AtlIPersistPropertyBag_Load(LPPROPERTYBAG pPropBag, LPERRORLOG pE
     return S_OK;
 }
 
+/***********************************************************************
+ *           AtlIPersistPropertyBag_Save     [atl100.@]
+ */
+HRESULT WINAPI AtlIPersistPropertyBag_Save(LPPROPERTYBAG pPropBag, BOOL fClearDirty,
+                                           BOOL fSaveAll, ATL_PROPMAP_ENTRY *pMap,
+                                           void *pThis, IUnknown *pUnk)
+{
+    FIXME("(%p, %d, %d, %p, %p, %p)\n", pPropBag, fClearDirty, fSaveAll, pMap, pThis, pUnk);
+
+    return S_OK;
+}
+
 /***********************************************************************
  *           AtlModuleAddTermFunc            [atl100.@]
  */
