@@ -1035,15 +1035,15 @@ static enum content get_content_type(LPCOLESTR name, CLSID *control_id)
 }
 
 /***********************************************************************
- *           AtlAxCreateControlEx            [atl100.@]
+ *           AtlAxCreateControlLicEx         [atl100.@]
  *
  * REMARKS
  *   See http://www.codeproject.com/com/cwebpage.asp for some background
  *
  */
-HRESULT WINAPI AtlAxCreateControlEx(LPCOLESTR lpszName, HWND hWnd,
+HRESULT WINAPI AtlAxCreateControlLicEx(LPCOLESTR lpszName, HWND hWnd,
         IStream *pStream, IUnknown **ppUnkContainer, IUnknown **ppUnkControl,
-        REFIID iidSink, IUnknown *punkSink)
+        REFIID iidSink, IUnknown *punkSink, BSTR lic)
 {
     CLSID controlId;
     HRESULT hRes;
@@ -1053,8 +1053,11 @@ HRESULT WINAPI AtlAxCreateControlEx(LPCOLESTR lpszName, HWND hWnd,
     IUnknown *pContainer = NULL;
     enum content content;
 
-    TRACE("(%s %p %p %p %p %p %p)\n", debugstr_w(lpszName), hWnd, pStream,
-            ppUnkContainer, ppUnkControl, iidSink, punkSink);
+    TRACE("(%s %p %p %p %p %p %p %s)\n", debugstr_w(lpszName), hWnd, pStream,
+            ppUnkContainer, ppUnkControl, iidSink, punkSink, debugstr_w(lic));
+
+    if (lic)
+        FIXME("semi stub\n");
 
     if (ppUnkContainer) *ppUnkContainer = NULL;
     if (ppUnkControl) *ppUnkControl = NULL;
@@ -1427,28 +1430,21 @@ INT_PTR WINAPI AtlAxDialogBoxA(HINSTANCE hInstance, LPCSTR lpTemplateName, HWND
     return 0;
 }
 
-#if _ATL_VER >= _ATL_VER_80
-
 /***********************************************************************
  *           AtlAxCreateControlLic        [atl100.59]
  *
  */
 HRESULT WINAPI AtlAxCreateControlLic(const WCHAR *lpTricsData, HWND hwnd, IStream *stream, IUnknown **container, BSTR lic)
 {
-    FIXME("(%s %p %p %p %s)\n", debugstr_w(lpTricsData), hwnd, stream, container, debugstr_w(lic));
-    return E_NOTIMPL;
+    return AtlAxCreateControlLicEx(lpTricsData, hwnd, stream, container, NULL, NULL, NULL, lic);
 }
 
 /***********************************************************************
- *           AtlAxCreateControlLicEx      [atl100.60]
+ *           AtlAxCreateControlEx         [atl100.@]
  *
  */
-HRESULT WINAPI AtlAxCreateControlLicEx(const WCHAR *lpTricsData, HWND hwnd, IStream *stream,
-        IUnknown **container, IUnknown **control, REFIID iidSink, IUnknown *punkSink, BSTR lic)
+HRESULT WINAPI AtlAxCreateControlEx(const WCHAR *lpTricsData, HWND hwnd, IStream *stream,
+        IUnknown **container, IUnknown **control, REFIID iidSink, IUnknown *punkSink)
 {
-    FIXME("(%s %p %p %p %p %s %p %s)\n", debugstr_w(lpTricsData), hwnd, stream, container, control,
-          debugstr_guid(iidSink), punkSink, debugstr_w(lic));
-    return E_NOTIMPL;
+    return AtlAxCreateControlLicEx(lpTricsData, hwnd, stream, container, control, iidSink, punkSink, NULL);
 }
-
-#endif
