@@ -262,6 +262,7 @@ static void test_monthcal(void)
     SYSTEMTIME st[2], st1[2], today;
     int res, month_range;
     DWORD limits;
+    BOOL r;
 
     hwnd = CreateWindowA(MONTHCAL_CLASSA, "MonthCal", WS_POPUP | WS_VISIBLE, CW_USEDEFAULT,
                          0, 300, 300, 0, 0, NULL, NULL);
@@ -304,6 +305,9 @@ static void test_monthcal(void)
     expect(0, st[1].wSecond);
     expect(0, st[1].wMilliseconds);
 
+    limits = SendMessageA(hwnd, MCM_GETRANGE, 0, 0);
+    ok(limits == 0, "got %u\n", limits);
+
     GetSystemTime(&st[0]);
     st[1] = st[0];
 
@@ -450,6 +454,40 @@ static void test_monthcal(void)
     expect(0, st1[1].wSecond);
     expect(0, st1[1].wMilliseconds);
 
+    /* 0 limit flags */
+    limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
+    ok(limits == GDTR_MIN, "got 0x%08x\n", limits);
+
+    GetSystemTime(st);
+    st[1] = st[0];
+    st[1].wYear++;
+    r = SendMessageA(hwnd, MCM_SETRANGE, 0, (LPARAM)st);
+    ok(r, "got %d\n", r);
+
+    limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st);
+    ok(limits == 0, "got 0x%08x\n", limits);
+    ok(st[0].wYear == 0 && st[1].wYear == 0, "got %u, %u\n", st[0].wYear, st[1].wYear);
+
+    /* flags are 0, set min limit */
+    GetSystemTime(st);
+    st[1] = st[0];
+    st[1].wYear++;
+
+    r = SendMessageA(hwnd, MCM_SETRANGE, GDTR_MIN, (LPARAM)st);
+    ok(r, "got %d\n", r);
+
+    limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
+    ok(limits == GDTR_MIN, "got 0x%08x\n", limits);
+    ok(st1[1].wYear == 0, "got %u\n", st1[1].wYear);
+
+    /* now set max limit, check flags */
+    r = SendMessageA(hwnd, MCM_SETRANGE, GDTR_MAX, (LPARAM)st);
+    ok(r, "got %d\n", r);
+
+    limits = SendMessageA(hwnd, MCM_GETRANGE, 0, (LPARAM)st1);
+    ok(limits == GDTR_MAX, "got 0x%08x\n", limits);
+    ok(st1[0].wYear == 0, "got %u\n", st1[0].wYear);
+
     DestroyWindow(hwnd);
 }
 
@@ -612,7 +650,9 @@ static LRESULT WINAPI monthcal_subclass_proc(HWND hwnd, UINT message, WPARAM wPa
 static HWND create_monthcal_control(DWORD style)
 {
     WNDPROC oldproc;
+    RECT rect;
     HWND hwnd;
+    BOOL ret;
 
     hwnd = CreateWindowExA(0, MONTHCAL_CLASSA, "", WS_CHILD | WS_BORDER | WS_VISIBLE | style,
                     0, 0, 300, 400, parent_wnd, NULL, GetModuleHandleA(NULL), NULL);
@@ -625,6 +665,13 @@ static HWND create_monthcal_control(DWORD style)
 
     SendMessageA(hwnd, WM_SETFONT, (WPARAM)GetStockObject(SYSTEM_FONT), 0);
 
+    /* make sure calendar grid is 2x1 */
+    ret = SendMessageA(hwnd, MCM_GETMINREQRECT, 0, (LPARAM)&rect);
+    ok(ret, "got %d\n", ret);
+
+    ret = SetWindowPos(hwnd, NULL, 0, 0, rect.right * 5 / 2, rect.bottom * 3 / 2, SWP_NOMOVE);
+    ok(ret, "got %d\n", ret);
+
     return hwnd;
 }
 
@@ -1173,14 +1220,9 @@ if (0)
                 if (sizeof(title_hits) / sizeof(title_hits[0]) <= title_index)
                     break;
 
-                if (title_hits[title_index].todo) {
-                    todo_wine
+                todo_wine_if(title_hits[title_index].todo)
                     ok(title_hits[title_index].ht == res, "Expected %x, got %x, pos %d\n",
                                                           title_hits[title_index].ht, res, x);
-                } else {
-                    ok(title_hits[title_index].ht == res, "Expected %x, got %x, pos %d\n",
-                                                          title_hits[title_index].ht, res, x);
-                }
             }
             old_res = res;
         }
