@@ -52,11 +52,17 @@ struct d2d_clip_stack
     unsigned int count;
 };
 
+struct d2d_error_state
+{
+    HRESULT code;
+    D2D1_TAG tag1, tag2;
+};
+
 struct d2d_shape_resources
 {
     ID3D10InputLayout *il;
     ID3D10VertexShader *vs;
-    ID3D10PixelShader *ps[D2D_BRUSH_TYPE_COUNT];
+    ID3D10PixelShader *ps[D2D_BRUSH_TYPE_COUNT][D2D_BRUSH_TYPE_COUNT + 1];
 };
 
 struct d2d_d3d_render_target
@@ -76,8 +82,10 @@ struct d2d_d3d_render_target
     ID3D10RasterizerState *rs;
     ID3D10BlendState *bs;
 
+    struct d2d_error_state error;
     D2D1_DRAWING_STATE_DESCRIPTION drawing_state;
     IDWriteRenderingParams *text_rendering_params;
+    IDWriteRenderingParams *default_text_rendering_params;
 
     D2D1_PIXEL_FORMAT format;
     D2D1_SIZE_U pixel_size;
@@ -153,13 +161,13 @@ void d2d_solid_color_brush_init(struct d2d_brush *brush, ID2D1Factory *factory,
 void d2d_linear_gradient_brush_init(struct d2d_brush *brush, ID2D1Factory *factory,
         const D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES *gradient_brush_desc, const D2D1_BRUSH_PROPERTIES *brush_desc,
         ID2D1GradientStopCollection *gradient) DECLSPEC_HIDDEN;
-HRESULT d2d_bitmap_brush_init(struct d2d_brush *brush, ID2D1Factory *factory,
+void d2d_bitmap_brush_init(struct d2d_brush *brush, ID2D1Factory *factory,
         ID2D1Bitmap *bitmap, const D2D1_BITMAP_BRUSH_PROPERTIES *bitmap_brush_desc,
         const D2D1_BRUSH_PROPERTIES *brush_desc) DECLSPEC_HIDDEN;
-void d2d_brush_bind_resources(struct d2d_brush *brush, struct d2d_d3d_render_target *render_target,
-        enum d2d_shape_type shape_type) DECLSPEC_HIDDEN;
-HRESULT d2d_brush_get_ps_cb(struct d2d_brush *brush, struct d2d_d3d_render_target *render_target,
-        ID3D10Buffer **ps_cb) DECLSPEC_HIDDEN;
+void d2d_brush_bind_resources(struct d2d_brush *brush, struct d2d_brush *opacity_brush,
+        struct d2d_d3d_render_target *render_target, enum d2d_shape_type shape_type) DECLSPEC_HIDDEN;
+HRESULT d2d_brush_get_ps_cb(struct d2d_brush *brush, struct d2d_brush *opacity_brush,
+        struct d2d_d3d_render_target *render_target, ID3D10Buffer **ps_cb) DECLSPEC_HIDDEN;
 struct d2d_brush *unsafe_impl_from_ID2D1Brush(ID2D1Brush *iface) DECLSPEC_HIDDEN;
 
 struct d2d_stroke_style
@@ -249,6 +257,8 @@ struct d2d_geometry
 
     ID2D1Factory *factory;
 
+    D2D_MATRIX_3X2_F transform;
+
     D2D1_POINT_2F *vertices;
     size_t vertex_count;
 
@@ -277,12 +287,18 @@ struct d2d_geometry
         {
             D2D1_RECT_F rect;
         } rectangle;
+        struct
+        {
+            ID2D1Geometry *src_geometry;
+        } transformed;
     } u;
 };
 
 void d2d_path_geometry_init(struct d2d_geometry *geometry, ID2D1Factory *factory) DECLSPEC_HIDDEN;
 HRESULT d2d_rectangle_geometry_init(struct d2d_geometry *geometry,
         ID2D1Factory *factory, const D2D1_RECT_F *rect) DECLSPEC_HIDDEN;
+void d2d_transformed_geometry_init(struct d2d_geometry *geometry, ID2D1Factory *factory,
+        ID2D1Geometry *src_geometry, const D2D_MATRIX_3X2_F *transform) DECLSPEC_HIDDEN;
 struct d2d_geometry *unsafe_impl_from_ID2D1Geometry(ID2D1Geometry *iface) DECLSPEC_HIDDEN;
 
 static inline void d2d_matrix_multiply(D2D_MATRIX_3X2_F *a, const D2D_MATRIX_3X2_F *b)
