@@ -101,11 +101,27 @@ static HRESULT WINAPI layer_create(enum dxgi_device_layer_id id, void **layer_ba
     return S_OK;
 }
 
+static void WINAPI layer_set_feature_level(enum dxgi_device_layer_id id, void *device,
+        D3D_FEATURE_LEVEL feature_level)
+{
+    struct d3d_device *d3d_device = device;
+
+    TRACE("id %#x, device %p, feature_level %u.\n", id, device, feature_level);
+
+    if (id != DXGI_DEVICE_LAYER_D3D10_DEVICE)
+    {
+        WARN("Unknown layer id %#x.\n", id);
+        return;
+    }
+
+    d3d_device->feature_level = feature_level;
+}
+
 HRESULT WINAPI D3D11CoreRegisterLayers(void)
 {
     static const struct dxgi_device_layer layers[] =
     {
-        {DXGI_DEVICE_LAYER_D3D10_DEVICE, layer_init, layer_get_size, layer_create},
+        {DXGI_DEVICE_LAYER_D3D10_DEVICE, layer_init, layer_get_size, layer_create, layer_set_feature_level},
     };
 
     DXGID3D10RegisterLayers(layers, sizeof(layers)/sizeof(*layers));
@@ -123,10 +139,8 @@ HRESULT WINAPI D3D11CoreCreateDevice(IDXGIFactory *factory, IDXGIAdapter *adapte
     TRACE("factory %p, adapter %p, flags %#x, feature_levels %p, levels %u, device %p.\n",
             factory, adapter, flags, feature_levels, levels, device);
 
-    FIXME("Ignoring feature levels.\n");
-
     d3d11 = GetModuleHandleA("d3d11.dll");
-    hr = DXGID3D10CreateDevice(d3d11, factory, adapter, flags, 0, (void **)&dxgi_device);
+    hr = DXGID3D10CreateDevice(d3d11, factory, adapter, flags, feature_levels, levels, (void **)&dxgi_device);
     if (FAILED(hr))
     {
         WARN("Failed to create device, returning %#x.\n", hr);
@@ -148,6 +162,15 @@ HRESULT WINAPI D3D11CreateDevice(IDXGIAdapter *adapter, D3D_DRIVER_TYPE driver_t
         const D3D_FEATURE_LEVEL *feature_levels, UINT levels, UINT sdk_version, ID3D11Device **device_out,
         D3D_FEATURE_LEVEL *obtained_feature_level, ID3D11DeviceContext **immediate_context)
 {
+    static const D3D_FEATURE_LEVEL default_feature_levels[] =
+    {
+        D3D_FEATURE_LEVEL_11_0,
+        D3D_FEATURE_LEVEL_10_1,
+        D3D_FEATURE_LEVEL_10_0,
+        D3D_FEATURE_LEVEL_9_3,
+        D3D_FEATURE_LEVEL_9_2,
+        D3D_FEATURE_LEVEL_9_1,
+    };
     IDXGIFactory *factory;
     ID3D11Device *device;
     HRESULT hr;
@@ -176,7 +199,7 @@ HRESULT WINAPI D3D11CreateDevice(IDXGIAdapter *adapter, D3D_DRIVER_TYPE driver_t
     }
     else
     {
-        hr = CreateDXGIFactory(&IID_IDXGIFactory, (void **)&factory);
+        hr = CreateDXGIFactory1(&IID_IDXGIFactory, (void **)&factory);
         if (FAILED(hr))
         {
             WARN("Failed to create dxgi factory, returning %#x.\n", hr);
@@ -245,6 +268,11 @@ HRESULT WINAPI D3D11CreateDevice(IDXGIAdapter *adapter, D3D_DRIVER_TYPE driver_t
         }
     }
 
+    if (!feature_levels)
+    {
+        feature_levels = default_feature_levels;
+        levels = sizeof(default_feature_levels) / sizeof(default_feature_levels[0]);
+    }
     hr = D3D11CoreCreateDevice(factory, adapter, flags, feature_levels, levels, &device);
     IDXGIAdapter_Release(adapter);
     IDXGIFactory_Release(factory);
@@ -331,7 +359,7 @@ HRESULT WINAPI D3D11CreateDeviceAndSwapChain(IDXGIAdapter *adapter, D3D_DRIVER_T
             goto cleanup;
         }
 
-        TRACE("Created IDXGISwapChain %p.\n", swapchain);
+        TRACE("Created IDXGISwapChain %p.\n", *swapchain);
     }
 
     if (device_out)
@@ -342,14 +370,12 @@ HRESULT WINAPI D3D11CreateDeviceAndSwapChain(IDXGIAdapter *adapter, D3D_DRIVER_T
     return S_OK;
 
 cleanup:
-    if (device)
-        ID3D11Device_Release(device);
+    ID3D11Device_Release(device);
     if (obtained_feature_level)
         *obtained_feature_level = 0;
     if (immediate_context)
     {
-        /* FIXME: Remove the following NULL check once the d3d11_device_GetImmediateContext() is implemented. */
-        if (*immediate_context) ID3D11DeviceContext_Release(*immediate_context);
+        ID3D11DeviceContext_Release(*immediate_context);
         *immediate_context = NULL;
     }
 
