@@ -24,22 +24,22 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d11);
 
-#define WINE_D3D10_TO_STR(x) case x: return #x
+#define WINE_D3D_TO_STR(x) case x: return #x
 
 const char *debug_d3d10_primitive_topology(D3D10_PRIMITIVE_TOPOLOGY topology)
 {
     switch (topology)
     {
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_UNDEFINED);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_POINTLIST);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINELIST);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINELIST_ADJ);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP_ADJ);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST_ADJ);
-        WINE_D3D10_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP_ADJ);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_UNDEFINED);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_POINTLIST);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINELIST);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINELIST_ADJ);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP_ADJ);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST_ADJ);
+        WINE_D3D_TO_STR(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP_ADJ);
         default:
             FIXME("Unrecognized D3D10_PRIMITIVE_TOPOLOGY %#x\n", topology);
             return "unrecognized";
@@ -50,102 +50,117 @@ const char *debug_dxgi_format(DXGI_FORMAT format)
 {
     switch(format)
     {
-        WINE_D3D10_TO_STR(DXGI_FORMAT_UNKNOWN);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32A32_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32A32_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32A32_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32A32_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32B32_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16B16A16_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G32_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32G8X24_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_D32_FLOAT_S8X24_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_X32_TYPELESS_G8X24_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R10G10B10A2_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R10G10B10A2_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R10G10B10A2_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R11G11B10_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_UNORM_SRGB);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8B8A8_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16G16_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_D32_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R32_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R24G8_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_D24_UNORM_S8_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R24_UNORM_X8_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_X24_TYPELESS_G8_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_FLOAT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_D16_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R16_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8_UINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8_SINT);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_A8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R1_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R9G9B9E5_SHAREDEXP);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_R8G8_B8G8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_G8R8_G8B8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC1_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC1_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC1_UNORM_SRGB);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC2_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC2_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC2_UNORM_SRGB);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC3_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC3_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC3_UNORM_SRGB);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC4_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC4_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC4_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC5_TYPELESS);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC5_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_BC5_SNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_B5G6R5_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_B5G5R5A1_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_B8G8R8A8_UNORM);
-        WINE_D3D10_TO_STR(DXGI_FORMAT_B8G8R8X8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_UNKNOWN);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32A32_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32A32_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32A32_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32A32_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32B32_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16B16A16_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G32_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32G8X24_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_D32_FLOAT_S8X24_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_X32_TYPELESS_G8X24_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R10G10B10A2_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R10G10B10A2_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R10G10B10A2_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R11G11B10_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8B8A8_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16G16_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_D32_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R32_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R24G8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_D24_UNORM_S8_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R24_UNORM_X8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_X24_TYPELESS_G8_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_FLOAT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_D16_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R16_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8_UINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8_SINT);
+        WINE_D3D_TO_STR(DXGI_FORMAT_A8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R1_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R9G9B9E5_SHAREDEXP);
+        WINE_D3D_TO_STR(DXGI_FORMAT_R8G8_B8G8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_G8R8_G8B8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC1_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC1_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC1_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC2_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC2_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC2_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC3_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC3_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC3_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC4_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC4_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC4_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC5_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC5_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC5_SNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B5G6R5_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B5G5R5A1_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8A8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8X8_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8A8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8A8_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8X8_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_B8G8R8X8_UNORM_SRGB);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC7_TYPELESS);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC7_UNORM);
+        WINE_D3D_TO_STR(DXGI_FORMAT_BC7_UNORM_SRGB);
         default:
-            FIXME("Unrecognized DXGI_FORMAT %#x\n", format);
+            FIXME("Unrecognized DXGI_FORMAT %#x.\n", format);
             return "unrecognized";
     }
 }
 
-#undef WINE_D3D10_TO_STR
+#undef WINE_D3D_TO_STR
+
+const char *debug_float4(const float *values)
+{
+    if (!values)
+        return "(null)";
+    return wine_dbg_sprintf("{%.8e, %.8e, %.8e, %.8e}",
+            values[0], values[1], values[2], values[3]);
+}
 
 DXGI_FORMAT dxgi_format_from_wined3dformat(enum wined3d_format_id format)
 {
@@ -240,6 +255,13 @@ DXGI_FORMAT dxgi_format_from_wined3dformat(enum wined3d_format_id format)
         case WINED3DFMT_B5G5R5A1_UNORM: return DXGI_FORMAT_B5G5R5A1_UNORM;
         case WINED3DFMT_B8G8R8A8_UNORM: return DXGI_FORMAT_B8G8R8A8_UNORM;
         case WINED3DFMT_B8G8R8X8_UNORM: return DXGI_FORMAT_B8G8R8X8_UNORM;
+        case WINED3DFMT_B8G8R8A8_TYPELESS: return DXGI_FORMAT_B8G8R8A8_TYPELESS;
+        case WINED3DFMT_B8G8R8A8_UNORM_SRGB: return DXGI_FORMAT_B8G8R8A8_UNORM_SRGB;
+        case WINED3DFMT_B8G8R8X8_TYPELESS: return DXGI_FORMAT_B8G8R8X8_TYPELESS;
+        case WINED3DFMT_B8G8R8X8_UNORM_SRGB: return DXGI_FORMAT_B8G8R8X8_UNORM_SRGB;
+        case WINED3DFMT_BC7_TYPELESS: return DXGI_FORMAT_BC7_TYPELESS;
+        case WINED3DFMT_BC7_UNORM: return DXGI_FORMAT_BC7_UNORM;
+        case WINED3DFMT_BC7_UNORM_SRGB: return DXGI_FORMAT_BC7_UNORM_SRGB;
         default:
             FIXME("Unhandled wined3d format %#x.\n", format);
             return DXGI_FORMAT_UNKNOWN;
@@ -339,8 +361,15 @@ enum wined3d_format_id wined3dformat_from_dxgi_format(DXGI_FORMAT format)
         case DXGI_FORMAT_B5G5R5A1_UNORM: return WINED3DFMT_B5G5R5A1_UNORM;
         case DXGI_FORMAT_B8G8R8A8_UNORM: return WINED3DFMT_B8G8R8A8_UNORM;
         case DXGI_FORMAT_B8G8R8X8_UNORM: return WINED3DFMT_B8G8R8X8_UNORM;
+        case DXGI_FORMAT_B8G8R8A8_TYPELESS: return WINED3DFMT_B8G8R8A8_TYPELESS;
+        case DXGI_FORMAT_B8G8R8A8_UNORM_SRGB: return WINED3DFMT_B8G8R8A8_UNORM_SRGB;
+        case DXGI_FORMAT_B8G8R8X8_TYPELESS: return WINED3DFMT_B8G8R8X8_TYPELESS;
+        case DXGI_FORMAT_B8G8R8X8_UNORM_SRGB: return WINED3DFMT_B8G8R8X8_UNORM_SRGB;
+        case DXGI_FORMAT_BC7_TYPELESS: return WINED3DFMT_BC7_TYPELESS;
+        case DXGI_FORMAT_BC7_UNORM: return WINED3DFMT_BC7_UNORM;
+        case DXGI_FORMAT_BC7_UNORM_SRGB: return WINED3DFMT_BC7_UNORM_SRGB;
         default:
-            FIXME("Unhandled DXGI_FORMAT %#x\n", format);
+            FIXME("Unhandled DXGI_FORMAT %#x.\n", format);
             return WINED3DFMT_UNKNOWN;
     }
 }
@@ -518,6 +547,10 @@ struct wined3d_resource *wined3d_resource_from_d3d11_resource(ID3D11Resource *re
             return wined3d_texture_get_resource(unsafe_impl_from_ID3D11Texture2D(
                     (ID3D11Texture2D *)resource)->wined3d_texture);
 
+        case D3D11_RESOURCE_DIMENSION_TEXTURE3D:
+            return wined3d_texture_get_resource(unsafe_impl_from_ID3D11Texture3D(
+                    (ID3D11Texture3D *)resource)->wined3d_texture);
+
         default:
             FIXME("Unhandled resource dimension %#x.\n", dimension);
             return NULL;
@@ -546,20 +579,20 @@ struct wined3d_resource *wined3d_resource_from_d3d10_resource(ID3D10Resource *re
     }
 }
 
-DWORD wined3d_map_flags_from_d3d10_map_type(D3D10_MAP map_type)
+DWORD wined3d_map_flags_from_d3d11_map_type(D3D11_MAP map_type)
 {
     switch (map_type)
     {
-        case D3D10_MAP_READ_WRITE:
+        case D3D11_MAP_READ_WRITE:
             return 0;
 
-        case D3D10_MAP_READ:
+        case D3D11_MAP_READ:
             return WINED3D_MAP_READONLY;
 
-        case D3D10_MAP_WRITE_DISCARD:
+        case D3D11_MAP_WRITE_DISCARD:
             return WINED3D_MAP_DISCARD;
 
-        case D3D10_MAP_WRITE_NO_OVERWRITE:
+        case D3D11_MAP_WRITE_NO_OVERWRITE:
             return WINED3D_MAP_NOOVERWRITE;
 
         default:
@@ -568,6 +601,24 @@ DWORD wined3d_map_flags_from_d3d10_map_type(D3D10_MAP map_type)
     }
 }
 
+DWORD wined3d_clear_flags_from_d3d11_clear_flags(UINT clear_flags)
+{
+    DWORD wined3d_clear_flags = 0;
+
+    if (clear_flags & D3D11_CLEAR_DEPTH)
+        wined3d_clear_flags |= WINED3DCLEAR_ZBUFFER;
+    if (clear_flags & D3D11_CLEAR_STENCIL)
+        wined3d_clear_flags |= WINED3DCLEAR_STENCIL;
+
+    if (clear_flags & ~(D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL))
+    {
+        FIXME("Unhandled clear flags %#x.\n",
+                clear_flags & ~(D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL));
+    }
+
+    return wined3d_clear_flags;
+}
+
 HRESULT d3d_get_private_data(struct wined3d_private_store *store,
         REFGUID guid, UINT *data_size, void *data)
 {
