@@ -1069,7 +1069,7 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
             return hr;
     }
 
-    TRACE("Mesh name is '%s'\n", This->name ? This->name : "");
+    TRACE("Mesh name is %s\n", debugstr_a(This->name));
 
     This->nb_normals = 0;
 
@@ -1250,20 +1250,21 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
                     IDirectXFileData *data;
                     char **filename;
 
-                    hr = IDirectXFileObject_QueryInterface(material_child, &IID_IDirectXFileData, (void **)&data);
-                    if (FAILED(hr))
+                    if (FAILED(hr = IDirectXFileObject_QueryInterface(material_child,
+                            &IID_IDirectXFileData, (void **)&data)))
                     {
                         IDirectXFileDataReference *reference;
 
-                        hr = IDirectXFileObject_QueryInterface(material_child, &IID_IDirectXFileDataReference, (void **)&reference);
-                        if (FAILED(hr))
-                            goto end;
-
-                        hr = IDirectXFileDataReference_Resolve(reference, &data);
-                        IDirectXFileDataReference_Release(reference);
-                        if (FAILED(hr))
-                            goto end;
+                        if (SUCCEEDED(IDirectXFileObject_QueryInterface(material_child,
+                                &IID_IDirectXFileDataReference, (void **)&reference)))
+                        {
+                            hr = IDirectXFileDataReference_Resolve(reference, &data);
+                            IDirectXFileDataReference_Release(reference);
+                        }
                     }
+                    IDirectXFileObject_Release(material_child);
+                    if (FAILED(hr))
+                        goto end;
 
                     hr = IDirectXFileData_GetType(data, &guid);
                     if (hr != DXFILE_OK)
@@ -1308,6 +1309,7 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
                             }
                         }
                     }
+                    IDirectXFileData_Release(data);
                 }
                 else if (hr != DXFILEERR_NOMOREOBJECTS)
                 {
@@ -1366,7 +1368,10 @@ HRESULT load_mesh_data(IDirect3DRMMeshBuilder3 *iface, IDirectXFileData *pData,
         if (!This->nb_normals)
         {
             /* Compute face normal */
-            if (nb_face_indexes > 2)
+            if (nb_face_indexes > 2
+                    && faces_vertex_idx_ptr[0] < This->nb_vertices
+                    && faces_vertex_idx_ptr[1] < This->nb_vertices
+                    && faces_vertex_idx_ptr[2] < This->nb_vertices)
             {
                 D3DVECTOR a, b;
 
