@@ -60,6 +60,7 @@ static void ddraw_enumerate_secondary_devices(struct wined3d *wined3d, LPDDENUMC
                                               void *context)
 {
     struct wined3d_adapter_identifier adapter_id;
+    struct wined3d_output_desc output_desc;
     BOOL cont_enum = TRUE;
     HRESULT hr = S_OK;
     UINT adapter = 0;
@@ -76,13 +77,14 @@ static void ddraw_enumerate_secondary_devices(struct wined3d *wined3d, LPDDENUMC
         adapter_id.description = DriverDescription;
         adapter_id.description_size = sizeof(DriverDescription);
         wined3d_mutex_lock();
-        hr = wined3d_get_adapter_identifier(wined3d, adapter, 0x0, &adapter_id);
+        if (SUCCEEDED(hr = wined3d_get_adapter_identifier(wined3d, adapter, 0x0, &adapter_id)))
+            hr = wined3d_get_output_desc(wined3d, adapter, &output_desc);
         wined3d_mutex_unlock();
         if (SUCCEEDED(hr))
         {
             TRACE("Interface %d: %s\n", adapter, wine_dbgstr_guid(&adapter_id.device_identifier));
             cont_enum = callback(&adapter_id.device_identifier, adapter_id.description,
-                    adapter_id.device_name, context, wined3d_get_adapter_monitor(wined3d, adapter));
+                    adapter_id.device_name, context, output_desc.monitor);
         }
     }
 }
