@@ -184,10 +184,6 @@ static void test_COM_track(void)
         /* Same refcount for all DirectMusicTrack interfaces */
         hr = CoCreateInstance(class[i].clsid, NULL, CLSCTX_INPROC_SERVER, &IID_IDirectMusicTrack8,
                 (void**)&dmt8);
-        if (hr == E_NOINTERFACE && !dmt8) {
-            skip("%s not created with CoCreateInstance()\n", class[i].name);
-            continue;
-        }
         ok(hr == S_OK, "%s create failed: %08x, expected S_OK\n", class[i].name, hr);
         refcount = IDirectMusicTrack8_AddRef(dmt8);
         ok(refcount == 2, "refcount == %u, expected 2\n", refcount);
@@ -251,14 +247,14 @@ static void test_track(void)
         REFCLSID clsid;
         const char *name;
         BOOL has_save;
-        BOOL todo;
+        BOOL has_join;
     } class[] = {
-        { X(DirectMusicAuditionTrack), TRUE, TRUE },
-        { X(DirectMusicChordTrack), TRUE, FALSE },
+        { X(DirectMusicAuditionTrack), TRUE, FALSE },
+        { X(DirectMusicChordTrack), TRUE, TRUE },
         { X(DirectMusicCommandTrack), TRUE, TRUE },
-        { X(DirectMusicMotifTrack), FALSE, TRUE },
-        { X(DirectMusicMuteTrack), TRUE, TRUE },
-        { X(DirectMusicStyleTrack), FALSE, FALSE },
+        { X(DirectMusicMotifTrack), FALSE, FALSE },
+        { X(DirectMusicMuteTrack), TRUE, FALSE },
+        { X(DirectMusicStyleTrack), FALSE, TRUE },
     };
 #undef X
     unsigned int i;
@@ -267,42 +263,75 @@ static void test_track(void)
         trace("Testing %s\n", class[i].name);
         hr = CoCreateInstance(class[i].clsid, NULL, CLSCTX_INPROC_SERVER, &IID_IDirectMusicTrack8,
                 (void**)&dmt8);
-        if (hr == E_NOINTERFACE && !dmt8) {
-            skip("%s not created with CoCreateInstance()\n", class[i].name);
-            continue;
-        }
         ok(hr == S_OK, "%s create failed: %08x, expected S_OK\n", class[i].name, hr);
 
         /* IDirectMusicTrack8 */
+        hr = IDirectMusicTrack8_Init(dmt8, NULL);
+        todo_wine ok(hr == E_POINTER, "IDirectMusicTrack8_Init failed: %08x\n", hr);
+        if (class[i].clsid != &CLSID_DirectMusicChordTrack &&
+                class[i].clsid != &CLSID_DirectMusicCommandTrack) {
+            /* Crashes on native */
+            hr = IDirectMusicTrack8_InitPlay(dmt8, NULL, NULL, NULL, 0, 0);
+            if (class[i].clsid == &CLSID_DirectMusicMuteTrack)
+                ok(hr == S_OK, "IDirectMusicTrack8_InitPlay failed: %08x\n", hr);
+            else
+                todo_wine ok(hr == E_POINTER, "IDirectMusicTrack8_InitPlay failed: %08x\n", hr);
+        }
+        todo_wine {
+        hr = IDirectMusicTrack8_GetParam(dmt8, NULL, 0, NULL, NULL);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_GetParam failed: %08x\n", hr);
+        hr = IDirectMusicTrack8_SetParam(dmt8, NULL, 0, NULL);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_SetParam failed: %08x\n", hr);
+        }
         hr = IDirectMusicTrack8_IsParamSupported(dmt8, NULL);
         ok(hr == E_POINTER, "IDirectMusicTrack8_IsParamSupported failed: %08x\n", hr);
+        if (class[i].clsid == &CLSID_DirectMusicMuteTrack) {
+            hr = IDirectMusicTrack8_AddNotificationType(dmt8, NULL);
+            ok(hr == E_NOTIMPL, "IDirectMusicTrack8_AddNotificationType failed: %08x\n", hr);
+            hr = IDirectMusicTrack8_RemoveNotificationType(dmt8, NULL);
+            ok(hr == E_NOTIMPL, "IDirectMusicTrack8_RemoveNotificationType failed: %08x\n", hr);
+        } else todo_wine {
+            hr = IDirectMusicTrack8_AddNotificationType(dmt8, NULL);
+            ok(hr == E_POINTER, "IDirectMusicTrack8_AddNotificationType failed: %08x\n", hr);
+            hr = IDirectMusicTrack8_RemoveNotificationType(dmt8, NULL);
+            ok(hr == E_POINTER, "IDirectMusicTrack8_RemoveNotificationType failed: %08x\n", hr);
+        }
+        todo_wine {
+        hr = IDirectMusicTrack8_Clone(dmt8, 0, 0, NULL);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_Clone failed: %08x\n", hr);
+        hr = IDirectMusicTrack8_PlayEx(dmt8, NULL, 0, 0, 0, 0, NULL, NULL, 0);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_PlayEx failed: %08x\n", hr);
+        hr = IDirectMusicTrack8_GetParamEx(dmt8, NULL, 0, NULL, NULL, NULL, 0);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_GetParamEx failed: %08x\n", hr);
+        hr = IDirectMusicTrack8_SetParamEx(dmt8, NULL, 0, NULL, NULL, 0);
+        ok(hr == E_POINTER, "IDirectMusicTrack8_SetParamEx failed: %08x\n", hr);
+        }
+        hr = IDirectMusicTrack8_Compose(dmt8, NULL, 0, NULL);
+        ok(hr == E_NOTIMPL, "IDirectMusicTrack8_Compose failed: %08x\n", hr);
+        if (class[i].has_join) {
+            hr = IDirectMusicTrack8_Join(dmt8, NULL, 0, NULL, 0, NULL);
+            todo_wine ok(hr == E_POINTER, "IDirectMusicTrack8_Join failed: %08x\n", hr);
+        } else {
+            hr = IDirectMusicTrack8_Join(dmt8, NULL, 0, NULL, 0, NULL);
+            ok(hr == E_NOTIMPL, "IDirectMusicTrack8_Join failed: %08x\n", hr);
+        }
 
         /* IPersistStream */
         hr = IDirectMusicTrack8_QueryInterface(dmt8, &IID_IPersistStream, (void**)&ps);
         ok(hr == S_OK, "QueryInterface for IID_IPersistStream failed: %08x\n", hr);
         hr = IPersistStream_GetClassID(ps, &classid);
-        if (class[i].todo) {
-            todo_wine {
-                ok(hr == S_OK, "IPersistStream_GetClassID failed: %08x\n", hr);
-                ok(IsEqualGUID(&classid, class[i].clsid),
-                        "Expected class %s got %s\n", class[i].name, wine_dbgstr_guid(&classid));
-                hr = IPersistStream_IsDirty(ps);
-                ok(hr == S_FALSE, "IPersistStream_IsDirty failed: %08x\n", hr);
-            }
-        } else {
-            ok(hr == S_OK, "IPersistStream_GetClassID failed: %08x\n", hr);
-            ok(IsEqualGUID(&classid, class[i].clsid),
-                    "Expected class %s got %s\n", class[i].name, wine_dbgstr_guid(&classid));
-            hr = IPersistStream_IsDirty(ps);
-            ok(hr == S_FALSE, "IPersistStream_IsDirty failed: %08x\n", hr);
-        }
+        ok(hr == S_OK, "IPersistStream_GetClassID failed: %08x\n", hr);
+        ok(IsEqualGUID(&classid, class[i].clsid),
+                "Expected class %s got %s\n", class[i].name, wine_dbgstr_guid(&classid));
+        hr = IPersistStream_IsDirty(ps);
+        ok(hr == S_FALSE, "IPersistStream_IsDirty failed: %08x\n", hr);
 
         hr = IPersistStream_GetSizeMax(ps, &size);
         ok(hr == E_NOTIMPL, "IPersistStream_GetSizeMax failed: %08x\n", hr);
 
         hr = IPersistStream_Save(ps, NULL, TRUE);
         if (class[i].has_save)
-            todo_wine ok(hr == E_POINTER, "IPersistStream_Save failed: %08x\n", hr);
+            ok(hr == E_POINTER, "IPersistStream_Save failed: %08x\n", hr);
         else
             ok(hr == E_NOTIMPL, "IPersistStream_Save failed: %08x\n", hr);
 
