@@ -805,9 +805,7 @@ static HRESULT test_secondary(LPGUID lpGuid, int play,
         bufdesc.dwFlags=DSBCAPS_GETCURRENTPOSITION2;
         if (has_3dbuffer)
             bufdesc.dwFlags|=DSBCAPS_CTRL3D;
-        else
-            bufdesc.dwFlags|=
-                (DSBCAPS_CTRLFREQUENCY|DSBCAPS_CTRLVOLUME|DSBCAPS_CTRLPAN);
+        bufdesc.dwFlags|= DSBCAPS_CTRLFREQUENCY|DSBCAPS_CTRLVOLUME|DSBCAPS_CTRLPAN;
         bufdesc.dwBufferBytes=align(wfx.nAvgBytesPerSec*BUFFER_LEN/1000,
                                     wfx.nBlockAlign);
         bufdesc.lpwfxFormat=&wfx;
@@ -888,7 +886,15 @@ static HRESULT test_secondary(LPGUID lpGuid, int play,
                 ok(rc==DS_OK,"IDirectSoundBuffer_SetVolume(secondary) failed: %08x\n",rc);
                 rc=IDirectSoundBuffer_SetPan(secondary,0);
                 ok(rc==DS_OK,"IDirectSoundBuffer_SetPan(secondary) failed: %08x\n",rc);
+            } else if (has_3dbuffer) {
+                LONG pan;
+
+                rc=IDirectSoundBuffer_GetPan(secondary,&pan);
+                ok(rc==DS_OK,"IDirectSoundBuffer_GetPan() failed, returned: %08x\n", rc);
+                rc=IDirectSoundBuffer_SetPan(secondary,0);
+                ok(rc==DS_OK,"IDirectSoundBuffer_SetPan() failed, returned: %08x\n", rc);
             }
+
             if (has_duplicate) {
                 LPDIRECTSOUNDBUFFER duplicated=NULL;
 
