@@ -22,8 +22,6 @@
 #include "config.h"
 #include <stdarg.h>
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #define COBJMACROS
 #include "windef.h"
 #include "winbase.h"
@@ -252,3 +250,12 @@ HRESULT WINAPI DwmDetachMilContent(HWND hwnd)
     FIXME("(%p) stub\n", hwnd);
     return E_NOTIMPL;
 }
+
+/**********************************************************************
+ *           DwmUpdateThumbnailProperties         (DWMAPI.@)
+ */
+HRESULT WINAPI DwmUpdateThumbnailProperties(HTHUMBNAIL thumbnail, const DWM_THUMBNAIL_PROPERTIES *props)
+{
+    FIXME("(%p, %p) stub\n", thumbnail, props);
+    return E_NOTIMPL;
+}
