@@ -52,7 +52,7 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(bidi);
 
-extern const unsigned short bidi_bracket_table[];
+extern const unsigned short bidi_bracket_table[] DECLSPEC_HIDDEN;
 
 #define ASSERT(x) do { if (!(x)) FIXME("assert failed: %s\n", #x); } while(0)
 #define MAX_DEPTH 125
@@ -199,7 +199,7 @@ static void bidi_classify(const WCHAR *string, UINT8 *chartype, UINT32 count)
 
 WCHAR bidi_get_mirrored_char(WCHAR ch)
 {
-    extern const WCHAR wine_mirror_map[];
+    extern const WCHAR wine_mirror_map[] DECLSPEC_HIDDEN;
     return ch + wine_mirror_map[wine_mirror_map[ch >> 8] + (ch & 0xff)];
 }
 
@@ -543,6 +543,11 @@ static inline void iso_dump_types(const char* header, IsolatedRun *run)
     Note: On input only these directional classes are expected
           AL, HL, R, L,  ON, BN, NSM, AN, EN, ES, ET, CS,
 ------------------------------------------------------------------------*/
+static BOOL bidi_is_isolate(UINT8 class)
+{
+    return class == LRI || class == RLI || class == FSI || class == PDI;
+}
+
 static void bidi_resolve_weak(IsolatedRun *iso_run)
 {
     int i;
@@ -553,7 +558,7 @@ static void bidi_resolve_weak(IsolatedRun *iso_run)
             int j = get_prev_valid_char_from_run(iso_run, i);
             if (j == -1)
                 *iso_run->item[i].class = iso_run->sos;
-            else if (*iso_run->item[j].class >= LRI)
+            else if (bidi_is_isolate(*iso_run->item[j].class))
                 *iso_run->item[i].class = ON;
             else
                 *iso_run->item[i].class = *iso_run->item[j].class;
@@ -754,16 +759,18 @@ static void bidi_resolve_neutrals(IsolatedRun *run)
 
     /* Translate isolates into NI */
     for (i = 0; i < run->length; i++) {
-        if (*run->item[i].class >= LRI)
-            *run->item[i].class = NI;
-
         switch (*run->item[i].class) {
             case B:
             case S:
-            case WS: *run->item[i].class = NI;
+            case WS:
+            case FSI:
+            case LRI:
+            case RLI:
+            case PDI: *run->item[i].class = NI;
         }
 
-        ASSERT(*run->item[i].class < 5 || *run->item[i].class == BN); /* "Only NI, L, R,  AN, EN and BN are allowed" */
+        /* "Only NI, L, R, AN, EN and BN are allowed" */
+        ASSERT(*run->item[i].class <= EN || *run->item[i].class == BN);
     }
 
     /* N0: Skipping bracketed pairs for now */
@@ -895,8 +902,8 @@ static void bidi_resolve_implicit(const UINT8 *classes, UINT8 *levels, int sos,
         if (classes[i] == BN)
             continue;
 
-        ASSERT(classes[i] > 0); /* "No Neutrals allowed to survive here." */
-        ASSERT(classes[i] < 5); /* "Out of range." */
+        ASSERT(classes[i] != ON); /* "No Neutrals allowed to survive here." */
+        ASSERT(classes[i] <= EN); /* "Out of range." */
 
         if (odd(levels[i]) && (classes[i] == L || classes[i] == EN || classes[i] == AN))
             levels[i]++;
