@@ -32,7 +32,6 @@
 #endif /* HAVE_FT2BUILD_H */
 
 #include "windef.h"
-#include "dwrite_2.h"
 #include "wine/library.h"
 #include "wine/debug.h"
 
@@ -65,15 +64,19 @@ typedef struct
 
 #define MAKE_FUNCPTR(f) static typeof(f) * p##f = NULL
 MAKE_FUNCPTR(FT_Done_FreeType);
+MAKE_FUNCPTR(FT_Done_Glyph);
 MAKE_FUNCPTR(FT_Get_First_Char);
 MAKE_FUNCPTR(FT_Get_Kerning);
 MAKE_FUNCPTR(FT_Get_Sfnt_Table);
+MAKE_FUNCPTR(FT_Glyph_Copy);
 MAKE_FUNCPTR(FT_Glyph_Get_CBox);
+MAKE_FUNCPTR(FT_Glyph_Transform);
 MAKE_FUNCPTR(FT_Init_FreeType);
 MAKE_FUNCPTR(FT_Library_Version);
 MAKE_FUNCPTR(FT_Load_Glyph);
 MAKE_FUNCPTR(FT_New_Memory_Face);
 MAKE_FUNCPTR(FT_Outline_Copy);
+MAKE_FUNCPTR(FT_Outline_Decompose);
 MAKE_FUNCPTR(FT_Outline_Done);
 MAKE_FUNCPTR(FT_Outline_Get_Bitmap);
 MAKE_FUNCPTR(FT_Outline_New);
@@ -148,15 +151,19 @@ BOOL init_freetype(void)
 
 #define LOAD_FUNCPTR(f) if((p##f = wine_dlsym(ft_handle, #f, NULL, 0)) == NULL){WARN("Can't find symbol %s\n", #f); goto sym_not_found;}
     LOAD_FUNCPTR(FT_Done_FreeType)
+    LOAD_FUNCPTR(FT_Done_Glyph)
     LOAD_FUNCPTR(FT_Get_First_Char)
     LOAD_FUNCPTR(FT_Get_Kerning)
     LOAD_FUNCPTR(FT_Get_Sfnt_Table)
+    LOAD_FUNCPTR(FT_Glyph_Copy)
     LOAD_FUNCPTR(FT_Glyph_Get_CBox)
+    LOAD_FUNCPTR(FT_Glyph_Transform)
     LOAD_FUNCPTR(FT_Init_FreeType)
     LOAD_FUNCPTR(FT_Library_Version)
     LOAD_FUNCPTR(FT_Load_Glyph)
     LOAD_FUNCPTR(FT_New_Memory_Face)
     LOAD_FUNCPTR(FT_Outline_Copy)
+    LOAD_FUNCPTR(FT_Outline_Decompose)
     LOAD_FUNCPTR(FT_Outline_Done)
     LOAD_FUNCPTR(FT_Outline_Get_Bitmap)
     LOAD_FUNCPTR(FT_Outline_New)
@@ -210,14 +217,14 @@ void release_freetype(void)
     pFT_Done_FreeType(library);
 }
 
-void freetype_notify_cacheremove(IDWriteFontFace2 *fontface)
+void freetype_notify_cacheremove(IDWriteFontFace3 *fontface)
 {
     EnterCriticalSection(&freetype_cs);
     pFTC_Manager_RemoveFaceID(cache_manager, fontface);
     LeaveCriticalSection(&freetype_cs);
 }
 
-HRESULT freetype_get_design_glyph_metrics(IDWriteFontFace2 *fontface, UINT16 unitsperEm, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
+HRESULT freetype_get_design_glyph_metrics(IDWriteFontFace3 *fontface, UINT16 unitsperEm, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
 {
     FTC_ScalerRec scaler;
     FT_Size size;
@@ -248,29 +255,82 @@ HRESULT freetype_get_design_glyph_metrics(IDWriteFontFace2 *fontface, UINT16 uni
     return S_OK;
 }
 
-BOOL freetype_is_monospaced(IDWriteFontFace2 *fontface)
+BOOL freetype_is_monospaced(IDWriteFontFace3 *fontface)
 {
     BOOL is_monospaced = FALSE;
     FT_Face face;
 
     EnterCriticalSection(&freetype_cs);
     if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
-        is_monospaced = FT_IS_FIXED_WIDTH(face);
+        is_monospaced = !!FT_IS_FIXED_WIDTH(face);
     LeaveCriticalSection(&freetype_cs);
 
     return is_monospaced;
 }
 
-static inline void ft_vector_to_d2d_point(const FT_Vector *v, D2D1_POINT_2F *p)
+struct decompose_context {
+    IDWriteGeometrySink *sink;
+    FLOAT xoffset;
+    FLOAT yoffset;
+    BOOL figure_started;
+    BOOL figure_closed;
+    BOOL move_to;     /* last call was 'move_to' */
+    FT_Vector origin; /* 'pen' position from last call */
+};
+
+static inline void ft_vector_to_d2d_point(const FT_Vector *v, FLOAT xoffset, FLOAT yoffset, D2D1_POINT_2F *p)
 {
-    p->x = v->x / 64.0;
-    p->y = v->y / 64.0;
+    p->x = (v->x / 64.0f) + xoffset;
+    p->y = (v->y / 64.0f) + yoffset;
 }
 
-/* Convert the quadratic Beziers to cubic Beziers. */
-static void get_cubic_glyph_outline(const FT_Outline *outline, short point, short first_pt,
-    short contour, FT_Vector *cubic_control)
+static int decompose_move_to(const FT_Vector *to, void *user)
 {
+    struct decompose_context *ctxt = (struct decompose_context*)user;
+    D2D1_POINT_2F point;
+
+    if (ctxt->figure_started) {
+        ID2D1SimplifiedGeometrySink_EndFigure(ctxt->sink, D2D1_FIGURE_END_CLOSED);
+        ctxt->figure_closed = TRUE;
+    }
+    else
+        ctxt->figure_closed = FALSE;
+    ctxt->figure_started = TRUE;
+
+    ft_vector_to_d2d_point(to, ctxt->xoffset, ctxt->yoffset, &point);
+    ID2D1SimplifiedGeometrySink_BeginFigure(ctxt->sink, point, D2D1_FIGURE_BEGIN_FILLED);
+    ctxt->move_to = TRUE;
+    ctxt->origin = *to;
+    return 0;
+}
+
+static int decompose_line_to(const FT_Vector *to, void *user)
+{
+    struct decompose_context *ctxt = (struct decompose_context*)user;
+    /* special case for empty contours, in a way freetype returns them */
+    if (ctxt->move_to && !memcmp(to, &ctxt->origin, sizeof(*to))) {
+        ID2D1SimplifiedGeometrySink_EndFigure(ctxt->sink, D2D1_FIGURE_END_CLOSED);
+        ctxt->figure_closed = TRUE;
+    }
+    else {
+        D2D1_POINT_2F point;
+        ft_vector_to_d2d_point(to, ctxt->xoffset, ctxt->yoffset, &point);
+        ID2D1SimplifiedGeometrySink_AddLines(ctxt->sink, &point, 1);
+        ctxt->figure_closed = FALSE;
+    }
+    ctxt->move_to = FALSE;
+    ctxt->origin = *to;
+    return 0;
+}
+
+static int decompose_conic_to(const FT_Vector *control, const FT_Vector *to, void *user)
+{
+    struct decompose_context *ctxt = (struct decompose_context*)user;
+    D2D1_POINT_2F points[3];
+    FT_Vector cubic[3];
+
+    /* convert from quadratic to cubic */
+
     /*
        The parametric eqn for a cubic Bezier is, from PLRM:
        r(t) = at^3 + bt^2 + ct + r0
@@ -288,108 +348,85 @@ static void get_cubic_glyph_outline(const FT_Outline *outline, short point, shor
        and of course r0 = p0, r3 = p2
     */
 
-    /* FIXME: Possible optimization in endpoint calculation
-       if there are two consecutive curves */
-    cubic_control[0] = outline->points[point-1];
-    if (!(outline->tags[point-1] & FT_Curve_Tag_On)) {
-        cubic_control[0].x += outline->points[point].x + 1;
-        cubic_control[0].y += outline->points[point].y + 1;
-        cubic_control[0].x >>= 1;
-        cubic_control[0].y >>= 1;
-    }
-    if (point+1 > outline->contours[contour])
-        cubic_control[3] = outline->points[first_pt];
-    else {
-        cubic_control[3] = outline->points[point+1];
-        if (!(outline->tags[point+1] & FT_Curve_Tag_On)) {
-            cubic_control[3].x += outline->points[point].x + 1;
-            cubic_control[3].y += outline->points[point].y + 1;
-            cubic_control[3].x >>= 1;
-            cubic_control[3].y >>= 1;
-        }
-    }
-
     /* r1 = 1/3 p0 + 2/3 p1
        r2 = 1/3 p2 + 2/3 p1 */
-    cubic_control[1].x = (2 * outline->points[point].x + 1) / 3;
-    cubic_control[1].y = (2 * outline->points[point].y + 1) / 3;
-    cubic_control[2] = cubic_control[1];
-    cubic_control[1].x += (cubic_control[0].x + 1) / 3;
-    cubic_control[1].y += (cubic_control[0].y + 1) / 3;
-    cubic_control[2].x += (cubic_control[3].x + 1) / 3;
-    cubic_control[2].y += (cubic_control[3].y + 1) / 3;
+    cubic[0].x = (2 * control->x + 1) / 3;
+    cubic[0].y = (2 * control->y + 1) / 3;
+    cubic[1] = cubic[0];
+    cubic[0].x += (ctxt->origin.x + 1) / 3;
+    cubic[0].y += (ctxt->origin.y + 1) / 3;
+    cubic[1].x += (to->x + 1) / 3;
+    cubic[1].y += (to->y + 1) / 3;
+    cubic[2] = *to;
+
+    ft_vector_to_d2d_point(cubic, ctxt->xoffset, ctxt->yoffset, points);
+    ft_vector_to_d2d_point(cubic + 1, ctxt->xoffset, ctxt->yoffset, points + 1);
+    ft_vector_to_d2d_point(cubic + 2, ctxt->xoffset, ctxt->yoffset, points + 2);
+    ID2D1SimplifiedGeometrySink_AddBeziers(ctxt->sink, (D2D1_BEZIER_SEGMENT*)points, 1);
+    ctxt->figure_closed = FALSE;
+    ctxt->move_to = FALSE;
+    ctxt->origin = *to;
+    return 0;
 }
 
-static short get_outline_data(const FT_Outline *outline, struct glyph_outline *ret)
+static int decompose_cubic_to(const FT_Vector *control1, const FT_Vector *control2,
+    const FT_Vector *to, void *user)
 {
-    short contour, point = 0, first_pt, count;
+    struct decompose_context *ctxt = (struct decompose_context*)user;
+    D2D1_POINT_2F points[3];
 
-    for (contour = 0, count = 0; contour < outline->n_contours; contour++) {
-        first_pt = point;
-        if (ret) {
-            ft_vector_to_d2d_point(&outline->points[point], &ret->points[count]);
-            ret->tags[count] = OUTLINE_POINT_START;
-            if (count)
-                ret->tags[count-1] |= OUTLINE_POINT_END;
-        }
-
-        point++;
-        count++;
-
-        while (point <= outline->contours[contour]) {
-            do {
-                if (outline->tags[point] & FT_Curve_Tag_On) {
-                    if (ret) {
-                        ft_vector_to_d2d_point(&outline->points[point], &ret->points[count]);
-                        ret->tags[count] |= OUTLINE_POINT_LINE;
-                    }
-
-                    point++;
-                    count++;
-                }
-                else {
-
-                    if (ret) {
-                        FT_Vector cubic_control[4];
-
-                        get_cubic_glyph_outline(outline, point, first_pt, contour, cubic_control);
-                        ft_vector_to_d2d_point(&cubic_control[1], &ret->points[count]);
-                        ft_vector_to_d2d_point(&cubic_control[2], &ret->points[count+1]);
-                        ft_vector_to_d2d_point(&cubic_control[3], &ret->points[count+2]);
-                        ret->tags[count] = OUTLINE_POINT_BEZIER;
-                        ret->tags[count+1] = OUTLINE_POINT_BEZIER;
-                        ret->tags[count+2] = OUTLINE_POINT_BEZIER;
-                    }
-
-                    count += 3;
-                    point++;
-                }
-            } while (point <= outline->contours[contour] &&
-                    (outline->tags[point] & FT_Curve_Tag_On) ==
-                    (outline->tags[point-1] & FT_Curve_Tag_On));
-
-            if (point <= outline->contours[contour] &&
-               outline->tags[point] & FT_Curve_Tag_On)
-            {
-                /* This is the closing pt of a bezier, but we've already
-                   added it, so just inc point and carry on */
-                point++;
-            }
-        }
-    }
-
-    if (ret)
-        ret->tags[count-1] |= OUTLINE_POINT_END;
-
-    return count;
+    ft_vector_to_d2d_point(control1, ctxt->xoffset, ctxt->yoffset, points);
+    ft_vector_to_d2d_point(control2, ctxt->xoffset, ctxt->yoffset, points + 1);
+    ft_vector_to_d2d_point(to, ctxt->xoffset, ctxt->yoffset, points + 2);
+    ID2D1SimplifiedGeometrySink_AddBeziers(ctxt->sink, (D2D1_BEZIER_SEGMENT*)points, 1);
+    ctxt->figure_closed = FALSE;
+    ctxt->move_to = FALSE;
+    ctxt->origin = *to;
+    return 0;
 }
 
-HRESULT freetype_get_glyph_outline(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, USHORT simulations, struct glyph_outline **ret)
+static void decompose_outline(FT_Outline *outline, FLOAT xoffset, FLOAT yoffset, IDWriteGeometrySink *sink)
+{
+    static const FT_Outline_Funcs decompose_funcs = {
+        decompose_move_to,
+        decompose_line_to,
+        decompose_conic_to,
+        decompose_cubic_to,
+        0,
+        0
+    };
+    struct decompose_context context;
+
+    context.sink = sink;
+    context.xoffset = xoffset;
+    context.yoffset = yoffset;
+    context.figure_started = FALSE;
+    context.figure_closed = FALSE;
+    context.move_to = FALSE;
+    context.origin.x = 0;
+    context.origin.y = 0;
+
+    pFT_Outline_Decompose(outline, &decompose_funcs, &context);
+
+    if (!context.figure_closed && outline->n_points)
+        ID2D1SimplifiedGeometrySink_EndFigure(sink, D2D1_FIGURE_END_CLOSED);
+}
+
+HRESULT freetype_get_glyphrun_outline(IDWriteFontFace3 *fontface, FLOAT emSize, UINT16 const *glyphs, FLOAT const *advances,
+    DWRITE_GLYPH_OFFSET const *offsets, UINT32 count, BOOL is_rtl, IDWriteGeometrySink *sink)
 {
     FTC_ScalerRec scaler;
+    USHORT simulations;
     HRESULT hr = S_OK;
     FT_Size size;
 
+    if (!count)
+        return S_OK;
+
+    ID2D1SimplifiedGeometrySink_SetFillMode(sink, D2D1_FILL_MODE_WINDING);
+
+    simulations = IDWriteFontFace3_GetSimulations(fontface);
+
     scaler.face_id = fontface;
     scaler.width  = emSize;
     scaler.height = emSize;
@@ -399,32 +436,51 @@ HRESULT freetype_get_glyph_outline(IDWriteFontFace2 *fontface, FLOAT emSize, UIN
 
     EnterCriticalSection(&freetype_cs);
     if (pFTC_Manager_LookupSize(cache_manager, &scaler, &size) == 0) {
-         if (pFT_Load_Glyph(size->face, index, FT_LOAD_NO_BITMAP) == 0) {
-             FT_Outline *outline = &size->face->glyph->outline;
-             short count;
-             FT_Matrix m;
+        FLOAT advance = 0.0f;
+        UINT32 g;
 
-             m.xx = 1 << 16;
-             m.xy = simulations & DWRITE_FONT_SIMULATIONS_OBLIQUE ? (1 << 16) / 3 : 0;
-             m.yx = 0;
-             m.yy = -(1 << 16); /* flip Y axis */
+        for (g = 0; g < count; g++) {
+            if (pFT_Load_Glyph(size->face, glyphs[g], FT_LOAD_NO_BITMAP) == 0) {
+                FLOAT ft_advance = size->face->glyph->metrics.horiAdvance >> 6;
+                FT_Outline *outline = &size->face->glyph->outline;
+                FLOAT xoffset = 0.0f, yoffset = 0.0f;
+                FT_Matrix m;
 
-             pFT_Outline_Transform(outline, &m);
+                m.xx = 1 << 16;
+                m.xy = simulations & DWRITE_FONT_SIMULATIONS_OBLIQUE ? (1 << 16) / 3 : 0;
+                m.yx = 0;
+                m.yy = -(1 << 16); /* flip Y axis */
 
-             count = get_outline_data(outline, NULL);
-             hr = new_glyph_outline(count, ret);
-             if (hr == S_OK) {
-                 get_outline_data(outline, *ret);
-                 (*ret)->advance = size->face->glyph->metrics.horiAdvance >> 6;
-             }
-         }
+                pFT_Outline_Transform(outline, &m);
+
+                /* glyph offsets act as current glyph adjustment */
+                if (offsets) {
+                    xoffset += is_rtl ? -offsets[g].advanceOffset : offsets[g].advanceOffset;
+                    yoffset -= offsets[g].ascenderOffset;
+                }
+
+                if (g == 0 && is_rtl)
+                    advance = advances ? -advances[g] : -ft_advance;
+
+                xoffset += advance;
+                decompose_outline(outline, xoffset, yoffset, sink);
+
+                /* update advance to next glyph */
+                if (advances)
+                    advance += is_rtl ? -advances[g] : advances[g];
+                else
+                    advance += is_rtl ? -ft_advance : ft_advance;
+            }
+        }
     }
+    else
+        hr = E_FAIL;
     LeaveCriticalSection(&freetype_cs);
 
     return hr;
 }
 
-UINT16 freetype_get_glyphcount(IDWriteFontFace2 *fontface)
+UINT16 freetype_get_glyphcount(IDWriteFontFace3 *fontface)
 {
     UINT16 count = 0;
     FT_Face face;
@@ -437,39 +493,41 @@ UINT16 freetype_get_glyphcount(IDWriteFontFace2 *fontface)
     return count;
 }
 
-UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint, INT charmap)
+void freetype_get_glyphs(IDWriteFontFace3 *fontface, INT charmap, UINT32 const *codepoints, UINT32 count,
+    UINT16 *glyphs)
 {
-    UINT16 glyph;
+    UINT32 i;
 
     EnterCriticalSection(&freetype_cs);
-    if (charmap == -1)
-        glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint);
-    else {
-        /* special handling for symbol fonts */
-        if (codepoint < 0x100) codepoint += 0xf000;
-        glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint);
-        if (!glyph)
-            glyph = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint - 0xf000);
+    for (i = 0; i < count; i++) {
+        if (charmap == -1)
+            glyphs[i] = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoints[i]);
+        else {
+            UINT32 codepoint = codepoints[i];
+            /* special handling for symbol fonts */
+            if (codepoint < 0x100) codepoint += 0xf000;
+            glyphs[i] = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint);
+            if (!glyphs[i])
+                glyphs[i] = pFTC_CMapCache_Lookup(cmap_cache, fontface, charmap, codepoint - 0xf000);
+        }
     }
     LeaveCriticalSection(&freetype_cs);
-
-    return glyph;
 }
 
-BOOL freetype_has_kerning_pairs(IDWriteFontFace2 *fontface)
+BOOL freetype_has_kerning_pairs(IDWriteFontFace3 *fontface)
 {
     BOOL has_kerning_pairs = FALSE;
     FT_Face face;
 
     EnterCriticalSection(&freetype_cs);
     if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
-        has_kerning_pairs = FT_HAS_KERNING(face);
+        has_kerning_pairs = !!FT_HAS_KERNING(face);
     LeaveCriticalSection(&freetype_cs);
 
     return has_kerning_pairs;
 }
 
-INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 left, UINT16 right)
+INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace3 *fontface, UINT16 left, UINT16 right)
 {
     INT32 adjustment = 0;
     FT_Face face;
@@ -487,20 +545,61 @@ INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 le
     return adjustment;
 }
 
+static inline void ft_matrix_from_dwrite_matrix(const DWRITE_MATRIX *m, FT_Matrix *ft_matrix)
+{
+    ft_matrix->xx =  m->m11 * 0x10000;
+    ft_matrix->xy = -m->m21 * 0x10000;
+    ft_matrix->yx = -m->m12 * 0x10000;
+    ft_matrix->yy =  m->m22 * 0x10000;
+}
+
+/* Should be used only while holding 'freetype_cs' */
+static BOOL is_face_scalable(IDWriteFontFace3 *fontface)
+{
+    FT_Face face;
+    if (pFTC_Manager_LookupFace(cache_manager, fontface, &face) == 0)
+        return FT_IS_SCALABLE(face);
+    else
+        return FALSE;
+}
+
 void freetype_get_glyph_bbox(struct dwrite_glyphbitmap *bitmap)
 {
     FTC_ImageTypeRec imagetype;
     FT_BBox bbox = { 0 };
     FT_Glyph glyph;
 
+    EnterCriticalSection(&freetype_cs);
+
+    /* Some fonts provide mostly bitmaps and very few outlines, for example for .notdef,
+       disable transform if that's the case. */
+    if (bitmap->m) {
+        if (!is_face_scalable(bitmap->fontface))
+            bitmap->m = NULL;
+    }
+
     imagetype.face_id = bitmap->fontface;
     imagetype.width = 0;
     imagetype.height = bitmap->emsize;
-    imagetype.flags = FT_LOAD_DEFAULT;
+    imagetype.flags = bitmap->m ? FT_LOAD_NO_BITMAP : FT_LOAD_DEFAULT;
+
+    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, bitmap->index, &glyph, NULL) == 0) {
+        if (bitmap->m) {
+            FT_Glyph glyph_copy;
+
+            if (pFT_Glyph_Copy(glyph, &glyph_copy) == 0) {
+                FT_Matrix ft_matrix;
+
+                ft_matrix_from_dwrite_matrix(bitmap->m, &ft_matrix);
+                pFT_Glyph_Transform(glyph_copy, &ft_matrix, NULL);
+                pFT_Glyph_Get_CBox(glyph_copy, FT_GLYPH_BBOX_PIXELS, &bbox);
+                pFT_Done_Glyph(glyph_copy);
+            }
+        }
+        else
+            pFT_Glyph_Get_CBox(glyph, FT_GLYPH_BBOX_PIXELS, &bbox);
+    }
 
-    EnterCriticalSection(&freetype_cs);
-    if (pFTC_ImageCache_Lookup(image_cache, &imagetype, bitmap->index, &glyph, NULL) == 0)
-        pFT_Glyph_Get_CBox(glyph, FT_GLYPH_BBOX_PIXELS, &bbox);
     LeaveCriticalSection(&freetype_cs);
 
     /* flip Y axis */
@@ -607,24 +706,48 @@ BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
     BOOL ret = FALSE;
     FT_Glyph glyph;
 
+    EnterCriticalSection(&freetype_cs);
+
+    if (bitmap->m) {
+        if (!is_face_scalable(bitmap->fontface))
+            bitmap->m = NULL;
+    }
+
     imagetype.face_id = bitmap->fontface;
     imagetype.width = 0;
     imagetype.height = bitmap->emsize;
-    imagetype.flags = FT_LOAD_DEFAULT;
+    imagetype.flags = bitmap->m ? FT_LOAD_NO_BITMAP : FT_LOAD_DEFAULT;
 
-    EnterCriticalSection(&freetype_cs);
     if (pFTC_ImageCache_Lookup(image_cache, &imagetype, bitmap->index, &glyph, NULL) == 0) {
+        FT_Glyph glyph_copy;
+
+        if (bitmap->m) {
+            if (pFT_Glyph_Copy(glyph, &glyph_copy) == 0) {
+                FT_Matrix ft_matrix;
+
+                ft_matrix_from_dwrite_matrix(bitmap->m, &ft_matrix);
+                pFT_Glyph_Transform(glyph_copy, &ft_matrix, NULL);
+                glyph = glyph_copy;
+            }
+        }
+        else
+            glyph_copy = NULL;
+
         if (bitmap->type == DWRITE_TEXTURE_CLEARTYPE_3x1)
             ret = freetype_get_aa_glyph_bitmap(bitmap, glyph);
         else
             ret = freetype_get_aliased_glyph_bitmap(bitmap, glyph);
+
+        if (glyph_copy)
+            pFT_Done_Glyph(glyph_copy);
     }
+
     LeaveCriticalSection(&freetype_cs);
 
     return ret;
 }
 
-INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
+INT freetype_get_charmap_index(IDWriteFontFace3 *fontface, BOOL *is_symbol)
 {
     INT charmap_index = -1;
     FT_Face face;
@@ -656,7 +779,7 @@ INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
     return charmap_index;
 }
 
-INT32 freetype_get_glyph_advance(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
+INT32 freetype_get_glyph_advance(IDWriteFontFace3 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
 {
     FTC_ImageTypeRec imagetype;
     FT_Glyph glyph;
@@ -690,42 +813,43 @@ void release_freetype(void)
 {
 }
 
-void freetype_notify_cacheremove(IDWriteFontFace2 *fontface)
+void freetype_notify_cacheremove(IDWriteFontFace3 *fontface)
 {
 }
 
-HRESULT freetype_get_design_glyph_metrics(IDWriteFontFace2 *fontface, UINT16 unitsperEm, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
+HRESULT freetype_get_design_glyph_metrics(IDWriteFontFace3 *fontface, UINT16 unitsperEm, UINT16 glyph, DWRITE_GLYPH_METRICS *ret)
 {
     return E_NOTIMPL;
 }
 
-BOOL freetype_is_monospaced(IDWriteFontFace2 *fontface)
+BOOL freetype_is_monospaced(IDWriteFontFace3 *fontface)
 {
     return FALSE;
 }
 
-HRESULT freetype_get_glyph_outline(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, USHORT simulations, struct glyph_outline **ret)
+HRESULT freetype_get_glyphrun_outline(IDWriteFontFace3 *fontface, FLOAT emSize, UINT16 const *glyphs, FLOAT const *advances,
+    DWRITE_GLYPH_OFFSET const *offsets, UINT32 count, BOOL is_rtl, IDWriteGeometrySink *sink)
 {
-    *ret = NULL;
     return E_NOTIMPL;
 }
 
-UINT16 freetype_get_glyphcount(IDWriteFontFace2 *fontface)
+UINT16 freetype_get_glyphcount(IDWriteFontFace3 *fontface)
 {
     return 0;
 }
 
-UINT16 freetype_get_glyphindex(IDWriteFontFace2 *fontface, UINT32 codepoint, INT charmap)
+void freetype_get_glyphs(IDWriteFontFace3 *fontface, INT charmap, UINT32 const *codepoints, UINT32 count,
+    UINT16 *glyphs)
 {
-    return 0;
+    memset(glyphs, 0, count * sizeof(*glyphs));
 }
 
-BOOL freetype_has_kerning_pairs(IDWriteFontFace2 *fontface)
+BOOL freetype_has_kerning_pairs(IDWriteFontFace3 *fontface)
 {
     return FALSE;
 }
 
-INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace2 *fontface, UINT16 left, UINT16 right)
+INT32 freetype_get_kerning_pair_adjustment(IDWriteFontFace3 *fontface, UINT16 left, UINT16 right)
 {
     return 0;
 }
@@ -740,13 +864,13 @@ BOOL freetype_get_glyph_bitmap(struct dwrite_glyphbitmap *bitmap)
     return FALSE;
 }
 
-INT freetype_get_charmap_index(IDWriteFontFace2 *fontface, BOOL *is_symbol)
+INT freetype_get_charmap_index(IDWriteFontFace3 *fontface, BOOL *is_symbol)
 {
     *is_symbol = FALSE;
     return -1;
 }
 
-INT32 freetype_get_glyph_advance(IDWriteFontFace2 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
+INT32 freetype_get_glyph_advance(IDWriteFontFace3 *fontface, FLOAT emSize, UINT16 index, DWRITE_MEASURING_MODE mode)
 {
     return 0;
 }
