@@ -45,6 +45,7 @@ struct dib_data {
 
 struct rendertarget {
     IDWriteBitmapRenderTarget1 IDWriteBitmapRenderTarget1_iface;
+    ID2D1SimplifiedGeometrySink ID2D1SimplifiedGeometrySink_iface;
     LONG ref;
 
     IDWriteFactory *factory;
@@ -99,11 +100,111 @@ static inline struct rendertarget *impl_from_IDWriteBitmapRenderTarget1(IDWriteB
     return CONTAINING_RECORD(iface, struct rendertarget, IDWriteBitmapRenderTarget1_iface);
 }
 
+static inline struct rendertarget *impl_from_ID2D1SimplifiedGeometrySink(ID2D1SimplifiedGeometrySink *iface)
+{
+    return CONTAINING_RECORD(iface, struct rendertarget, ID2D1SimplifiedGeometrySink_iface);
+}
+
 static inline struct gdiinterop *impl_from_IDWriteGdiInterop(IDWriteGdiInterop *iface)
 {
     return CONTAINING_RECORD(iface, struct gdiinterop, IDWriteGdiInterop_iface);
 }
 
+static HRESULT WINAPI rendertarget_sink_QueryInterface(ID2D1SimplifiedGeometrySink *iface, REFIID riid, void **obj)
+{
+    if (IsEqualIID(riid, &IID_ID2D1SimplifiedGeometrySink) ||
+        IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        ID2D1SimplifiedGeometrySink_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI rendertarget_sink_AddRef(ID2D1SimplifiedGeometrySink *iface)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    return IDWriteBitmapRenderTarget1_AddRef(&This->IDWriteBitmapRenderTarget1_iface);
+}
+
+static ULONG WINAPI rendertarget_sink_Release(ID2D1SimplifiedGeometrySink *iface)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    return IDWriteBitmapRenderTarget1_Release(&This->IDWriteBitmapRenderTarget1_iface);
+}
+
+static void WINAPI rendertarget_sink_SetFillMode(ID2D1SimplifiedGeometrySink *iface, D2D1_FILL_MODE mode)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    SetPolyFillMode(This->hdc, mode == D2D1_FILL_MODE_ALTERNATE ? ALTERNATE : WINDING);
+}
+
+static void WINAPI rendertarget_sink_SetSegmentFlags(ID2D1SimplifiedGeometrySink *iface, D2D1_PATH_SEGMENT vertexFlags)
+{
+}
+
+static void WINAPI rendertarget_sink_BeginFigure(ID2D1SimplifiedGeometrySink *iface, D2D1_POINT_2F startPoint, D2D1_FIGURE_BEGIN figureBegin)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    MoveToEx(This->hdc, startPoint.x, startPoint.y, NULL);
+}
+
+static void WINAPI rendertarget_sink_AddLines(ID2D1SimplifiedGeometrySink *iface, const D2D1_POINT_2F *points, UINT32 count)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+
+    while (count--) {
+        LineTo(This->hdc, points->x, points->y);
+        points++;
+    }
+}
+
+static void WINAPI rendertarget_sink_AddBeziers(ID2D1SimplifiedGeometrySink *iface, const D2D1_BEZIER_SEGMENT *beziers, UINT32 count)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    POINT points[3];
+
+    while (count--) {
+        points[0].x = beziers->point1.x;
+        points[0].y = beziers->point1.y;
+        points[1].x = beziers->point2.x;
+        points[1].y = beziers->point2.y;
+        points[2].x = beziers->point3.x;
+        points[2].y = beziers->point3.y;
+
+        PolyBezierTo(This->hdc, points, 3);
+        beziers++;
+    }
+}
+
+static void WINAPI rendertarget_sink_EndFigure(ID2D1SimplifiedGeometrySink *iface, D2D1_FIGURE_END figureEnd)
+{
+    struct rendertarget *This = impl_from_ID2D1SimplifiedGeometrySink(iface);
+    CloseFigure(This->hdc);
+}
+
+static HRESULT WINAPI rendertarget_sink_Close(ID2D1SimplifiedGeometrySink *iface)
+{
+    return S_OK;
+}
+
+static const ID2D1SimplifiedGeometrySinkVtbl rendertargetsinkvtbl = {
+    rendertarget_sink_QueryInterface,
+    rendertarget_sink_AddRef,
+    rendertarget_sink_Release,
+    rendertarget_sink_SetFillMode,
+    rendertarget_sink_SetSegmentFlags,
+    rendertarget_sink_BeginFigure,
+    rendertarget_sink_AddLines,
+    rendertarget_sink_AddBeziers,
+    rendertarget_sink_EndFigure,
+    rendertarget_sink_Close
+};
+
 static HRESULT WINAPI rendertarget_QueryInterface(IDWriteBitmapRenderTarget1 *iface, REFIID riid, void **obj)
 {
     struct rendertarget *This = impl_from_IDWriteBitmapRenderTarget1(iface);
@@ -198,6 +299,11 @@ static void blit_subpixel_888(struct dib_data *dib, int dib_width, const BYTE *s
     }
 }
 
+static inline DWORD colorref_to_pixel_888(COLORREF color)
+{
+    return (((color >> 16) & 0xff) | (color & 0xff00) | ((color << 16) & 0xff0000));
+}
+
 static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *iface,
     FLOAT originX, FLOAT originY, DWRITE_MEASURING_MODE measuring_mode,
     DWRITE_GLYPH_RUN const *run, IDWriteRenderingParams *params, COLORREF color,
@@ -207,23 +313,85 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
     IDWriteGlyphRunAnalysis *analysis;
     DWRITE_RENDERING_MODE rendermode;
     DWRITE_TEXTURE_TYPE texturetype;
+    IDWriteFontFace1 *fontface1;
     RECT target, bounds;
     HRESULT hr;
 
     TRACE("(%p)->(%.2f %.2f %d %p %p 0x%08x %p)\n", This, originX, originY,
         measuring_mode, run, params, color, bbox_ret);
 
+    SetRectEmpty(bbox_ret);
+
     if (!This->dib.ptr)
         return S_OK;
 
-    hr = IDWriteFontFace_GetRecommendedRenderingMode(run->fontFace, run->fontEmSize,
-        This->ppdip, measuring_mode, params, &rendermode);
+    hr = IDWriteFontFace_QueryInterface(run->fontFace, &IID_IDWriteFontFace1, (void**)&fontface1);
+    if (hr == S_OK) {
+        hr = IDWriteFontFace1_GetRecommendedRenderingMode(fontface1, run->fontEmSize, This->ppdip * 96.0f,
+            This->ppdip * 96.0f, NULL, run->isSideways, DWRITE_OUTLINE_THRESHOLD_ALIASED, measuring_mode,
+            &rendermode);
+        IDWriteFontFace1_Release(fontface1);
+    }
+    else
+        hr = IDWriteFontFace_GetRecommendedRenderingMode(run->fontFace, run->fontEmSize,
+            This->ppdip, measuring_mode, params, &rendermode);
+
     if (FAILED(hr))
         return hr;
 
-    /* FIXME: outline mode rendering is not supported for this target yet */
-    if (rendermode == DWRITE_RENDERING_MODE_OUTLINE)
-        rendermode = DWRITE_RENDERING_MODE_ALIASED;
+    target.left = target.top = 0;
+    target.right = This->size.cx;
+    target.bottom = This->size.cy;
+
+    if (rendermode == DWRITE_RENDERING_MODE_OUTLINE) {
+        static const XFORM identity = { 1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f };
+        const DWRITE_MATRIX *m = &This->m;
+        XFORM xform;
+
+        /* target allows any transform to be set, filter it here */
+        if (m->m11 * m->m22 == m->m12 * m->m21) {
+            xform.eM11 = 1.0f;
+            xform.eM12 = 0.0f;
+            xform.eM21 = 0.0f;
+            xform.eM22 = 1.0f;
+            xform.eDx  = originX;
+            xform.eDy  = originY;
+        } else {
+            xform.eM11 = m->m11;
+            xform.eM12 = m->m12;
+            xform.eM21 = m->m21;
+            xform.eM22 = m->m22;
+            xform.eDx  = m->m11 * originX + m->m21 * originY + m->dx;
+            xform.eDy  = m->m12 * originX + m->m22 * originY + m->dy;
+        }
+        SetWorldTransform(This->hdc, &xform);
+
+        BeginPath(This->hdc);
+
+        hr = IDWriteFontFace_GetGlyphRunOutline(run->fontFace, run->fontEmSize * This->ppdip,
+            run->glyphIndices, run->glyphAdvances, run->glyphOffsets, run->glyphCount,
+            run->isSideways, run->bidiLevel & 1, &This->ID2D1SimplifiedGeometrySink_iface);
+
+        EndPath(This->hdc);
+
+        if (hr == S_OK) {
+            HBRUSH brush = CreateSolidBrush(color);
+
+            SelectObject(This->hdc, brush);
+
+            FillPath(This->hdc);
+
+            /* FIXME: one way to get affected rectangle bounds is to use region fill */
+            if (bbox_ret)
+                *bbox_ret = target;
+
+            DeleteObject(brush);
+        }
+
+        SetWorldTransform(This->hdc, &identity);
+
+        return hr;
+    }
 
     hr = IDWriteFactory_CreateGlyphRunAnalysis(This->factory,
         run, This->ppdip, &This->m, rendermode, measuring_mode,
@@ -238,21 +406,27 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
     hr = IDWriteGlyphRunAnalysis_GetAlphaTextureBounds(analysis, DWRITE_TEXTURE_ALIASED_1x1, &bounds);
     if (FAILED(hr) || IsRectEmpty(&bounds)) {
         hr = IDWriteGlyphRunAnalysis_GetAlphaTextureBounds(analysis, DWRITE_TEXTURE_CLEARTYPE_3x1, &bounds);
+        if (FAILED(hr)) {
+            WARN("GetAlphaTextureBounds() failed, 0x%08x\n", hr);
+            IDWriteGlyphRunAnalysis_Release(analysis);
+            return hr;
+        }
         texturetype = DWRITE_TEXTURE_CLEARTYPE_3x1;
     }
 
-    target.left = target.top = 0;
-    target.right = This->size.cx;
-    target.bottom = This->size.cy;
-
-    SetRectEmpty(bbox_ret);
     if (IntersectRect(&target, &target, &bounds)) {
         UINT32 size = (target.right - target.left) * (target.bottom - target.top);
         BYTE *bitmap;
 
+        color = colorref_to_pixel_888(color);
         if (texturetype == DWRITE_TEXTURE_CLEARTYPE_3x1)
             size *= 3;
         bitmap = heap_alloc_zero(size);
+        if (!bitmap) {
+            IDWriteGlyphRunAnalysis_Release(analysis);
+            return E_OUTOFMEMORY;
+        }
+
         hr = IDWriteGlyphRunAnalysis_CreateAlphaTexture(analysis, texturetype, &target, bitmap, size);
         if (hr == S_OK) {
             /* blit to target dib */
@@ -261,7 +435,7 @@ static HRESULT WINAPI rendertarget_DrawGlyphRun(IDWriteBitmapRenderTarget1 *ifac
             else
                 blit_subpixel_888(&This->dib, This->size.cx, bitmap, &target, color);
 
-            *bbox_ret = target;
+            if (bbox_ret) *bbox_ret = target;
         }
 
         heap_free(bitmap);
@@ -292,7 +466,7 @@ static HRESULT WINAPI rendertarget_SetPixelsPerDip(IDWriteBitmapRenderTarget1 *i
 
     TRACE("(%p)->(%.2f)\n", This, ppdip);
 
-    if (ppdip <= 0.0)
+    if (ppdip <= 0.0f)
         return E_INVALIDARG;
 
     This->ppdip = ppdip;
@@ -387,9 +561,11 @@ static HRESULT create_rendertarget(IDWriteFactory *factory, HDC hdc, UINT32 widt
     if (!target) return E_OUTOFMEMORY;
 
     target->IDWriteBitmapRenderTarget1_iface.lpVtbl = &rendertargetvtbl;
+    target->ID2D1SimplifiedGeometrySink_iface.lpVtbl = &rendertargetsinkvtbl;
     target->ref = 1;
 
     target->hdc = CreateCompatibleDC(hdc);
+    SetGraphicsMode(target->hdc, GM_ADVANCED);
     hr = create_target_dibsection(target, width, height);
     if (FAILED(hr)) {
         IDWriteBitmapRenderTarget1_Release(&target->IDWriteBitmapRenderTarget1_iface);
@@ -397,7 +573,7 @@ static HRESULT create_rendertarget(IDWriteFactory *factory, HDC hdc, UINT32 widt
     }
 
     target->m = identity;
-    target->ppdip = 1.0;
+    target->ppdip = GetDeviceCaps(target->hdc, LOGPIXELSX) / 96.0f;
     target->antialiasmode = DWRITE_TEXT_ANTIALIAS_MODE_CLEARTYPE;
     target->factory = factory;
     IDWriteFactory_AddRef(factory);
@@ -535,11 +711,11 @@ static HRESULT WINAPI gdiinterop_ConvertFontToLOGFONT(IDWriteGdiInterop *iface,
     if (FAILED(hr) || !exists)
         return hr;
 
-    IDWriteLocalizedStrings_FindLocaleName(name, enusW, &index, &exists);
-    IDWriteLocalizedStrings_GetString(name, index, logfont->lfFaceName, sizeof(logfont->lfFaceName)/sizeof(WCHAR));
+    hr = IDWriteLocalizedStrings_FindLocaleName(name, enusW, &index, &exists);
+    if (hr == S_OK)
+        hr = IDWriteLocalizedStrings_GetString(name, index, logfont->lfFaceName, sizeof(logfont->lfFaceName)/sizeof(WCHAR));
     IDWriteLocalizedStrings_Release(name);
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT WINAPI gdiinterop_ConvertFontFaceToLOGFONT(IDWriteGdiInterop *iface,
@@ -575,7 +751,7 @@ static HRESULT WINAPI gdiinterop_ConvertFontFaceToLOGFONT(IDWriteGdiInterop *ifa
     index = IDWriteFontFace_GetIndex(fontface);
     face_type = IDWriteFontFace_GetType(fontface);
     opentype_get_font_properties(stream, face_type, index, &props);
-    hr = get_family_names_from_stream(stream, index, face_type, &familynames);
+    hr = opentype_get_font_familyname(stream, index, face_type, &familynames);
     IDWriteFontFile_Release(file);
     IDWriteFontFileStream_Release(stream);
     if (FAILED(hr))
@@ -607,31 +783,87 @@ static HRESULT WINAPI gdiinterop_ConvertFontFaceToLOGFONT(IDWriteGdiInterop *ifa
     return hr;
 }
 
+struct font_realization_info {
+    DWORD size;
+    DWORD flags;
+    DWORD cache_num;
+    DWORD instance_id;
+    DWORD unk;
+    WORD  face_index;
+    WORD  simulations;
+};
+
+struct font_fileinfo {
+    FILETIME writetime;
+    LARGE_INTEGER size;
+    WCHAR path[1];
+};
+
+/* Undocumented gdi32 exports, used to access actually selected font information */
+extern BOOL WINAPI GetFontRealizationInfo(HDC hdc, struct font_realization_info *info);
+extern BOOL WINAPI GetFontFileInfo(DWORD instance_id, DWORD unknown, struct font_fileinfo *info, DWORD size, DWORD *needed);
+
 static HRESULT WINAPI gdiinterop_CreateFontFaceFromHdc(IDWriteGdiInterop *iface,
     HDC hdc, IDWriteFontFace **fontface)
 {
     struct gdiinterop *This = impl_from_IDWriteGdiInterop(iface);
-    IDWriteFont *font;
-    LOGFONTW logfont;
-    HFONT hfont;
+    struct font_realization_info info;
+    struct font_fileinfo *fileinfo;
+    DWRITE_FONT_FILE_TYPE filetype;
+    DWRITE_FONT_FACE_TYPE facetype;
+    IDWriteFontFile *file;
+    BOOL is_supported;
+    UINT32 facenum;
+    DWORD needed;
     HRESULT hr;
 
     TRACE("(%p)->(%p %p)\n", This, hdc, fontface);
 
     *fontface = NULL;
 
-    hfont = GetCurrentObject(hdc, OBJ_FONT);
-    if (!hfont)
+    if (!hdc)
         return E_INVALIDARG;
-    GetObjectW(hfont, sizeof(logfont), &logfont);
 
-    hr = IDWriteGdiInterop_CreateFontFromLOGFONT(iface, &logfont, &font);
+    /* get selected font id  */
+    info.size = sizeof(info);
+    if (!GetFontRealizationInfo(hdc, &info)) {
+        WARN("failed to get selected font id\n");
+        return E_FAIL;
+    }
+
+    needed = 0;
+    GetFontFileInfo(info.instance_id, 0, NULL, 0, &needed);
+    if (needed == 0) {
+        WARN("failed to get font file info size\n");
+        return E_FAIL;
+    }
+
+    fileinfo = heap_alloc(needed);
+    if (!fileinfo)
+        return E_OUTOFMEMORY;
+
+    if (!GetFontFileInfo(info.instance_id, 0, fileinfo, needed, &needed)) {
+        heap_free(fileinfo);
+        return E_FAIL;
+    }
+
+    hr = IDWriteFactory2_CreateFontFileReference(This->factory, fileinfo->path, &fileinfo->writetime,
+        &file);
+    heap_free(fileinfo);
     if (FAILED(hr))
         return hr;
 
-    hr = IDWriteFont_CreateFontFace(font, fontface);
-    IDWriteFont_Release(font);
+    is_supported = FALSE;
+    hr = IDWriteFontFile_Analyze(file, &is_supported, &filetype, &facetype, &facenum);
+    if (FAILED(hr) || !is_supported) {
+        IDWriteFontFile_Release(file);
+        return hr;
+    }
 
+    /* Simulations flags values match DWRITE_FONT_SIMULATIONS */
+    hr = IDWriteFactory2_CreateFontFace(This->factory, facetype, 1, &file, info.face_index, info.simulations,
+        fontface);
+    IDWriteFontFile_Release(file);
     return hr;
 }
 
