@@ -438,7 +438,7 @@ HRESULT add_localizedstring(IDWriteLocalizedStrings *iface, const WCHAR *locale,
 }
 
 HRESULT clone_localizedstring(IDWriteLocalizedStrings *iface, IDWriteLocalizedStrings **ret)
- {
+{
     struct localizedstrings *strings, *strings_clone;
     int i;
 
@@ -471,6 +471,21 @@ HRESULT clone_localizedstring(IDWriteLocalizedStrings *iface, IDWriteLocalizedSt
     return S_OK;
 }
 
+void set_en_localizedstring(IDWriteLocalizedStrings *iface, const WCHAR *string)
+{
+    static const WCHAR enusW[] = {'e','n','-','U','S',0};
+    struct localizedstrings *This = impl_from_IDWriteLocalizedStrings(iface);
+    UINT32 i;
+
+    for (i = 0; i < This->count; i++) {
+        if (!strcmpiW(This->data[i].locale, enusW)) {
+            heap_free(This->data[i].string);
+            This->data[i].string = heap_strdupW(string);
+            break;
+        }
+    }
+}
+
 struct collectionloader
 {
     struct list entry;
@@ -497,6 +512,7 @@ struct dwritefactory {
     IDWriteFontCollection *system_collection;
     IDWriteFontCollection *eudc_collection;
     IDWriteGdiInterop *gdiinterop;
+    IDWriteFontFallback *fallback;
 
     IDWriteLocalFontFileLoader* localfontfileloader;
     struct list localfontfaces;
@@ -510,6 +526,12 @@ static inline struct dwritefactory *impl_from_IDWriteFactory2(IDWriteFactory2 *i
     return CONTAINING_RECORD(iface, struct dwritefactory, IDWriteFactory2_iface);
 }
 
+void notify_factory_fallback_removed(IDWriteFactory2 *iface)
+{
+    struct dwritefactory *factory = impl_from_IDWriteFactory2(iface);
+    factory->fallback = NULL;
+}
+
 static void release_fontface_cache(struct list *fontfaces)
 {
     struct fontfacecached *fontface, *fontface2;
@@ -552,6 +574,8 @@ static void release_dwritefactory(struct dwritefactory *factory)
         IDWriteFontCollection_Release(factory->eudc_collection);
     if (factory->gdiinterop)
         release_gdiinterop(factory->gdiinterop);
+    if (factory->fallback)
+        release_system_fontfallback(factory->fallback);
     heap_free(factory);
 }
 
@@ -764,6 +788,8 @@ static HRESULT WINAPI dwritefactory_CreateCustomFontFileReference(IDWriteFactory
 
     TRACE("(%p)->(%p %u %p %p)\n", This, reference_key, key_size, loader, font_file);
 
+    *font_file = NULL;
+
     if (!loader || !factory_get_file_loader(This, loader))
         return E_INVALIDARG;
 
@@ -780,7 +806,7 @@ static HRESULT WINAPI dwritefactory_CreateFontFace(IDWriteFactory2 *iface,
     IDWriteFontFileLoader *loader;
     struct fontfacecached *cached;
     struct list *fontfaces;
-    IDWriteFontFace2 *face;
+    IDWriteFontFace3 *face;
     UINT32 key_size, count;
     BOOL is_supported;
     const void *key;
@@ -803,9 +829,12 @@ static HRESULT WINAPI dwritefactory_CreateFontFace(IDWriteFactory2 *iface,
     if (FAILED(hr))
         return hr;
 
-    if (!is_supported || (face_type != req_facetype))
+    if (!is_supported)
         return E_FAIL;
 
+    if (face_type != req_facetype)
+        return DWRITE_E_FILEFORMAT;
+
     hr = IDWriteFontFile_GetReferenceKey(*font_files, &key, &key_size);
     if (FAILED(hr))
         return hr;
@@ -864,8 +893,8 @@ static HRESULT WINAPI dwritefactory_CreateFontFace(IDWriteFactory2 *iface,
     /* new cache entry */
     cached = heap_alloc(sizeof(*cached));
     if (!cached) {
-        IDWriteFontFace2_Release(face);
-        return hr;
+        IDWriteFontFace3_Release(face);
+        return E_OUTOFMEMORY;
     }
 
     cached->fontface = (IDWriteFontFace*)face;
@@ -903,7 +932,7 @@ static HRESULT WINAPI dwritefactory_CreateMonitorRenderingParams(IDWriteFactory2
     if (!fixme_once++)
         FIXME("(%p): monitor setting ignored\n", monitor);
 
-    hr = IDWriteFactory2_CreateCustomRenderingParams(iface, 0.0, 0.0, 1.0, 0.0, DWRITE_PIXEL_GEOMETRY_FLAT, DWRITE_RENDERING_MODE_DEFAULT,
+    hr = IDWriteFactory2_CreateCustomRenderingParams(iface, 0.0f, 0.0f, 1.0f, 0.0f, DWRITE_PIXEL_GEOMETRY_FLAT, DWRITE_RENDERING_MODE_DEFAULT,
         DWRITE_GRID_FIT_MODE_DEFAULT, &params2);
     *params = (IDWriteRenderingParams*)params2;
     return hr;
@@ -918,7 +947,7 @@ static HRESULT WINAPI dwritefactory_CreateCustomRenderingParams(IDWriteFactory2
 
     TRACE("(%p)->(%f %f %f %d %d %p)\n", This, gamma, enhancedContrast, cleartype_level, geometry, mode, params);
 
-    hr = IDWriteFactory2_CreateCustomRenderingParams(iface, gamma, enhancedContrast, 1.0, cleartype_level, geometry,
+    hr = IDWriteFactory2_CreateCustomRenderingParams(iface, gamma, enhancedContrast, 1.0f, cleartype_level, geometry,
         mode, DWRITE_GRID_FIT_MODE_DEFAULT, &params2);
     *params = (IDWriteRenderingParams*)params2;
     return hr;
@@ -1027,10 +1056,10 @@ static HRESULT WINAPI dwritefactory_CreateTextLayout(IDWriteFactory2 *iface, WCH
     UINT32 len, IDWriteTextFormat *format, FLOAT max_width, FLOAT max_height, IDWriteTextLayout **layout)
 {
     struct dwritefactory *This = impl_from_IDWriteFactory2(iface);
+
     TRACE("(%p)->(%s:%u %p %f %f %p)\n", This, debugstr_wn(string, len), len, format, max_width, max_height, layout);
 
-    if (!format) return E_INVALIDARG;
-    return create_textlayout(string, len, format, max_width, max_height, layout);
+    return create_textlayout(iface, string, len, format, max_width, max_height, layout);
 }
 
 static HRESULT WINAPI dwritefactory_CreateGdiCompatibleTextLayout(IDWriteFactory2 *iface, WCHAR const* string,
@@ -1042,8 +1071,7 @@ static HRESULT WINAPI dwritefactory_CreateGdiCompatibleTextLayout(IDWriteFactory
     TRACE("(%p)->(%s:%u %p %f %f %f %p %d %p)\n", This, debugstr_wn(string, len), len, format, layout_width, layout_height,
         pixels_per_dip, transform, use_gdi_natural, layout);
 
-    if (!format) return E_INVALIDARG;
-    return create_gdicompat_textlayout(string, len, format, layout_width, layout_height, pixels_per_dip, transform,
+    return create_gdicompat_textlayout(iface, string, len, format, layout_width, layout_height, pixels_per_dip, transform,
         use_gdi_natural, layout);
 }
 
@@ -1079,7 +1107,12 @@ static HRESULT WINAPI dwritefactory_CreateGlyphRunAnalysis(IDWriteFactory2 *ifac
     TRACE("(%p)->(%p %.2f %p %d %d %.2f %.2f %p)\n", This, run, ppdip, transform, rendering_mode,
         measuring_mode, originX, originY, analysis);
 
-    return create_glyphrunanalysis(rendering_mode, measuring_mode, run, 1.0f, DWRITE_GRID_FIT_MODE_DEFAULT,
+    if (ppdip <= 0.0f) {
+        *analysis = NULL;
+        return E_INVALIDARG;
+    }
+
+    return create_glyphrunanalysis(rendering_mode, measuring_mode, run, ppdip, transform, DWRITE_GRID_FIT_MODE_DEFAULT,
         DWRITE_TEXT_ANTIALIAS_MODE_CLEARTYPE, originX, originY, analysis);
 }
 
@@ -1124,8 +1157,20 @@ static HRESULT WINAPI dwritefactory1_CreateCustomRenderingParams(IDWriteFactory2
 static HRESULT WINAPI dwritefactory2_GetSystemFontFallback(IDWriteFactory2 *iface, IDWriteFontFallback **fallback)
 {
     struct dwritefactory *This = impl_from_IDWriteFactory2(iface);
-    FIXME("(%p)->(%p): stub\n", This, fallback);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, fallback);
+
+    *fallback = NULL;
+
+    if (!This->fallback) {
+        HRESULT hr = create_system_fontfallback(iface, &This->fallback);
+        if (FAILED(hr))
+            return hr;
+    }
+
+    *fallback = This->fallback;
+    IDWriteFontFallback_AddRef(*fallback);
+    return S_OK;
 }
 
 static HRESULT WINAPI dwritefactory2_CreateFontFallbackBuilder(IDWriteFactory2 *iface, IDWriteFontFallbackBuilder **fallbackbuilder)
@@ -1165,7 +1210,7 @@ static HRESULT WINAPI dwritefactory2_CreateGlyphRunAnalysis(IDWriteFactory2 *ifa
     TRACE("(%p)->(%p %p %d %d %d %d %.2f %.2f %p)\n", This, run, transform, rendering_mode, measuring_mode,
         gridfit_mode, aa_mode, originX, originY, analysis);
 
-    return create_glyphrunanalysis(rendering_mode, measuring_mode, run, 1.0f, gridfit_mode, aa_mode, originX, originY, analysis);
+    return create_glyphrunanalysis(rendering_mode, measuring_mode, run, 1.0f, transform, gridfit_mode, aa_mode, originX, originY, analysis);
 }
 
 static const struct IDWriteFactory2Vtbl dwritefactoryvtbl = {
@@ -1258,6 +1303,7 @@ static void init_dwritefactory(struct dwritefactory *factory, DWRITE_FACTORY_TYP
     factory->system_collection = NULL;
     factory->eudc_collection = NULL;
     factory->gdiinterop = NULL;
+    factory->fallback = NULL;
 
     list_init(&factory->collection_loaders);
     list_init(&factory->file_loaders);
