@@ -27,8 +27,7 @@
 
 #include "initguid.h"
 #include "windows.h"
-#include "dwrite.h"
-#include "dwrite_2.h"
+#include "dwrite_3.h"
 
 #include "wine/test.h"
 
@@ -966,21 +965,44 @@ struct linebreaks_test {
 };
 
 static struct linebreaks_test linebreaks_tests[] = {
-    { {'A','-','B',' ','C',0x58a,'D',0x2010,'E',0x2012,'F',0x2013,'\t',0},
+    { {'A','-','B',' ','C',0x58a,'D',0x2010,'E',0x2012,'F',0x2013,'\t',0xc,0xb,0x2028,0x2029,0x200b,0},
       {
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     TRUE,  FALSE },
-          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, FALSE, FALSE },
-          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     TRUE,  FALSE }
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     1, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 1, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MUST_BREAK,    1, 0 },
+          { DWRITE_BREAK_CONDITION_MUST_BREAK,    DWRITE_BREAK_CONDITION_MUST_BREAK,    1, 0 },
+          { DWRITE_BREAK_CONDITION_MUST_BREAK,    DWRITE_BREAK_CONDITION_MUST_BREAK,    1, 0 },
+          { DWRITE_BREAK_CONDITION_MUST_BREAK,    DWRITE_BREAK_CONDITION_MUST_BREAK,    1, 0 },
+          { DWRITE_BREAK_CONDITION_MUST_BREAK,    DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+      }
+    },
+    /* Soft hyphen, visible word dividers */
+    { {'A',0xad,'B',0x5be,'C',0xf0b,'D',0x1361,'E',0x17d8,'F',0x17da,'G',0},
+      {
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 1 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, 0, 0 },
+          { DWRITE_BREAK_CONDITION_MAY_NOT_BREAK, DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
+          { DWRITE_BREAK_CONDITION_CAN_BREAK,     DWRITE_BREAK_CONDITION_CAN_BREAK,     0, 0 },
       }
     },
     { { 0 } }
@@ -988,6 +1010,7 @@ static struct linebreaks_test linebreaks_tests[] = {
 
 static void compare_breakpoints(const struct linebreaks_test *test, DWRITE_LINE_BREAKPOINT *actual)
 {
+    static const char *conditions[] = {"N","CB","NB","B"};
     const WCHAR *text = test->text;
     int cmp = memcmp(test->bp, actual, sizeof(*actual)*BREAKPOINT_COUNT);
     ok(!cmp, "%s: got wrong breakpoint data\n", wine_dbgstr_w(test->text));
@@ -995,16 +1018,19 @@ static void compare_breakpoints(const struct linebreaks_test *test, DWRITE_LINE_
         int i = 0;
         while (*text) {
             ok(!memcmp(&test->bp[i], &actual[i], sizeof(*actual)),
-                "%s: got (%d, %d, %d, %d), expected (%d, %d, %d, %d)\n",
+                "%s: got [%s, %s] (%s, %s), expected [%s, %s] (%s, %s)\n",
                 wine_dbgstr_wn(&test->text[i], 1),
-                g_actual_bp[i].breakConditionBefore,
-                g_actual_bp[i].breakConditionAfter,
-                g_actual_bp[i].isWhitespace,
-                g_actual_bp[i].isSoftHyphen,
-                test->bp[i].breakConditionBefore,
-                test->bp[i].breakConditionAfter,
-                test->bp[i].isWhitespace,
-                test->bp[i].isSoftHyphen);
+                conditions[g_actual_bp[i].breakConditionBefore],
+                conditions[g_actual_bp[i].breakConditionAfter],
+                g_actual_bp[i].isWhitespace ? "WS"  : "0",
+                g_actual_bp[i].isSoftHyphen ? "SHY" : "0",
+                conditions[test->bp[i].breakConditionBefore],
+                conditions[test->bp[i].breakConditionAfter],
+                test->bp[i].isWhitespace ? "WS"  : "0",
+                test->bp[i].isSoftHyphen ? "SHY" : "0");
+            if (g_actual_bp[i].isSoftHyphen)
+                ok(!g_actual_bp[i].isWhitespace, "%s: soft hyphen marked as a whitespace\n",
+                    wine_dbgstr_wn(&test->text[i], 1));
             text++;
             i++;
         }
@@ -1016,6 +1042,7 @@ static void test_AnalyzeLineBreakpoints(void)
     static const WCHAR emptyW[] = {0};
     const struct linebreaks_test *ptr = linebreaks_tests;
     IDWriteTextAnalyzer *analyzer;
+    UINT32 i = 0;
     HRESULT hr;
 
     hr = IDWriteFactory_CreateTextAnalyzer(factory, &analyzer);
@@ -1027,13 +1054,24 @@ static void test_AnalyzeLineBreakpoints(void)
 
     while (*ptr->text)
     {
+        UINT32 len;
+
         g_source = ptr->text;
+        len = lstrlenW(g_source);
+
+        if (len > BREAKPOINT_COUNT) {
+            ok(0, "test %u: increase BREAKPOINT_COUNT to at least %u\n", i, len);
+            i++;
+            ptr++;
+            continue;
+        }
 
         memset(g_actual_bp, 0, sizeof(g_actual_bp));
-        hr = IDWriteTextAnalyzer_AnalyzeLineBreakpoints(analyzer, &analysissource, 0, lstrlenW(g_source), &analysissink);
+        hr = IDWriteTextAnalyzer_AnalyzeLineBreakpoints(analyzer, &analysissource, 0, len, &analysissink);
         ok(hr == S_OK, "got 0x%08x\n", hr);
         compare_breakpoints(ptr, g_actual_bp);
 
+        i++;
         ptr++;
     }
 
@@ -1197,6 +1235,35 @@ static void test_numbersubstitution(void)
     IDWriteNumberSubstitution_Release(substitution);
 }
 
+static void get_fontface_glyphs(IDWriteFontFace *fontface, const WCHAR *str, UINT16 *glyphs)
+{
+    while (*str) {
+        UINT32 codepoint = *str;
+        HRESULT hr;
+
+        hr = IDWriteFontFace_GetGlyphIndices(fontface, &codepoint, 1, glyphs++);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+        str++;
+    }
+}
+
+static void get_fontface_advances(IDWriteFontFace *fontface, FLOAT emsize, const UINT16 *glyphs, FLOAT *advances, UINT32 count)
+{
+    DWRITE_FONT_METRICS fontmetrics;
+    UINT32 i;
+
+    IDWriteFontFace_GetMetrics(fontface, &fontmetrics);
+    for (i = 0; i < count; i++) {
+        DWRITE_GLYPH_METRICS metrics;
+        HRESULT hr;
+
+        hr = IDWriteFontFace_GetDesignGlyphMetrics(fontface, glyphs + i, 1, &metrics, FALSE);
+        ok(hr == S_OK, "got 0x%08x\n", hr);
+
+        advances[i] = (FLOAT)metrics.advanceWidth * emsize / (FLOAT)fontmetrics.designUnitsPerEm;
+    }
+}
+
 static void test_GetGlyphs(void)
 {
     static const WCHAR test1W[] = {'<','B',' ','C',0};
@@ -1205,6 +1272,7 @@ static void test_GetGlyphs(void)
     DWRITE_SHAPING_GLYPH_PROPERTIES shapingprops[20];
     DWRITE_SHAPING_TEXT_PROPERTIES props[20];
     UINT32 maxglyphcount, actual_count;
+    FLOAT advances[10], advances2[10];
     IDWriteTextAnalyzer *analyzer;
     IDWriteFontFace *fontface;
     DWRITE_SCRIPT_ANALYSIS sa;
@@ -1212,7 +1280,6 @@ static void test_GetGlyphs(void)
     UINT16 clustermap[10];
     UINT16 glyphs1[10];
     UINT16 glyphs2[10];
-    FLOAT advances[10];
     HRESULT hr;
 
     hr = IDWriteFactory_CreateTextAnalyzer(factory, &analyzer);
@@ -1275,13 +1342,16 @@ if (0) {
     ok(glyphs1[0] != glyphs2[0], "got %d\n", glyphs1[0]);
 
     /* embedded control codes, with unknown script id 0 */
+    get_fontface_glyphs(fontface, test3W, glyphs2);
+    get_fontface_advances(fontface, 10.0, glyphs2, advances2, 2);
+
     actual_count = 0;
     hr = IDWriteTextAnalyzer_GetGlyphs(analyzer, test3W, lstrlenW(test3W), fontface, FALSE, TRUE, &sa, NULL,
         NULL, NULL, NULL, 0, maxglyphcount, clustermap, props, glyphs1, shapingprops, &actual_count);
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok(actual_count == 2, "got %d\n", actual_count);
-    ok(glyphs1[0] == 0, "got %d\n", glyphs1[0]);
-    ok(glyphs1[1] == 0, "got %d\n", glyphs1[1]);
+    ok(glyphs1[0] == glyphs2[0], "got %u, expected %u\n", glyphs1[0], glyphs2[0]);
+    ok(glyphs1[1] == glyphs2[1], "got %u, expected %u\n", glyphs1[1], glyphs2[1]);
     ok(shapingprops[0].isClusterStart == 1, "got %d\n", shapingprops[0].isClusterStart);
     ok(shapingprops[0].isZeroWidthSpace == 0, "got %d\n", shapingprops[0].isZeroWidthSpace);
     ok(shapingprops[1].isClusterStart == 1, "got %d\n", shapingprops[1].isClusterStart);
@@ -1294,8 +1364,8 @@ if (0) {
         glyphs1, shapingprops, actual_count, fontface, 10.0, FALSE, FALSE, &sa, NULL, NULL,
         NULL, 0, advances, offsets);
     ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(advances[0] == 10.0, "got %.2f\n", advances[0]);
-    ok(advances[1] == 10.0, "got %.2f\n", advances[1]);
+    ok(advances[0] == advances2[0], "got %.2f, expected %.2f\n", advances[0], advances2[0]);
+    ok(advances[1] == advances2[1], "got %.2f, expected %.2f\n", advances[1], advances2[1]);
 
     /* embedded control codes with proper script */
     sa.script = 0;
@@ -1306,8 +1376,8 @@ if (0) {
         NULL, NULL, NULL, 0, maxglyphcount, clustermap, props, glyphs1, shapingprops, &actual_count);
     ok(hr == S_OK, "got 0x%08x\n", hr);
     ok(actual_count == 2, "got %d\n", actual_count);
-    ok(glyphs1[0] == 0, "got %d\n", glyphs1[0]);
-    ok(glyphs1[1] == 0, "got %d\n", glyphs1[1]);
+    ok(glyphs1[0] == glyphs2[0], "got %u, expected %u\n", glyphs1[0], glyphs2[0]);
+    ok(glyphs1[1] == glyphs2[1], "got %u, expected %u\n", glyphs1[1], glyphs2[1]);
     ok(shapingprops[0].isClusterStart == 1, "got %d\n", shapingprops[0].isClusterStart);
     ok(shapingprops[0].isZeroWidthSpace == 0, "got %d\n", shapingprops[0].isZeroWidthSpace);
     ok(shapingprops[1].isClusterStart == 1, "got %d\n", shapingprops[1].isClusterStart);
@@ -1320,8 +1390,20 @@ if (0) {
         glyphs1, shapingprops, actual_count, fontface, 10.0, FALSE, FALSE, &sa, NULL, NULL,
         NULL, 0, advances, offsets);
     ok(hr == S_OK, "got 0x%08x\n", hr);
-    ok(advances[0] == 10.0, "got %.2f\n", advances[0]);
-    ok(advances[1] == 10.0, "got %.2f\n", advances[1]);
+    ok(advances[0] == advances2[0], "got %.2f, expected %.2f\n", advances[0], advances2[0]);
+    ok(advances[1] == advances2[1], "got %.2f, expected %.2f\n", advances[1], advances2[1]);
+
+    /* DWRITE_SCRIPT_SHAPES_NO_VISUAL run */
+    maxglyphcount = 10;
+    actual_count = 0;
+    sa.script = 0;
+    sa.shapes = DWRITE_SCRIPT_SHAPES_NO_VISUAL;
+    hr = IDWriteTextAnalyzer_GetGlyphs(analyzer, test1W, lstrlenW(test1W), fontface, FALSE, FALSE, &sa, NULL,
+        NULL, NULL, NULL, 0, maxglyphcount, clustermap, props, glyphs1, shapingprops, &actual_count);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(actual_count == 4, "got %d\n", actual_count);
+    ok(sa.script == 0, "got %u\n", sa.script);
+    ok(!shapingprops[0].isZeroWidthSpace, "got %d\n", shapingprops[0].isZeroWidthSpace);
 
     IDWriteTextAnalyzer_Release(analyzer);
     IDWriteFontFace_Release(fontface);
@@ -1512,6 +1594,33 @@ static void test_GetGlyphPlacements(void)
     ok(offsets[0].advanceOffset == 0.0 && offsets[0].ascenderOffset == 0.0, "got %.2f,%.2f\n",
         offsets[0].advanceOffset, offsets[0].ascenderOffset);
 
+    /* DWRITE_SCRIPT_SHAPES_NO_VISUAL has no effect on placement */
+    sa.shapes = DWRITE_SCRIPT_SHAPES_NO_VISUAL;
+    advances[0] = advances[1] = 1.0f;
+    memset(offsets, 0xcc, sizeof(offsets));
+    hr = IDWriteTextAnalyzer_GetGlyphPlacements(analyzer, aW, clustermap, textprops,
+        len, glyphs, glyphprops, len, fontface, 2048.0f, FALSE, FALSE, &sa, NULL, NULL,
+        NULL, 0, advances, offsets);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(advances[0] == 1000.0f, "got %.2f\n", advances[0]);
+    ok(advances[1] == 1000.0f, "got %.2f\n", advances[1]);
+    ok(offsets[0].advanceOffset == 0.0f && offsets[0].ascenderOffset == 0.0f, "got %.2f,%.2f\n",
+        offsets[0].advanceOffset, offsets[0].ascenderOffset);
+
+    /* isZeroWidthSpace */
+    sa.shapes = DWRITE_SCRIPT_SHAPES_DEFAULT;
+    advances[0] = advances[1] = 1.0f;
+    memset(offsets, 0xcc, sizeof(offsets));
+    glyphprops[0].isZeroWidthSpace = 1;
+    hr = IDWriteTextAnalyzer_GetGlyphPlacements(analyzer, aW, clustermap, textprops,
+        len, glyphs, glyphprops, len, fontface, 2048.0f, FALSE, FALSE, &sa, NULL, NULL,
+        NULL, 0, advances, offsets);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(advances[0] == 0.0f, "got %.2f\n", advances[0]);
+    ok(advances[1] == 1000.0f, "got %.2f\n", advances[1]);
+    ok(offsets[0].advanceOffset == 0.0f && offsets[0].ascenderOffset == 0.0f, "got %.2f,%.2f\n",
+        offsets[0].advanceOffset, offsets[0].ascenderOffset);
+
     IDWriteTextAnalyzer_Release(analyzer);
     IDWriteFontFace_Release(fontface);
     DELETE_FONTFILE(path);
