@@ -21,6 +21,12 @@
 #include "d3d11.h"
 #include "wine/test.h"
 
+enum frame_latency
+{
+    DEFAULT_FRAME_LATENCY =  3,
+    MAX_FRAME_LATENCY     = 16,
+};
+
 static DEVMODEW registry_mode;
 
 static HRESULT (WINAPI *pCreateDXGIFactory1)(REFIID iid, void **factory);
@@ -54,53 +60,6 @@ success:
     return dxgi_device;
 }
 
-static void test_device_interfaces(void)
-{
-    IDXGIDevice *device;
-    IUnknown *iface;
-    ULONG refcount;
-    HRESULT hr;
-
-    if (!(device = create_device()))
-    {
-        skip("Failed to create device, skipping tests.\n");
-        return;
-    }
-
-    hr = IDXGIDevice_QueryInterface(device, &IID_IUnknown, (void **)&iface);
-    ok(SUCCEEDED(hr), "Failed to query IUnknown interface, hr %#x.\n", hr);
-    IUnknown_Release(iface);
-
-    hr = IDXGIDevice_QueryInterface(device, &IID_IDXGIObject, (void **)&iface);
-    ok(SUCCEEDED(hr), "Failed to query IDXGIObject interface, hr %#x.\n", hr);
-    IUnknown_Release(iface);
-
-    hr = IDXGIDevice_QueryInterface(device, &IID_IDXGIDevice, (void **)&iface);
-    ok(SUCCEEDED(hr), "Failed to query IDXGIDevice interface, hr %#x.\n", hr);
-    IUnknown_Release(iface);
-
-    hr = IDXGIDevice_QueryInterface(device, &IID_ID3D10Device, (void **)&iface);
-    ok(SUCCEEDED(hr), "Failed to query ID3D10Device interface, hr %#x.\n", hr);
-    IUnknown_Release(iface);
-
-    hr = IDXGIDevice_QueryInterface(device, &IID_ID3D10Multithread, (void **)&iface);
-    ok(SUCCEEDED(hr), "Failed to query ID3D10Multithread interface, hr %#x.\n", hr);
-    IUnknown_Release(iface);
-
-    if (SUCCEEDED(hr = IDXGIDevice_QueryInterface(device, &IID_ID3D10Device1, (void **)&iface)))
-        IUnknown_Release(iface);
-    ok(SUCCEEDED(hr) || broken(hr == E_NOINTERFACE) /* Not available on all Windows versions. */,
-            "Failed to query ID3D10Device1 interface, hr %#x.\n", hr);
-
-    if (SUCCEEDED(hr = IDXGIDevice_QueryInterface(device, &IID_ID3D11Device, (void **)&iface)))
-        IUnknown_Release(iface);
-    ok(SUCCEEDED(hr) || broken(hr == E_NOINTERFACE) /* Not available on all Windows versions. */,
-            "Failed to query ID3D11Device interface, hr %#x.\n", hr);
-
-    refcount = IDXGIDevice_Release(device);
-    ok(!refcount, "Device has %u references left.\n", refcount);
-}
-
 static void test_adapter_desc(void)
 {
     DXGI_ADAPTER_DESC1 desc1;
@@ -170,6 +129,60 @@ done:
     ok(!refcount, "Device has %u references left.\n", refcount);
 }
 
+static void test_check_interface_support(void)
+{
+    LARGE_INTEGER driver_version;
+    IDXGIAdapter *adapter;
+    IDXGIDevice *device;
+    IUnknown *iface;
+    ULONG refcount;
+    HRESULT hr;
+
+    if (!(device = create_device()))
+    {
+        skip("Failed to create device.\n");
+        return;
+    }
+
+    hr = IDXGIDevice_GetAdapter(device, &adapter);
+    ok(SUCCEEDED(hr), "GetAdapter failed, hr %#x.\n", hr);
+
+    hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D10Device, NULL);
+    ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+    hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D10Device, &driver_version);
+    ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+
+    trace("UMD version: %u.%u.%u.%u.\n",
+            HIWORD(U(driver_version).HighPart), LOWORD(U(driver_version).HighPart),
+            HIWORD(U(driver_version).LowPart), LOWORD(U(driver_version).LowPart));
+
+    hr = IDXGIDevice_QueryInterface(device, &IID_ID3D10Device1, (void **)&iface);
+    if (SUCCEEDED(hr))
+    {
+        IUnknown_Release(iface);
+        hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D10Device1, NULL);
+        ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D10Device1, &driver_version);
+        ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+    }
+    else
+    {
+        win_skip("D3D10.1 is not supported.\n");
+    }
+
+    hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D11Device, NULL);
+    ok(hr == DXGI_ERROR_UNSUPPORTED, "Got unexpected hr %#x.\n", hr);
+    driver_version.HighPart = driver_version.LowPart = 0xdeadbeef;
+    hr = IDXGIAdapter_CheckInterfaceSupport(adapter, &IID_ID3D11Device, &driver_version);
+    ok(hr == DXGI_ERROR_UNSUPPORTED, "Got unexpected hr %#x.\n", hr);
+    ok(driver_version.HighPart == 0xdeadbeef, "Got unexpected driver version %#x.\n", driver_version.HighPart);
+    ok(driver_version.LowPart == 0xdeadbeef, "Got unexpected driver version %#x.\n", driver_version.LowPart);
+
+    IDXGIAdapter_Release(adapter);
+    refcount = IDXGIDevice_Release(device);
+    ok(!refcount, "Device has %u references left.\n", refcount);
+}
+
 static void test_create_surface(void)
 {
     DXGI_SURFACE_DESC desc;
@@ -465,19 +478,13 @@ static void test_createswapchain(void)
         hr = IDXGISwapChain_GetDesc(swapchain, &result_desc);
         ok(SUCCEEDED(hr), "GetDesc failed, hr %#x.\n", hr);
 
-        if (refresh_list[i].numerator_should_pass)
+        todo_wine_if (!refresh_list[i].numerator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
                 "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
-                "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
 
-        if (refresh_list[i].denominator_should_pass)
+        todo_wine_if (!refresh_list[i].denominator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
                     "Denominator %u is %u.\n", i ,result_desc.BufferDesc.RefreshRate.Denominator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
-                    "Denominator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Denominator);
 
         IDXGISwapChain_Release(swapchain);
     }
@@ -495,19 +502,13 @@ static void test_createswapchain(void)
         hr = IDXGISwapChain_GetDesc(swapchain, &result_desc);
         ok(SUCCEEDED(hr), "GetDesc failed, hr %#x.\n", hr);
 
-        if (refresh_list[i].numerator_should_pass)
+        todo_wine_if (!refresh_list[i].numerator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
                     "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Numerator == refresh_list[i].numerator,
-                    "Numerator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Numerator);
 
-        if (refresh_list[i].denominator_should_pass)
+        todo_wine_if (!refresh_list[i].denominator_should_pass)
             ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
                     "Denominator %u is %u.\n", i ,result_desc.BufferDesc.RefreshRate.Denominator);
-        else
-            todo_wine ok(result_desc.BufferDesc.RefreshRate.Denominator == refresh_list[i].denominator,
-                    "Denominator %u is %u.\n", i, result_desc.BufferDesc.RefreshRate.Denominator);
 
         hr = IDXGISwapChain_SetFullscreenState(swapchain, FALSE, NULL);
         todo_wine ok(SUCCEEDED(hr), "SetFullscreenState failed, hr %#x.\n", hr);
@@ -1038,7 +1039,7 @@ static void test_swapchain_parameters(void)
     HRESULT hr;
     unsigned int i, j;
     ULONG refcount;
-    DXGI_USAGE usage, expected_usage;
+    DXGI_USAGE usage, expected_usage, broken_usage;
     HWND window;
     static const struct
     {
@@ -1064,15 +1065,20 @@ static void test_swapchain_parameters(void)
         {TRUE,   1, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {TRUE,   2, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL,  1},
         {TRUE,   3, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL,  2},
-        {TRUE,   0, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
-        {TRUE,   1, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
-        {TRUE,   2, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   0, DXGI_SWAP_EFFECT_FLIP_DISCARD,    DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   1, DXGI_SWAP_EFFECT_FLIP_DISCARD,    DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   2, DXGI_SWAP_EFFECT_FLIP_DISCARD,    S_OK,                    DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   0, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   1, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,   2, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {TRUE,  16, DXGI_SWAP_EFFECT_DISCARD,         S_OK,                    S_OK,                     0},
         {TRUE,  16, DXGI_SWAP_EFFECT_SEQUENTIAL,      S_OK,                    S_OK,                    15},
         {TRUE,  16, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL, 15},
+        {TRUE,  16, DXGI_SWAP_EFFECT_FLIP_DISCARD,    S_OK,                    DXGI_ERROR_INVALID_CALL,  0},
         {TRUE,  17, DXGI_SWAP_EFFECT_DISCARD,         DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {TRUE,  17, DXGI_SWAP_EFFECT_SEQUENTIAL,      DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {TRUE,  17, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {TRUE,  17, DXGI_SWAP_EFFECT_DISCARD,         DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
 
         {FALSE,  0, DXGI_SWAP_EFFECT_DISCARD,         DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {FALSE,  1, DXGI_SWAP_EFFECT_DISCARD,         S_OK,                    S_OK,                     0},
@@ -1088,15 +1094,23 @@ static void test_swapchain_parameters(void)
         {FALSE,  1, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {FALSE,  2, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL,  1},
         {FALSE,  3, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL,  2},
-        {FALSE,  0, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
-        {FALSE,  1, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
-        {FALSE,  2, 4 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  0, DXGI_SWAP_EFFECT_FLIP_DISCARD,    DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  1, DXGI_SWAP_EFFECT_FLIP_DISCARD,    DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  2, DXGI_SWAP_EFFECT_FLIP_DISCARD,    S_OK,                    DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  0, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  1, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE,  2, 5 /* undefined */,                DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {FALSE, 16, DXGI_SWAP_EFFECT_DISCARD,         S_OK,                    S_OK,                     0},
         {FALSE, 16, DXGI_SWAP_EFFECT_SEQUENTIAL,      S_OK,                    S_OK,                    15},
         {FALSE, 16, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, S_OK,                    DXGI_ERROR_INVALID_CALL, 15},
+        /* The following test fails on Nvidia with E_OUTOFMEMORY and leaks device references in the
+         * process. Disable it for now.
+        {FALSE, 16, DXGI_SWAP_EFFECT_FLIP_DISCARD,    S_OK,                    DXGI_ERROR_INVALID_CALL,  0},
+         */
         {FALSE, 17, DXGI_SWAP_EFFECT_DISCARD,         DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {FALSE, 17, DXGI_SWAP_EFFECT_SEQUENTIAL,      DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
         {FALSE, 17, DXGI_SWAP_EFFECT_FLIP_SEQUENTIAL, DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
+        {FALSE, 17, DXGI_SWAP_EFFECT_FLIP_DISCARD,    DXGI_ERROR_INVALID_CALL, DXGI_ERROR_INVALID_CALL,  0},
     };
 
     if (!(device = create_device()))
@@ -1165,13 +1179,14 @@ static void test_swapchain_parameters(void)
             hr = IDXGISwapChain_GetBuffer(swapchain, j, &IID_IDXGIResource, (void **)&resource);
             ok(SUCCEEDED(hr), "GetBuffer(%u) failed, hr %#x, test %u.\n", hr, i, j);
 
-            expected_usage = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_BACK_BUFFER;
-
             /* Buffers > 0 are supposed to be read only. This is the case except that in
-             * fullscreen mode the last backbuffer (BufferCount - 1) is writeable. This
-             * is not the case if an unsupported refresh rate is passed for some reason,
-             * probably because the invalid refresh rate triggers a kinda-sorta windowed
-             * mode.
+             * fullscreen mode on Windows <= 8 the last backbuffer (BufferCount - 1) is
+             * writable. This is not the case if an unsupported refresh rate is passed
+             * for some reason, probably because the invalid refresh rate triggers a
+             * kinda-sorta windowed mode.
+             *
+             * On Windows 10 all buffers > 0 are read-only. Mark the earlier behavior
+             * broken.
              *
              * This last buffer acts as a shadow frontbuffer. Writing to it doesn't show
              * the draw on the screen right away (Aero on or off doesn't matter), but
@@ -1180,12 +1195,15 @@ static void test_swapchain_parameters(void)
              * Note that if the application doesn't have focused creating a fullscreen
              * swapchain returns DXGI_STATUS_OCCLUDED and we get a windowed swapchain,
              * so use the Windowed property of the swapchain that was actually created. */
+            expected_usage = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_BACK_BUFFER | DXGI_USAGE_READ_ONLY;
+            broken_usage = DXGI_USAGE_RENDER_TARGET_OUTPUT | DXGI_USAGE_BACK_BUFFER;
+
             if (desc.Windowed || j < tests[i].highest_accessible_buffer)
-                expected_usage |= DXGI_USAGE_READ_ONLY;
+                broken_usage |= DXGI_USAGE_READ_ONLY;
 
             hr = IDXGIResource_GetUsage(resource, &usage);
             ok(SUCCEEDED(hr), "Failed to get resource usage, hr %#x, test %u, buffer %u.\n", hr, i, j);
-            ok(usage == expected_usage, "Got usage %x, expected %x, test %u, buffer %u.\n",
+            ok(usage == expected_usage || broken(usage == broken_usage), "Got usage %x, expected %x, test %u, buffer %u.\n",
                     usage, expected_usage, i, j);
 
             IDXGIResource_Release(resource);
@@ -1207,6 +1225,112 @@ static void test_swapchain_parameters(void)
     DestroyWindow(window);
 }
 
+static void test_maximum_frame_latency(void)
+{
+    IDXGIDevice1 *device1;
+    IDXGIDevice *device;
+    UINT max_latency;
+    ULONG refcount;
+    HRESULT hr;
+
+    if (!(device = create_device()))
+    {
+        skip("Failed to create device.\n");
+        return;
+    }
+
+    if (SUCCEEDED(IDXGIDevice_QueryInterface(device, &IID_IDXGIDevice1, (void **)&device1)))
+    {
+        hr = IDXGIDevice1_GetMaximumFrameLatency(device1, &max_latency);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        ok(max_latency == DEFAULT_FRAME_LATENCY, "Got unexpected maximum frame latency %u.\n", max_latency);
+
+        hr = IDXGIDevice1_SetMaximumFrameLatency(device1, MAX_FRAME_LATENCY);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        hr = IDXGIDevice1_GetMaximumFrameLatency(device1, &max_latency);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        todo_wine ok(max_latency == MAX_FRAME_LATENCY, "Got unexpected maximum frame latency %u.\n", max_latency);
+
+        hr = IDXGIDevice1_SetMaximumFrameLatency(device1, MAX_FRAME_LATENCY + 1);
+        ok(hr == DXGI_ERROR_INVALID_CALL, "Got unexpected hr %#x.\n", hr);
+        hr = IDXGIDevice1_GetMaximumFrameLatency(device1, &max_latency);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        todo_wine ok(max_latency == MAX_FRAME_LATENCY, "Got unexpected maximum frame latency %u.\n", max_latency);
+
+        hr = IDXGIDevice1_SetMaximumFrameLatency(device1, 0);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        hr = IDXGIDevice1_GetMaximumFrameLatency(device1, &max_latency);
+        todo_wine ok(hr == S_OK, "Got unexpected hr %#x.\n", hr);
+        /* 0 does not reset to the default frame latency on all Windows versions. */
+        ok(max_latency == DEFAULT_FRAME_LATENCY || broken(!max_latency),
+                "Got unexpected maximum frame latency %u.\n", max_latency);
+
+        IDXGIDevice1_Release(device1);
+    }
+    else
+    {
+        win_skip("IDXGIDevice1 is not implemented.\n");
+    }
+
+    refcount = IDXGIDevice_Release(device);
+    ok(!refcount, "Device has %u references left.\n", refcount);
+}
+
+static void test_output_desc(void)
+{
+    DXGI_OUTPUT_DESC desc;
+    IDXGIFactory *factory;
+    IDXGIAdapter *adapter;
+    IDXGIOutput *output;
+    unsigned int i, j;
+    HRESULT hr;
+
+    hr = CreateDXGIFactory(&IID_IDXGIFactory, (void **)&factory);
+    ok(SUCCEEDED(hr), "Failed to create DXGI factory, hr %#x.\n", hr);
+
+    for (i = 0; ; ++i)
+    {
+        hr = IDXGIFactory_EnumAdapters(factory, i, &adapter);
+        if (hr == DXGI_ERROR_NOT_FOUND)
+            break;
+        ok(SUCCEEDED(hr), "Failed to enumerate adapter %u, hr %#x.\n", i, hr);
+
+        for (j = 0; ; ++j)
+        {
+            MONITORINFOEXW monitor_info;
+            BOOL ret;
+
+            hr = IDXGIAdapter_EnumOutputs(adapter, j, &output);
+            if (hr == DXGI_ERROR_NOT_FOUND)
+                break;
+            ok(SUCCEEDED(hr), "Failed to enumerate output %u on adapter %u, hr %#x.\n", j, i, hr);
+
+            hr = IDXGIOutput_GetDesc(output, NULL);
+            ok(hr == E_INVALIDARG, "Got unexpected hr %#x for output %u on adapter %u.\n", hr, j, i);
+            hr = IDXGIOutput_GetDesc(output, &desc);
+            ok(SUCCEEDED(hr), "Failed to get desc for output %u on adapter %u, hr %#x.\n", j, i, hr);
+
+            monitor_info.cbSize = sizeof(monitor_info);
+            ret = GetMonitorInfoW(desc.Monitor, (MONITORINFO *)&monitor_info);
+            ok(ret, "Failed to get monitor info.\n");
+            ok(!lstrcmpW(desc.DeviceName, monitor_info.szDevice), "Got unexpected device name %s, expected %s.\n",
+                    wine_dbgstr_w(desc.DeviceName), wine_dbgstr_w(monitor_info.szDevice));
+            ok(!memcmp(&desc.DesktopCoordinates, &monitor_info.rcMonitor, sizeof(desc.DesktopCoordinates)),
+                    "Got unexpected desktop coordinates {%d, %d, %d, %d}, expected {%d, %d, %d, %d}.\n",
+                    desc.DesktopCoordinates.left, desc.DesktopCoordinates.top,
+                    desc.DesktopCoordinates.right, desc.DesktopCoordinates.bottom,
+                    monitor_info.rcMonitor.left, monitor_info.rcMonitor.top,
+                    monitor_info.rcMonitor.right, monitor_info.rcMonitor.bottom);
+
+            IDXGIOutput_Release(output);
+        }
+
+        IDXGIAdapter_Release(adapter);
+    }
+
+    IDXGIFactory_Release(factory);
+}
+
 START_TEST(device)
 {
     pCreateDXGIFactory1 = (void *)GetProcAddress(GetModuleHandleA("dxgi.dll"), "CreateDXGIFactory1");
@@ -1215,7 +1339,7 @@ START_TEST(device)
     ok(EnumDisplaySettingsW(NULL, ENUM_REGISTRY_SETTINGS, &registry_mode), "Failed to get display mode.\n");
 
     test_adapter_desc();
-    test_device_interfaces();
+    test_check_interface_support();
     test_create_surface();
     test_parents();
     test_output();
@@ -1224,4 +1348,6 @@ START_TEST(device)
     test_private_data();
     test_swapchain_resize();
     test_swapchain_parameters();
+    test_maximum_frame_latency();
+    test_output_desc();
 }
