@@ -264,7 +264,7 @@ static HRESULT GAMEUX_updateStatisticsFile(struct GAMEUX_STATS *stats)
                     }
 
                     if(SUCCEEDED(hr))
-                        hr = IXMLDOMNode_appendChild(categoryNode, statisticsNode, &statisticsNode);
+                        hr = IXMLDOMNode_appendChild(categoryNode, statisticsNode, NULL);
 
                     IXMLDOMElement_Release(statisticsElement);
                     IXMLDOMNode_Release(statisticsNode);
@@ -578,6 +578,8 @@ static HRESULT GAMEUX_loadStatisticsFromFile(struct GAMEUX_STATS *data)
                             }
                         }
 
+                        IXMLDOMNodeList_Release(categoryChildren);
+
                         if(SUCCEEDED(hr))
                             hr = S_OK;
                     }
