@@ -33,16 +33,18 @@ WINE_DEFAULT_DEBUG_CHANNEL(hid);
 
 #define USAGE_MAX 10
 
+/* Flags that are defined in the document
+   "Device Class Definition for Human Interface Devices" */
 enum {
-    INPUT_DATA = 0x01,
-    INPUT_ARRAY = 0x02,
-    INPUT_ABS = 0x04,
-    INPUT_WRAP = 0x08,
-    INPUT_LINEAR = 0x10,
-    INPUT_PREFSTATE = 0x20,
-    INPUT_NULL = 0x40,
-    INPUT_VOLATILE = 0x80,
-    INPUT_BITFIELD = 0x100
+    INPUT_DATA_CONST = 0x01, /* Data (0)             | Constant (1)       */
+    INPUT_ARRAY_VAR = 0x02,  /* Array (0)            | Variable (1)       */
+    INPUT_ABS_REL = 0x04,    /* Absolute (0)         | Relative (1)       */
+    INPUT_WRAP = 0x08,       /* No Wrap (0)          | Wrap (1)           */
+    INPUT_LINEAR = 0x10,     /* Linear (0)           | Non Linear (1)     */
+    INPUT_PREFSTATE = 0x20,  /* Preferred State (0)  | No Preferred (1)   */
+    INPUT_NULL = 0x40,       /* No Null position (0) | Null state(1)      */
+    INPUT_VOLATILE = 0x80,   /* Non Volatile (0)     | Volatile (1)       */
+    INPUT_BITFIELD = 0x100   /* Bit Field (0)        | Buffered Bytes (1) */
 };
 
 enum {
@@ -399,7 +401,9 @@ static int getValue(int bsize, int source)
     return value;
 }
 
-void parse_io_feature(unsigned int bSize, int itemVal, int bTag, unsigned int *feature_index, struct feature *feature)
+static void parse_io_feature(unsigned int bSize, int itemVal, int bTag,
+                             unsigned int *feature_index,
+                             struct feature *feature)
 {
     if (bSize <= 0)
     {
@@ -407,55 +411,29 @@ void parse_io_feature(unsigned int bSize, int itemVal, int bTag, unsigned int *f
     }
     else
     {
-        if ((itemVal & INPUT_DATA) == 0)
-            feature->isData = TRUE;
-        else
-            feature->isData = FALSE; /* Const */
-        if ((itemVal & INPUT_ARRAY) == 0)
-            feature->isArray= TRUE;
-        else
-            feature->isArray= TRUE; /* Var */
-        if ((itemVal & INPUT_ABS) == 0)
-            feature->IsAbsolute = TRUE;
-        else
-            feature->IsAbsolute = FALSE; /* Rel */
-        if ((itemVal & INPUT_WRAP) == 0)
-            feature->Wrap = FALSE;
-        else
-            feature->Wrap = TRUE;
-        if ((itemVal & INPUT_LINEAR) == 0)
-            feature->Linear = TRUE;
-        else
-            feature->Linear = FALSE;
-        if ((itemVal & INPUT_PREFSTATE) == 0)
-            feature->prefState = TRUE;
-        else
-            feature->prefState = FALSE;
-        if ((itemVal & INPUT_NULL) == 0)
-            feature->HasNull = FALSE;
-        else
-            feature->HasNull = TRUE;
+        feature->isData = ((itemVal & INPUT_DATA_CONST) == 0);
+        feature->isArray =  ((itemVal & INPUT_ARRAY_VAR) == 0);
+        feature->IsAbsolute = ((itemVal & INPUT_ABS_REL) == 0);
+        feature->Wrap = ((itemVal & INPUT_WRAP) != 0);
+        feature->Linear = ((itemVal & INPUT_LINEAR) == 0);
+        feature->prefState = ((itemVal & INPUT_PREFSTATE) == 0);
+        feature->HasNull = ((itemVal & INPUT_NULL) != 0);
 
         if (bTag != TAG_MAIN_INPUT)
         {
-            if ((itemVal & INPUT_VOLATILE) == 0)
-                feature->Volatile = FALSE;
-            else
-                feature->Volatile = TRUE;
+            feature->Volatile = ((itemVal & INPUT_VOLATILE) != 0);
         }
         if (bSize > 1)
         {
-            if ((itemVal & INPUT_BITFIELD) == 0)
-                feature->BitField = TRUE;
-            else
-                feature->BitField = FALSE; /* Buffered Bytes */
+            feature->BitField = ((itemVal & INPUT_BITFIELD) == 0);
         }
         feature->index = *feature_index;
         *feature_index = *feature_index + 1;
     }
 }
 
-void parse_collection(unsigned int bSize, int itemVal, struct collection *collection)
+static void parse_collection(unsigned int bSize, int itemVal,
+                             struct collection *collection)
 {
     if (bSize <= 0)
         return;
@@ -797,7 +775,7 @@ static void count_elements(struct feature* feature, USHORT *buttons, USHORT *val
     }
 }
 
-WINE_HIDP_PREPARSED_DATA* build_PreparseData(
+static WINE_HIDP_PREPARSED_DATA* build_PreparseData(
                        struct feature **features, int feature_count,
                        struct feature **input_features, int i_count,
                        struct feature **output_features, int o_count,
@@ -989,7 +967,7 @@ WINE_HIDP_PREPARSED_DATA* build_PreparseData(
             if (feature_features[i]->caps.ReportID != wine_report->reportID)
             {
                 wine_report->dwSize += (sizeof(WINE_HID_ELEMENT) * wine_report->elementCount);
-                wine_report = (WINE_HID_REPORT*)((BYTE*)wine_report)+wine_report->dwSize;
+                wine_report = (WINE_HID_REPORT*)((BYTE*)wine_report+wine_report->dwSize);
                 new_report(wine_report, feature_features[i]);
                 data->dwFeatureReportCount++;
                 bitLength = max(bitOffset, bitLength);
@@ -1050,7 +1028,11 @@ WINE_HIDP_PREPARSED_DATA* ParseDescriptor(BYTE *descriptor, unsigned int length)
     {
         TRACE("Descriptor[%i]: ", length);
         for (cidx = 0; cidx < length; cidx++)
+        {
             TRACE("%x ",descriptor[cidx]);
+            if ((cidx+1) % 80 == 0)
+                TRACE("\n");
+        }
         TRACE("\n");
     }
 
