@@ -167,6 +167,9 @@ static ULONG WINAPI WebBrowser_Release(IWebBrowser2 *iface)
     TRACE("(%p) ref=%d\n", This, ref);
 
     if(!ref) {
+        if(This->sink)
+            IAdviseSink_Release(This->sink);
+
         if(This->doc_host.document)
             IUnknown_Release(This->doc_host.document);
 
@@ -1194,26 +1197,31 @@ static ULONG WebBrowser_release(DocHost *iface)
     return IWebBrowser2_Release(&This->IWebBrowser2_iface);
 }
 
-static void WINAPI DocHostContainer_GetDocObjRect(DocHost* This, RECT* rc)
+static void DocHostContainer_get_docobj_rect(DocHost *This, RECT *rc)
 {
     GetClientRect(This->frame_hwnd, rc);
 }
 
-static HRESULT WINAPI DocHostContainer_SetStatusText(DocHost* This, LPCWSTR text)
+static HRESULT DocHostContainer_set_status_text(DocHost *This, const WCHAR *text)
 {
     return E_NOTIMPL;
 }
 
-static void WINAPI DocHostContainer_SetURL(DocHost* This, LPCWSTR url)
+static void DocHostContainer_on_command_state_change(DocHost *This, LONG command, BOOL enable)
+{
+}
+
+static void DocHostContainer_set_url(DocHost *This, const WCHAR *url)
 {
 }
 
 static const IDocHostContainerVtbl DocHostContainerVtbl = {
     WebBrowser_addref,
     WebBrowser_release,
-    DocHostContainer_GetDocObjRect,
-    DocHostContainer_SetStatusText,
-    DocHostContainer_SetURL
+    DocHostContainer_get_docobj_rect,
+    DocHostContainer_set_status_text,
+    DocHostContainer_on_command_state_change,
+    DocHostContainer_set_url
 };
 
 static HRESULT create_webbrowser(int version, IUnknown *outer, REFIID riid, void **ppv)
