@@ -94,9 +94,14 @@ static const WCHAR setYearW[] = {'s','e','t','Y','e','a','r',0};
 static const WCHAR UTCW[] = {'U','T','C',0};
 static const WCHAR parseW[] = {'p','a','r','s','e',0};
 
+static inline DateInstance *date_from_jsdisp(jsdisp_t *jsdisp)
+{
+    return CONTAINING_RECORD(jsdisp, DateInstance, dispex);
+}
+
 static inline DateInstance *date_this(vdisp_t *jsthis)
 {
-    return is_vclass(jsthis, JSCLASS_DATE) ? (DateInstance*)jsthis->u.jsdisp : NULL;
+    return is_vclass(jsthis, JSCLASS_DATE) ? date_from_jsdisp(jsthis->u.jsdisp) : NULL;
 }
 
 /*ECMA-262 3rd Edition    15.9.1.2 */
@@ -1917,20 +1922,11 @@ static HRESULT Date_setYear(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsi
     return S_OK;
 }
 
-static HRESULT Date_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, unsigned argc, jsval_t *argv,
-        jsval_t *r)
+static HRESULT Date_get_value(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t *r)
 {
     TRACE("\n");
 
-    switch(flags) {
-    case INVOKE_FUNC:
-        return throw_type_error(ctx, JS_E_FUNCTION_EXPECTED, NULL);
-    default:
-        FIXME("unimplemented flags %x\n", flags);
-        return E_NOTIMPL;
-    }
-
-    return S_OK;
+    return dateobj_to_string(date_from_jsdisp(jsthis), r);
 }
 
 static const builtin_prop_t Date_props[] = {
@@ -1982,7 +1978,7 @@ static const builtin_prop_t Date_props[] = {
 
 static const builtin_info_t Date_info = {
     JSCLASS_DATE,
-    {NULL, Date_value, 0},
+    {NULL, NULL,0, Date_get_value},
     sizeof(Date_props)/sizeof(*Date_props),
     Date_props,
     NULL,
@@ -1991,7 +1987,7 @@ static const builtin_info_t Date_info = {
 
 static const builtin_info_t DateInst_info = {
     JSCLASS_DATE,
-    {NULL, Date_value, 0},
+    {NULL, NULL,0, Date_get_value},
     0, NULL,
     NULL,
     NULL
@@ -2136,7 +2132,7 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
                 }
             }
             else if(parse[i]=='-' || parse[i]=='/') {
-                /* Short date */
+                /* Short or long date */
                 if(set_day || set_month || set_year) break;
                 set_day = TRUE;
                 set_month = TRUE;
@@ -2156,6 +2152,13 @@ static inline HRESULT date_parse(jsstr_t *input_str, double *ret) {
                 if(parse[i]<'0' || parse[i]>'9') break;
                 year = atoiW(&parse[i]);
                 while(parse[i]>='0' && parse[i]<='9') i++;
+
+                if(tmp >= 70){
+                        /* long date */
+                        month = day - 1;
+                        day = year;
+                        year = tmp;
+		}
             }
             else if(tmp<0) break;
             else if(tmp<70) {
@@ -2506,7 +2509,7 @@ static const builtin_prop_t DateConstr_props[] = {
 
 static const builtin_info_t DateConstr_info = {
     JSCLASS_FUNCTION,
-    {NULL, Function_value, 0},
+    DEFAULT_FUNCTION_VALUE,
     sizeof(DateConstr_props)/sizeof(*DateConstr_props),
     DateConstr_props,
     NULL,
