@@ -420,8 +420,12 @@ static HRESULT prop_get(jsdisp_t *This, dispex_prop_t *prop, DISPPARAMS *dp,
 
     switch(prop->type) {
     case PROP_BUILTIN:
-        if(prop->u.p->flags & PROPF_METHOD) {
+        if(prop->u.p->getter) {
+            hres = prop->u.p->getter(This->ctx, This, r);
+        }else {
             jsdisp_t *obj;
+
+            assert(prop->u.p->invoke != NULL);
             hres = create_builtin_function(This->ctx, prop->u.p->invoke, prop->u.p->name, NULL,
                     prop->u.p->flags, NULL, &obj);
             if(FAILED(hres))
@@ -432,12 +436,6 @@ static HRESULT prop_get(jsdisp_t *This, dispex_prop_t *prop, DISPPARAMS *dp,
 
             jsdisp_addref(obj);
             *r = jsval_obj(obj);
-        }else {
-            vdisp_t vthis;
-
-            set_jsdisp(&vthis, This);
-            hres = prop->u.p->invoke(This->ctx, &vthis, DISPATCH_PROPERTYGET, 0, NULL, r);
-            vdisp_release(&vthis);
         }
         break;
     case PROP_PROTREF:
@@ -472,13 +470,12 @@ static HRESULT prop_put(jsdisp_t *This, dispex_prop_t *prop, jsval_t val, IServi
 
     switch(prop->type) {
     case PROP_BUILTIN:
-        if(!(prop->flags & PROPF_METHOD)) {
-            vdisp_t vthis;
+        if(prop->u.p->setter)
+            return prop->u.p->setter(This->ctx, This, val);
 
-            set_jsdisp(&vthis, This);
-            hres = prop->u.p->invoke(This->ctx, &vthis, DISPATCH_PROPERTYPUT, 1, &val, NULL);
-            vdisp_release(&vthis);
-            return hres;
+        if(prop->u.p->setter) {
+            FIXME("getter with no setter\n");
+            return E_FAIL;
         }
         /* fall through */
     case PROP_PROTREF:
@@ -510,6 +507,12 @@ static HRESULT prop_put(jsdisp_t *This, dispex_prop_t *prop, jsval_t val, IServi
     return S_OK;
 }
 
+HRESULT builtin_set_const(script_ctx_t *ctx, jsdisp_t *jsthis, jsval_t value)
+{
+    TRACE("%p %s\n", jsthis, debugstr_jsval(value));
+    return S_OK;
+}
+
 static HRESULT fill_protrefs(jsdisp_t *This)
 {
     dispex_prop_t *iter, *prop;
@@ -907,7 +910,7 @@ HRESULT init_dispex(jsdisp_t *dispex, script_ctx_t *ctx, const builtin_info_t *b
         jsdisp_addref(prototype);
 
     dispex->prop_cnt = 1;
-    if(builtin_info->value_prop.invoke) {
+    if(builtin_info->value_prop.invoke || builtin_info->value_prop.getter) {
         dispex->props[0].type = PROP_BUILTIN;
         dispex->props[0].u.p = &builtin_info->value_prop;
     }else {
@@ -1057,11 +1060,18 @@ HRESULT jsdisp_call_value(jsdisp_t *jsfunc, IDispatch *jsthis, WORD flags, unsig
 {
     HRESULT hres;
 
+    assert(!(flags & ~(DISPATCH_METHOD|DISPATCH_CONSTRUCT)));
+
     if(is_class(jsfunc, JSCLASS_FUNCTION)) {
         hres = Function_invoke(jsfunc, jsthis, flags, argc, argv, r);
     }else {
         vdisp_t vdisp;
 
+        if(!jsfunc->builtin_info->value_prop.invoke) {
+            WARN("Not a function\n");
+            return throw_type_error(jsfunc->ctx, JS_E_FUNCTION_EXPECTED, NULL);
+        }
+
         set_disp(&vdisp, jsthis);
         hres = jsfunc->builtin_info->value_prop.invoke(jsfunc->ctx, &vdisp, flags, argc, argv, r);
         vdisp_release(&vdisp);
@@ -1190,13 +1200,10 @@ HRESULT disp_call_value(script_ctx_t *ctx, IDispatch *disp, IDispatch *jsthis, W
     unsigned i;
     HRESULT hres;
 
+    assert(!(flags & ~(DISPATCH_METHOD|DISPATCH_CONSTRUCT)));
+
     jsdisp = iface_to_jsdisp((IUnknown*)disp);
     if(jsdisp) {
-        if(flags & DISPATCH_PROPERTYPUT) {
-            FIXME("disp_call(propput) on builtin object\n");
-            return E_FAIL;
-        }
-
         hres = jsdisp_call_value(jsdisp, jsthis, flags, argc, argv, r);
         jsdisp_release(jsdisp);
         return hres;
@@ -1346,6 +1353,7 @@ HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, jsval_t val)
         jsdisp_release(jsdisp);
     }else {
         DISPID dispid = DISPID_PROPERTYPUT;
+        DWORD flags = DISPATCH_PROPERTYPUT;
         VARIANT var;
         DISPPARAMS dp  = {&var, &dispid, 1, 1};
         IDispatchEx *dispex;
@@ -1354,17 +1362,20 @@ HRESULT disp_propput(script_ctx_t *ctx, IDispatch *disp, DISPID id, jsval_t val)
         if(FAILED(hres))
             return hres;
 
+        if(V_VT(&var) == VT_DISPATCH)
+            flags |= DISPATCH_PROPERTYPUTREF;
+
         clear_ei(ctx);
         hres = IDispatch_QueryInterface(disp, &IID_IDispatchEx, (void**)&dispex);
         if(SUCCEEDED(hres)) {
-            hres = IDispatchEx_InvokeEx(dispex, id, ctx->lcid, DISPATCH_PROPERTYPUT, &dp, NULL, &ctx->ei.ei,
+            hres = IDispatchEx_InvokeEx(dispex, id, ctx->lcid, flags, &dp, NULL, &ctx->ei.ei,
                     &ctx->jscaller->IServiceProvider_iface);
             IDispatchEx_Release(dispex);
         }else {
             ULONG err = 0;
 
             TRACE("using IDispatch\n");
-            hres = IDispatch_Invoke(disp, id, &IID_NULL, ctx->lcid, DISPATCH_PROPERTYPUT, &dp, NULL, &ctx->ei.ei, &err);
+            hres = IDispatch_Invoke(disp, id, &IID_NULL, ctx->lcid, flags, &dp, NULL, &ctx->ei.ei, &err);
         }
 
         VariantClear(&var);
