@@ -53,6 +53,11 @@ const char *debugstr_jsval(const jsval_t v)
     return NULL;
 }
 
+BOOL is_finite(double n)
+{
+    return !isnan(n) && !isinf(n);
+}
+
 #define MIN_BLOCK_SIZE  128
 #define ARENA_FREE_FILLER  0xaa
 
@@ -246,6 +251,9 @@ HRESULT jsval_copy(jsval_t v, jsval_t *r)
 
 HRESULT variant_to_jsval(VARIANT *var, jsval_t *r)
 {
+    if(V_VT(var) == (VT_VARIANT|VT_BYREF))
+        var = V_VARIANTREF(var);
+
     switch(V_VT(var)) {
     case VT_EMPTY:
         *r = jsval_undefined();
@@ -288,6 +296,9 @@ HRESULT variant_to_jsval(VARIANT *var, jsval_t *r)
     case VT_INT:
         *r = jsval_number(V_INT(var));
         return S_OK;
+    case VT_UI4:
+        *r = jsval_number(V_UI4(var));
+        return S_OK;
     case VT_UNKNOWN:
         if(V_UNKNOWN(var)) {
             IDispatch *disp;
@@ -635,7 +646,7 @@ HRESULT to_int32(script_ctx_t *ctx, jsval_t v, INT *ret)
     if(FAILED(hres))
         return hres;
 
-    *ret = isnan(n) || isinf(n) ? 0 : n;
+    *ret = is_finite(n) ? n : 0;
     return S_OK;
 }
 
