@@ -39,6 +39,7 @@
 #include "mmsystem.h"
 #include "audioclient.h"
 #include "audiopolicy.h"
+#include "endpointvolume.h"
 
 static const unsigned int win_formats[][4] = {
     { 8000,  8, 1},   { 8000,  8, 2},   { 8000, 16, 1},   { 8000, 16, 2},
@@ -337,7 +338,8 @@ static void test_audioclient(void)
           (UINT)(t2/10000), (UINT)(t2 % 10000));
     ok(t2 >= t1 || broken(t2 >= t1/2 && pwfx->nSamplesPerSec > 48000) ||
             broken(t2 == 0) /* (!) win10 */,
-       "Latency < default period, delta %ldus (%llu vs %llu)\n", (long)((t2-t1)/10), t2, t1);
+       "Latency < default period, delta %dus (%x%08x vs %x%08x)\n",
+       (LONG)((t2-t1)/10), (DWORD)(t2 >> 32), (DWORD)t2, (DWORD)(t1 >> 32), (DWORD)t1);
     /* Native appears to add the engine period to the HW latency in shared mode */
     if(t2 == 0)
         win10 = TRUE;
@@ -2241,6 +2243,44 @@ static void test_marshal(void)
 
 }
 
+static void test_endpointvolume(void)
+{
+    HRESULT hr;
+    IAudioEndpointVolume *aev;
+    float mindb, maxdb, increment, volume;
+    BOOL mute;
+
+    hr = IMMDevice_Activate(dev, &IID_IAudioEndpointVolume,
+            CLSCTX_INPROC_SERVER, NULL, (void**)&aev);
+    ok(hr == S_OK, "Activation failed with %08x\n", hr);
+    if(hr != S_OK)
+        return;
+
+    hr = IAudioEndpointVolume_GetVolumeRange(aev, &mindb, NULL, NULL);
+    ok(hr == E_POINTER, "GetVolumeRange should have failed with E_POINTER: 0x%08x\n", hr);
+
+    hr = IAudioEndpointVolume_GetVolumeRange(aev, &mindb, &maxdb, &increment);
+    ok(hr == S_OK, "GetVolumeRange failed: 0x%08x\n", hr);
+    trace("got range: [%f,%f]/%f\n", mindb, maxdb, increment);
+
+    hr = IAudioEndpointVolume_SetMasterVolumeLevel(aev, mindb - increment, NULL);
+    ok(hr == E_INVALIDARG, "SetMasterVolumeLevel failed: 0x%08x\n", hr);
+
+    hr = IAudioEndpointVolume_GetMasterVolumeLevel(aev, &volume);
+    ok(hr == S_OK, "GetMasterVolumeLevel failed: 0x%08x\n", hr);
+
+    hr = IAudioEndpointVolume_SetMasterVolumeLevel(aev, volume, NULL);
+    ok(hr == S_OK, "SetMasterVolumeLevel failed: 0x%08x\n", hr);
+
+    hr = IAudioEndpointVolume_GetMute(aev, &mute);
+    ok(hr == S_OK, "GetMute failed: %08x\n", hr);
+
+    hr = IAudioEndpointVolume_SetMute(aev, mute, NULL);
+    ok(hr == S_OK || hr == S_FALSE, "SetMute failed: %08x\n", hr);
+
+    IAudioEndpointVolume_Release(aev);
+}
+
 START_TEST(render)
 {
     HRESULT hr;
@@ -2282,6 +2322,7 @@ START_TEST(render)
     test_volume_dependence();
     test_session_creation();
     test_worst_case();
+    test_endpointvolume();
 
     IMMDevice_Release(dev);
 
