@@ -107,14 +107,14 @@ static ULONG WINAPI HTMLFormElement_Release(IHTMLFormElement *iface)
 static HRESULT WINAPI HTMLFormElement_GetTypeInfoCount(IHTMLFormElement *iface, UINT *pctinfo)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->element.node.dispex.IDispatchEx_iface, pctinfo);
+    return IDispatchEx_GetTypeInfoCount(&This->element.node.event_target.dispex.IDispatchEx_iface, pctinfo);
 }
 
 static HRESULT WINAPI HTMLFormElement_GetTypeInfo(IHTMLFormElement *iface, UINT iTInfo,
                                               LCID lcid, ITypeInfo **ppTInfo)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->element.node.dispex.IDispatchEx_iface, iTInfo, lcid,
+    return IDispatchEx_GetTypeInfo(&This->element.node.event_target.dispex.IDispatchEx_iface, iTInfo, lcid,
             ppTInfo);
 }
 
@@ -123,7 +123,7 @@ static HRESULT WINAPI HTMLFormElement_GetIDsOfNames(IHTMLFormElement *iface, REF
                                                 LCID lcid, DISPID *rgDispId)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->element.node.dispex.IDispatchEx_iface, riid, rgszNames,
+    return IDispatchEx_GetIDsOfNames(&This->element.node.event_target.dispex.IDispatchEx_iface, riid, rgszNames,
             cNames, lcid, rgDispId);
 }
 
@@ -132,7 +132,7 @@ static HRESULT WINAPI HTMLFormElement_Invoke(IHTMLFormElement *iface, DISPID dis
                             VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
-    return IDispatchEx_Invoke(&This->element.node.dispex.IDispatchEx_iface, dispIdMember, riid,
+    return IDispatchEx_Invoke(&This->element.node.event_target.dispex.IDispatchEx_iface, dispIdMember, riid,
             lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 }
 
@@ -384,13 +384,15 @@ static HRESULT WINAPI HTMLFormElement_submit(IHTMLFormElement *iface)
 {
     HTMLFormElement *This = impl_from_IHTMLFormElement(iface);
     HTMLOuterWindow *window = NULL, *this_window = NULL;
+    nsAString action_uri_str, target_str, method_str;
     nsIInputStream *post_stream;
-    nsAString action_uri_str, target_str;
+    BOOL is_post_submit = FALSE;
     IUri *uri;
     nsresult nsres;
     HRESULT hres;
+    BOOL use_new_window = FALSE;
 
-    TRACE("(%p)->()\n", This);
+    TRACE("(%p)\n", This);
 
     if(This->element.node.doc) {
         HTMLDocumentNode *doc = This->element.node.doc;
@@ -404,22 +406,33 @@ static HRESULT WINAPI HTMLFormElement_submit(IHTMLFormElement *iface)
 
     nsAString_Init(&target_str, NULL);
     nsres = nsIDOMHTMLFormElement_GetTarget(This->nsform, &target_str);
-    if(NS_SUCCEEDED(nsres)) {
-        BOOL use_new_window;
+    if(NS_SUCCEEDED(nsres))
         window = get_target_window(this_window, &target_str, &use_new_window);
-        if(use_new_window)
-            FIXME("submit to new window is not supported\n");
-    }
-    nsAString_Finish(&target_str);
-    if(!window)
+
+    if(!window && !use_new_window) {
+        nsAString_Finish(&target_str);
         return S_OK;
+    }
+
+    nsAString_Init(&method_str, NULL);
+    nsres = nsIDOMHTMLFormElement_GetMethod(This->nsform, &method_str);
+    if(NS_SUCCEEDED(nsres)) {
+        const PRUnichar *method;
+
+        static const PRUnichar postW[] = {'p','o','s','t',0};
+
+        nsAString_GetData(&method_str, &method);
+        TRACE("method is %s\n", debugstr_w(method));
+        is_post_submit = !strcmpiW(method, postW);
+    }
+    nsAString_Finish(&method_str);
 
     /*
-     * FIXME: We currently don't use our submit implementation for sub-windows because
-     * load_nsuri can't support post data. We should fix it.
+     * FIXME: We currently use our submit implementation for POST submit. We should always use it.
      */
-    if(!window->doc_obj || window->doc_obj->basedoc.window != window) {
+    if(window && !is_post_submit) {
         nsres = nsIDOMHTMLFormElement_Submit(This->nsform);
+        nsAString_Finish(&target_str);
         IHTMLWindow2_Release(&window->base.IHTMLWindow2_iface);
         if(NS_FAILED(nsres)) {
             ERR("Submit failed: %08x\n", nsres);
@@ -442,13 +455,16 @@ static HRESULT WINAPI HTMLFormElement_submit(IHTMLFormElement *iface)
     }
     nsAString_Finish(&action_uri_str);
     if(SUCCEEDED(hres)) {
-        window->readystate_locked++;
-        hres = submit_form(window, uri, post_stream);
-        window->readystate_locked--;
+        const PRUnichar *target;
+
+        nsAString_GetData(&target_str, &target);
+        hres = submit_form(window, target, uri, post_stream);
         IUri_Release(uri);
     }
 
-    IHTMLWindow2_Release(&window->base.IHTMLWindow2_iface);
+    nsAString_Finish(&target_str);
+    if(window)
+        IHTMLWindow2_Release(&window->base.IHTMLWindow2_iface);
     if(post_stream)
         nsIInputStream_Release(post_stream);
     return hres;
