@@ -74,7 +74,7 @@ static ULONG WINAPI HTMLObjectElement_Release(IHTMLObjectElement *iface)
 static HRESULT WINAPI HTMLObjectElement_GetTypeInfoCount(IHTMLObjectElement *iface, UINT *pctinfo)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             pctinfo);
 }
 
@@ -82,7 +82,7 @@ static HRESULT WINAPI HTMLObjectElement_GetTypeInfo(IHTMLObjectElement *iface, U
                                               LCID lcid, ITypeInfo **ppTInfo)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             iTInfo, lcid, ppTInfo);
 }
 
@@ -91,7 +91,7 @@ static HRESULT WINAPI HTMLObjectElement_GetIDsOfNames(IHTMLObjectElement *iface,
                                                 LCID lcid, DISPID *rgDispId)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             riid, rgszNames, cNames, lcid, rgDispId);
 }
 
@@ -100,7 +100,7 @@ static HRESULT WINAPI HTMLObjectElement_Invoke(IHTMLObjectElement *iface, DISPID
                             VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement(iface);
-    return IDispatchEx_Invoke(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_Invoke(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             dispIdMember, riid, lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 }
 
@@ -535,7 +535,7 @@ static ULONG WINAPI HTMLObjectElement2_Release(IHTMLObjectElement2 *iface)
 static HRESULT WINAPI HTMLObjectElement2_GetTypeInfoCount(IHTMLObjectElement2 *iface, UINT *pctinfo)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetTypeInfoCount(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             pctinfo);
 }
 
@@ -543,7 +543,7 @@ static HRESULT WINAPI HTMLObjectElement2_GetTypeInfo(IHTMLObjectElement2 *iface,
                                               LCID lcid, ITypeInfo **ppTInfo)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetTypeInfo(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             iTInfo, lcid, ppTInfo);
 }
 
@@ -551,7 +551,7 @@ static HRESULT WINAPI HTMLObjectElement2_GetIDsOfNames(IHTMLObjectElement2 *ifac
         LPOLESTR *rgszNames, UINT cNames, LCID lcid, DISPID *rgDispId)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_GetIDsOfNames(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             riid, rgszNames, cNames, lcid, rgDispId);
 }
 
@@ -560,7 +560,7 @@ static HRESULT WINAPI HTMLObjectElement2_Invoke(IHTMLObjectElement2 *iface, DISP
         EXCEPINFO *pExcepInfo, UINT *puArgErr)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    return IDispatchEx_Invoke(&This->plugin_container.element.node.dispex.IDispatchEx_iface,
+    return IDispatchEx_Invoke(&This->plugin_container.element.node.event_target.dispex.IDispatchEx_iface,
             dispIdMember, riid, lcid, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr);
 }
 
@@ -575,8 +575,29 @@ static HRESULT WINAPI HTMLObjectElement2_namedRecordset(IHTMLObjectElement2 *ifa
 static HRESULT WINAPI HTMLObjectElement2_put_classid(IHTMLObjectElement2 *iface, BSTR v)
 {
     HTMLObjectElement *This = impl_from_IHTMLObjectElement2(iface);
-    FIXME("(%p)->(%s)\n", This, debugstr_w(v));
-    return E_NOTIMPL;
+    HRESULT hres;
+
+    static const WCHAR classidW[] = {'c','l','a','s','s','i','d',0};
+
+    FIXME("(%p)->(%s) semi-stub\n", This, debugstr_w(v));
+
+    hres = elem_string_attr_setter(&This->plugin_container.element, classidW, v);
+    if(FAILED(hres))
+        return hres;
+
+    if(This->plugin_container.plugin_host) {
+        FIXME("Host already associated.\n");
+        return E_NOTIMPL;
+    }
+
+    /*
+     * NOTE:
+     * If the element is not yet in DOM tree, we should embed it as soon as it's added.
+     * However, Gecko for some reason decides not to create NP plugin in this case,
+     * so this won't work.
+     */
+
+    return create_plugin_host(This->plugin_container.element.node.doc, &This->plugin_container);
 }
 
 static HRESULT WINAPI HTMLObjectElement2_get_classid(IHTMLObjectElement2 *iface, BSTR *p)
@@ -624,21 +645,20 @@ static HRESULT HTMLObjectElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
 {
     HTMLObjectElement *This = impl_from_HTMLDOMNode(iface);
 
+    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+
     if(IsEqualGUID(&IID_IUnknown, riid)) {
-        TRACE("(%p)->(IID_IUnknown %p)\n", This, ppv);
         *ppv = &This->IHTMLObjectElement_iface;
     }else if(IsEqualGUID(&IID_IDispatch, riid)) {
-        TRACE("(%p)->(IID_IDispatch %p)\n", This, ppv);
         *ppv = &This->IHTMLObjectElement_iface;
     }else if(IsEqualGUID(&IID_IHTMLObjectElement, riid)) {
-        TRACE("(%p)->(IID_IHTMLObjectElement %p)\n", This, ppv);
         *ppv = &This->IHTMLObjectElement_iface;
     }else if(IsEqualGUID(&IID_IHTMLObjectElement2, riid)) {
-        TRACE("(%p)->(IID_IHTMLObjectElement2 %p)\n", This, ppv);
         *ppv = &This->IHTMLObjectElement2_iface;
     }else if(IsEqualGUID(&IID_HTMLPluginContainer, riid)) {
-        TRACE("(%p)->(IID_HTMLPluginContainer %p)\n", This, ppv);
+        /* Special pseudo-interface returning HTMLPluginContainse struct. */
         *ppv = &This->plugin_container;
+        node_addref(&This->plugin_container.element.node);
         return S_OK;
     }else {
         HRESULT hres;
@@ -745,9 +765,9 @@ static const NodeImplVtbl HTMLObjectElementImplVtbl = {
 };
 
 static const tid_t HTMLObjectElement_iface_tids[] = {
-    HTMLELEMENT_TIDS,
-    IHTMLObjectElement_tid,
     IHTMLObjectElement2_tid,
+    IHTMLObjectElement_tid,
+    HTMLELEMENT_TIDS,
     0
 };
 static dispex_static_data_t HTMLObjectElement_dispex = {
