@@ -380,6 +380,11 @@ HRESULT get_frame_by_name(HTMLOuterWindow *This, const WCHAR *name, BOOL deep, H
         return E_FAIL;
     }
 
+    if(!nsframes) {
+        WARN("nsIDOMWindow_GetFrames returned NULL nsframes: %p\n", This->nswindow);
+        return DISP_E_MEMBERNOTFOUND;
+    }
+
     nsAString_InitDepend(&name_str, name);
     nsres = nsIDOMWindowCollection_NamedItem(nsframes, &name_str, &nswindow);
     nsAString_Finish(&name_str);
@@ -1344,8 +1349,10 @@ static HRESULT WINAPI HTMLWindow2_scroll(IHTMLWindow2 *iface, LONG x, LONG y)
 static HRESULT WINAPI HTMLWindow2_get_clientInformation(IHTMLWindow2 *iface, IOmNavigator **p)
 {
     HTMLWindow *This = impl_from_IHTMLWindow2(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return IHTMLWindow2_get_navigator(&This->IHTMLWindow2_iface, p);
 }
 
 static HRESULT WINAPI HTMLWindow2_setInterval(IHTMLWindow2 *iface, BSTR expression,
@@ -2623,10 +2630,14 @@ static HRESULT WINAPI WindowDispEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID
 
     TRACE("(%p)->(%x %x %x %p %p %p %p)\n", This, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 
-    if(id == DISPID_IHTMLWINDOW2_LOCATION && (wFlags & DISPATCH_PROPERTYPUT)) {
+    switch(id) {
+    case DISPID_IHTMLWINDOW2_LOCATION: {
         HTMLLocation *location;
         HRESULT hres;
 
+        if(!(wFlags & DISPATCH_PROPERTYPUT))
+            break;
+
         TRACE("forwarding to location.href\n");
 
         hres = get_location(window, &location);
@@ -2638,6 +2649,27 @@ static HRESULT WINAPI WindowDispEx_InvokeEx(IDispatchEx *iface, DISPID id, LCID
         IHTMLLocation_Release(&location->IHTMLLocation_iface);
         return hres;
     }
+    case DISPID_IHTMLWINDOW2_SETTIMEOUT:
+    case DISPID_IHTMLWINDOW3_SETTIMEOUT: {
+        VARIANT args[2];
+        DISPPARAMS dp = {args, NULL, 2, 0};
+
+        /*
+         * setTimeout calls should use default value 0 for the second argument if only one is provided,
+         * but IDL file does not reflect that. We fixup arguments here instead.
+         */
+        if(!(wFlags & DISPATCH_METHOD) || pdp->cArgs != 1 || pdp->cNamedArgs)
+            break;
+
+        TRACE("Fixing args\n");
+
+        V_VT(args) = VT_I4;
+        V_I4(args) = 0;
+        args[1] = *pdp->rgvarg;
+        return IDispatchEx_InvokeEx(&window->event_target.dispex.IDispatchEx_iface, id, lcid,
+                wFlags, &dp, pvarRes, pei, pspCaller);
+    }
+    }
 
     return IDispatchEx_InvokeEx(&window->event_target.dispex.IDispatchEx_iface, id, lcid, wFlags, pdp, pvarRes, pei, pspCaller);
 }
@@ -2907,7 +2939,8 @@ static dispex_static_data_t HTMLWindow_dispex = {
     &HTMLWindow_dispex_vtbl,
     DispHTMLWindow2_tid,
     NULL,
-    HTMLWindow_iface_tids
+    HTMLWindow_iface_tids,
+    IHTMLWindow5_tid
 };
 
 static void *alloc_window(size_t size)
