@@ -38,6 +38,7 @@
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
 #define NSCMD_COPY "cmd_copy"
+#define NSCMD_SELECTALL           "cmd_selectAll"
 
 void do_ns_command(HTMLDocument *This, const char *cmd, nsICommandParams *nsparam)
 {
@@ -62,6 +63,28 @@ void do_ns_command(HTMLDocument *This, const char *cmd, nsICommandParams *nspara
     nsICommandManager_Release(cmdmgr);
 }
 
+static nsIClipboardCommands *get_clipboard_commands(HTMLDocument *doc)
+{
+    nsIClipboardCommands *clipboard_commands;
+    nsIDocShell *doc_shell;
+    nsresult nsres;
+
+    nsres = get_nsinterface((nsISupports*)doc->window->nswindow, &IID_nsIDocShell, (void**)&doc_shell);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDocShell interface\n");
+        return NULL;
+    }
+
+    nsres = nsIDocShell_QueryInterface(doc_shell, &IID_nsIClipboardCommands, (void**)&clipboard_commands);
+    nsIDocShell_Release(doc_shell);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIClipboardCommands interface\n");
+        return NULL;
+    }
+
+    return clipboard_commands;
+}
+
 /**********************************************************
  * IOleCommandTarget implementation
  */
@@ -333,8 +356,10 @@ static HRESULT exec_cut(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, V
 
 static HRESULT exec_copy(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
 {
-    FIXME("(%p)->(%d %p %p)\n", This, nCmdexecopt, pvaIn, pvaOut);
-    return E_NOTIMPL;
+    TRACE("(%p)->(%d %s %p)\n", This, nCmdexecopt, debugstr_variant(pvaIn), pvaOut);
+
+    do_ns_command(This, NSCMD_COPY, NULL);
+    return S_OK;
 }
 
 static HRESULT exec_paste(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
@@ -361,10 +386,18 @@ static HRESULT exec_rendo(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn,
     return E_NOTIMPL;
 }
 
-static HRESULT exec_select_all(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
+static HRESULT exec_select_all(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *in, VARIANT *out)
 {
-    FIXME("(%p)->(%d %p %p)\n", This, nCmdexecopt, pvaIn, pvaOut);
-    return E_NOTIMPL;
+    TRACE("(%p)\n", This);
+
+    if(in || out)
+        FIXME("unsupported args\n");
+
+    if(This->doc_obj->nscontainer)
+        do_ns_command(This, NSCMD_SELECTALL, NULL);
+
+    update_doc(This, UPDATE_UI);
+    return S_OK;
 }
 
 static HRESULT exec_clear_selection(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
@@ -524,6 +557,19 @@ static HRESULT exec_get_print_template(HTMLDocument *This, DWORD nCmdexecopt, VA
     return E_NOTIMPL;
 }
 
+static HRESULT exec_optical_zoom(HTMLDocument *This, DWORD nCmdexecopt, VARIANT *pvaIn, VARIANT *pvaOut)
+{
+    TRACE("(%p)->(%d %s %p)\n", This, nCmdexecopt, debugstr_variant(pvaIn), pvaOut);
+
+    if(!pvaIn || V_VT(pvaIn) != VT_I4) {
+        FIXME("Unsupported argument %s\n", debugstr_variant(pvaIn));
+        return E_NOTIMPL;
+    }
+
+    set_viewer_zoom(This->doc_obj->nscontainer, (float)V_I4(pvaIn)/100);
+    return S_OK;
+}
+
 static HRESULT query_mshtml_copy(HTMLDocument *This, OLECMD *cmd)
 {
     FIXME("(%p)\n", This);
@@ -551,13 +597,26 @@ static HRESULT query_mshtml_cut(HTMLDocument *This, OLECMD *cmd)
 
 static HRESULT exec_mshtml_cut(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
 {
+    nsIClipboardCommands *clipboard_commands;
+    nsresult nsres;
+
     TRACE("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
 
     if(This->doc_obj->usermode == EDITMODE)
         return editor_exec_cut(This, cmdexecopt, in, out);
 
-    FIXME("Unimplemented in browse mode\n");
-    return E_NOTIMPL;
+    clipboard_commands = get_clipboard_commands(This);
+    if(!clipboard_commands)
+        return E_UNEXPECTED;
+
+    nsres = nsIClipboardCommands_CutSelection(clipboard_commands);
+    nsIClipboardCommands_Release(clipboard_commands);
+    if(NS_FAILED(nsres)) {
+        ERR("Paste failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
 }
 
 static HRESULT query_mshtml_paste(HTMLDocument *This, OLECMD *cmd)
@@ -569,13 +628,34 @@ static HRESULT query_mshtml_paste(HTMLDocument *This, OLECMD *cmd)
 
 static HRESULT exec_mshtml_paste(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
 {
+    nsIClipboardCommands *clipboard_commands;
+    nsresult nsres;
+
     TRACE("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
 
     if(This->doc_obj->usermode == EDITMODE)
         return editor_exec_paste(This, cmdexecopt, in, out);
 
-    FIXME("Unimplemented in browse mode\n");
-    return E_NOTIMPL;
+    clipboard_commands = get_clipboard_commands(This);
+    if(!clipboard_commands)
+        return E_UNEXPECTED;
+
+    nsres = nsIClipboardCommands_Paste(clipboard_commands);
+    nsIClipboardCommands_Release(clipboard_commands);
+    if(NS_FAILED(nsres)) {
+        ERR("Paste failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
+}
+
+static HRESULT query_selall_status(HTMLDocument *This, OLECMD *cmd)
+{
+    TRACE("(%p)->(%p)\n", This, cmd);
+
+    cmd->cmdf = OLECMDF_SUPPORTED|OLECMDF_ENABLED;
+    return S_OK;
 }
 
 static HRESULT exec_browsemode(HTMLDocument *This, DWORD cmdexecopt, VARIANT *in, VARIANT *out)
@@ -615,8 +695,13 @@ static HRESULT exec_baselinefont3(HTMLDocument *This, DWORD cmdexecopt, VARIANT
 static HRESULT exec_respectvisibility_indesign(HTMLDocument *This, DWORD cmdexecopt,
         VARIANT *in, VARIANT *out)
 {
-    FIXME("(%p)->(%08x %p %p)\n", This, cmdexecopt, in, out);
-    return E_NOTIMPL;
+    TRACE("(%p)->(%x %s %p)\n", This, cmdexecopt, debugstr_variant(in), out);
+
+    /* This is turned on by default in Gecko. */
+    if(!in || V_VT(in) != VT_BOOL || !V_BOOL(in))
+        FIXME("Unsupported argument %s\n", debugstr_variant(in));
+
+    return S_OK;
 }
 
 static HRESULT query_enabled_stub(HTMLDocument *This, OLECMD *cmd)
@@ -642,7 +727,7 @@ static HRESULT query_enabled_stub(HTMLDocument *This, OLECMD *cmd)
 static const struct {
     OLECMDF cmdf;
     HRESULT (*func)(HTMLDocument*,DWORD,VARIANT*,VARIANT*);
-} exec_table[OLECMDID_GETPRINTTEMPLATE+1] = {
+} exec_table[] = {
     {0},
     { OLECMDF_SUPPORTED,                  exec_open                 }, /* OLECMDID_OPEN */
     { OLECMDF_SUPPORTED,                  exec_new                  }, /* OLECMDID_NEW */
@@ -682,13 +767,16 @@ static const struct {
     { OLECMDF_SUPPORTED,                  exec_close                }, /* OLECMDID_CLOSE */
     {0},{0},{0},
     { OLECMDF_SUPPORTED,                  exec_set_print_template   }, /* OLECMDID_SETPRINTTEMPLATE */
-    { OLECMDF_SUPPORTED,                  exec_get_print_template   }  /* OLECMDID_GETPRINTTEMPLATE */
+    { OLECMDF_SUPPORTED,                  exec_get_print_template   }, /* OLECMDID_GETPRINTTEMPLATE */
+    {0},{0},{0},{0},{0},{0},{0},{0},{0},{0},
+    { 0, /* not reported as supported */  exec_optical_zoom         }  /* OLECMDID_OPTICAL_ZOOM */
 };
 
 static const cmdtable_t base_cmds[] = {
     {IDM_COPY,             query_mshtml_copy,     exec_mshtml_copy},
     {IDM_PASTE,            query_mshtml_paste,    exec_mshtml_paste},
     {IDM_CUT,              query_mshtml_cut,      exec_mshtml_cut},
+    {IDM_SELECTALL,        query_selall_status,   exec_select_all},
     {IDM_BROWSEMODE,       NULL,                  exec_browsemode},
     {IDM_EDITMODE,         NULL,                  exec_editmode},
     {IDM_PRINT,            query_enabled_stub,    exec_print},
@@ -737,18 +825,22 @@ static HRESULT WINAPI OleCommandTarget_QueryStatus(IOleCommandTarget *iface, con
         ULONG cCmds, OLECMD prgCmds[], OLECMDTEXT *pCmdText)
 {
     HTMLDocument *This = impl_from_IOleCommandTarget(iface);
-    HRESULT hres = S_OK, hr;
+    HRESULT hres;
 
     TRACE("(%p)->(%s %d %p %p)\n", This, debugstr_guid(pguidCmdGroup), cCmds, prgCmds, pCmdText);
 
+    if(pCmdText)
+        FIXME("Unsupported pCmdText\n");
+    if(!cCmds)
+        return S_OK;
+
     if(!pguidCmdGroup) {
         ULONG i;
 
         for(i=0; i<cCmds; i++) {
-            if(prgCmds[i].cmdID<OLECMDID_OPEN || prgCmds[i].cmdID>OLECMDID_GETPRINTTEMPLATE) {
+            if(prgCmds[i].cmdID < OLECMDID_OPEN || prgCmds[i].cmdID >= sizeof(exec_table)/sizeof(*exec_table)) {
                 WARN("Unsupported cmdID = %d\n", prgCmds[i].cmdID);
                 prgCmds[i].cmdf = 0;
-                hres = OLECMDERR_E_NOTSUPPORTED;
             }else {
                 if(prgCmds[i].cmdID == OLECMDID_OPEN || prgCmds[i].cmdID == OLECMDID_NEW) {
                     IOleCommandTarget *cmdtrg = NULL;
@@ -756,14 +848,14 @@ static HRESULT WINAPI OleCommandTarget_QueryStatus(IOleCommandTarget *iface, con
 
                     prgCmds[i].cmdf = OLECMDF_SUPPORTED;
                     if(This->doc_obj->client) {
-                        hr = IOleClientSite_QueryInterface(This->doc_obj->client, &IID_IOleCommandTarget,
+                        hres = IOleClientSite_QueryInterface(This->doc_obj->client, &IID_IOleCommandTarget,
                                 (void**)&cmdtrg);
-                        if(SUCCEEDED(hr)) {
+                        if(SUCCEEDED(hres)) {
                             olecmd.cmdID = prgCmds[i].cmdID;
                             olecmd.cmdf = 0;
 
-                            hr = IOleCommandTarget_QueryStatus(cmdtrg, NULL, 1, &olecmd, NULL);
-                            if(SUCCEEDED(hr) && olecmd.cmdf)
+                            hres = IOleCommandTarget_QueryStatus(cmdtrg, NULL, 1, &olecmd, NULL);
+                            if(SUCCEEDED(hres) && olecmd.cmdf)
                                 prgCmds[i].cmdf = olecmd.cmdf;
                         }
                     }else {
@@ -773,33 +865,28 @@ static HRESULT WINAPI OleCommandTarget_QueryStatus(IOleCommandTarget *iface, con
                     prgCmds[i].cmdf = exec_table[prgCmds[i].cmdID].cmdf;
                     TRACE("cmdID = %d  returning %x\n", prgCmds[i].cmdID, prgCmds[i].cmdf);
                 }
-                hres = S_OK;
             }
         }
 
-        if(pCmdText)
-            FIXME("Set pCmdText\n");
-    }else if(IsEqualGUID(&CGID_MSHTML, pguidCmdGroup)) {
+        return (prgCmds[cCmds-1].cmdf & OLECMDF_SUPPORTED) ? S_OK : OLECMDERR_E_NOTSUPPORTED;
+    }
+
+    if(IsEqualGUID(&CGID_MSHTML, pguidCmdGroup)) {
         ULONG i;
 
         for(i=0; i<cCmds; i++) {
-            HRESULT hres = query_from_table(This, base_cmds, prgCmds+i);
+            hres = query_from_table(This, base_cmds, prgCmds+i);
             if(hres == OLECMDERR_E_NOTSUPPORTED)
                 hres = query_from_table(This, editmode_cmds, prgCmds+i);
             if(hres == OLECMDERR_E_NOTSUPPORTED)
                 FIXME("CGID_MSHTML: unsupported cmdID %d\n", prgCmds[i].cmdID);
         }
 
-        hres = prgCmds[i-1].cmdf ? S_OK : OLECMDERR_E_NOTSUPPORTED;
-
-        if(pCmdText)
-            FIXME("Set pCmdText\n");
-    }else {
-        FIXME("Unsupported pguidCmdGroup %s\n", debugstr_guid(pguidCmdGroup));
-        hres = OLECMDERR_E_UNKNOWNGROUP;
+        return (prgCmds[cCmds-1].cmdf & OLECMDF_SUPPORTED) ? S_OK : OLECMDERR_E_NOTSUPPORTED;
     }
 
-    return hres;
+    FIXME("Unsupported pguidCmdGroup %s\n", debugstr_guid(pguidCmdGroup));
+    return OLECMDERR_E_UNKNOWNGROUP;
 }
 
 static HRESULT exec_from_table(HTMLDocument *This, const cmdtable_t *cmdtable, DWORD cmdid,
@@ -822,7 +909,7 @@ static HRESULT WINAPI OleCommandTarget_Exec(IOleCommandTarget *iface, const GUID
     HTMLDocument *This = impl_from_IOleCommandTarget(iface);
 
     if(!pguidCmdGroup) {
-        if(nCmdID<OLECMDID_OPEN || nCmdID>OLECMDID_GETPRINTTEMPLATE || !exec_table[nCmdID].func) {
+        if(nCmdID < OLECMDID_OPEN || nCmdID >= sizeof(exec_table)/sizeof(*exec_table) || !exec_table[nCmdID].func) {
             WARN("Unsupported cmdID = %d\n", nCmdID);
             return OLECMDERR_E_NOTSUPPORTED;
         }
