@@ -110,6 +110,26 @@ static void set_script_prop(ScriptHost *script_host, DWORD property, VARIANT *va
         WARN("SetProperty(%x) failed: %08x\n", property, hres);
 }
 
+static BOOL is_quirks_mode(HTMLDocumentNode *doc)
+{
+    const WCHAR *compat_mode;
+    nsAString nsstr;
+    nsresult nsres;
+    BOOL ret = FALSE;
+
+    static const WCHAR BackCompatW[] = {'B','a','c','k','C','o','m','p','a','t',0};
+
+    nsAString_Init(&nsstr, NULL);
+    nsres = nsIDOMHTMLDocument_GetCompatMode(doc->nsdoc, &nsstr);
+    if(NS_SUCCEEDED(nsres)) {
+        nsAString_GetData(&nsstr, &compat_mode);
+        if(!strcmpW(compat_mode, BackCompatW))
+            ret = TRUE;
+    }
+    nsAString_Finish(&nsstr);
+    return ret;
+}
+
 static BOOL init_script_engine(ScriptHost *script_host)
 {
     IObjectSafety *safety;
@@ -148,7 +168,7 @@ static BOOL init_script_engine(ScriptHost *script_host)
         return FALSE;
 
     V_VT(&var) = VT_I4;
-    V_I4(&var) = 1;
+    V_I4(&var) = is_quirks_mode(script_host->window->doc) ? 1 : 2;
     set_script_prop(script_host, SCRIPTPROP_INVOKEVERSIONING, &var);
 
     V_VT(&var) = VT_BOOL;
@@ -1314,10 +1334,9 @@ IDispatch *get_script_disp(ScriptHost *script_host)
     return disp;
 }
 
-static event_target_t **find_event_target(HTMLDocumentNode *doc, HTMLScriptElement *script_elem, HTMLDOMNode **ret_target_node)
+static EventTarget *find_event_target(HTMLDocumentNode *doc, HTMLScriptElement *script_elem)
 {
-    HTMLDOMNode *target_node = NULL;
-    event_target_t **target = NULL;
+    EventTarget *event_target = NULL;
     const PRUnichar *target_id;
     nsAString target_id_str;
     nsresult nsres;
@@ -1335,24 +1354,24 @@ static event_target_t **find_event_target(HTMLDocumentNode *doc, HTMLScriptEleme
     if(!*target_id) {
         FIXME("Empty for attribute\n");
     }else if(!strcmpW(target_id, documentW)) {
-        target = &doc->node.event_target;
-        target_node = &doc->node;
-        IHTMLDOMNode_AddRef(&target_node->IHTMLDOMNode_iface);
+        event_target = &doc->node.event_target;
+        htmldoc_addref(&doc->basedoc);
     }else if(!strcmpW(target_id, windowW)) {
-        target = &doc->body_event_target;
+        if(doc->window) {
+            event_target = &doc->window->event_target;
+            IDispatchEx_AddRef(&event_target->dispex.IDispatchEx_iface);
+        }
     }else {
         HTMLElement *target_elem;
 
         hres = get_doc_elem_by_id(doc, target_id, &target_elem);
         if(SUCCEEDED(hres) && target_elem) {
-            target_node = &target_elem->node;
-            target = &target_elem->node.event_target;
+            event_target = &target_elem->node.event_target;
         }
     }
-    nsAString_Finish(&target_id_str);
 
-    *ret_target_node = target_node;
-    return target;
+    nsAString_Finish(&target_id_str);
+    return event_target;
 }
 
 static BOOL parse_event_str(WCHAR *event, const WCHAR **args)
@@ -1447,9 +1466,8 @@ void bind_event_scripts(HTMLDocumentNode *doc)
     HTMLPluginContainer *plugin_container;
     nsIDOMHTMLScriptElement *nsscript;
     HTMLScriptElement *script_elem;
-    event_target_t **event_target;
+    EventTarget *event_target;
     nsIDOMNodeList *node_list;
-    HTMLDOMNode *target_node;
     nsIDOMNode *script_node;
     nsAString selector_str;
     IDispatch *event_disp;
@@ -1496,24 +1514,18 @@ void bind_event_scripts(HTMLDocumentNode *doc)
 
         event_disp = parse_event_elem(doc, script_elem, &event);
         if(event_disp) {
-            event_target = find_event_target(doc, script_elem, &target_node);
+            event_target = find_event_target(doc, script_elem);
             if(event_target) {
-                if(target_node)
-                    hres = IHTMLDOMNode_QueryInterface(&target_node->IHTMLDOMNode_iface, &IID_HTMLPluginContainer,
-                            (void**)&plugin_container);
-                else
-                    hres = E_NOINTERFACE;
-
+                hres = IDispatchEx_QueryInterface(&event_target->dispex.IDispatchEx_iface, &IID_HTMLPluginContainer,
+                        (void**)&plugin_container);
                 if(SUCCEEDED(hres))
                     bind_activex_event(doc, plugin_container, event, event_disp);
                 else
-                    bind_node_event(doc, event_target, target_node, event, event_disp);
+                    bind_target_event(doc, event_target, event, event_disp);
 
-                if(target_node) {
-                    IHTMLDOMNode_Release(&target_node->IHTMLDOMNode_iface);
-                    if(plugin_container)
-                        node_release(&plugin_container->element.node);
-                }
+                IDispatchEx_Release(&event_target->dispex.IDispatchEx_iface);
+                if(plugin_container)
+                    node_release(&plugin_container->element.node);
             }
 
             heap_free(event);
