@@ -24,6 +24,54 @@
 #include "wine/test.h"
 #include "winbase.h"
 
+#undef __thiscall
+#ifdef __i386__
+#define __thiscall __stdcall
+#else
+#define __thiscall __cdecl
+#endif
+
+/* Emulate a __thiscall */
+#ifdef __i386__
+
+#include "pshpack1.h"
+struct thiscall_thunk
+{
+    BYTE pop_eax;    /* popl  %eax (ret addr) */
+    BYTE pop_edx;    /* popl  %edx (func) */
+    BYTE pop_ecx;    /* popl  %ecx (this) */
+    BYTE push_eax;   /* pushl %eax */
+    WORD jmp_edx;    /* jmp  *%edx */
+};
+#include "poppack.h"
+
+static void * (WINAPI *call_thiscall_func1)( void *func, void *this );
+static void * (WINAPI *call_thiscall_func2)( void *func, void *this, const void *a );
+
+static void init_thiscall_thunk(void)
+{
+    struct thiscall_thunk *thunk = VirtualAlloc( NULL, sizeof(*thunk),
+            MEM_COMMIT, PAGE_EXECUTE_READWRITE );
+    thunk->pop_eax  = 0x58;   /* popl  %eax */
+    thunk->pop_edx  = 0x5a;   /* popl  %edx */
+    thunk->pop_ecx  = 0x59;   /* popl  %ecx */
+    thunk->push_eax = 0x50;   /* pushl %eax */
+    thunk->jmp_edx  = 0xe2ff; /* jmp  *%edx */
+    call_thiscall_func1 = (void *)thunk;
+    call_thiscall_func2 = (void *)thunk;
+}
+
+#define call_func1(func,_this) call_thiscall_func1(func,_this)
+#define call_func2(func,_this,a) call_thiscall_func2(func,_this,(const void*)(a))
+
+#else
+
+#define init_thiscall_thunk()
+#define call_func1(func,_this) func(_this)
+#define call_func2(func,_this,a) func(_this,a)
+
+#endif /* __i386__ */
+
 static inline float __port_infinity(void)
 {
     static const unsigned __inf_bytes = 0x7f800000;
@@ -126,12 +174,17 @@ static int (__cdecl *p_tr2_sys__Rename_wchar)(WCHAR const*, WCHAR const*);
 static struct space_info* (__cdecl *p_tr2_sys__Statvfs)(struct space_info*, char const*);
 static struct space_info* (__cdecl *p_tr2_sys__Statvfs_wchar)(struct space_info*, WCHAR const*);
 static enum file_type (__cdecl *p_tr2_sys__Stat)(char const*, int *);
+static enum file_type (__cdecl *p_tr2_sys__Stat_wchar)(WCHAR const*, int *);
 static enum file_type (__cdecl *p_tr2_sys__Lstat)(char const*, int *);
+static enum file_type (__cdecl *p_tr2_sys__Lstat_wchar)(WCHAR const*, int *);
 static __int64 (__cdecl *p_tr2_sys__Last_write_time)(char const*);
 static void (__cdecl *p_tr2_sys__Last_write_time_set)(char const*, __int64);
 static void* (__cdecl *p_tr2_sys__Open_dir)(char*, char const*, int *, enum file_type*);
 static char* (__cdecl *p_tr2_sys__Read_dir)(char*, void*, enum file_type*);
 static void (__cdecl *p_tr2_sys__Close_dir)(void*);
+static int (__cdecl *p_tr2_sys__Link)(char const*, char const*);
+static int (__cdecl *p_tr2_sys__Symlink)(char const*, char const*);
+static int (__cdecl *p_tr2_sys__Unlink)(char const*);
 
 /* thrd */
 typedef struct
@@ -150,6 +203,7 @@ static void (__cdecl *p__Thrd_sleep)(const xtime*);
 static _Thrd_t (__cdecl *p__Thrd_current)(void);
 static int (__cdecl *p__Thrd_create)(_Thrd_t*, _Thrd_start_t, void*);
 static int (__cdecl *p__Thrd_join)(_Thrd_t, int*);
+static int (__cdecl *p__Thrd_detach)(_Thrd_t);
 
 #ifdef __i386__
 static ULONGLONG (__cdecl *p_i386_Thrd_current)(void);
@@ -165,7 +219,30 @@ static _Thrd_t __cdecl i386_Thrd_current(void)
 #endif
 
 /* mtx */
-typedef void *_Mtx_t;
+typedef struct cs_queue
+{
+    struct cs_queue *next;
+    BOOL free;
+    int unknown;
+} cs_queue;
+
+typedef struct
+{
+    ULONG_PTR unk_thread_id;
+    cs_queue unk_active;
+    void *unknown[2];
+    cs_queue *head;
+    void *tail;
+} critical_section;
+
+typedef struct
+{
+    DWORD flags;
+    critical_section cs;
+    DWORD thread_id;
+    DWORD count;
+} *_Mtx_t;
+
 static int (__cdecl *p__Mtx_init)(_Mtx_t*, int);
 static void (__cdecl *p__Mtx_destroy)(_Mtx_t*);
 static int (__cdecl *p__Mtx_lock)(_Mtx_t*);
@@ -181,6 +258,23 @@ static int (__cdecl *p__Cnd_timedwait)(_Cnd_t*, _Mtx_t*, const xtime*);
 static int (__cdecl *p__Cnd_broadcast)(_Cnd_t*);
 static int (__cdecl *p__Cnd_signal)(_Cnd_t*);
 
+/* _Pad */
+typedef void (*vtable_ptr)(void);
+
+typedef struct
+{
+    const vtable_ptr *vtable;
+    _Cnd_t cnd;
+    _Mtx_t mtx;
+    MSVCP_bool launched;
+} _Pad;
+
+static _Pad* (__thiscall *p__Pad_ctor)(_Pad*);
+static _Pad* (__thiscall *p__Pad_copy_ctor)(_Pad*, const _Pad*);
+static void (__thiscall *p__Pad_dtor)(_Pad*);
+static _Pad* (__thiscall *p__Pad_op_assign)(_Pad*, const _Pad*);
+static void (__thiscall *p__Pad__Launch)(_Pad*, _Thrd_t*);
+static void (__thiscall *p__Pad__Release)(_Pad*);
 
 static HMODULE msvcp;
 #define SETNOFAIL(x,y) x = (void*)GetProcAddress(msvcp,y)
@@ -253,8 +347,12 @@ static BOOL init(void)
                 "?_Statvfs@sys@tr2@std@@YA?AUspace_info@123@PEB_W@Z");
         SET(p_tr2_sys__Stat,
                 "?_Stat@sys@tr2@std@@YA?AW4file_type@123@PEBDAEAH@Z");
+        SET(p_tr2_sys__Stat_wchar,
+                "?_Stat@sys@tr2@std@@YA?AW4file_type@123@PEB_WAEAH@Z");
         SET(p_tr2_sys__Lstat,
                 "?_Lstat@sys@tr2@std@@YA?AW4file_type@123@PEBDAEAH@Z");
+        SET(p_tr2_sys__Lstat_wchar,
+                "?_Lstat@sys@tr2@std@@YA?AW4file_type@123@PEB_WAEAH@Z");
         SET(p_tr2_sys__Last_write_time,
                 "?_Last_write_time@sys@tr2@std@@YA_JPEBD@Z");
         SET(p_tr2_sys__Last_write_time_set,
@@ -265,8 +363,26 @@ static BOOL init(void)
                 "?_Read_dir@sys@tr2@std@@YAPEADAEAY0BAE@DPEAXAEAW4file_type@123@@Z");
         SET(p_tr2_sys__Close_dir,
                 "?_Close_dir@sys@tr2@std@@YAXPEAX@Z");
+        SET(p_tr2_sys__Link,
+                "?_Link@sys@tr2@std@@YAHPEBD0@Z");
+        SET(p_tr2_sys__Symlink,
+                "?_Symlink@sys@tr2@std@@YAHPEBD0@Z");
+        SET(p_tr2_sys__Unlink,
+                "?_Unlink@sys@tr2@std@@YAHPEBD@Z");
         SET(p__Thrd_current,
                 "_Thrd_current");
+        SET(p__Pad_ctor,
+                "??0_Pad@std@@QEAA@XZ");
+        SET(p__Pad_copy_ctor,
+                "??0_Pad@std@@QEAA@AEBV01@@Z");
+        SET(p__Pad_dtor,
+                "??1_Pad@std@@QEAA@XZ");
+        SET(p__Pad_op_assign,
+                "??4_Pad@std@@QEAAAEAV01@AEBV01@@Z");
+        SET(p__Pad__Launch,
+                "?_Launch@_Pad@std@@QEAAXPEAU_Thrd_imp_t@@@Z");
+        SET(p__Pad__Release,
+                "?_Release@_Pad@std@@QEAAXXZ");
     } else {
         SET(p_tr2_sys__File_size,
                 "?_File_size@sys@tr2@std@@YA_KPBD@Z");
@@ -306,8 +422,12 @@ static BOOL init(void)
                 "?_Statvfs@sys@tr2@std@@YA?AUspace_info@123@PB_W@Z");
         SET(p_tr2_sys__Stat,
                 "?_Stat@sys@tr2@std@@YA?AW4file_type@123@PBDAAH@Z");
+        SET(p_tr2_sys__Stat_wchar,
+                "?_Stat@sys@tr2@std@@YA?AW4file_type@123@PB_WAAH@Z");
         SET(p_tr2_sys__Lstat,
                 "?_Lstat@sys@tr2@std@@YA?AW4file_type@123@PBDAAH@Z");
+        SET(p_tr2_sys__Lstat_wchar,
+                "?_Lstat@sys@tr2@std@@YA?AW4file_type@123@PB_WAAH@Z");
         SET(p_tr2_sys__Last_write_time,
                 "?_Last_write_time@sys@tr2@std@@YA_JPBD@Z");
         SET(p_tr2_sys__Last_write_time_set,
@@ -318,13 +438,43 @@ static BOOL init(void)
                 "?_Read_dir@sys@tr2@std@@YAPADAAY0BAE@DPAXAAW4file_type@123@@Z");
         SET(p_tr2_sys__Close_dir,
                 "?_Close_dir@sys@tr2@std@@YAXPAX@Z");
+        SET(p_tr2_sys__Link,
+                "?_Link@sys@tr2@std@@YAHPBD0@Z");
+        SET(p_tr2_sys__Symlink,
+                "?_Symlink@sys@tr2@std@@YAHPBD0@Z");
+        SET(p_tr2_sys__Unlink,
+                "?_Unlink@sys@tr2@std@@YAHPBD@Z");
 #ifdef __i386__
         SET(p_i386_Thrd_current,
                 "_Thrd_current");
         p__Thrd_current = i386_Thrd_current;
+        SET(p__Pad_ctor,
+                "??0_Pad@std@@QAE@XZ");
+        SET(p__Pad_copy_ctor,
+                "??0_Pad@std@@QAE@ABV01@@Z");
+        SET(p__Pad_dtor,
+                "??1_Pad@std@@QAE@XZ");
+        SET(p__Pad_op_assign,
+                "??4_Pad@std@@QAEAAV01@ABV01@@Z");
+        SET(p__Pad__Launch,
+                "?_Launch@_Pad@std@@QAEXPAU_Thrd_imp_t@@@Z");
+        SET(p__Pad__Release,
+                "?_Release@_Pad@std@@QAEXXZ");
 #else
         SET(p__Thrd_current,
                 "_Thrd_current");
+        SET(p__Pad_ctor,
+                "??0_Pad@std@@QAA@XZ");
+        SET(p__Pad_copy_ctor,
+                "??0_Pad@std@@QAA@ABV01@@Z");
+        SET(p__Pad_dtor,
+                "??1_Pad@std@@QAA@XZ");
+        SET(p__Pad_op_assign,
+                "??4_Pad@std@@QAAAAV01@ABV01@@Z");
+        SET(p__Pad__Launch,
+                "?_Launch@_Pad@std@@QAAXPAU_Thrd_imp_t@@@Z");
+        SET(p__Pad__Release,
+                "?_Release@_Pad@std@@QAAXXZ");
 #endif
     }
     SET(p__Thrd_equal,
@@ -337,6 +487,8 @@ static BOOL init(void)
             "_Thrd_create");
     SET(p__Thrd_join,
             "_Thrd_join");
+    SET(p__Thrd_detach,
+            "_Thrd_detach");
 
     SET(p__Mtx_init,
             "_Mtx_init");
@@ -364,6 +516,8 @@ static BOOL init(void)
     p_setlocale = (void*)GetProcAddress(msvcr, "setlocale");
     p__setmbcp = (void*)GetProcAddress(msvcr, "_setmbcp");
     p_isleadbyte = (void*)GetProcAddress(msvcr, "isleadbyte");
+
+    init_thiscall_thunk();
     return TRUE;
 }
 
@@ -961,14 +1115,9 @@ static void test_tr2_sys__Copy_file(void)
     for(i=0; i<sizeof(tests)/sizeof(tests[0]); i++) {
         errno = 0xdeadbeef;
         ret = p_tr2_sys__Copy_file(tests[i].source, tests[i].dest, tests[i].fail_if_exists);
-        if(tests[i].is_todo)
-            todo_wine ok(ret == tests[i].last_error || ret == tests[i].last_error2,
-                    "test_tr2_sys__Copy_file(): test %d expect: %d, got %d\n",
-                    i+1, tests[i].last_error, ret);
-        else
+        todo_wine_if(tests[i].is_todo)
             ok(ret == tests[i].last_error || ret == tests[i].last_error2,
-                    "test_tr2_sys__Copy_file(): test %d expect: %d, got %d\n",
-                    i+1, tests[i].last_error, ret);
+                    "test_tr2_sys__Copy_file(): test %d expect: %d, got %d\n", i+1, tests[i].last_error, ret);
         ok(errno == 0xdeadbeef, "test_tr2_sys__Copy_file(): test %d errno expect 0xdeadbeef, got %d\n", i+1, errno);
         if(ret == ERROR_SUCCESS)
             ok(p_tr2_sys__File_size(tests[i].source) == p_tr2_sys__File_size(tests[i].dest),
@@ -1120,6 +1269,8 @@ static void test_tr2_sys__Stat(void)
         { "tr2_test_dir\\f1_link" ,   regular_file, ERROR_SUCCESS, TRUE },
         { "tr2_test_dir\\dir_link", directory_file, ERROR_SUCCESS, TRUE },
     };
+    WCHAR testW[] = {'t','r','2','_','t','e','s','t','_','d','i','r',0};
+    WCHAR testW2[] = {'t','r','2','_','t','e','s','t','_','d','i','r','/','f','1',0};
 
     CreateDirectoryA("tr2_test_dir", NULL);
     file = CreateFileA("tr2_test_dir/f1", 0, 0, NULL, CREATE_ALWAYS, 0, NULL);
@@ -1161,10 +1312,7 @@ static void test_tr2_sys__Stat(void)
     for(i=0; i<sizeof(tests)/sizeof(tests[0]); i++) {
         err_code = 0xdeadbeef;
         val = p_tr2_sys__Stat(tests[i].path, &err_code);
-        if(tests[i].is_todo)
-            todo_wine ok(tests[i].ret == val, "tr2_sys__Stat(): test %d expect: %d, got %d\n",
-                    i+1, tests[i].ret, val);
-        else
+        todo_wine_if(tests[i].is_todo)
             ok(tests[i].ret == val, "tr2_sys__Stat(): test %d expect: %d, got %d\n", i+1, tests[i].ret, val);
         ok(tests[i].err_code == err_code, "tr2_sys__Stat(): test %d err_code expect: %d, got %d\n",
                 i+1, tests[i].err_code, err_code);
@@ -1172,16 +1320,21 @@ static void test_tr2_sys__Stat(void)
         /* test tr2_sys__Lstat */
         err_code = 0xdeadbeef;
         val = p_tr2_sys__Lstat(tests[i].path, &err_code);
-        if(tests[i].is_todo)
-            todo_wine ok(tests[i].ret == val, "tr2_sys__Lstat(): test %d expect: %d, got %d\n",
-                    i+1, tests[i].ret, val);
-        else
+        todo_wine_if(tests[i].is_todo)
             ok(tests[i].ret == val, "tr2_sys__Lstat(): test %d expect: %d, got %d\n", i+1, tests[i].ret, val);
-
         ok(tests[i].err_code == err_code, "tr2_sys__Lstat(): test %d err_code expect: %d, got %d\n",
                 i+1, tests[i].err_code, err_code);
     }
 
+    err_code = 0xdeadbeef;
+    val = p_tr2_sys__Stat_wchar(testW, &err_code);
+    ok(directory_file == val, "tr2_sys__Stat_wchar() expect directory_file, got %d\n", val);
+    ok(ERROR_SUCCESS == err_code, "tr2_sys__Stat_wchar(): err_code expect ERROR_SUCCESS, got %d\n", err_code);
+    err_code = 0xdeadbeef;
+    val = p_tr2_sys__Lstat_wchar(testW2, &err_code);
+    ok(regular_file == val, "tr2_sys__Lstat_wchar() expect regular_file, got %d\n", val);
+    ok(ERROR_SUCCESS == err_code, "tr2_sys__Lstat_wchar(): err_code expect ERROR_SUCCESS, got %d\n", err_code);
+
     if(ret) {
         todo_wine ok(DeleteFileA("tr2_test_dir/f1_link"), "expect tr2_test_dir/f1_link to exist\n");
         todo_wine ok(RemoveDirectoryA("tr2_test_dir/dir_link"), "expect tr2_test_dir/dir_link to exist\n");
@@ -1319,11 +1472,216 @@ static void test_tr2_sys__dir_operation(void)
     ok(RemoveDirectoryA("tr2_test_dir"), "expect tr2_test_dir to exist\n");
 }
 
+static void test_tr2_sys__Link(void)
+{
+    int ret, i;
+    HANDLE file, h1, h2;
+    BY_HANDLE_FILE_INFORMATION info1, info2;
+    char temp_path[MAX_PATH], current_path[MAX_PATH];
+    LARGE_INTEGER file_size;
+    struct {
+        char const *existing_path;
+        char const *new_path;
+        MSVCP_bool fail_if_exists;
+        int last_error;
+    } tests[] = {
+        { "f1", "f1_link", TRUE, ERROR_SUCCESS },
+        { "f1", "tr2_test_dir\\f1_link", TRUE, ERROR_SUCCESS },
+        { "tr2_test_dir\\f1_link", "tr2_test_dir\\f1_link_link", TRUE, ERROR_SUCCESS },
+        { "tr2_test_dir", "dir_link", TRUE, ERROR_ACCESS_DENIED },
+        { NULL, "NULL_link", TRUE, ERROR_INVALID_PARAMETER },
+        { "f1", NULL, TRUE, ERROR_INVALID_PARAMETER },
+        { "not_exist", "not_exist_link", TRUE, ERROR_FILE_NOT_FOUND },
+        { "f1", "not_exist_dir\\f1_link", TRUE, ERROR_PATH_NOT_FOUND }
+    };
+
+    memset(current_path, 0, MAX_PATH);
+    GetCurrentDirectoryA(MAX_PATH, current_path);
+    memset(temp_path, 0, MAX_PATH);
+    GetTempPathA(MAX_PATH, temp_path);
+    ok(SetCurrentDirectoryA(temp_path), "SetCurrentDirectoryA to temp_path failed\n");
+
+    ret = p_tr2_sys__Make_dir("tr2_test_dir");
+    ok(ret == 1, "test_tr2_sys__Make_dir(): expect 1 got %d\n", ret);
+    file = CreateFileA("f1", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    file_size.QuadPart = 7;
+    ok(SetFilePointerEx(file, file_size, NULL, FILE_BEGIN), "SetFilePointerEx failed\n");
+    ok(SetEndOfFile(file), "SetEndOfFile failed\n");
+    CloseHandle(file);
+
+    for(i=0; i<sizeof(tests)/sizeof(tests[0]); i++) {
+        errno = 0xdeadbeef;
+        ret = p_tr2_sys__Link(tests[i].existing_path, tests[i].new_path);
+        ok(ret == tests[i].last_error, "tr2_sys__Link(): test %d expect: %d, got %d\n",
+                i+1, tests[i].last_error, ret);
+        ok(errno == 0xdeadbeef, "tr2_sys__Link(): test %d errno expect 0xdeadbeef, got %d\n", i+1, errno);
+        if(ret == ERROR_SUCCESS)
+            ok(p_tr2_sys__File_size(tests[i].existing_path) == p_tr2_sys__File_size(tests[i].new_path),
+                    "tr2_sys__Link(): test %d failed, two files' size are not equal\n", i+1);
+    }
+
+    ok(DeleteFileA("f1"), "expect f1 to exist\n");
+    ok(p_tr2_sys__File_size("f1_link") == p_tr2_sys__File_size("tr2_test_dir/f1_link") &&
+            p_tr2_sys__File_size("tr2_test_dir/f1_link") == p_tr2_sys__File_size("tr2_test_dir/f1_link_link"),
+            "tr2_sys__Link(): expect links' size are equal, got %s\n", debugstr_longlong(p_tr2_sys__File_size("tr2_test_dir/f1_link_link")));
+    ok(p_tr2_sys__File_size("f1_link") == 7, "tr2_sys__Link(): expect f1_link's size equals to 7, got %s\n", debugstr_longlong(p_tr2_sys__File_size("f1_link")));
+
+    file = CreateFileA("f1_link", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    file_size.QuadPart = 20;
+    ok(SetFilePointerEx(file, file_size, NULL, FILE_BEGIN), "SetFilePointerEx failed\n");
+    ok(SetEndOfFile(file), "SetEndOfFile failed\n");
+    CloseHandle(file);
+    h1 = CreateFileA("f1_link", 0, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL, OPEN_EXISTING, 0, 0);
+    ok(h1 != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    ok(GetFileInformationByHandle(h1, &info1), "GetFileInformationByHandle failed\n");
+    CloseHandle(h1);
+    h2 = CreateFileA("tr2_test_dir/f1_link", 0, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE,
+            NULL, OPEN_EXISTING, 0, 0);
+    ok(h2 != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    ok(GetFileInformationByHandle(h2, &info2), "GetFileInformationByHandle failed\n");
+    CloseHandle(h2);
+    ok(info1.nFileIndexHigh == info2.nFileIndexHigh
+            && info1.nFileIndexLow == info2.nFileIndexLow,
+            "tr2_sys__Link(): test %d expect two files equivalent\n", i+1);
+    ok(p_tr2_sys__File_size("f1_link") == 20, "tr2_sys__Link(): expect f1_link's size equals to 20, got %s\n", debugstr_longlong(p_tr2_sys__File_size("f1_link")));
+
+    ok(DeleteFileA("f1_link"), "expect f1_link to exist\n");
+    ok(DeleteFileA("tr2_test_dir/f1_link"), "expect tr2_test_dir/f1_link to exist\n");
+    ok(DeleteFileA("tr2_test_dir/f1_link_link"), "expect tr2_test_dir/f1_link_link to exist\n");
+    ret = p_tr2_sys__Remove_dir("tr2_test_dir");
+    ok(ret == 1, "tr2_sys__Remove_dir(): expect 1 got %d\n", ret);
+    ok(SetCurrentDirectoryA(current_path), "SetCurrentDirectoryA failed\n");
+}
+
+static void test_tr2_sys__Symlink(void)
+{
+    int ret, i;
+    HANDLE file;
+    LARGE_INTEGER file_size;
+    struct {
+        char const *existing_path;
+        char const *new_path;
+        int last_error;
+        MSVCP_bool is_todo;
+    } tests[] = {
+        { "f1", "f1_link", ERROR_SUCCESS, FALSE },
+        { "f1", "tr2_test_dir\\f1_link", ERROR_SUCCESS, FALSE },
+        { "tr2_test_dir\\f1_link", "tr2_test_dir\\f1_link_link", ERROR_SUCCESS, FALSE },
+        { "tr2_test_dir", "dir_link", ERROR_SUCCESS, FALSE },
+        { NULL, "NULL_link", ERROR_INVALID_PARAMETER, FALSE },
+        { "f1", NULL, ERROR_INVALID_PARAMETER, FALSE },
+        { "not_exist",  "not_exist_link", ERROR_SUCCESS, FALSE },
+        { "f1", "not_exist_dir\\f1_link", ERROR_PATH_NOT_FOUND, TRUE }
+    };
+
+    ret = p_tr2_sys__Make_dir("tr2_test_dir");
+    ok(ret == 1, "test_tr2_sys__Make_dir(): expect 1 got %d\n", ret);
+    file = CreateFileA("f1", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    file_size.QuadPart = 7;
+    ok(SetFilePointerEx(file, file_size, NULL, FILE_BEGIN), "SetFilePointerEx failed\n");
+    ok(SetEndOfFile(file), "SetEndOfFile failed\n");
+    CloseHandle(file);
+
+    for(i=0; i<sizeof(tests)/sizeof(tests[0]); i++) {
+        errno = 0xdeadbeef;
+        SetLastError(0xdeadbeef);
+        ret = p_tr2_sys__Symlink(tests[i].existing_path, tests[i].new_path);
+        if(!i && (ret==ERROR_PRIVILEGE_NOT_HELD || ret==ERROR_INVALID_FUNCTION)) {
+            win_skip("Privilege not held or symbolic link not supported, skipping symbolic link tests.\n");
+            ok(DeleteFileA("f1"), "expect f1 to exist\n");
+            ret = p_tr2_sys__Remove_dir("tr2_test_dir");
+            ok(ret == 1, "tr2_sys__Remove_dir(): expect 1 got %d\n", ret);
+            return;
+        }
+
+        ok(errno == 0xdeadbeef, "tr2_sys__Symlink(): test %d errno expect 0xdeadbeef, got %d\n", i+1, errno);
+        todo_wine_if(tests[i].is_todo)
+            ok(ret == tests[i].last_error, "tr2_sys__Symlink(): test %d expect: %d, got %d\n", i+1, tests[i].last_error, ret);
+        if(ret == ERROR_SUCCESS)
+            ok(p_tr2_sys__File_size(tests[i].new_path) == 0, "tr2_sys__Symlink(): expect 0, got %s\n", debugstr_longlong(p_tr2_sys__File_size(tests[i].new_path)));
+    }
+
+    ok(DeleteFileA("f1"), "expect f1 to exist\n");
+    todo_wine ok(DeleteFileA("f1_link"), "expect f1_link to exist\n");
+    todo_wine ok(DeleteFileA("tr2_test_dir/f1_link"), "expect tr2_test_dir/f1_link to exist\n");
+    todo_wine ok(DeleteFileA("tr2_test_dir/f1_link_link"), "expect tr2_test_dir/f1_link_link to exist\n");
+    todo_wine ok(DeleteFileA("not_exist_link"), "expect not_exist_link to exist\n");
+    todo_wine ok(DeleteFileA("dir_link"), "expect dir_link to exist\n");
+    ret = p_tr2_sys__Remove_dir("tr2_test_dir");
+    ok(ret == 1, "tr2_sys__Remove_dir(): expect 1 got %d\n", ret);
+}
+
+static void test_tr2_sys__Unlink(void)
+{
+    char temp_path[MAX_PATH], current_path[MAX_PATH];
+    int ret, i;
+    HANDLE file;
+    LARGE_INTEGER file_size;
+    struct {
+        char const *path;
+        int last_error;
+        MSVCP_bool is_todo;
+    } tests[] = {
+        { "tr2_test_dir\\f1_symlink", ERROR_SUCCESS, TRUE },
+        { "tr2_test_dir\\f1_link", ERROR_SUCCESS, FALSE },
+        { "tr2_test_dir\\f1", ERROR_SUCCESS, FALSE },
+        { "tr2_test_dir", ERROR_ACCESS_DENIED, FALSE },
+        { "not_exist", ERROR_FILE_NOT_FOUND, FALSE },
+        { "not_exist_dir\\not_exist_file", ERROR_PATH_NOT_FOUND, FALSE },
+        { NULL, ERROR_PATH_NOT_FOUND, FALSE }
+    };
+
+    GetCurrentDirectoryA(MAX_PATH, current_path);
+    GetTempPathA(MAX_PATH, temp_path);
+    ok(SetCurrentDirectoryA(temp_path), "SetCurrentDirectoryA to temp_path failed\n");
+
+    ret = p_tr2_sys__Make_dir("tr2_test_dir");
+    ok(ret == 1, "tr2_sys__Make_dir(): expect 1 got %d\n", ret);
+    file = CreateFileA("tr2_test_dir/f1", GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, NULL);
+    ok(file != INVALID_HANDLE_VALUE, "create file failed: INVALID_HANDLE_VALUE\n");
+    file_size.QuadPart = 7;
+    ok(SetFilePointerEx(file, file_size, NULL, FILE_BEGIN), "SetFilePointerEx failed\n");
+    ok(SetEndOfFile(file), "SetEndOfFile failed\n");
+    CloseHandle(file);
+
+    ret = p_tr2_sys__Symlink("tr2_test_dir/f1", "tr2_test_dir/f1_symlink");
+    if(ret==ERROR_PRIVILEGE_NOT_HELD || ret==ERROR_INVALID_FUNCTION) {
+        tests[0].last_error = ERROR_FILE_NOT_FOUND;
+        win_skip("Privilege not held or symbolic link not supported, skipping symbolic link tests.\n");
+    }else {
+        ok(ret == ERROR_SUCCESS, "tr2_sys__Symlink(): expect: ERROR_SUCCESS, got %d\n", ret);
+    }
+    ret = p_tr2_sys__Link("tr2_test_dir/f1", "tr2_test_dir/f1_link");
+    ok(ret == ERROR_SUCCESS, "tr2_sys__Link(): expect: ERROR_SUCCESS, got %d\n", ret);
+
+    for(i=0; i<sizeof(tests)/sizeof(tests[0]); i++) {
+        errno = 0xdeadbeef;
+        ret = p_tr2_sys__Unlink(tests[i].path);
+        todo_wine_if(tests[i].is_todo)
+            ok(ret == tests[i].last_error, "tr2_sys__Unlink(): test %d expect: %d, got %d\n",
+                    i+1, tests[i].last_error, ret);
+        ok(errno == 0xdeadbeef, "tr2_sys__Unlink(): test %d errno expect: 0xdeadbeef, got %d\n", i+1, ret);
+    }
+
+    ok(!DeleteFileA("tr2_test_dir/f1"), "expect tr2_test_dir/f1 not to exist\n");
+    ok(!DeleteFileA("tr2_test_dir/f1_link"), "expect tr2_test_dir/f1_link not to exist\n");
+    ok(!DeleteFileA("tr2_test_dir/f1_symlink"), "expect tr2_test_dir/f1_symlink not to exist\n");
+    ret = p_tr2_sys__Remove_dir("tr2_test_dir");
+    ok(ret == 1, "tr2_sys__Remove_dir(): expect 1 got %d\n", ret);
+
+    ok(SetCurrentDirectoryA(current_path), "SetCurrentDirectoryA failed\n");
+}
+
 static int __cdecl thrd_thread(void *arg)
 {
     _Thrd_t *thr = arg;
 
-    *thr = p__Thrd_current();
+    if(thr)
+        *thr = p__Thrd_current();
     return 0x42;
 }
 
@@ -1405,7 +1763,14 @@ static void test_thrd(void)
     ok(ta.id == tb.id, "expected %d, got %d\n", ta.id, tb.id);
     ok(ta.hnd != tb.hnd, "same handles, got %p\n", ta.hnd);
     ok(r == 0x42, "expected 0x42, got %d\n", r);
-    ok(!CloseHandle(ta.hnd), "handle %p not closed\n", ta.hnd);
+    ret = p__Thrd_detach(ta);
+    ok(ret == 4, "_Thrd_detach should have failed with error 4, got %d\n", ret);
+
+    ret = p__Thrd_create(&ta, thrd_thread, NULL);
+    ok(!ret, "failed to create thread, got %d\n", ret);
+    ret = p__Thrd_detach(ta);
+    ok(!ret, "_Thrd_detach failed, got %d\n", ret);
+
 }
 
 #define NUM_THREADS 10
@@ -1549,6 +1914,117 @@ static void test_cnd(void)
     CloseHandle(cm.initialized);
 }
 
+static struct {
+    int value[2];
+    const char* export_name;
+} vbtable_size_exports_list[] = {
+    {{0x20, 0x20}, "??_8?$basic_iostream@DU?$char_traits@D@std@@@std@@7B?$basic_istream@DU?$char_traits@D@std@@@1@@"},
+    {{0x10, 0x10}, "??_8?$basic_iostream@DU?$char_traits@D@std@@@std@@7B?$basic_ostream@DU?$char_traits@D@std@@@1@@"},
+    {{0x20, 0x20}, "??_8?$basic_iostream@GU?$char_traits@G@std@@@std@@7B?$basic_istream@GU?$char_traits@G@std@@@1@@"},
+    {{0x10, 0x10}, "??_8?$basic_iostream@GU?$char_traits@G@std@@@std@@7B?$basic_ostream@GU?$char_traits@G@std@@@1@@"},
+    {{0x20, 0x20}, "??_8?$basic_iostream@_WU?$char_traits@_W@std@@@std@@7B?$basic_istream@_WU?$char_traits@_W@std@@@1@@"},
+    {{0x10, 0x10}, "??_8?$basic_iostream@_WU?$char_traits@_W@std@@@std@@7B?$basic_ostream@_WU?$char_traits@_W@std@@@1@@"},
+    {{0x18, 0x18}, "??_8?$basic_istream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{0x18, 0x18}, "??_8?$basic_istream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{0x18, 0x18}, "??_8?$basic_istream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{ 0x8, 0x10}, "??_8?$basic_ostream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{ 0x8, 0x10}, "??_8?$basic_ostream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{ 0x8, 0x10}, "??_8?$basic_ostream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{ 0x0,  0x0}, 0}
+};
+
+static void test_vbtable_size_exports(void)
+{
+    int i;
+    const int *p_vbtable;
+    int arch_idx = (sizeof(void*) == 8);
+
+    for (i = 0; vbtable_size_exports_list[i].export_name; i++)
+    {
+        SET(p_vbtable, vbtable_size_exports_list[i].export_name);
+
+        ok(p_vbtable[0] == 0, "vbtable[0] wrong, got 0x%x\n", p_vbtable[0]);
+        ok(p_vbtable[1] == vbtable_size_exports_list[i].value[arch_idx],
+                "%d: %s[1] wrong, got 0x%x\n", i, vbtable_size_exports_list[i].export_name, p_vbtable[1]);
+    }
+}
+
+HANDLE _Pad__Launch_returned;
+_Pad pad;
+#ifdef __i386__
+/* TODO: this should be a __thiscall function */
+static unsigned int __stdcall vtbl_func__Go(void)
+#else
+static unsigned int __cdecl vtbl_func__Go(_Pad *this)
+#endif
+{
+    DWORD ret;
+
+    ret = WaitForSingleObject(_Pad__Launch_returned, 100);
+    ok(ret == WAIT_TIMEOUT, "WiatForSingleObject returned %x\n", ret);
+    ok(!pad.mtx->count, "pad.mtx.count = %d\n", pad.mtx->count);
+    ok(!pad.launched, "pad.launched = %x\n", pad.launched);
+    call_func1(p__Pad__Release, &pad);
+    ok(pad.launched, "pad.launched = %x\n", pad.launched);
+    ret = WaitForSingleObject(_Pad__Launch_returned, 100);
+    ok(ret == WAIT_OBJECT_0, "WiatForSingleObject returned %x\n", ret);
+    ok(pad.mtx->count == 1, "pad.mtx.count = %d\n", pad.mtx->count);
+    return 0;
+}
+
+static void test__Pad(void)
+{
+    _Pad pad_copy;
+    _Thrd_t thrd;
+    vtable_ptr pfunc = (vtable_ptr)&vtbl_func__Go;
+
+    _Pad__Launch_returned = CreateEventW(NULL, FALSE, FALSE, NULL);
+
+    pad.vtable = (void*)1;
+    pad.cnd = (void*)2;
+    pad.mtx = (void*)3;
+    pad.launched = TRUE;
+    memset(&pad_copy, 0, sizeof(pad_copy));
+    call_func2(p__Pad_copy_ctor, &pad_copy, &pad);
+    ok(pad_copy.vtable != (void*)1, "pad_copy.vtable was not set\n");
+    ok(pad_copy.cnd == (void*)2, "pad_copy.cnd = %p\n", pad_copy.cnd);
+    ok(pad_copy.mtx == (void*)3, "pad_copy.mtx = %p\n", pad_copy.mtx);
+    ok(pad_copy.launched, "pad_copy.launched = %x\n", pad_copy.launched);
+
+    memset(&pad_copy, 0xde, sizeof(pad_copy));
+    pad_copy.vtable = (void*)4;
+    pad_copy.cnd = (void*)5;
+    pad_copy.mtx = (void*)6;
+    pad_copy.launched = FALSE;
+    call_func2(p__Pad_op_assign, &pad_copy, &pad);
+    ok(pad_copy.vtable == (void*)4, "pad_copy.vtable was set\n");
+    ok(pad_copy.cnd == (void*)2, "pad_copy.cnd = %p\n", pad_copy.cnd);
+    ok(pad_copy.mtx == (void*)3, "pad_copy.mtx = %p\n", pad_copy.mtx);
+    ok(pad_copy.launched, "pad_copy.launched = %x\n", pad_copy.launched);
+
+    call_func1(p__Pad_ctor, &pad);
+    call_func2(p__Pad_copy_ctor, &pad_copy, &pad);
+    ok(pad.vtable == pad_copy.vtable, "pad.vtable = %p, pad_copy.vtable = %p\n", pad.vtable, pad_copy.vtable);
+    ok(pad.cnd == pad_copy.cnd, "pad.cnd = %p, pad_copy.cnd = %p\n", pad.cnd, pad_copy.cnd);
+    ok(pad.mtx == pad_copy.mtx, "pad.mtx = %p, pad_copy.mtx = %p\n", pad.mtx, pad_copy.mtx);
+    ok(pad.launched == pad_copy.launched, "pad.launched = %x, pad_copy.launched = %x\n", pad.launched, pad_copy.launched);
+    call_func1(p__Pad_dtor, &pad);
+    /* call_func1(p__Pad_dtor, &pad_copy);  - copy constructor is broken, this causes a crash */
+
+    memset(&pad, 0xfe, sizeof(pad));
+    call_func1(p__Pad_ctor, &pad);
+    ok(!pad.launched, "pad.launched = %x\n", pad.launched);
+    ok(pad.mtx->count == 1, "pad.mtx.count = %d\n", pad.mtx->count);
+
+    pad.vtable = &pfunc;
+    call_func2(p__Pad__Launch, &pad, &thrd);
+    SetEvent(_Pad__Launch_returned);
+    ok(!p__Thrd_join(thrd, NULL), "_Thrd_join failed\n");
+
+    call_func1(p__Pad_dtor, &pad);
+    CloseHandle(_Pad__Launch_returned);
+}
+
 START_TEST(msvcp120)
 {
     if(!init()) return;
@@ -1573,9 +2049,15 @@ START_TEST(msvcp120)
     test_tr2_sys__Stat();
     test_tr2_sys__Last_write_time();
     test_tr2_sys__dir_operation();
+    test_tr2_sys__Link();
+    test_tr2_sys__Symlink();
+    test_tr2_sys__Unlink();
 
     test_thrd();
     test_cnd();
+    test__Pad();
+
+    test_vbtable_size_exports();
 
     FreeLibrary(msvcp);
 }
