@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <locale.h>
 #include <wctype.h>
+#include <math.h>
 #include <float.h>
 #include <errno.h>
 
@@ -26,6 +27,29 @@
 #include <winbase.h>
 #include "wine/test.h"
 
+static inline float __port_infinity(void)
+{
+        static const unsigned __inf_bytes = 0x7f800000;
+            return *(const float *)&__inf_bytes;
+}
+#define INFINITY __port_infinity()
+
+static inline float __port_nan(void)
+{
+        static const unsigned __nan_bytes = 0x7fc00000;
+            return *(const float *)&__nan_bytes;
+}
+#define NAN __port_nan()
+
+static inline float __port_ind(void)
+{
+        static const unsigned __ind_bytes = 0xffc00000;
+            return *(const float *)&__ind_bytes;
+}
+#define IND __port_ind()
+
+typedef double LDOUBLE;  /* long double is just a double */
+
 typedef struct {
     LCID handle;
     unsigned page;
@@ -110,6 +134,10 @@ static void (__thiscall *p_basic_ostringstream_char_dtor)(/*basic_ostringstream_
 static void (__thiscall *p_basic_ostringstream_char_vbase_dtor)(/*basic_ostringstream_char*/void*);
 static void (__thiscall *p_basic_ios_char_dtor)(/*basic_ios_char*/void*);
 
+static BOOL (__cdecl *p_std_Ctraits_float__Isnan)(float);
+static BOOL (__cdecl *p_std_Ctraits_double__Isnan)(double);
+static BOOL (__cdecl *p_std_Ctraits_long_double__Isnan)(LDOUBLE);
+
 static complex_float* (__thiscall *p_complex_float_ctor)(complex_float*, const float*, const float*);
 static complex_float* (__cdecl *p_complex_float_add)(complex_float*, const complex_float*, const complex_float*);
 static complex_float* (__cdecl *p_complex_float_div)(complex_float*, const complex_float*, const complex_float*);
@@ -216,6 +244,11 @@ static BOOL init(void)
     SET(p_wctrans, "wctrans");
     SET(p_towctrans, "towctrans");
     SET(basic_ostringstream_char_vbtable, "??_8?$basic_ostringstream@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@7B@");
+
+    SET(p_std_Ctraits_float__Isnan, "?_Isnan@?$_Ctraits@M@std@@SA_NM@Z");
+    SET(p_std_Ctraits_double__Isnan, "?_Isnan@?$_Ctraits@N@std@@SA_NN@Z");
+    SET(p_std_Ctraits_long_double__Isnan, "?_Isnan@?$_Ctraits@O@std@@SA_NO@Z");
+
     if(sizeof(void*) == 8) { /* 64-bit initialization */
         SET(p_char_assign, "?assign@?$char_traits@D@std@@SAXAEADAEBD@Z");
         SET(p_wchar_assign, "?assign@?$char_traits@_W@std@@SAXAEA_WAEB_W@Z");
@@ -688,6 +721,44 @@ static BOOL almost_eq(float f1, float f2)
     return f1 < 0.0001;
 }
 
+static void test_Ctraits_math_functions(void)
+{
+    BYTE ret;
+
+    ret = p_std_Ctraits_float__Isnan(0.0);
+    ok(ret == FALSE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_float__Isnan(INFINITY);
+    ok(ret == FALSE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_float__Isnan(-INFINITY);
+    ok(ret == FALSE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_float__Isnan(IND);
+    ok(ret == TRUE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_float__Isnan(NAN);
+    ok(ret == TRUE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_double__Isnan(INFINITY);
+    ok(ret == FALSE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_double__Isnan(IND);
+    ok(ret == TRUE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_double__Isnan(NAN);
+    ok(ret == TRUE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_long_double__Isnan(INFINITY);
+    ok(ret == FALSE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_long_double__Isnan(IND);
+    ok(ret == TRUE, "ret = %d\n", ret);
+
+    ret = p_std_Ctraits_long_double__Isnan(NAN);
+    ok(ret == TRUE, "ret = %d\n", ret);
+}
+
 static void test_complex(void)
 {
     complex_float c1, c2, c3;
@@ -920,6 +991,65 @@ static void test_complex(void)
     ok(almost_eq(c2.imag, 0.00035447), "c2.imag = %g\n", c2.imag);
 }
 
+static struct {
+    int value[2];
+    const char* export_name;
+} vbtable_size_exports_list[] = {
+    {{0x5c, 0xa8}, "??_8?$basic_fstream@DU?$char_traits@D@std@@@std@@7B?$basic_istream@DU?$char_traits@D@std@@@1@@"},
+    {{0x54, 0x98}, "??_8?$basic_fstream@DU?$char_traits@D@std@@@std@@7B?$basic_ostream@DU?$char_traits@D@std@@@1@@"},
+    {{0x5c, 0xa8}, "??_8?$basic_fstream@GU?$char_traits@G@std@@@std@@7B?$basic_istream@GU?$char_traits@G@std@@@1@@"},
+    {{0x54, 0x98}, "??_8?$basic_fstream@GU?$char_traits@G@std@@@std@@7B?$basic_ostream@GU?$char_traits@G@std@@@1@@"},
+    {{0x5c, 0xa8}, "??_8?$basic_fstream@_WU?$char_traits@_W@std@@@std@@7B?$basic_istream@_WU?$char_traits@_W@std@@@1@@"},
+    {{0x54, 0x98}, "??_8?$basic_fstream@_WU?$char_traits@_W@std@@@std@@7B?$basic_ostream@_WU?$char_traits@_W@std@@@1@@"},
+    {{0x58, 0xa0}, "??_8?$basic_ifstream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{0x58, 0xa0}, "??_8?$basic_ifstream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{0x58, 0xa0}, "??_8?$basic_ifstream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{ 0xc, 0x18}, "??_8?$basic_iostream@DU?$char_traits@D@std@@@std@@7B?$basic_istream@DU?$char_traits@D@std@@@1@@"},
+    {{ 0x4,  0x8}, "??_8?$basic_iostream@DU?$char_traits@D@std@@@std@@7B?$basic_ostream@DU?$char_traits@D@std@@@1@@"},
+    {{ 0xc, 0x18}, "??_8?$basic_iostream@GU?$char_traits@G@std@@@std@@7B?$basic_istream@GU?$char_traits@G@std@@@1@@"},
+    {{ 0x4,  0x8}, "??_8?$basic_iostream@GU?$char_traits@G@std@@@std@@7B?$basic_ostream@GU?$char_traits@G@std@@@1@@"},
+    {{ 0xc, 0x18}, "??_8?$basic_iostream@_WU?$char_traits@_W@std@@@std@@7B?$basic_istream@_WU?$char_traits@_W@std@@@1@@"},
+    {{ 0x4,  0x8}, "??_8?$basic_iostream@_WU?$char_traits@_W@std@@@std@@7B?$basic_ostream@_WU?$char_traits@_W@std@@@1@@"},
+    {{ 0x8, 0x10}, "??_8?$basic_istream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{ 0x8, 0x10}, "??_8?$basic_istream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{ 0x8, 0x10}, "??_8?$basic_istream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{0x50, 0x90}, "??_8?$basic_istringstream@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@7B@"},
+    {{0x50, 0x90}, "??_8?$basic_istringstream@GU?$char_traits@G@std@@V?$allocator@G@2@@std@@7B@"},
+    {{0x50, 0x90}, "??_8?$basic_istringstream@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@7B@"},
+    {{0x54, 0x98}, "??_8?$basic_ofstream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{0x54, 0x98}, "??_8?$basic_ofstream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{0x54, 0x98}, "??_8?$basic_ofstream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{ 0x4,  0x8}, "??_8?$basic_ostream@DU?$char_traits@D@std@@@std@@7B@"},
+    {{ 0x4,  0x8}, "??_8?$basic_ostream@GU?$char_traits@G@std@@@std@@7B@"},
+    {{ 0x4,  0x8}, "??_8?$basic_ostream@_WU?$char_traits@_W@std@@@std@@7B@"},
+    {{0x4c, 0x88}, "??_8?$basic_ostringstream@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@7B@"},
+    {{0x4c, 0x88}, "??_8?$basic_ostringstream@GU?$char_traits@G@std@@V?$allocator@G@2@@std@@7B@"},
+    {{0x4c, 0x88}, "??_8?$basic_ostringstream@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@7B@"},
+    {{0x54, 0x98}, "??_8?$basic_stringstream@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@7B?$basic_istream@DU?$char_traits@D@std@@@1@@"},
+    {{0x4c, 0x88}, "??_8?$basic_stringstream@DU?$char_traits@D@std@@V?$allocator@D@2@@std@@7B?$basic_ostream@DU?$char_traits@D@std@@@1@@"},
+    {{0x54, 0x98}, "??_8?$basic_stringstream@GU?$char_traits@G@std@@V?$allocator@G@2@@std@@7B?$basic_istream@GU?$char_traits@G@std@@@1@@"},
+    {{0x4c, 0x88}, "??_8?$basic_stringstream@GU?$char_traits@G@std@@V?$allocator@G@2@@std@@7B?$basic_ostream@GU?$char_traits@G@std@@@1@@"},
+    {{0x54, 0x98}, "??_8?$basic_stringstream@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@7B?$basic_istream@_WU?$char_traits@_W@std@@@1@@"},
+    {{0x4c, 0x88}, "??_8?$basic_stringstream@_WU?$char_traits@_W@std@@V?$allocator@_W@2@@std@@7B?$basic_ostream@_WU?$char_traits@_W@std@@@1@@"},
+    {{ 0x0,  0x0}, 0}
+};
+
+static void test_vbtable_size_exports(void)
+{
+    int i;
+    const int *p_vbtable;
+    int arch_idx = (sizeof(void*) == 8);
+
+    for (i = 0; vbtable_size_exports_list[i].export_name; i++)
+    {
+        SET(p_vbtable, vbtable_size_exports_list[i].export_name);
+
+        ok(p_vbtable[0] == 0, "vbtable[0] wrong, got 0x%x\n", p_vbtable[0]);
+        ok(p_vbtable[1] == vbtable_size_exports_list[i].value[arch_idx],
+                "%d: %s[1] wrong, got 0x%x\n", i, vbtable_size_exports_list[i].export_name, p_vbtable[1]);
+    }
+}
+
 START_TEST(misc)
 {
     if(!init())
@@ -935,7 +1065,9 @@ START_TEST(misc)
     test_virtual_call();
     test_virtual_base_dtors();
     test_allocator_char();
+    test_Ctraits_math_functions();
     test_complex();
+    test_vbtable_size_exports();
 
     ok(!invalid_parameter, "invalid_parameter_handler was invoked too many times\n");
 
