@@ -21,13 +21,30 @@
 #include <stdlib.h>
 #include <wchar.h>
 #include <stdio.h>
+#include <float.h>
+#include <limits.h>
 
 #include <windef.h>
 #include <winbase.h>
+#include <winnls.h>
 #include "wine/test.h"
 
 #include <locale.h>
 
+static inline float __port_infinity(void)
+{
+    static const unsigned __inf_bytes = 0x7f800000;
+    return *(const float *)&__inf_bytes;
+}
+#define INFINITY __port_infinity()
+
+static inline float __port_nan(void)
+{
+    static const unsigned __nan_bytes = 0x7fc00000;
+    return *(const float *)&__nan_bytes;
+}
+#define NAN __port_nan()
+
 struct MSVCRT_lconv
 {
     char* decimal_point;
@@ -63,8 +80,14 @@ static struct MSVCRT_lconv* (CDECL *p_localeconv)(void);
 static size_t (CDECL *p_wcstombs_s)(size_t *ret, char* dest, size_t sz, const wchar_t* src, size_t max);
 static int (CDECL *p__dsign)(double);
 static int (CDECL *p__fdsign)(float);
+static int (__cdecl *p__dpcomp)(double x, double y);
 static wchar_t** (CDECL *p____lc_locale_name_func)(void);
 static unsigned int (CDECL *p__GetConcurrency)(void);
+static void* (CDECL *p__W_Gettnames)(void);
+static void (CDECL *p_free)(void*);
+static float (CDECL *p_strtof)(const char *, char **);
+static int (CDECL *p__finite)(double);
+static float (CDECL *p_wcstof)(const wchar_t*, wchar_t**);
 
 static BOOL init(void)
 {
@@ -82,8 +105,14 @@ static BOOL init(void)
     p_wcstombs_s = (void*)GetProcAddress(module, "wcstombs_s");
     p__dsign = (void*)GetProcAddress(module, "_dsign");
     p__fdsign = (void*)GetProcAddress(module, "_fdsign");
+    p__dpcomp = (void*)GetProcAddress(module, "_dpcomp");
     p____lc_locale_name_func = (void*)GetProcAddress(module, "___lc_locale_name_func");
     p__GetConcurrency = (void*)GetProcAddress(module,"?_GetConcurrency@details@Concurrency@@YAIXZ");
+    p__W_Gettnames = (void*)GetProcAddress(module, "_W_Gettnames");
+    p_free = (void*)GetProcAddress(module, "free");
+    p_strtof = (void*)GetProcAddress(module, "strtof");
+    p__finite = (void*)GetProcAddress(module, "_finite");
+    p_wcstof = (void*)GetProcAddress(module, "wcstof");
     return TRUE;
 }
 
@@ -175,6 +204,26 @@ static void test__dsign(void)
     ok(ret == 0x8000, "p_fdsign(-1) = %x\n", ret);
 }
 
+static void test__dpcomp(void)
+{
+    struct {
+        double x, y;
+        int ret;
+    } tests[] = {
+        {0, 0, 2}, {1, 1, 2}, {-1, -1, 2},
+        {-2, -1, 1}, {-1, 1, 1}, {1, 2, 1},
+        {1, -1, 4}, {2, 1, 4}, {-1, -2, 4},
+        {NAN, NAN, 0}, {NAN, 1, 0}, {1, NAN, 0},
+        {INFINITY, INFINITY, 2}, {-1, INFINITY, 1}, {1, INFINITY, 1},
+    };
+    int i, ret;
+
+    for(i=0; i<sizeof(tests)/sizeof(*tests); i++) {
+        ret = p__dpcomp(tests[i].x, tests[i].y);
+        ok(ret == tests[i].ret, "%d) dpcomp(%f, %f) = %x\n", i, tests[i].x, tests[i].y, ret);
+    }
+}
+
 static void test____lc_locale_name_func(void)
 {
     struct {
@@ -204,13 +253,9 @@ static void test____lc_locale_name_func(void)
 
         lc_names = p____lc_locale_name_func();
         ok(lc_names[0] == NULL, "%d - lc_names[0] = %s\n", i, wine_dbgstr_w(lc_names[0]));
-        if(tests[i].todo) {
-            todo_wine ok(!lstrcmpW(lc_names[1], tests[i].name) || broken(!lstrcmpW(lc_names[1], tests[i].broken_name)),
-                    "%d - lc_names[1] = %s\n", i, wine_dbgstr_w(lc_names[1]));
-        } else {
+        todo_wine_if(tests[i].todo)
             ok(!lstrcmpW(lc_names[1], tests[i].name) || broken(!lstrcmpW(lc_names[1], tests[i].broken_name)),
                     "%d - lc_names[1] = %s\n", i, wine_dbgstr_w(lc_names[1]));
-        }
 
         for(j=LC_MIN+2; j<=LC_MAX; j++) {
             ok(!lstrcmpW(lc_names[1], lc_names[j]), "%d - lc_names[%d] = %s, expected %s\n",
@@ -233,11 +278,126 @@ static void test__GetConcurrency(void)
     ok(c == si.dwNumberOfProcessors, "expected %u, got %u\n", si.dwNumberOfProcessors, c);
 }
 
+static void test__W_Gettnames(void)
+{
+    static const char *str[] = {
+        "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat",
+        "Sunday", "Monday", "Tuesday", "Wednesday",
+        "Thursday", "Friday", "Saturday",
+        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
+        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
+        "January", "February", "March", "April", "May", "June", "July",
+        "August", "September", "October", "November", "December",
+        "AM", "PM", "M/d/yyyy"
+    };
+
+    struct {
+        char *str[43];
+        int  unk[2];
+        wchar_t *wstr[43];
+        wchar_t *locname;
+        char data[1];
+    } *ret;
+    int i, size;
+    WCHAR buf[64];
+
+    if(!p_setlocale(LC_ALL, "english"))
+        return;
+
+    ret = p__W_Gettnames();
+    size = ret->str[0]-(char*)ret;
+    if(sizeof(void*) == 8)
+        ok(size==0x2c0, "structure size: %x\n", size);
+    else
+        ok(size==0x164, "structure size: %x\n", size);
+
+    for(i=0; i<sizeof(str)/sizeof(*str); i++) {
+        ok(!strcmp(ret->str[i], str[i]), "ret->str[%d] = %s, expected %s\n",
+                i, ret->str[i], str[i]);
+
+        MultiByteToWideChar(CP_ACP, 0, str[i], strlen(str[i])+1,
+                buf, sizeof(buf)/sizeof(*buf));
+        ok(!lstrcmpW(ret->wstr[i], buf), "ret->wstr[%d] = %s, expected %s\n",
+                i, wine_dbgstr_w(ret->wstr[i]), wine_dbgstr_w(buf));
+    }
+    p_free(ret);
+
+    p_setlocale(LC_ALL, "C");
+}
+
+static void test__strtof(void)
+{
+    const char float1[] = "12.0";
+    const char float2[] = "3.402823466e+38";          /* FLT_MAX */
+    const char float3[] = "-3.402823466e+38";
+    const char float4[] = "1.7976931348623158e+308";  /* DBL_MAX */
+
+    const WCHAR twelve[] = {'1','2','.','0',0};
+    const WCHAR arabic23[] = { 0x662, 0x663, 0};
+
+    char *end;
+    float f;
+
+    f = p_strtof(float1, &end);
+    ok(f == 12.0, "f = %lf\n", f);
+    ok(end == float1+4, "incorrect end (%d)\n", (int)(end-float1));
+
+    f = p_strtof(float2, &end);
+    ok(f == FLT_MAX, "f = %lf\n", f);
+    ok(end == float2+15, "incorrect end (%d)\n", (int)(end-float2));
+
+    f = p_strtof(float3, &end);
+    ok(f == -FLT_MAX, "f = %lf\n", f);
+    ok(end == float3+16, "incorrect end (%d)\n", (int)(end-float3));
+
+    f = p_strtof(float4, &end);
+    ok(!p__finite(f), "f = %lf\n", f);
+    ok(end == float4+23, "incorrect end (%d)\n", (int)(end-float4));
+
+    f = p_strtof("inf", NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    f = p_strtof("INF", NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    f = p_strtof("1.#inf", NULL);
+    ok(f == 1, "f = %lf\n", f);
+
+    f = p_strtof("INFINITY", NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    f = p_strtof("0x12", NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    f = p_wcstof(twelve, NULL);
+    ok(f == 12.0, "f = %lf\n", f);
+
+    f = p_wcstof(arabic23, NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    if(!p_setlocale(LC_ALL, "Arabic")) {
+        win_skip("Arabic locale not available\n");
+        return;
+    }
+
+    f = p_wcstof(twelve, NULL);
+    ok(f == 12.0, "f = %lf\n", f);
+
+    f = p_wcstof(arabic23, NULL);
+    ok(f == 0, "f = %lf\n", f);
+
+    p_setlocale(LC_ALL, "C");
+}
+
 START_TEST(msvcr120)
 {
     if (!init()) return;
+    test__strtof();
     test_lconv();
     test__dsign();
+    test__dpcomp();
     test____lc_locale_name_func();
     test__GetConcurrency();
+    test__W_Gettnames();
+    test__strtof();
 }
