@@ -574,6 +574,16 @@ int __thiscall MSVCRT_type_info_before(type_info * _this, const type_info * rhs)
     return ret;
 }
 
+/******************************************************************
+ *		__std_type_info_compare (MSVCRT.@)
+ */
+int CDECL MSVCRT_type_info_compare(type_info * _this, const type_info * rhs)
+{
+    int ret = strcmp(_this->mangled + 1, rhs->mangled + 1);
+    TRACE("(%p %p) returning %d\n", _this, rhs, ret);
+    return ret;
+}
+
 /******************************************************************
  *		??1type_info@@UAE@XZ (MSVCRT.@)
  */
@@ -904,7 +914,7 @@ const type_info* CDECL MSVCRT___RTtypeid(void *cppobj)
     {
         __non_rtti_object e;
         MSVCRT___non_rtti_object_ctor( &e, "Bad read pointer - no RTTI data!" );
-        _CxxThrowException( &e, &bad_typeid_exception_type );
+        _CxxThrowException( &e, &__non_rtti_object_exception_type );
         return NULL;
     }
     __ENDTRY
@@ -941,7 +951,7 @@ const type_info* CDECL MSVCRT___RTtypeid(void *cppobj)
     {
         __non_rtti_object e;
         MSVCRT___non_rtti_object_ctor( &e, "Bad read pointer - no RTTI data!" );
-        _CxxThrowException( &e, &bad_typeid_exception_type );
+        _CxxThrowException( &e, &__non_rtti_object_exception_type );
         return NULL;
     }
     __ENDTRY
@@ -1028,7 +1038,7 @@ void* CDECL MSVCRT___RTDynamicCast(void *cppobj, int unknown,
     {
         __non_rtti_object e;
         MSVCRT___non_rtti_object_ctor( &e, "Access violation - no RTTI data!" );
-        _CxxThrowException( &e, &bad_typeid_exception_type );
+        _CxxThrowException( &e, &__non_rtti_object_exception_type );
         return NULL;
     }
     __ENDTRY
@@ -1091,7 +1101,7 @@ void* CDECL MSVCRT___RTDynamicCast(void *cppobj, int unknown,
     {
         __non_rtti_object e;
         MSVCRT___non_rtti_object_ctor( &e, "Access violation - no RTTI data!" );
-        _CxxThrowException( &e, &bad_typeid_exception_type );
+        _CxxThrowException( &e, &__non_rtti_object_exception_type );
         return NULL;
     }
     __ENDTRY
@@ -1131,7 +1141,7 @@ void* CDECL MSVCRT___RTCastToVoid(void *cppobj)
     {
         __non_rtti_object e;
         MSVCRT___non_rtti_object_ctor( &e, "Access violation - no RTTI data!" );
-        _CxxThrowException( &e, &bad_typeid_exception_type );
+        _CxxThrowException( &e, &__non_rtti_object_exception_type );
         return NULL;
     }
     __ENDTRY
@@ -1142,6 +1152,7 @@ void* CDECL MSVCRT___RTCastToVoid(void *cppobj)
 /*********************************************************************
  *		_CxxThrowException (MSVCRT.@)
  */
+#ifndef __x86_64__
 void WINAPI _CxxThrowException( exception *object, const cxx_exception_type *type )
 {
     ULONG_PTR args[3];
@@ -1151,6 +1162,18 @@ void WINAPI _CxxThrowException( exception *object, const cxx_exception_type *typ
     args[2] = (ULONG_PTR)type;
     RaiseException( CXX_EXCEPTION, EH_NONCONTINUABLE, 3, args );
 }
+#else
+void WINAPI _CxxThrowException( exception *object, const cxx_exception_type *type )
+{
+    ULONG_PTR args[4];
+
+    args[0] = CXX_FRAME_MAGIC_VC6;
+    args[1] = (ULONG_PTR)object;
+    args[2] = (ULONG_PTR)type;
+    RtlPcToFileHeader( (void*)type, (void**)&args[3]);
+    RaiseException( CXX_EXCEPTION, EH_NONCONTINUABLE, 4, args );
+}
+#endif
 
 /*********************************************************************
  * ?_is_exception_typeof@@YAHABVtype_info@@PAU_EXCEPTION_POINTERS@@@Z
