@@ -48,6 +48,8 @@
 
 #include "msxml_private.h"
 
+#ifdef HAVE_LIBXML2
+
 WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 
 /* We use a chained hashtable, which can hold any number of schemas
@@ -58,8 +60,6 @@ WINE_DEFAULT_DEBUG_CHANNEL(msxml);
 /* This is just the number of buckets, should be prime */
 #define DEFAULT_HASHTABLE_SIZE 17
 
-#ifdef HAVE_LIBXML2
-
 xmlDocPtr XDR_to_XSD_doc(xmlDocPtr xdr_doc, xmlChar const* nsURI);
 
 static const xmlChar XSD_schema[] = "schema";
@@ -1105,6 +1105,18 @@ static HRESULT WINAPI schema_cache_QueryInterface(IXMLDOMSchemaCollection2* ifac
     {
         *ppvObject = iface;
     }
+    else if(This->version == MSXML6 && IsEqualIID(riid, &CLSID_XMLSchemaCache60))
+    {
+        /*
+         * Version 6 can be queried for an interface with IID equal to CLSID.
+         * There is no public interface with that IID and returned pointer
+         * is equal to returned IXMLDOMSchemaCollection2 iface. We assume
+         * that it's just another way for querying IXMLDOMSchemaCollection2
+         * interface. Office 2013 ClickToRun installer uses this.
+         */
+        WARN("riid CLSID_XMLSchemaCache60, returning IXMLDOMSchemaCollection2 interface.\n");
+        *ppvObject = iface;
+    }
     else if (dispex_query_interface(&This->dispex, riid, ppvObject))
     {
         return *ppvObject ? S_OK : E_NOINTERFACE;
@@ -1226,12 +1238,13 @@ static HRESULT WINAPI schema_cache_add(IXMLDOMSchemaCollection2* iface, BSTR uri
             break;
 
         case VT_DISPATCH:
+        case VT_UNKNOWN:
             {
                 xmlDocPtr doc = NULL;
                 cache_entry* entry;
                 CacheEntryType type;
                 IXMLDOMNode* domnode = NULL;
-                IDispatch_QueryInterface(V_DISPATCH(&var), &IID_IXMLDOMNode, (void**)&domnode);
+                IUnknown_QueryInterface(V_UNKNOWN(&var), &IID_IXMLDOMNode, (void**)&domnode);
 
                 if (domnode)
                     doc = xmlNodePtr_from_domnode(domnode, XML_DOCUMENT_NODE)->doc;
@@ -1275,10 +1288,9 @@ static HRESULT WINAPI schema_cache_add(IXMLDOMSchemaCollection2* iface, BSTR uri
             break;
 
         default:
-            {
-                heap_free(name);
-                return E_INVALIDARG;
-            }
+            FIXME("arg type is not supported, %s\n", debugstr_variant(&var));
+            heap_free(name);
+            return E_INVALIDARG;
     }
     heap_free(name);
     return S_OK;
