@@ -33,10 +33,9 @@
 #define RUNNING_ON_VALGRIND 0
 #endif
 
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
+#define NONAMELESSUNION
 #include "windef.h"
 #include "winnt.h"
 #include "winternl.h"
@@ -267,7 +266,8 @@ static inline void notify_free( void const *ptr )
 static inline void notify_realloc( void const *ptr, SIZE_T size_old, SIZE_T size_new )
 {
 #ifdef VALGRIND_RESIZEINPLACE_BLOCK
-    VALGRIND_RESIZEINPLACE_BLOCK( ptr, size_old, size_new, 0 );
+    /* zero is not a valid size */
+    VALGRIND_RESIZEINPLACE_BLOCK( ptr, size_old ? size_old : 1, size_new ? size_new : 1, 0 );
 #endif
 }
 
@@ -2252,3 +2252,12 @@ NTSTATUS WINAPI RtlQueryHeapInformation( HANDLE heap, HEAP_INFORMATION_CLASS inf
         return STATUS_INVALID_INFO_CLASS;
     }
 }
+
+/***********************************************************************
+ *           RtlSetHeapInformation    (NTDLL.@)
+ */
+NTSTATUS WINAPI RtlSetHeapInformation( HANDLE heap, HEAP_INFORMATION_CLASS info_class, PVOID info, SIZE_T size)
+{
+    FIXME("%p %d %p %ld stub\n", heap, info_class, info, size);
+    return STATUS_SUCCESS;
+}
