@@ -614,10 +614,11 @@ unsigned int server_select( const select_op_t *select_op, data_size_t size, UINT
         if (ret != STATUS_USER_APC) break;
         if (invoke_apc( &call, &result ))
         {
-            /* if we ran a user apc we have to check once more if an object got signaled,
+            /* if we ran a user apc we have to check once more if additional apcs are queued,
              * but we don't want to wait */
             abs_timeout = 0;
             user_apc = TRUE;
+            size = 0;
         }
 
         /* don't signal multiple times */
@@ -676,7 +677,6 @@ unsigned int server_queue_process_apc( HANDLE process, const apc_call_t *call, a
             SERVER_END_REQ;
 
             if (!ret && result->type == APC_NONE) continue;  /* APC didn't run, try again */
-            if (ret) NtClose( handle );
         }
         return ret;
     }
