@@ -60,6 +60,7 @@ static struct testfile_s {
     int nfound;               /* How many were found (expect 1) */
     WCHAR nameW[20];          /* unicode version of name (filled in later) */
 } testfiles[] = {
+    { 0, 0, FILE_ATTRIBUTE_NORMAL,    "longfilename.tmp", NULL, "normal" },
     { 0, 0, FILE_ATTRIBUTE_NORMAL,    "n.tmp", NULL, "normal" },
     { 1, 0, FILE_ATTRIBUTE_HIDDEN,    "h.tmp", NULL, "hidden" },
     { 1, 0, FILE_ATTRIBUTE_SYSTEM,    "s.tmp", NULL, "system" },
@@ -147,12 +148,8 @@ static void tally_test_file(FILE_BOTH_DIRECTORY_INFORMATION *dir_info)
         if (namelen != len || memcmp(nameW, testfiles[i].nameW, len*sizeof(WCHAR)))
             continue;
         if (!testfiles[i].attr_done) {
-            if (testfiles[i].todo) {
-                todo_wine
+            todo_wine_if (testfiles[i].todo)
                 ok (attrib == (testfiles[i].attr & attribmask), "file %s: expected %s (%x), got %x (is your linux new enough?)\n", testfiles[i].name, testfiles[i].description, testfiles[i].attr, attrib);
-            } else {
-                ok (attrib == (testfiles[i].attr & attribmask), "file %s: expected %s (%x), got %x (is your linux new enough?)\n", testfiles[i].name, testfiles[i].description, testfiles[i].attr, attrib);
-            }
             testfiles[i].attr_done = TRUE;
         }
         testfiles[i].nfound++;
@@ -234,10 +231,17 @@ static void test_flags_NtQueryDirectoryFile(OBJECT_ATTRIBUTES *attr, const char
 static void test_NtQueryDirectoryFile(void)
 {
     OBJECT_ATTRIBUTES attr;
-    UNICODE_STRING ntdirname;
+    UNICODE_STRING ntdirname, mask;
     char testdirA[MAX_PATH];
     WCHAR testdirW[MAX_PATH];
     int i;
+    IO_STATUS_BLOCK io;
+    WCHAR short_name[12];
+    UINT data_size;
+    BYTE data[8192];
+    FILE_BOTH_DIRECTORY_INFORMATION *fbdi = (FILE_BOTH_DIRECTORY_INFORMATION*)data;
+    DWORD status;
+    HANDLE dirh;
 
     /* Clean up from prior aborted run, if any, then set up test files */
     ok(GetTempPathA(MAX_PATH, testdirA), "couldn't get temp dir\n");
@@ -260,8 +264,6 @@ static void test_NtQueryDirectoryFile(void)
 
     for (i = 0; testfiles[i].name; i++)
     {
-        UNICODE_STRING mask;
-
         if (testfiles[i].nameW[0] == '.') continue;  /* . and .. as masks are broken on Windows */
         mask.Buffer = testfiles[i].nameW;
         mask.Length = mask.MaximumLength = lstrlenW(testfiles[i].nameW) * sizeof(WCHAR);
@@ -271,6 +273,35 @@ static void test_NtQueryDirectoryFile(void)
         test_flags_NtQueryDirectoryFile(&attr, testdirA, &mask, TRUE, FALSE);
     }
 
+    /* short path passed as mask */
+    status = pNtOpenFile(&dirh, SYNCHRONIZE | FILE_LIST_DIRECTORY, &attr, &io, FILE_SHARE_READ,
+            FILE_SYNCHRONOUS_IO_NONALERT | FILE_OPEN_FOR_BACKUP_INTENT | FILE_DIRECTORY_FILE);
+    ok(status == STATUS_SUCCESS, "failed to open dir '%s'\n", testdirA);
+    if (status != STATUS_SUCCESS) {
+        skip("can't test if we can't open the directory\n");
+        return;
+    }
+    mask.Buffer = testfiles[0].nameW;
+    mask.Length = mask.MaximumLength = lstrlenW(testfiles[0].nameW) * sizeof(WCHAR);
+    data_size = offsetof(FILE_BOTH_DIRECTORY_INFORMATION, FileName[256]);
+    pNtQueryDirectoryFile(dirh, 0, NULL, NULL, &io, data, data_size,
+            FileBothDirectoryInformation, TRUE, &mask, FALSE);
+    ok(U(io).Status == STATUS_SUCCESS, "failed to query directory; status %x\n", U(io).Status);
+    ok(fbdi->ShortName[0], "ShortName is empty\n");
+
+    mask.Length = mask.MaximumLength = fbdi->ShortNameLength;
+    memcpy(short_name, fbdi->ShortName, mask.Length);
+    mask.Buffer = short_name;
+    pNtQueryDirectoryFile(dirh, 0, NULL, NULL, &io, data, data_size,
+            FileBothDirectoryInformation, TRUE, &mask, TRUE);
+    ok(U(io).Status == STATUS_SUCCESS, "failed to query directory status %x\n", U(io).Status);
+    ok(fbdi->FileNameLength == strlen(testfiles[0].name)*sizeof(WCHAR) &&
+            !memcmp(fbdi->FileName, testfiles[0].nameW, fbdi->FileNameLength),
+            "incorrect long file name: %s\n", wine_dbgstr_wn(fbdi->FileName,
+                fbdi->FileNameLength/sizeof(WCHAR)));
+
+    pNtClose(dirh);
+
 done:
     tear_down_attribute_test(testdirA);
     pRtlFreeUnicodeString(&ntdirname);
@@ -350,7 +381,7 @@ static void test_NtQueryDirectoryFile_case(void)
     }
 
     mask.Buffer = testmask;
-    mask.Length = mask.MaximumLength = lstrlenW(testmask) * sizeof(WCHAR);
+    mask.Length = mask.MaximumLength = sizeof(testmask);
     pNtQueryDirectoryFile(dirh, NULL, NULL, NULL, &io, data, data_size,
                           FileBothDirectoryInformation, TRUE, &mask, FALSE);
     ok(U(io).Status == STATUS_SUCCESS, "failed to query directory; status %x\n", U(io).Status);
