@@ -280,6 +280,12 @@ HANDLE thread_init(void)
 #endif
 #endif
 
+    /*
+     * Starting with Vista, the first user to log on has session id 1.
+     * Session id 0 is for processes that don't interact with the user (like services).
+     */
+    peb->SessionId = 1;
+
     /* allocate and initialize the initial TEB */
 
     signal_alloc_thread( &teb );
@@ -457,7 +463,7 @@ NTSTATUS WINAPI RtlCreateUserThread( HANDLE process, const SECURITY_DESCRIPTOR *
     pthread_t pthread_id;
     pthread_attr_t attr;
     struct ntdll_thread_data *thread_data;
-    struct startup_info *info = NULL;
+    struct startup_info *info;
     HANDLE handle = 0, actctx = 0;
     TEB *teb = NULL;
     DWORD tid = 0;
@@ -726,7 +732,7 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
 {
     NTSTATUS ret;
     DWORD dummy, i;
-    BOOL self = FALSE;
+    BOOL self;
 
 #ifdef __i386__
     /* on i386 debug registers always require a server call */
@@ -740,6 +746,8 @@ NTSTATUS WINAPI NtSetContextThread( HANDLE handle, const CONTEXT *context )
                 ntdll_get_thread_data()->dr6 == context->Dr6 &&
                 ntdll_get_thread_data()->dr7 == context->Dr7);
     }
+#else
+    self = FALSE;
 #endif
 
     if (!self)
@@ -824,9 +832,11 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
     DWORD needed_flags = context->ContextFlags;
     BOOL self = (handle == GetCurrentThread());
 
+    /* on i386/amd64 debug registers always require a server call */
 #ifdef __i386__
-    /* on i386 debug registers always require a server call */
     if (context->ContextFlags & (CONTEXT_DEBUG_REGISTERS & ~CONTEXT_i386)) self = FALSE;
+#elif defined(__x86_64__)
+    if (context->ContextFlags & (CONTEXT_DEBUG_REGISTERS & ~CONTEXT_AMD64)) self = FALSE;
 #endif
 
     if (!self)
