@@ -92,7 +92,7 @@ BindCtxImpl_QueryInterface(IBindCtx* iface,REFIID riid,void** ppvObject)
     if (IsEqualIID(&IID_IUnknown, riid) ||
         IsEqualIID(&IID_IBindCtx, riid))
     {
-        *ppvObject = This;
+        *ppvObject = &This->IBindCtx_iface;
         IBindCtx_AddRef(iface);
         return S_OK;
     }
