@@ -1490,6 +1490,7 @@ unsigned char * WINAPI WdtpInterfacePointer_UserUnmarshal(ULONG *pFlags, unsigne
     IStream *stm;
     DWORD size;
     void *ptr;
+    IUnknown *orig;
 
     TRACE("(%s, %p, %p, %s)\n", debugstr_user_flags(pFlags), pBuffer, ppunk, debugstr_guid(riid));
 
@@ -1517,11 +1518,14 @@ unsigned char * WINAPI WdtpInterfacePointer_UserUnmarshal(ULONG *pFlags, unsigne
     memcpy(ptr, pBuffer, size);
     GlobalUnlock(h);
 
+    orig = *ppunk;
     hr = CoUnmarshalInterface(stm, riid, (void**)ppunk);
     IStream_Release(stm);
 
     if(hr != S_OK) RaiseException(hr, 0, 0, NULL);
 
+    if(orig) IUnknown_Release(orig);
+
     return pBuffer + size;
 }
 
@@ -1846,7 +1850,10 @@ unsigned char * __RPC_USER STGMEDIUM_UserUnmarshal(ULONG *pFlags, unsigned char
             pBuffer = WdtpInterfacePointer_UserUnmarshal(pFlags, pBuffer, (IUnknown**)&pStgMedium->u.pstm, &IID_IStream);
         }
         else
+        {
+            if (pStgMedium->u.pstm) IStream_Release( pStgMedium->u.pstm );
             pStgMedium->u.pstm = NULL;
+        }
         break;
     case TYMED_ISTORAGE:
         TRACE("TYMED_ISTORAGE\n");
@@ -1855,7 +1862,10 @@ unsigned char * __RPC_USER STGMEDIUM_UserUnmarshal(ULONG *pFlags, unsigned char
             pBuffer = WdtpInterfacePointer_UserUnmarshal(pFlags, pBuffer, (IUnknown**)&pStgMedium->u.pstg, &IID_IStorage);
         }
         else
+        {
+            if (pStgMedium->u.pstg) IStorage_Release( pStgMedium->u.pstg );
             pStgMedium->u.pstg = NULL;
+        }
         break;
     case TYMED_GDI:
         TRACE("TYMED_GDI\n");
@@ -1884,9 +1894,10 @@ unsigned char * __RPC_USER STGMEDIUM_UserUnmarshal(ULONG *pFlags, unsigned char
         RaiseException(DV_E_TYMED, 0, 0, NULL);
     }
 
-    pStgMedium->pUnkForRelease = NULL;
     if (releaseunk)
         pBuffer = WdtpInterfacePointer_UserUnmarshal(pFlags, pBuffer, &pStgMedium->pUnkForRelease, &IID_IUnknown);
+    /* Unlike the IStream / IStorage ifaces, the existing pUnkForRelease
+       is left intact if a NULL ptr is unmarshalled - see the tests. */
 
     return pBuffer;
 }
@@ -2772,13 +2783,39 @@ HRESULT __RPC_STUB IDataObject_GetData_Stub(
     return IDataObject_GetData(This, pformatetcIn, pRemoteMedium);
 }
 
-HRESULT CALLBACK IDataObject_GetDataHere_Proxy(
-    IDataObject* This,
-    FORMATETC *pformatetc,
-    STGMEDIUM *pmedium)
+HRESULT CALLBACK IDataObject_GetDataHere_Proxy(IDataObject *iface, FORMATETC *fmt, STGMEDIUM *med)
 {
-    TRACE("(%p)->(%p, %p)\n", This, pformatetc, pmedium);
-    return IDataObject_RemoteGetDataHere_Proxy(This, pformatetc, pmedium);
+    IUnknown *release;
+    IStorage *stg = NULL;
+    HRESULT hr;
+
+    TRACE("(%p)->(%p, %p)\n", iface, fmt, med);
+
+    if ((med->tymed & (TYMED_HGLOBAL | TYMED_FILE | TYMED_ISTREAM | TYMED_ISTORAGE)) == 0)
+        return DV_E_TYMED;
+    if (med->tymed != fmt->tymed)
+        return DV_E_TYMED;
+
+    release = med->pUnkForRelease;
+    med->pUnkForRelease = NULL;
+
+    if (med->tymed == TYMED_ISTREAM || med->tymed == TYMED_ISTORAGE)
+    {
+        stg = med->u.pstg; /* This may actually be a stream, but that's ok */
+        if (stg) IStorage_AddRef( stg );
+    }
+
+    hr = IDataObject_RemoteGetDataHere_Proxy(iface, fmt, med);
+
+    med->pUnkForRelease = release;
+    if (stg)
+    {
+        if (med->u.pstg)
+            IStorage_Release( med->u.pstg );
+        med->u.pstg = stg;
+    }
+
+    return hr;
 }
 
 HRESULT __RPC_STUB IDataObject_GetDataHere_Stub(
