@@ -373,6 +373,7 @@ static BOOL transfer_file_http(BackgroundCopyFileImpl *file, URL_COMPONENTSW *uc
     DWORD flags = (uc->nScheme == INTERNET_SCHEME_HTTPS) ? WINHTTP_FLAG_SECURE : 0;
     char buf[4096];
     BOOL ret = FALSE;
+    DWORD written;
 
     transitionJobState(job, BG_JOB_STATE_QUEUED, BG_JOB_STATE_CONNECTING);
 
@@ -405,7 +406,7 @@ static BOOL transfer_file_http(BackgroundCopyFileImpl *file, URL_COMPONENTSW *uc
             break;
         }
         if (!file->read_size) break;
-        if (!(ret = WriteFile(handle, buf, file->read_size, NULL, NULL))) break;
+        if (!(ret = WriteFile(handle, buf, file->read_size, &written, NULL))) break;
 
         EnterCriticalSection(&job->cs);
         file->fileProgress.BytesTransferred += file->read_size;
@@ -476,7 +477,7 @@ BOOL processFile(BackgroundCopyFileImpl *file, BackgroundCopyJobImpl *job)
 {
     static const WCHAR prefix[] = {'B','I','T', 0};
     WCHAR tmpDir[MAX_PATH], tmpName[MAX_PATH];
-    WCHAR host[MAX_PATH], path[MAX_PATH];
+    WCHAR host[MAX_PATH];
     URL_COMPONENTSW uc;
     BOOL ret;
 
@@ -518,10 +519,8 @@ BOOL processFile(BackgroundCopyFileImpl *file, BackgroundCopyJobImpl *job)
     uc.lpszHostName      = host;
     uc.dwHostNameLength  = sizeof(host)/sizeof(host[0]);
     uc.nPort             = 0;
-    uc.lpszUrlPath       = path;
-    uc.dwUrlPathLength   = sizeof(path)/sizeof(path[0]);
-    uc.lpszExtraInfo     = NULL;
-    uc.dwExtraInfoLength = 0;
+    uc.lpszUrlPath       = NULL;
+    uc.dwUrlPathLength   = ~0u;
     ret = WinHttpCrackUrl(file->info.RemoteName, 0, 0, &uc);
     if (!ret)
     {
