@@ -46,7 +46,6 @@ static const REFERENCE_TIME DSoundRenderer_Max_Fill = 150 * 10000;
 static const IBaseFilterVtbl DSoundRender_Vtbl;
 static const IBasicAudioVtbl IBasicAudio_Vtbl;
 static const IReferenceClockVtbl IReferenceClock_Vtbl;
-static const IMediaSeekingVtbl IMediaSeeking_Vtbl;
 static const IAMDirectSoundVtbl IAMDirectSound_Vtbl;
 static const IAMFilterMiscFlagsVtbl IAMFilterMiscFlags_Vtbl;
 
@@ -917,7 +916,7 @@ static LONG cookie_counter = 1;
 
 static DWORD WINAPI DSoundAdviseThread(LPVOID lpParam) {
     DSoundRenderImpl *This = lpParam;
-    struct dsoundrender_timer head = { };
+    struct dsoundrender_timer head = {NULL};
     MSG msg;
 
     TRACE("(%p): Main Loop\n", This);
