@@ -29,6 +29,8 @@
 #include <winbase.h>
 #include <objbase.h>
 #include <richedit.h>
+#include <tom.h>
+#include <richole.h>
 #include <initguid.h>
 #include <imm.h>
 #include <textserv.h>
@@ -504,8 +506,6 @@ static ITextHostVtbl itextHostVtbl = {
     ITextHostImpl_TxGetSelectionBarWidth
 };
 
-static ITextServices *txtserv = NULL;
-static ITextHostTestImpl *dummyTextHost;
 static void *wrapperCodeMem = NULL;
 
 #include "pshpack1.h"
@@ -604,14 +604,15 @@ static void setup_thiscall_wrappers(void)
 /* Conformance test functions. */
 
 /* Initialize the test texthost structure */
-static BOOL init_texthost(void)
+static BOOL init_texthost(ITextServices **txtserv, ITextHost **ret)
 {
+    ITextHostTestImpl *dummyTextHost;
     IUnknown *init;
     HRESULT result;
 
     dummyTextHost = CoTaskMemAlloc(sizeof(*dummyTextHost));
     if (dummyTextHost == NULL) {
-        skip("Insufficient memory to create ITextHost interface\n");
+        win_skip("Insufficient memory to create ITextHost interface\n");
         return FALSE;
     }
     dummyTextHost->ITextHost_iface.lpVtbl = &itextHostVtbl;
@@ -620,54 +621,53 @@ static BOOL init_texthost(void)
     /* MSDN states that an IUnknown object is returned by
        CreateTextServices which is then queried to obtain a
        ITextServices object. */
-    result = (*pCreateTextServices)(NULL, &dummyTextHost->ITextHost_iface, &init);
+    result = pCreateTextServices(NULL, &dummyTextHost->ITextHost_iface, &init);
     ok(result == S_OK, "Did not return S_OK when created (result =  %x)\n", result);
     if (result != S_OK) {
         CoTaskMemFree(dummyTextHost);
-        skip("CreateTextServices failed.\n");
+        win_skip("CreateTextServices failed.\n");
         return FALSE;
     }
 
-    result = IUnknown_QueryInterface(init, pIID_ITextServices,
-                                     (void **)&txtserv);
-    ok((result == S_OK) && (txtserv != NULL), "Querying interface failed (result = %x, txtserv = %p)\n", result, txtserv);
+    result = IUnknown_QueryInterface(init, pIID_ITextServices, (void**)txtserv);
+    ok((result == S_OK) && (*txtserv != NULL), "Querying interface failed (result = %x, txtserv = %p)\n", result, *txtserv);
     IUnknown_Release(init);
-    if (!((result == S_OK) && (txtserv != NULL))) {
+    if (!((result == S_OK) && (*txtserv != NULL))) {
         CoTaskMemFree(dummyTextHost);
-        skip("Could not retrieve ITextServices interface\n");
+        win_skip("Could not retrieve ITextServices interface\n");
         return FALSE;
     }
 
+    *ret = &dummyTextHost->ITextHost_iface;
     return TRUE;
 }
 
-static void free_texthost(void)
-{
-    ITextServices_Release(txtserv);
-    CoTaskMemFree(dummyTextHost);
-}
-
 static void test_TxGetText(void)
 {
+    ITextServices *txtserv;
+    ITextHost *host;
     HRESULT hres;
     BSTR rettext;
 
-    if (!init_texthost())
+    if (!init_texthost(&txtserv, &host))
         return;
 
     hres = ITextServices_TxGetText(txtserv, &rettext);
     ok(hres == S_OK, "ITextServices_TxGetText failed (result = %x)\n", hres);
 
-    free_texthost();
+    ITextServices_Release(txtserv);
+    ITextHost_Release(host);
 }
 
 static void test_TxSetText(void)
 {
+    ITextServices *txtserv;
+    ITextHost *host;
     HRESULT hres;
     BSTR rettext;
     WCHAR settext[] = {'T','e','s','t',0};
 
-    if (!init_texthost())
+    if (!init_texthost(&txtserv, &host))
         return;
 
     hres = ITextServices_TxSetText(txtserv, settext);
@@ -682,10 +682,14 @@ static void test_TxSetText(void)
                  "String returned differs\n");
 
     SysFreeString(rettext);
-    free_texthost();
+    ITextServices_Release(txtserv);
+    ITextHost_Release(host);
 }
 
-static void test_TxGetNaturalSize(void) {
+static void test_TxGetNaturalSize(void)
+{
+    ITextServices *txtserv;
+    ITextHost *host;
     HRESULT result;
     BOOL ret;
 
@@ -708,7 +712,7 @@ static void test_TxGetNaturalSize(void) {
     INT charwidth_caps_text[26];
     TEXTMETRICA tmInfo_text;
 
-    if (!init_texthost())
+    if (!init_texthost(&txtserv, &host))
         return;
 
     hdcDraw = GetDC(NULL);
@@ -757,11 +761,14 @@ static void test_TxGetNaturalSize(void) {
 cleanup:
     RestoreDC(hdcDraw,1);
     ReleaseDC(NULL,hdcDraw);
-    free_texthost();
+    ITextServices_Release(txtserv);
+    ITextHost_Release(host);
 }
 
 static void test_TxDraw(void)
 {
+    ITextServices *txtserv;
+    ITextHost *host;
     HDC tmphdc = GetDC(NULL);
     DWORD dwAspect = DVASPECT_CONTENT;
     HDC hicTargetDev = NULL; /* Means "default" device */
@@ -770,7 +777,8 @@ static void test_TxDraw(void)
     HRESULT result;
     RECTL client = {0,0,100,100};
 
-    if (!init_texthost())
+
+    if (!init_texthost(&txtserv, &host))
         return;
 
     todo_wine {
@@ -780,8 +788,8 @@ static void test_TxDraw(void)
         ok(result == S_OK, "TxDraw failed (result = %x)\n", result);
     }
 
-    free_texthost();
-
+    ITextServices_Release(txtserv);
+    ITextHost_Release(host);
 }
 
 DEFINE_GUID(expected_iid_itextservices, 0x8d33f740, 0xcf58, 0x11ce, 0xa8, 0x9d, 0x00, 0xaa, 0x00, 0x6c, 0xad, 0xc5);
@@ -862,8 +870,80 @@ static void test_COM(void)
     IUnknown_Release(unk_obj.inner_unk);
 }
 
+static ULONG get_refcount(IUnknown *iface)
+{
+    IUnknown_AddRef(iface);
+    return IUnknown_Release(iface);
+}
+
+static void test_QueryInterface(void)
+{
+    ITextServices *txtserv;
+    ITextHost *host;
+    HRESULT hres;
+    IRichEditOle *reole, *txtsrv_reole;
+    ITextDocument *txtdoc, *txtsrv_txtdoc;
+    ULONG refcount;
+
+    if(!init_texthost(&txtserv, &host))
+        return;
+
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 1, "got wrong ref count: %d\n", refcount);
+
+    /* IID_IRichEditOle */
+    hres = ITextServices_QueryInterface(txtserv, &IID_IRichEditOle, (void **)&txtsrv_reole);
+    ok(hres == S_OK, "ITextServices_QueryInterface: 0x%08x\n", hres);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+    refcount = get_refcount((IUnknown *)txtsrv_reole);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+
+    hres = IRichEditOle_QueryInterface(txtsrv_reole, &IID_ITextDocument, (void **)&txtdoc);
+    ok(hres == S_OK, "IRichEditOle_QueryInterface: 0x%08x\n", hres);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 3, "got wrong ref count: %d\n", refcount);
+    refcount = get_refcount((IUnknown *)txtsrv_reole);
+    ok(refcount == 3, "got wrong ref count: %d\n", refcount);
+
+    ITextDocument_Release(txtdoc);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+    IRichEditOle_Release(txtsrv_reole);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 1, "got wrong ref count: %d\n", refcount);
+
+    /* IID_ITextDocument */
+    hres = ITextServices_QueryInterface(txtserv, &IID_ITextDocument, (void **)&txtsrv_txtdoc);
+    ok(hres == S_OK, "ITextServices_QueryInterface: 0x%08x\n", hres);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+    refcount = get_refcount((IUnknown *)txtsrv_txtdoc);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+
+    hres = ITextDocument_QueryInterface(txtsrv_txtdoc, &IID_IRichEditOle, (void **)&reole);
+    ok(hres == S_OK, "ITextDocument_QueryInterface: 0x%08x\n", hres);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 3, "got wrong ref count: %d\n", refcount);
+    refcount = get_refcount((IUnknown *)txtsrv_txtdoc);
+    ok(refcount == 3, "got wrong ref count: %d\n", refcount);
+
+    IRichEditOle_Release(reole);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 2, "got wrong ref count: %d\n", refcount);
+    ITextDocument_Release(txtsrv_txtdoc);
+    refcount = get_refcount((IUnknown *)txtserv);
+    ok(refcount == 1, "got wrong ref count: %d\n", refcount);
+
+    ITextServices_Release(txtserv);
+    ITextHost_Release(host);
+}
+
 START_TEST( txtsrv )
 {
+    ITextServices *txtserv;
+    ITextHost *host;
+
     setup_thiscall_wrappers();
 
     /* Must explicitly LoadLibrary(). The test has no references to functions in
@@ -879,14 +959,16 @@ START_TEST( txtsrv )
     test_IIDs();
     test_COM();
 
-    if (init_texthost())
+    if (init_texthost(&txtserv, &host))
     {
-        free_texthost();
+        ITextServices_Release(txtserv);
+        ITextHost_Release(host);
 
         test_TxGetText();
         test_TxSetText();
         test_TxGetNaturalSize();
         test_TxDraw();
+        test_QueryInterface();
     }
     if (wrapperCodeMem) VirtualFree(wrapperCodeMem, 0, MEM_RELEASE);
 }
