@@ -634,6 +634,7 @@ static void test_CreateViewWindow(void)
     IDropTarget *dt;
     HRESULT hr;
     RECT r = {0};
+    ULONG ref1, ref2;
 
     hr = SHGetDesktopFolder(&desktop);
     ok(hr == S_OK, "got (0x%08x)\n", hr);
@@ -667,7 +668,7 @@ if (0)
     hwnd2 = (HWND)0xdeadbeef;
     hr = IShellView_CreateViewWindow(view, NULL, &settings, &test_shellbrowser, &r, &hwnd2);
     ok(hr == E_UNEXPECTED, "got (0x%08x)\n", hr);
-    ok(hwnd2 == NULL, "got %p\n", hwnd_view);
+    ok(hwnd2 == NULL, "got %p\n", hwnd2);
 
     /* ::DragLeave without drag operation */
     hr = IShellView_QueryInterface(view, &IID_IDropTarget, (void**)&dt);
@@ -676,7 +677,29 @@ if (0)
     ok(hr == S_OK, "got (0x%08x)\n", hr);
     IDropTarget_Release(dt);
 
-    IShellView_Release(view);
+    IShellView_AddRef(view);
+    ref1 = IShellView_Release(view);
+    hr = IShellView_DestroyViewWindow(view);
+    ok(hr == S_OK, "got (0x%08x)\n", hr);
+    ok(!IsWindow(hwnd_view), "hwnd %p still valid\n", hwnd_view);
+    ref2 = IShellView_Release(view);
+    ok(ref1 > ref2, "expected %u > %u\n", ref1, ref2);
+    ref1 = ref2;
+
+    /* Show that releasing the shell view does not destroy the window */
+    hr = IShellFolder_CreateViewObject(desktop, NULL, &IID_IShellView, (void**)&view);
+    ok(hr == S_OK, "got (0x%08x)\n", hr);
+    hwnd_view = NULL;
+    hr = IShellView_CreateViewWindow(view, NULL, &settings, &test_shellbrowser, &r, &hwnd_view);
+    ok(hr == S_OK || broken(hr == S_FALSE), "got (0x%08x)\n", hr);
+    ok(hwnd_view != NULL, "got %p\n", hwnd_view);
+    ok(IsWindow(hwnd_view), "hwnd %p still valid\n", hwnd_view);
+    ref2 = IShellView_Release(view);
+    ok(ref2 != 0, "ref2 = %u\n", ref2);
+    ok(ref2 > ref1, "expected %u > %u\n", ref2, ref1);
+    ok(IsWindow(hwnd_view), "hwnd %p still valid\n", hwnd_view);
+    DestroyWindow(hwnd_view);
+
     IShellFolder_Release(desktop);
 }
 
@@ -688,12 +711,12 @@ static void test_IFolderView(void)
     IShellBrowser *browser;
     IFolderView2 *fv2;
     IFolderView *fv;
+    IUnknown *unk;
     HWND hwnd_view, hwnd_list;
     PITEMID_CHILD pidl;
     HRESULT hr;
     INT ret, count;
     POINT pt;
-    LONG ref1, ref2;
     RECT r;
 
     hr = SHGetDesktopFolder(&desktop);
@@ -821,21 +844,24 @@ if (0)
     hr = IFolderView_GetFolder(fv, &IID_IShellFolder, NULL);
     ok(hr == E_POINTER, "got (0x%08x)\n", hr);
 
-    ref1 = IShellFolder_AddRef(desktop);
-    IShellFolder_Release(desktop);
     hr = IFolderView_GetFolder(fv, &IID_IShellFolder, (void**)&folder);
     ok(hr == S_OK, "got (0x%08x)\n", hr);
-    ref2 = IShellFolder_AddRef(desktop);
-    IShellFolder_Release(desktop);
-    ok(ref1 == ref2 || ref1 + 1 == ref2, /* >= vista */
-       "expected same refcount, got %d\n", ref2);
     ok(desktop == folder, "\n");
+    if (folder) IShellFolder_Release(folder);
+
+    hr = IFolderView_GetFolder(fv, &IID_IUnknown, (void**)&unk);
+    ok(hr == S_OK, "got (0x%08x)\n", hr);
+    if (unk) IUnknown_Release(unk);
 
     hr = IFolderView_QueryInterface(fv, &IID_IFolderView2, (void**)&fv2);
     if (hr != S_OK)
         win_skip("IFolderView2 is not supported.\n");
     if (fv2) IFolderView2_Release(fv2);
 
+    hr = IShellView_DestroyViewWindow(view);
+    ok(hr == S_OK, "got (0x%08x)\n", hr);
+    ok(!IsWindow(hwnd_view), "hwnd %p still valid\n", hwnd_view);
+
     IShellBrowser_Release(browser);
     IFolderView_Release(fv);
     IShellView_Release(view);
