@@ -93,7 +93,7 @@ static HRESULT WINAPI TransformFilter_Input_CheckMediaType(BasePin *iface, const
 
 static HRESULT WINAPI TransformFilter_Input_Receive(BaseInputPin *This, IMediaSample *pInSample)
 {
-    HRESULT hr = S_FALSE;
+    HRESULT hr;
     TransformFilter * pTransform;
     TRACE("%p\n", This);
     pTransform = impl_from_IBaseFilter(This->pin.pinInfo.pFilter);
@@ -365,6 +365,8 @@ HRESULT WINAPI TransformFilterImpl_Stop(IBaseFilter * iface)
         This->filter.state = State_Stopped;
         if (This->pFuncsTable->pfnStopStreaming)
             hr = This->pFuncsTable->pfnStopStreaming(This);
+        if (SUCCEEDED(hr))
+            hr = BaseOutputPinImpl_Inactive(impl_BaseOutputPin_from_IPin(This->ppPins[1]));
     }
     LeaveCriticalSection(&This->csReceive);
 
