@@ -70,30 +70,6 @@ typedef struct
 static BOOL bCBHasChanged = FALSE;
 
 
-/**************************************************************************
- *                      CLIPBOARD_SetClipboardOwner
- *
- * Set the global wineserver clipboard owner. The current process will
- * be the owner and <hWnd> will get the render notifications.
- */
-static BOOL CLIPBOARD_SetClipboardOwner(HWND hWnd)
-{
-    BOOL bRet;
-
-    TRACE(" hWnd(%p)\n", hWnd);
-
-    SERVER_START_REQ( set_clipboard_info )
-    {
-        req->flags = SET_CB_OWNER;
-        req->owner = wine_server_user_handle( hWnd );
-        bRet = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-
-    return bRet;
-}
-
-
 /**************************************************************************
  *                      CLIPBOARD_GetClipboardInfo
  */
@@ -121,26 +97,15 @@ static BOOL CLIPBOARD_GetClipboardInfo(LPCLIPBOARDINFO cbInfo)
 /**************************************************************************
  *	CLIPBOARD_ReleaseOwner
  */
-BOOL CLIPBOARD_ReleaseOwner(void)
+void CLIPBOARD_ReleaseOwner( HWND hwnd )
 {
-    BOOL bRet = FALSE;
-
     SERVER_START_REQ( set_clipboard_info )
     {
         req->flags = SET_CB_RELOWNER | SET_CB_SEQNO;
-
-        if (wine_server_call_err( req ))
-        {
-            ERR("Failed to set clipboard.\n");
-        }
-        else
-        {
-            bRet = TRUE;
-        }
+        req->owner = wine_server_user_handle( hwnd );
+        wine_server_call( req );
     }
     SERVER_END_REQ;
-
-    return bRet;
 }
 
 
@@ -267,44 +232,25 @@ BOOL WINAPI CloseClipboard(void)
  */
 BOOL WINAPI EmptyClipboard(void)
 {
-    CLIPBOARDINFO cbinfo;
+    BOOL ret;
+    HWND owner = GetClipboardOwner();
 
     TRACE("()\n");
 
-    if (!CLIPBOARD_GetClipboardInfo(&cbinfo) ||
-        ~cbinfo.flags & CB_OPEN)
+    if (owner) SendMessageTimeoutW( owner, WM_DESTROYCLIPBOARD, 0, 0, SMTO_ABORTIFHUNG, 5000, NULL );
+
+    SERVER_START_REQ( empty_clipboard )
     {
-        WARN("Clipboard not opened by calling task!\n");
-        SetLastError(ERROR_CLIPBOARD_NOT_OPEN);
-        return FALSE;
+        ret = !wine_server_call_err( req );
     }
+    SERVER_END_REQ;
 
-    /* Destroy private objects */
-    if (cbinfo.hWndOwner)
-        SendMessageW(cbinfo.hWndOwner, WM_DESTROYCLIPBOARD, 0, 0);
-
-    /* Tell the driver to acquire the selection. The current owner
-     * will be signaled to delete its own cache. */
-
-    /* Assign ownership of the clipboard to the current client. We do
-     * this before acquiring the selection so that when we do acquire the
-     * selection and the selection loser gets notified, it can check if
-     * it has lost the Wine clipboard ownership. If it did then it knows
-     * that a WM_DESTROYCLIPBOARD has already been sent. Otherwise it
-     * lost the selection to a X app and it should send the
-     * WM_DESTROYCLIPBOARD itself. */
-    CLIPBOARD_SetClipboardOwner(cbinfo.hWndOpen);
-
-    /* Acquire the selection. This will notify the previous owner
-     * to clear its cache. */
-    USER_Driver->pAcquireClipboard(cbinfo.hWndOpen);
-
-    /* Empty the local cache */
-    USER_Driver->pEmptyClipboard(FALSE);
-
-    bCBHasChanged = TRUE;
-
-    return TRUE;
+    if (ret)
+    {
+        USER_Driver->pEmptyClipboard();
+        bCBHasChanged = TRUE;
+    }
+    return ret;
 }
 
 
@@ -416,6 +362,12 @@ HANDLE WINAPI SetClipboardData(UINT wFormat, HANDLE hData)
 
     TRACE("(%04X, %p) !\n", wFormat, hData);
 
+    if (!wFormat)
+    {
+        SetLastError( ERROR_CLIPBOARD_NOT_OPEN );
+        return 0;
+    }
+
     /* If it's not owned, data can only be set if the format isn't
        available and its rendering is not delayed */
     if (!CLIPBOARD_GetClipboardInfo(&cbinfo) ||
