@@ -394,17 +394,8 @@ static void test_GetNextDlgItem(void)
     {
         HWND a;
         a = (p->tab ? GetNextDlgTabItem : GetNextDlgGroupItem) (hwnd[p->dlg], hwnd[p->ctl], p->prev);
-        if (p->isok)
-        {
+        todo_wine_if (!p->isok)
             ok (a == hwnd[p->res], "Test %d: %s %s item of %d in %d was %d instead of %d\n", p->test, p->prev ? "Prev" : "Next", p->tab ? "Tab" : "Group", p->ctl, p->dlg, id(a), p->res);
-        }
-        else
-        {
-            todo_wine
-            {
-                ok (a == hwnd[p->res], "Test %d: %s %s item of %d in %d was actually  %d matching expected %d\n", p->test, p->prev ? "Prev" : "Next", p->tab ? "Tab" : "Group", p->ctl, p->dlg, id(a), p->res);
-            }
-        }
         p++;
     }
 }
@@ -795,7 +786,8 @@ static INT_PTR CALLBACK focusDlgWinProc (HWND hDlg, UINT uiMsg, WPARAM wParam,
     switch (uiMsg)
     {
     case WM_INITDIALOG:
-       return TRUE;
+        SetWindowTextA(GetDlgItem(hDlg, 200), "new caption");
+        return TRUE;
 
     case WM_COMMAND:
        if (LOWORD(wParam) == 200)
@@ -967,6 +959,34 @@ static void test_focus(void)
 
         DestroyWindow(hDlg);
     }
+
+    /* Test 5:
+     * Select textbox's text on creation */
+    {
+        HWND hDlg;
+        HRSRC hResource;
+        HANDLE hTemplate;
+        DLGTEMPLATE* pTemplate;
+        HWND edit;
+        DWORD selectionStart = 0xdead, selectionEnd = 0xbeef;
+
+        hResource = FindResourceA(g_hinst,"FOCUS_TEST_DIALOG_3", (LPCSTR)RT_DIALOG);
+        hTemplate = LoadResource(g_hinst, hResource);
+        pTemplate = LockResource(hTemplate);
+
+        hDlg = CreateDialogIndirectParamA(g_hinst, pTemplate, NULL, focusDlgWinProc, 0);
+        ok(hDlg != 0, "Failed to create test dialog.\n");
+        edit = GetDlgItem(hDlg, 200);
+
+        ok(GetFocus() == edit, "Focus not set to edit, focus=%p, dialog=%p, edit=%p\n",
+                GetFocus(), hDlg, edit);
+        SendMessageA(edit, EM_GETSEL, (WPARAM)&selectionStart, (LPARAM)&selectionEnd);
+        ok(selectionStart == 0 && selectionEnd == 11,
+                "Text selection after WM_SETFOCUS is [%i, %i) expected [0, 11)\n",
+                selectionStart, selectionEnd);
+
+        DestroyWindow(hDlg);
+    }
 }
 
 static void test_GetDlgItemText(void)
@@ -1227,7 +1247,12 @@ static void test_DisabledDialogTest(void)
 static INT_PTR CALLBACK messageBoxFontDlgWinProc (HWND hDlg, UINT uiMsg, WPARAM wParam,
         LPARAM lParam)
 {
-    return (uiMsg == WM_INITDIALOG);
+    if (uiMsg == WM_INITDIALOG) {
+        SetFocus(hDlg);
+        return 1;
+    }
+
+    return 0;
 }
 
 static void test_MessageBoxFontTest(void)
@@ -1337,6 +1362,12 @@ static void test_SaveRestoreFocus(void)
     foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
     ok (foundId == 1000, "First edit box should have gained focus on dialog creation. Expected: %d, Found: %ld\n", 1000, foundId);
 
+    SetFocus(GetNextDlgTabItem(hDlg, GetFocus(), FALSE));
+    SendMessageA(hDlg, WM_ACTIVATE, MAKEWPARAM(WA_ACTIVE, 0), 0);
+    foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
+    ok (foundId == 1001, "First edit box should have regained focus after dialog reactivation. Expected: %d, Found: %ld\n", 1001, foundId);
+    SetFocus(GetNextDlgTabItem(hDlg, NULL, FALSE));
+
     /* de- then reactivate the dialog */
     SendMessageA(hDlg, WM_ACTIVATE, MAKEWPARAM(WA_INACTIVE, 0), 0);
     SendMessageA(hDlg, WM_ACTIVATE, MAKEWPARAM(WA_ACTIVE, 0), 0);
@@ -1357,6 +1388,24 @@ static void test_SaveRestoreFocus(void)
     foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
     ok (foundId == 1001, "Second edit box should have gained focus after dialog reactivation. Expected: %d, Found: %ld\n", 1001, foundId);
 
+    /* set focus to the dialog */
+    SetFocus(hDlg);
+
+    foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
+    ok (foundId == 1000, "First edit box should have gained focus on dialog focus. Expected: %d, Found: %ld\n", 1000, foundId);
+
+    /* select second tabbable item */
+    SetFocus(GetNextDlgTabItem(hDlg, GetNextDlgTabItem(hDlg, NULL, FALSE), FALSE));
+
+    foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
+    ok (foundId == 1001, "Second edit box should have gained focus. Expected: %d, Found: %ld\n", 1001, foundId);
+
+    /* send WM_ACTIVATE message to already active dialog */
+    SendMessageA(hDlg, WM_ACTIVATE, MAKEWPARAM(WA_ACTIVE, 0), 0);
+
+    foundId = GetWindowLongPtrA(GetFocus(), GWLP_ID);
+    ok (foundId == 1001, "Second edit box should have gained focus. Expected: %d, Found: %ld\n", 1001, foundId);
+
     /* disable the 2nd box */
     EnableWindow(GetFocus(), FALSE);
 
