@@ -44,8 +44,9 @@ static void test_DrawTextCalcRect(void)
     static WCHAR emptystringW[] = { 0 };
     static CHAR wordbreak_text[] = "line1 line2";
     static WCHAR wordbreak_textW[] = {'l','i','n','e','1',' ','l','i','n','e','2',0};
+    static char tabstring[] = "one\ttwo";
     INT textlen, textheight, heightcheck;
-    RECT rect = { 0, 0, 100, 0 };
+    RECT rect = { 0, 0, 100, 0 }, rect2;
     BOOL ret;
     DRAWTEXTPARAMS dtp;
     BOOL conform_xp = TRUE;
@@ -315,16 +316,16 @@ static void test_DrawTextCalcRect(void)
     dtp.cbSize = sizeof(dtp);
     dtp.iLeftMargin = 0;
     dtp.iRightMargin = 0;
-    SetRect( &rect, 0, 0, 0, 0);
+    SetRectEmpty(&rect);
     DrawTextExA(hdc, text, -1, &rect, DT_CALCRECT, &dtp);
     textlen = rect.right; /* Width without margin */
     dtp.iLeftMargin = 8;
-    SetRect( &rect, 0, 0, 0, 0);
+    SetRectEmpty(&rect);
     DrawTextExA(hdc, text, -1, &rect, DT_CALCRECT, &dtp);
     ok(rect.right==dtp.iLeftMargin+textlen  ,"Incorrect left margin calculated  rc(%d,%d)\n", rect.left, rect.right);
     dtp.iLeftMargin = 0;
     dtp.iRightMargin = 8;
-    SetRect( &rect, 0, 0, 0, 0);
+    SetRectEmpty(&rect);
     DrawTextExA(hdc, text, -1, &rect, DT_CALCRECT, &dtp);
     ok(rect.right==dtp.iRightMargin+textlen  ,"Incorrect right margin calculated rc(%d,%d)\n", rect.left, rect.right);
 
@@ -535,7 +536,7 @@ static void test_DrawTextCalcRect(void)
     }
 
     /* More test cases from bug 12226 */
-    SetRect(&rect, 0, 0, 0, 0);
+    SetRectEmpty(&rect);
     textheight = DrawTextA(hdc, emptystring, -1, &rect, DT_CALCRECT | DT_LEFT | DT_SINGLELINE);
     ok(textheight, "DrawTextA error %u\n", GetLastError());
     ok(0 == rect.left, "expected 0, got %d\n", rect.left);
@@ -543,7 +544,7 @@ static void test_DrawTextCalcRect(void)
     ok(0 == rect.top, "expected 0, got %d\n", rect.top);
     ok(rect.bottom, "rect.bottom should not be 0\n");
 
-    SetRect(&rect, 0, 0, 0, 0);
+    SetRectEmpty(&rect);
     textheight = DrawTextW(hdc, emptystringW, -1, &rect, DT_CALCRECT | DT_LEFT | DT_SINGLELINE);
     if (!textheight && GetLastError() == ERROR_CALL_NOT_IMPLEMENTED)
     {
@@ -580,6 +581,42 @@ static void test_DrawTextCalcRect(void)
     ok(textheight >= heightcheck * 6, "Got unexpected textheight %d, expected at least %d.\n",
        textheight, heightcheck * 6);
 
+    /* DT_TABSTOP | DT_EXPANDTABS tests */
+    SetRect( &rect, 0,0, 10, 10);
+    textheight = DrawTextA(hdc, tabstring, -1, &rect, DT_TABSTOP | DT_EXPANDTABS );
+    ok(textheight >= heightcheck, "Got unexpected textheight %d\n", textheight);
+
+    SetRect( &rect, 0,0, 10, 10);
+    memset(&dtp, 0, sizeof(dtp));
+    dtp.cbSize = sizeof(dtp);
+    textheight = DrawTextExA(hdc, tabstring, -1, &rect, DT_CALCRECT, &dtp);
+    ok(textheight >= heightcheck, "Got unexpected textheight %d\n", textheight);
+    ok(dtp.iTabLength == 0, "invalid dtp.iTabLength = %i\n",dtp.iTabLength);
+
+    SetRect( &rect2, 0,0, 10, 10);
+    memset(&dtp, 0, sizeof(dtp));
+    dtp.cbSize = sizeof(dtp);
+    textheight = DrawTextExA(hdc, tabstring, -1, &rect2, DT_CALCRECT | DT_TABSTOP | DT_EXPANDTABS, &dtp);
+    ok(textheight >= heightcheck, "Got unexpected textheight %d\n", textheight);
+    ok(dtp.iTabLength == 0, "invalid dtp.iTabLength = %i\n",dtp.iTabLength);
+    ok(rect.left == rect2.left && rect.right != rect2.right && rect.top == rect2.top && rect.bottom == rect2.bottom,
+        "incorrect rect %d,%d-%d,%d rect2 %d,%d-%d,%d\n",
+        rect.left, rect.top, rect.right, rect.bottom, rect2.left, rect2.top, rect2.right, rect2.bottom );
+
+    SetRect( &rect, 0,0, 10, 10);
+    memset(&dtp, 0, sizeof(dtp));
+    dtp.cbSize = sizeof(dtp);
+    dtp.iTabLength = 8;
+    textheight = DrawTextExA(hdc, tabstring, -1, &rect, DT_CALCRECT | DT_TABSTOP | DT_EXPANDTABS, &dtp);
+    ok(textheight >= heightcheck, "Got unexpected textheight %d\n", textheight);
+    ok(dtp.iTabLength == 8, "invalid dtp.iTabLength = %i\n",dtp.iTabLength);
+    ok(rect.left == rect2.left, "unexpected value %d, got %d\n", rect.left, rect2.left);
+    /* XP, 2003 appear to not give the same values. */
+    ok(rect.right == rect2.right || broken(rect.right > rect2.right), "unexpected value %d, got %d\n",rect.right, rect2.right);
+    ok(rect.top == rect2.top, "unexpected value %d, got %d\n", rect.top, rect2.top);
+    ok(rect.bottom == rect2.bottom , "unexpected value %d, got %d\n", rect.bottom, rect2.bottom);
+
+
     SelectObject(hdc, hOldFont);
     ret = DeleteObject(hFont);
     ok( ret, "DeleteObject error %u\n", GetLastError());
@@ -604,15 +641,13 @@ static void strfmt( const char *str, char *strout)
 
 
 #define TABTEST( tabval, tabcount, string, _exp) \
-{ int i,x_act, x_exp; char strdisp[64];\
+{ int i; char strdisp[64];\
     for(i=0;i<8;i++) tabs[i]=(i+1)*(tabval); \
     extent = GetTabbedTextExtentA( hdc, string, strlen( string), (tabcount), tabs); \
     strfmt( string, strdisp); \
  /*   trace( "Extent is %08lx\n", extent); */\
-    x_act = LOWORD( extent); \
-    x_exp = (_exp); \
-    ok( x_act == x_exp, "Test case \"%s\". Text extent is %d, expected %d tab %d tabcount %d\n", \
-        strdisp, x_act, x_exp, tabval, tabcount); \
+    ok( extent == _exp, "Test case \"%s\". Text extent is 0x%x, expected 0x%x tab %d tabcount %d\n", \
+        strdisp, extent, _exp, tabval, tabcount); \
 } \
 
 
@@ -649,16 +684,16 @@ static void test_TabbedText(void)
         tab =  (cx *4 + t);
         /* test the special case tabcount =1 and the general array (80 of tabs */
         for( tabcount = 1; tabcount <= 8; tabcount +=7) { 
-            TABTEST( align * tab, tabcount, "\t", tab)
-            TABTEST( align * tab, tabcount, "xxx\t", tab)
-            TABTEST( align * tab, tabcount, "\tx", tab+cx)
-            TABTEST( align * tab, tabcount, "\t\t", tab*2)
-            TABTEST( align * tab, tabcount, "\tx\t", tab*2)
-            TABTEST( align * tab, tabcount, "x\tx", tab+cx)
-            TABTEST( align * tab, tabcount, "xx\tx", tab+cx)
-            TABTEST( align * tab, tabcount, "xxx\tx", tab+cx)
-            TABTEST( align * tab, tabcount, "xxxx\tx", t>0 ? tab + cx : 2*tab+cx)
-            TABTEST( align * tab, tabcount, "xxxxx\tx", 2*tab+cx)
+            TABTEST( align * tab, tabcount, "\t", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "xxx\t", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "\tx", MAKELONG(tab+cx, cy))
+            TABTEST( align * tab, tabcount, "\t\t", MAKELONG(tab*2, cy))
+            TABTEST( align * tab, tabcount, "\tx\t", MAKELONG(tab*2, cy))
+            TABTEST( align * tab, tabcount, "x\tx", MAKELONG(tab+cx, cy))
+            TABTEST( align * tab, tabcount, "xx\tx", MAKELONG(tab+cx, cy))
+            TABTEST( align * tab, tabcount, "xxx\tx", MAKELONG(tab+cx, cy))
+            TABTEST( align * tab, tabcount, "xxxx\tx", MAKELONG(t>0 ? tab + cx : 2*tab+cx, cy))
+            TABTEST( align * tab, tabcount, "xxxxx\tx", MAKELONG(2*tab+cx, cy))
         }
     }
     align=-1;
@@ -667,16 +702,16 @@ static void test_TabbedText(void)
         tab =  (cx *4 + t);
         /* test the special case tabcount =1 and the general array (8) of tabs */
         for( tabcount = 1; tabcount <= 8; tabcount +=7) { 
-            TABTEST( align * tab, tabcount, "\t", tab)
-            TABTEST( align * tab, tabcount, "xxx\t", tab)
-            TABTEST( align * tab, tabcount, "\tx", tab)
-            TABTEST( align * tab, tabcount, "\t\t", tab*2)
-            TABTEST( align * tab, tabcount, "\tx\t", tab*2)
-            TABTEST( align * tab, tabcount, "x\tx", tab)
-            TABTEST( align * tab, tabcount, "xx\tx", tab)
-            TABTEST( align * tab, tabcount, "xxx\tx", 4 * cx >= tab ? 2*tab :tab)
-            TABTEST( align * tab, tabcount, "xxxx\tx", 2*tab)
-            TABTEST( align * tab, tabcount, "xxxxx\tx", 2*tab)
+            TABTEST( align * tab, tabcount, "\t", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "xxx\t", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "\tx", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "\t\t", MAKELONG(tab*2, cy))
+            TABTEST( align * tab, tabcount, "\tx\t", MAKELONG(tab*2, cy))
+            TABTEST( align * tab, tabcount, "x\tx", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "xx\tx", MAKELONG(tab, cy))
+            TABTEST( align * tab, tabcount, "xxx\tx", MAKELONG(4 * cx >= tab ? 2*tab :tab, cy))
+            TABTEST( align * tab, tabcount, "xxxx\tx", MAKELONG(2*tab, cy))
+            TABTEST( align * tab, tabcount, "xxxxx\tx", MAKELONG(2*tab, cy))
         }
     }
 
