@@ -220,13 +220,12 @@ static HRESULT lookup_identifier(exec_ctx_t *ctx, BSTR name, vbdisp_invoke_type_
 }
 
 static HRESULT add_dynamic_var(exec_ctx_t *ctx, const WCHAR *name,
-        BOOL is_const, VARIANT *val, BOOL own_val, VARIANT **out_var)
+        BOOL is_const, VARIANT **out_var)
 {
     dynamic_var_t *new_var;
     heap_pool_t *heap;
     WCHAR *str;
     unsigned size;
-    HRESULT hres;
 
     heap = ctx->func->type == FUNC_GLOBAL ? &ctx->script->heap : &ctx->heap;
 
@@ -241,15 +240,7 @@ static HRESULT add_dynamic_var(exec_ctx_t *ctx, const WCHAR *name,
     memcpy(str, name, size);
     new_var->name = str;
     new_var->is_const = is_const;
-
-    if(own_val) {
-        new_var->v = *val;
-    }else {
-        V_VT(&new_var->v) = VT_EMPTY;
-        hres = VariantCopy(&new_var->v, val);
-        if(FAILED(hres))
-            return hres;
-    }
+    V_VT(&new_var->v) = VT_EMPTY;
 
     if(ctx->func->type == FUNC_GLOBAL) {
         new_var->next = ctx->script->global_vars;
@@ -259,9 +250,7 @@ static HRESULT add_dynamic_var(exec_ctx_t *ctx, const WCHAR *name,
         ctx->dynamic_vars = new_var;
     }
 
-    if(out_var)
-        *out_var = &new_var->v;
-
+    *out_var = &new_var->v;
     return S_OK;
 }
 
@@ -334,10 +323,9 @@ static HRESULT stack_pop_val(exec_ctx_t *ctx, variant_val_t *r)
     stack_pop_deref(ctx, r);
 
     if(V_VT(r->v) == VT_DISPATCH) {
-        DISPPARAMS dp = {0};
         HRESULT hres;
 
-        hres = disp_call(ctx->script, V_DISPATCH(r->v), DISPID_VALUE, &dp, &r->store);
+        hres = get_disp_value(ctx->script, V_DISPATCH(r->v), &r->store);
         if(r->owned)
             IDispatch_Release(V_DISPATCH(r->v));
         if(FAILED(hres))
@@ -365,12 +353,10 @@ static HRESULT stack_assume_val(exec_ctx_t *ctx, unsigned n)
     }
 
     if(V_VT(v) == VT_DISPATCH) {
-        DISPPARAMS dp = {0};
         IDispatch *disp;
 
         disp = V_DISPATCH(v);
-        V_VT(v) = VT_EMPTY;
-        hres = disp_call(ctx->script, disp, DISPID_VALUE, &dp, v);
+        hres = get_disp_value(ctx->script, disp, v);
         IDispatch_Release(disp);
         if(FAILED(hres))
             return hres;
@@ -620,9 +606,8 @@ static HRESULT do_icall(exec_ctx_t *ctx, VARIANT *res)
         break;
     case REF_NONE:
         if(res && !ctx->func->code_ctx->option_explicit && arg_cnt == 0) {
-            VARIANT v, *new;
-            VariantInit(&v);
-            hres = add_dynamic_var(ctx, identifier, FALSE, &v, FALSE, &new);
+            VARIANT *new;
+            hres = add_dynamic_var(ctx, identifier, FALSE, &new);
             if(FAILED(hres))
                 return hres;
             V_VT(res) = VT_BYREF|VT_VARIANT;
@@ -709,7 +694,29 @@ static HRESULT interp_mcallv(exec_ctx_t *ctx)
     return do_mcall(ctx, NULL);
 }
 
-static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, DISPPARAMS *dp)
+static HRESULT assign_value(exec_ctx_t *ctx, VARIANT *dst, VARIANT *src, WORD flags)
+{
+    HRESULT hres;
+
+    hres = VariantCopyInd(dst, src);
+    if(FAILED(hres))
+        return hres;
+
+    if(V_VT(dst) == VT_DISPATCH && !(flags & DISPATCH_PROPERTYPUTREF)) {
+        VARIANT value;
+
+        hres = get_disp_value(ctx->script, V_DISPATCH(dst), &value);
+        IDispatch_Release(V_DISPATCH(dst));
+        if(FAILED(hres))
+            return hres;
+
+        *dst = value;
+    }
+
+    return S_OK;
+}
+
+static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, WORD flags, DISPPARAMS *dp)
 {
     ref_t ref;
     HRESULT hres;
@@ -758,11 +765,11 @@ static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, DISPPARAMS *dp)
             return E_NOTIMPL;
         }
 
-        hres = VariantCopyInd(v, dp->rgvarg);
+        hres = assign_value(ctx, v, dp->rgvarg, flags);
         break;
     }
     case REF_DISP:
-        hres = disp_propput(ctx->script, ref.u.d.disp, ref.u.d.id, dp);
+        hres = disp_propput(ctx->script, ref.u.d.disp, ref.u.d.id, flags, dp);
         break;
     case REF_FUNC:
         FIXME("functions not implemented\n");
@@ -778,13 +785,17 @@ static HRESULT assign_ident(exec_ctx_t *ctx, BSTR name, DISPPARAMS *dp)
             FIXME("throw exception\n");
             hres = E_FAIL;
         }else {
+            VARIANT *new_var;
+
             if(arg_cnt(dp)) {
                 FIXME("arg_cnt %d not supported\n", arg_cnt(dp));
                 return E_NOTIMPL;
             }
 
             TRACE("creating variable %s\n", debugstr_w(name));
-            hres = add_dynamic_var(ctx, name, FALSE, dp->rgvarg, FALSE, NULL);
+            hres = add_dynamic_var(ctx, name, FALSE, &new_var);
+            if(SUCCEEDED(hres))
+                hres = assign_value(ctx, new_var, dp->rgvarg, flags);
         }
     }
 
@@ -800,12 +811,8 @@ static HRESULT interp_assign_ident(exec_ctx_t *ctx)
 
     TRACE("%s\n", debugstr_w(arg));
 
-    hres = stack_assume_val(ctx, arg_cnt);
-    if(FAILED(hres))
-        return hres;
-
     vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
-    hres = assign_ident(ctx, arg, &dp);
+    hres = assign_ident(ctx, arg, DISPATCH_PROPERTYPUT, &dp);
     if(FAILED(hres))
         return hres;
 
@@ -832,7 +839,7 @@ static HRESULT interp_set_ident(exec_ctx_t *ctx)
         return hres;
 
     vbstack_to_dp(ctx, 0, TRUE, &dp);
-    hres = assign_ident(ctx, ctx->instr->arg1.bstr, &dp);
+    hres = assign_ident(ctx, ctx->instr->arg1.bstr, DISPATCH_PROPERTYPUTREF, &dp);
     if(FAILED(hres))
         return hres;
 
@@ -860,14 +867,10 @@ static HRESULT interp_assign_member(exec_ctx_t *ctx)
         return E_FAIL;
     }
 
-    hres = stack_assume_val(ctx, arg_cnt);
-    if(FAILED(hres))
-        return hres;
-
     hres = disp_get_id(obj, identifier, VBDISP_LET, FALSE, &id);
     if(SUCCEEDED(hres)) {
         vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
-        hres = disp_propput(ctx->script, obj, id, &dp);
+        hres = disp_propput(ctx->script, obj, id, DISPATCH_PROPERTYPUT, &dp);
     }
     if(FAILED(hres))
         return hres;
@@ -908,7 +911,7 @@ static HRESULT interp_set_member(exec_ctx_t *ctx)
     hres = disp_get_id(obj, identifier, VBDISP_SET, FALSE, &id);
     if(SUCCEEDED(hres)) {
         vbstack_to_dp(ctx, arg_cnt, TRUE, &dp);
-        hres = disp_propput(ctx->script, obj, id, &dp);
+        hres = disp_propput(ctx->script, obj, id, DISPATCH_PROPERTYPUTREF, &dp);
     }
     if(FAILED(hres))
         return hres;
@@ -920,7 +923,7 @@ static HRESULT interp_set_member(exec_ctx_t *ctx)
 static HRESULT interp_const(exec_ctx_t *ctx)
 {
     BSTR arg = ctx->instr->arg1.bstr;
-    variant_val_t val;
+    VARIANT *v;
     ref_t ref;
     HRESULT hres;
 
@@ -937,11 +940,16 @@ static HRESULT interp_const(exec_ctx_t *ctx)
         return E_FAIL;
     }
 
-    hres = stack_pop_val(ctx, &val);
+    hres = stack_assume_val(ctx, 0);
     if(FAILED(hres))
         return hres;
 
-    return add_dynamic_var(ctx, arg, TRUE, val.v, val.owned, NULL);
+    hres = add_dynamic_var(ctx, arg, TRUE, &v);
+    if(FAILED(hres))
+        return hres;
+
+    *v = *stack_pop(ctx);
+    return S_OK;
 }
 
 static HRESULT interp_val(exec_ctx_t *ctx)
@@ -1176,7 +1184,7 @@ static HRESULT interp_enumnext(exec_ctx_t *ctx)
         return hres;
 
     do_continue = hres == S_OK;
-    hres = assign_ident(ctx, ident, &dp);
+    hres = assign_ident(ctx, ident, DISPATCH_PROPERTYPUT|DISPATCH_PROPERTYPUTREF, &dp);
     VariantClear(&v);
     if(FAILED(hres))
         return hres;
@@ -1366,6 +1374,18 @@ static HRESULT interp_nothing(exec_ctx_t *ctx)
     return stack_push(ctx, &v);
 }
 
+static HRESULT interp_hres(exec_ctx_t *ctx)
+{
+    const unsigned arg = ctx->instr->arg1.uint;
+    VARIANT v;
+
+    TRACE("%d\n", arg);
+
+    V_VT(&v) = VT_ERROR;
+    V_ERROR(&v) = arg;
+    return stack_push(ctx, &v);
+}
+
 static HRESULT interp_not(exec_ctx_t *ctx)
 {
     variant_val_t val;
