@@ -1199,7 +1199,6 @@ DWORD WINAPI VerFindFileA(
 
     GetSystemDirectoryA(systemDir, sizeof(systemDir));
     curDir = "";
-    destDir = "";
 
     if(flags & VFFF_ISSHAREDFILE)
     {
@@ -1296,7 +1295,6 @@ DWORD WINAPI VerFindFileW( DWORD flags,LPCWSTR lpszFilename,LPCWSTR lpszWinDir,
 
     GetSystemDirectoryW(systemDir, sizeof(systemDir)/sizeof(WCHAR));
     curDir = &emptyW;
-    destDir = &emptyW;
 
     if(flags & VFFF_ISSHAREDFILE)
     {
@@ -1372,7 +1370,7 @@ _fetch_versioninfo(LPSTR fn,VS_FIXEDFILEINFO **vffi) {
     alloclen = 1000;
     buf=HeapAlloc(GetProcessHeap(), 0, alloclen);
     if(buf == NULL) {
-        WARN("Memory exausted while fetching version info!\n");
+        WARN("Memory exhausted while fetching version info!\n");
         return NULL;
     }
     while (1) {
@@ -1386,7 +1384,7 @@ _fetch_versioninfo(LPSTR fn,VS_FIXEDFILEINFO **vffi) {
 	    HeapFree(GetProcessHeap(), 0, buf);
 	    buf = HeapAlloc(GetProcessHeap(), 0, alloclen);
             if(buf == NULL) {
-               WARN("Memory exausted while fetching version info!\n");
+               WARN("Memory exhausted while fetching version info!\n");
                return NULL;
             }
 	} else {
