@@ -25,7 +25,25 @@ struct xmlbuf
 };
 
 void *ws_alloc( WS_HEAP *, SIZE_T ) DECLSPEC_HIDDEN;
+void *ws_realloc( WS_HEAP *, void *, SIZE_T ) DECLSPEC_HIDDEN;
 void ws_free( WS_HEAP *, void * ) DECLSPEC_HIDDEN;
+const char *debugstr_xmlstr( const WS_XML_STRING * ) DECLSPEC_HIDDEN;
+WS_XML_STRING *alloc_xml_string( const unsigned char *, ULONG ) DECLSPEC_HIDDEN;
+WS_XML_UTF8_TEXT *alloc_utf8_text( const unsigned char *, ULONG ) DECLSPEC_HIDDEN;
+HRESULT append_attribute( WS_XML_ELEMENT_NODE *, WS_XML_ATTRIBUTE * ) DECLSPEC_HIDDEN;
+void free_attribute( WS_XML_ATTRIBUTE * ) DECLSPEC_HIDDEN;
+
+struct node
+{
+    WS_XML_ELEMENT_NODE hdr;
+    struct list         entry;
+    struct node        *parent;
+    struct list         children;
+};
+
+struct node *alloc_node( WS_XML_NODE_TYPE ) DECLSPEC_HIDDEN;
+void free_node( struct node * ) DECLSPEC_HIDDEN;
+void destroy_nodes( struct node * ) DECLSPEC_HIDDEN;
 
 static inline void *heap_alloc( SIZE_T size )
 {
@@ -42,6 +60,11 @@ static inline void *heap_realloc( void *mem, SIZE_T size )
     return HeapReAlloc( GetProcessHeap(), 0, mem, size );
 }
 
+static inline void *heap_realloc_zero( void *mem, SIZE_T size )
+{
+    return HeapReAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, mem, size );
+}
+
 static inline BOOL heap_free( void *mem )
 {
     return HeapFree( GetProcessHeap(), 0, mem );
