@@ -27,8 +27,6 @@
 #include "winbase.h"
 #include "winreg.h"
 #include "objbase.h"
-#include "wincodec.h"
-#include "wincodecsdk.h"
 
 #include "wincodecs_private.h"
 
@@ -56,6 +54,7 @@ static const WCHAR supportstransparency_valuename[] = {'S','u','p','p','o','r','
 static const WCHAR requiresfullstream_valuename[] = {'R','e','q','u','i','r','e','s','F','u','l','l','S','t','r','e','a','m',0};
 static const WCHAR supportspadding_valuename[] = {'S','u','p','p','o','r','t','s','P','a','d','d','i','n','g',0};
 static const WCHAR fileextensions_valuename[] = {'F','i','l','e','E','x','t','e','n','s','i','o','n','s',0};
+static const WCHAR containers_keyname[] = {'C','o','n','t','a','i','n','e','r','s',0};
 
 static HRESULT ComponentInfo_GetStringValue(HKEY classkey, LPCWSTR value,
     UINT buffer_size, WCHAR *buffer, UINT *actual_size)
@@ -151,7 +150,12 @@ static HRESULT ComponentInfo_GetGuidList(HKEY classkey, LPCWSTR subkeyname,
         return E_INVALIDARG;
 
     ret = RegOpenKeyExW(classkey, subkeyname, 0, KEY_READ, &subkey);
-    if (ret != ERROR_SUCCESS) return HRESULT_FROM_WIN32(ret);
+    if (ret == ERROR_FILE_NOT_FOUND)
+    {
+        *actual_size = 0;
+        return S_OK;
+    }
+    else if (ret != ERROR_SUCCESS) return HRESULT_FROM_WIN32(ret);
 
     if (buffer)
     {
@@ -224,7 +228,7 @@ static HRESULT WINAPI BitmapDecoderInfo_QueryInterface(IWICBitmapDecoderInfo *if
         IsEqualIID(&IID_IWICBitmapCodecInfo, iid) ||
         IsEqualIID(&IID_IWICBitmapDecoderInfo ,iid))
     {
-        *ppv = This;
+        *ppv = &This->IWICBitmapDecoderInfo_iface;
     }
     else
     {
@@ -617,8 +621,7 @@ static HRESULT WINAPI BitmapDecoderInfo_CreateInstance(IWICBitmapDecoderInfo *if
 
     TRACE("(%p,%p)\n", iface, ppIBitmapDecoder);
 
-    return CoCreateInstance(&This->clsid, NULL, CLSCTX_INPROC_SERVER,
-        &IID_IWICBitmapDecoder, (void**)ppIBitmapDecoder);
+    return create_instance(&This->clsid, &IID_IWICBitmapDecoder, (void**)ppIBitmapDecoder);
 }
 
 static const IWICBitmapDecoderInfoVtbl BitmapDecoderInfo_Vtbl = {
@@ -666,7 +669,7 @@ static HRESULT BitmapDecoderInfo_Constructor(HKEY classkey, REFCLSID clsid, IWIC
     This->classkey = classkey;
     memcpy(&This->clsid, clsid, sizeof(CLSID));
 
-    *ppIInfo = (IWICComponentInfo*)This;
+    *ppIInfo = (IWICComponentInfo *)&This->IWICBitmapDecoderInfo_iface;
     return S_OK;
 }
 
@@ -695,7 +698,7 @@ static HRESULT WINAPI BitmapEncoderInfo_QueryInterface(IWICBitmapEncoderInfo *if
         IsEqualIID(&IID_IWICBitmapCodecInfo, iid) ||
         IsEqualIID(&IID_IWICBitmapEncoderInfo ,iid))
     {
-        *ppv = This;
+        *ppv = &This->IWICBitmapEncoderInfo_iface;
     }
     else
     {
@@ -911,8 +914,7 @@ static HRESULT WINAPI BitmapEncoderInfo_CreateInstance(IWICBitmapEncoderInfo *if
 
     TRACE("(%p,%p)\n", iface, ppIBitmapEncoder);
 
-    return CoCreateInstance(&This->clsid, NULL, CLSCTX_INPROC_SERVER,
-        &IID_IWICBitmapEncoder, (void**)ppIBitmapEncoder);
+    return create_instance(&This->clsid, &IID_IWICBitmapEncoder, (void**)ppIBitmapEncoder);
 }
 
 static const IWICBitmapEncoderInfoVtbl BitmapEncoderInfo_Vtbl = {
@@ -958,7 +960,7 @@ static HRESULT BitmapEncoderInfo_Constructor(HKEY classkey, REFCLSID clsid, IWIC
     This->classkey = classkey;
     memcpy(&This->clsid, clsid, sizeof(CLSID));
 
-    *ppIInfo = (IWICComponentInfo*)This;
+    *ppIInfo = (IWICComponentInfo *)&This->IWICBitmapEncoderInfo_iface;
     return S_OK;
 }
 
@@ -986,7 +988,7 @@ static HRESULT WINAPI FormatConverterInfo_QueryInterface(IWICFormatConverterInfo
         IsEqualIID(&IID_IWICComponentInfo, iid) ||
         IsEqualIID(&IID_IWICFormatConverterInfo ,iid))
     {
-        *ppv = This;
+        *ppv = &This->IWICFormatConverterInfo_iface;
     }
     else
     {
@@ -1119,8 +1121,8 @@ static HRESULT WINAPI FormatConverterInfo_CreateInstance(IWICFormatConverterInfo
 
     TRACE("(%p,%p)\n", iface, ppIFormatConverter);
 
-    return CoCreateInstance(&This->clsid, NULL, CLSCTX_INPROC_SERVER,
-        &IID_IWICFormatConverter, (void**)ppIFormatConverter);
+    return create_instance(&This->clsid, &IID_IWICFormatConverter,
+            (void**)ppIFormatConverter);
 }
 
 static BOOL ConverterSupportsFormat(IWICFormatConverterInfo *iface, const WCHAR *formatguid)
@@ -1175,7 +1177,7 @@ static HRESULT FormatConverterInfo_Constructor(HKEY classkey, REFCLSID clsid, IW
     This->classkey = classkey;
     memcpy(&This->clsid, clsid, sizeof(CLSID));
 
-    *ppIInfo = (IWICComponentInfo*)This;
+    *ppIInfo = (IWICComponentInfo *)&This->IWICFormatConverterInfo_iface;
     return S_OK;
 }
 
@@ -1204,7 +1206,7 @@ static HRESULT WINAPI PixelFormatInfo_QueryInterface(IWICPixelFormatInfo2 *iface
         IsEqualIID(&IID_IWICPixelFormatInfo, iid) ||
         IsEqualIID(&IID_IWICPixelFormatInfo2 ,iid))
     {
-        *ppv = This;
+        *ppv = &This->IWICPixelFormatInfo2_iface;
     }
     else
     {
@@ -1469,7 +1471,7 @@ static HRESULT PixelFormatInfo_Constructor(HKEY classkey, REFCLSID clsid, IWICCo
     This->classkey = classkey;
     memcpy(&This->clsid, clsid, sizeof(CLSID));
 
-    *ppIInfo = (IWICComponentInfo*)This;
+    *ppIInfo = (IWICComponentInfo *)&This->IWICPixelFormatInfo2_iface;
     return S_OK;
 }
 
@@ -1500,7 +1502,7 @@ static HRESULT WINAPI MetadataReaderInfo_QueryInterface(IWICMetadataReaderInfo *
         IsEqualIID(&IID_IWICMetadataHandlerInfo, riid) ||
         IsEqualIID(&IID_IWICMetadataReaderInfo, riid))
     {
-        *ppv = This;
+        *ppv = &This->IWICMetadataReaderInfo_iface;
     }
     else
     {
@@ -1630,10 +1632,11 @@ static HRESULT WINAPI MetadataReaderInfo_GetMetadataFormat(IWICMetadataReaderInf
 static HRESULT WINAPI MetadataReaderInfo_GetContainerFormats(IWICMetadataReaderInfo *iface,
     UINT length, GUID *formats, UINT *actual_length)
 {
-    if (!actual_length) return E_INVALIDARG;
+    MetadataReaderInfo *This = impl_from_IWICMetadataReaderInfo(iface);
+    TRACE("(%p,%u,%p,%p)\n", iface, length, formats, actual_length);
 
-    FIXME("(%p,%u,%p,%p): stub\n", iface, length, formats, actual_length);
-    return E_NOTIMPL;
+    return ComponentInfo_GetGuidList(This->classkey, containers_keyname, length,
+        formats, actual_length);
 }
 
 static HRESULT WINAPI MetadataReaderInfo_GetDeviceManufacturer(IWICMetadataReaderInfo *iface,
@@ -1674,19 +1677,187 @@ static HRESULT WINAPI MetadataReaderInfo_DoesRequireFixedSize(IWICMetadataReader
 }
 
 static HRESULT WINAPI MetadataReaderInfo_GetPatterns(IWICMetadataReaderInfo *iface,
-    REFGUID container, UINT length, WICMetadataPattern *pattern, UINT *count, UINT *actual_length)
+    REFGUID container, UINT length, WICMetadataPattern *patterns, UINT *count, UINT *actual_length)
 {
-    if (!actual_length) return E_INVALIDARG;
+    MetadataReaderInfo *This = impl_from_IWICMetadataReaderInfo(iface);
+    HRESULT hr=S_OK;
+    LONG res;
+    UINT pattern_count=0, patterns_size=0;
+    DWORD valuesize, patternsize;
+    BYTE *bPatterns=(BYTE*)patterns;
+    HKEY containers_key, guid_key, pattern_key;
+    WCHAR subkeyname[11];
+    WCHAR guidkeyname[39];
+    int i;
+    static const WCHAR uintformatW[] = {'%','u',0};
+    static const WCHAR patternW[] = {'P','a','t','t','e','r','n',0};
+    static const WCHAR positionW[] = {'P','o','s','i','t','i','o','n',0};
+    static const WCHAR maskW[] = {'M','a','s','k',0};
+    static const WCHAR dataoffsetW[] = {'D','a','t','a','O','f','f','s','e','t',0};
 
-    FIXME("(%p,%s,%u,%p,%p,%p): stub\n", iface, debugstr_guid(container), length, pattern, count, actual_length);
-    return E_NOTIMPL;
+    TRACE("(%p,%s,%u,%p,%p,%p)\n", iface, debugstr_guid(container), length, patterns, count, actual_length);
+
+    if (!actual_length || !container) return E_INVALIDARG;
+
+    res = RegOpenKeyExW(This->classkey, containers_keyname, 0, KEY_READ, &containers_key);
+    if (res == ERROR_SUCCESS)
+    {
+        StringFromGUID2(container, guidkeyname, 39);
+
+        res = RegOpenKeyExW(containers_key, guidkeyname, 0, KEY_READ, &guid_key);
+        if (res == ERROR_FILE_NOT_FOUND) hr = WINCODEC_ERR_COMPONENTNOTFOUND;
+        else if (res != ERROR_SUCCESS) hr = HRESULT_FROM_WIN32(res);
+
+        RegCloseKey(containers_key);
+    }
+    else if (res == ERROR_FILE_NOT_FOUND) hr = WINCODEC_ERR_COMPONENTNOTFOUND;
+    else hr = HRESULT_FROM_WIN32(res);
+
+    if (SUCCEEDED(hr))
+    {
+        res = RegQueryInfoKeyW(guid_key, NULL, NULL, NULL, &pattern_count, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        if (res != ERROR_SUCCESS) hr = HRESULT_FROM_WIN32(res);
+
+        if (SUCCEEDED(hr))
+        {
+            patterns_size = pattern_count * sizeof(WICMetadataPattern);
+
+            for (i=0; i<pattern_count; i++)
+            {
+                snprintfW(subkeyname, 11, uintformatW, i);
+                res = RegOpenKeyExW(guid_key, subkeyname, 0, KEY_READ, &pattern_key);
+                if (res == ERROR_SUCCESS)
+                {
+                    res = RegGetValueW(pattern_key, NULL, patternW, RRF_RT_REG_BINARY, NULL,
+                        NULL, &patternsize);
+                    patterns_size += patternsize*2;
+
+                    if ((length >= patterns_size) && (res == ERROR_SUCCESS))
+                    {
+                        patterns[i].Length = patternsize;
+
+                        patterns[i].DataOffset.QuadPart = 0;
+                        valuesize = sizeof(ULARGE_INTEGER);
+                        RegGetValueW(pattern_key, NULL, dataoffsetW, RRF_RT_DWORD|RRF_RT_QWORD, NULL,
+                            &patterns[i].DataOffset, &valuesize);
+
+                        patterns[i].Position.QuadPart = 0;
+                        valuesize = sizeof(ULARGE_INTEGER);
+                        res = RegGetValueW(pattern_key, NULL, positionW, RRF_RT_DWORD|RRF_RT_QWORD, NULL,
+                            &patterns[i].Position, &valuesize);
+
+                        if (res == ERROR_SUCCESS)
+                        {
+                            patterns[i].Pattern = bPatterns+patterns_size-patternsize*2;
+                            valuesize = patternsize;
+                            res = RegGetValueW(pattern_key, NULL, patternW, RRF_RT_REG_BINARY, NULL,
+                                patterns[i].Pattern, &valuesize);
+                        }
+
+                        if (res == ERROR_SUCCESS)
+                        {
+                            patterns[i].Mask = bPatterns+patterns_size-patternsize;
+                            valuesize = patternsize;
+                            res = RegGetValueW(pattern_key, NULL, maskW, RRF_RT_REG_BINARY, NULL,
+                                patterns[i].Mask, &valuesize);
+                        }
+                    }
+
+                    RegCloseKey(pattern_key);
+                }
+                if (res != ERROR_SUCCESS)
+                {
+                    hr = HRESULT_FROM_WIN32(res);
+                    break;
+                }
+            }
+        }
+
+        RegCloseKey(guid_key);
+    }
+
+    if (hr == S_OK)
+    {
+        *count = pattern_count;
+        *actual_length = patterns_size;
+        if (patterns && length < patterns_size)
+            hr = WINCODEC_ERR_INSUFFICIENTBUFFER;
+    }
+
+    return hr;
 }
 
 static HRESULT WINAPI MetadataReaderInfo_MatchesPattern(IWICMetadataReaderInfo *iface,
     REFGUID container, IStream *stream, BOOL *matches)
 {
-    FIXME("(%p,%s,%p,%p): stub\n", iface, debugstr_guid(container), stream, matches);
-    return E_NOTIMPL;
+    HRESULT hr;
+    WICMetadataPattern *patterns;
+    UINT pattern_count=0, patterns_size=0;
+    ULONG datasize=0;
+    BYTE *data=NULL;
+    ULONG bytesread;
+    UINT i;
+    LARGE_INTEGER seekpos;
+    ULONG pos;
+
+    TRACE("(%p,%s,%p,%p)\n", iface, debugstr_guid(container), stream, matches);
+
+    hr = MetadataReaderInfo_GetPatterns(iface, container, 0, NULL, &pattern_count, &patterns_size);
+    if (FAILED(hr)) return hr;
+
+    patterns = HeapAlloc(GetProcessHeap(), 0, patterns_size);
+    if (!patterns) return E_OUTOFMEMORY;
+
+    hr = MetadataReaderInfo_GetPatterns(iface, container, patterns_size, patterns, &pattern_count, &patterns_size);
+    if (FAILED(hr)) goto end;
+
+    for (i=0; i<pattern_count; i++)
+    {
+        if (datasize < patterns[i].Length)
+        {
+            HeapFree(GetProcessHeap(), 0, data);
+            datasize = patterns[i].Length;
+            data = HeapAlloc(GetProcessHeap(), 0, patterns[i].Length);
+            if (!data)
+            {
+                hr = E_OUTOFMEMORY;
+                break;
+            }
+        }
+
+        seekpos.QuadPart = patterns[i].Position.QuadPart;
+        hr = IStream_Seek(stream, seekpos, STREAM_SEEK_SET, NULL);
+        if (FAILED(hr)) break;
+
+        hr = IStream_Read(stream, data, patterns[i].Length, &bytesread);
+        if (hr == S_FALSE || (hr == S_OK && bytesread != patterns[i].Length)) /* past end of stream */
+            continue;
+        if (FAILED(hr)) break;
+
+        for (pos=0; pos<patterns[i].Length; pos++)
+        {
+            if ((data[pos] & patterns[i].Mask[pos]) != patterns[i].Pattern[pos])
+                break;
+        }
+        if (pos == patterns[i].Length) /* matches pattern */
+        {
+            hr = S_OK;
+            *matches = TRUE;
+            break;
+        }
+    }
+
+    if (i == pattern_count) /* does not match any pattern */
+    {
+        hr = S_OK;
+        *matches = FALSE;
+    }
+
+end:
+    HeapFree(GetProcessHeap(), 0, patterns);
+    HeapFree(GetProcessHeap(), 0, data);
+
+    return hr;
 }
 
 static HRESULT WINAPI MetadataReaderInfo_CreateInstance(IWICMetadataReaderInfo *iface,
@@ -1696,8 +1867,7 @@ static HRESULT WINAPI MetadataReaderInfo_CreateInstance(IWICMetadataReaderInfo *
 
     TRACE("(%p,%p)\n", iface, reader);
 
-    return CoCreateInstance(&This->clsid, NULL, CLSCTX_INPROC_SERVER,
-                            &IID_IWICMetadataReader, (void **)reader);
+    return create_instance(&This->clsid, &IID_IWICMetadataReader, (void **)reader);
 }
 
 static const IWICMetadataReaderInfoVtbl MetadataReaderInfo_Vtbl = {
@@ -1740,7 +1910,7 @@ static HRESULT MetadataReaderInfo_Constructor(HKEY classkey, REFCLSID clsid, IWI
     This->classkey = classkey;
     This->clsid = *clsid;
 
-    *info = (IWICComponentInfo *)This;
+    *info = (IWICComponentInfo *)&This->IWICMetadataReaderInfo_iface;
     return S_OK;
 }
 
