@@ -99,7 +99,7 @@ static void drawStridedSlow(const struct wined3d_device *device, struct wined3d_
         const struct wined3d_stream_info *si, UINT NumVertexes, GLenum glPrimType,
         const void *idxData, UINT idxSize, UINT startIdx)
 {
-    unsigned int               textureNo    = 0;
+    unsigned int               textureNo;
     const WORD                *pIdxBufS     = NULL;
     const DWORD               *pIdxBufL     = NULL;
     UINT vx_index;
@@ -291,18 +291,13 @@ static void drawStridedSlow(const struct wined3d_device *device, struct wined3d_
 
             if (num_untracked_materials)
             {
-                DWORD diffuseColor = ((const DWORD *)ptrToCoords)[0];
+                struct wined3d_color color;
                 unsigned char i;
-                float color[4];
-
-                color[0] = D3DCOLOR_B_R(diffuseColor) / 255.0f;
-                color[1] = D3DCOLOR_B_G(diffuseColor) / 255.0f;
-                color[2] = D3DCOLOR_B_B(diffuseColor) / 255.0f;
-                color[3] = D3DCOLOR_B_A(diffuseColor) / 255.0f;
 
+                wined3d_color_from_d3dcolor(&color, *(const DWORD *)ptrToCoords);
                 for (i = 0; i < num_untracked_materials; ++i)
                 {
-                    gl_info->gl_ops.gl.p_glMaterialfv(GL_FRONT_AND_BACK, context->untracked_materials[i], color);
+                    gl_info->gl_ops.gl.p_glMaterialfv(GL_FRONT_AND_BACK, context->untracked_materials[i], &color.r);
                 }
             }
         }
@@ -611,20 +606,6 @@ void draw_primitive(struct wined3d_device *device, UINT start_idx, UINT index_co
 
     if (!index_count) return;
 
-    if (state->render_states[WINED3D_RS_COLORWRITEENABLE])
-    {
-        /* Invalidate the back buffer memory so LockRect will read it the next time */
-        for (i = 0; i < device->adapter->gl_info.limits.buffers; ++i)
-        {
-            struct wined3d_surface *target = wined3d_rendertarget_view_get_surface(device->fb.render_targets[i]);
-            if (target)
-            {
-                surface_load_location(target, target->container->resource.draw_binding);
-                surface_invalidate_location(target, ~target->container->resource.draw_binding);
-            }
-        }
-    }
-
     context = context_acquire(device, wined3d_rendertarget_view_get_surface(device->fb.render_targets[0]));
     if (!context->valid)
     {
@@ -634,6 +615,25 @@ void draw_primitive(struct wined3d_device *device, UINT start_idx, UINT index_co
     }
     gl_info = context->gl_info;
 
+    for (i = 0; i < device->adapter->gl_info.limits.buffers; ++i)
+    {
+        struct wined3d_rendertarget_view *rtv = device->fb.render_targets[i];
+        struct wined3d_surface *target = wined3d_rendertarget_view_get_surface(rtv);
+
+        if (target && rtv->format->id != WINED3DFMT_NULL)
+        {
+            if (state->render_states[WINED3D_RS_COLORWRITEENABLE])
+            {
+                surface_load_location(target, context, rtv->resource->draw_binding);
+                surface_invalidate_location(target, ~rtv->resource->draw_binding);
+            }
+            else
+            {
+                wined3d_surface_prepare(target, context, rtv->resource->draw_binding);
+            }
+        }
+    }
+
     if (device->fb.depth_stencil)
     {
         /* Note that this depends on the context_acquire() call above to set
@@ -643,9 +643,10 @@ void draw_primitive(struct wined3d_device *device, UINT start_idx, UINT index_co
          * that we never copy the stencil data.*/
         DWORD location = context->render_offscreen ? device->fb.depth_stencil->resource->draw_binding
                 : WINED3D_LOCATION_DRAWABLE;
+        struct wined3d_surface *ds = wined3d_rendertarget_view_get_surface(device->fb.depth_stencil);
+
         if (state->render_states[WINED3D_RS_ZWRITEENABLE] || state->render_states[WINED3D_RS_ZENABLE])
         {
-            struct wined3d_surface *ds = wined3d_rendertarget_view_get_surface(device->fb.depth_stencil);
             RECT current_rect, draw_rect, r;
 
             if (!context->render_offscreen && ds != device->onscreen_depth_stencil)
@@ -660,8 +661,12 @@ void draw_primitive(struct wined3d_device *device, UINT start_idx, UINT index_co
 
             IntersectRect(&r, &draw_rect, &current_rect);
             if (!EqualRect(&r, &draw_rect))
-                surface_load_ds_location(ds, context, location);
+                surface_load_location(ds, context, location);
+            else
+                wined3d_surface_prepare(ds, context, location);
         }
+        else
+            wined3d_surface_prepare(ds, context, location);
     }
 
     if (!context_apply_draw_state(context, device))
