@@ -38,7 +38,6 @@ enum wined3d_gl_extension
     WINED3D_GL_EXT_NONE,
 
     /* APPLE */
-    APPLE_CLIENT_STORAGE,
     APPLE_FENCE,
     APPLE_FLOAT_PIXELS,
     APPLE_FLUSH_BUFFER_RANGE,
@@ -52,6 +51,7 @@ enum wined3d_gl_extension
     ARB_DRAW_BUFFERS,
     ARB_DRAW_ELEMENTS_BASE_VERTEX,
     ARB_DRAW_INSTANCED,
+    ARB_ES2_COMPATIBILITY,
     ARB_FRAGMENT_PROGRAM,
     ARB_FRAGMENT_SHADER,
     ARB_FRAMEBUFFER_OBJECT,
@@ -60,6 +60,7 @@ enum wined3d_gl_extension
     ARB_HALF_FLOAT_PIXEL,
     ARB_HALF_FLOAT_VERTEX,
     ARB_INSTANCED_ARRAYS,
+    ARB_INTERNALFORMAT_QUERY,
     ARB_INTERNALFORMAT_QUERY2,
     ARB_MAP_BUFFER_ALIGNMENT,
     ARB_MAP_BUFFER_RANGE,
@@ -86,8 +87,10 @@ enum wined3d_gl_extension
     ARB_TEXTURE_MIRRORED_REPEAT,
     ARB_TEXTURE_MIRROR_CLAMP_TO_EDGE,
     ARB_TEXTURE_NON_POWER_OF_TWO,
+    ARB_TEXTURE_QUERY_LEVELS,
     ARB_TEXTURE_RECTANGLE,
     ARB_TEXTURE_RG,
+    ARB_TEXTURE_RGB10_A2UI,
     ARB_TIMER_QUERY,
     ARB_UNIFORM_BUFFER_OBJECT,
     ARB_VERTEX_ARRAY_BGRA,
@@ -116,6 +119,7 @@ enum wined3d_gl_extension
     EXT_GPU_PROGRAM_PARAMETERS,
     EXT_GPU_SHADER4,
     EXT_PACKED_DEPTH_STENCIL,
+    EXT_PACKED_FLOAT,
     EXT_POINT_PARAMETERS,
     EXT_PROVOKING_VERTEX,
     EXT_SECONDARY_COLOR,
@@ -127,6 +131,7 @@ enum wined3d_gl_extension
     EXT_TEXTURE_ENV_COMBINE,
     EXT_TEXTURE_ENV_DOT3,
     EXT_TEXTURE_FILTER_ANISOTROPIC,
+    EXT_TEXTURE_INTEGER,
     EXT_TEXTURE_LOD_BIAS,
     EXT_TEXTURE_MIRROR_CLAMP,
     EXT_TEXTURE_SNORM,
@@ -159,11 +164,14 @@ enum wined3d_gl_extension
     WGL_ARB_PIXEL_FORMAT,
     WGL_EXT_SWAP_CONTROL,
     WGL_WINE_PIXEL_FORMAT_PASSTHROUGH,
+    WGL_WINE_QUERY_RENDERER,
     /* Internally used */
     WINED3D_GL_BLEND_EQUATION,
     WINED3D_GL_NORMALIZED_TEXRECT,
     WINED3D_GL_LEGACY_CONTEXT,
     WINED3D_GL_VERSION_2_0,
+    WINED3D_GL_VERSION_3_2,
+    WINED3D_GL_VERSION_4_3,
 
     WINED3D_GL_EXT_COUNT,
 };
