@@ -16,7 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define NONAMELESSUNION
 #define COBJMACROS
 #include "config.h"
 
@@ -117,7 +116,7 @@ struct ACImpl {
 
     BOOL initted, playing;
     UINT64 written_frames, last_pos_frames;
-    UINT32 period_us, period_frames, bufsize_frames, held_frames, tmp_buffer_frames;
+    UINT32 period_us, period_frames, bufsize_frames, held_frames, tmp_buffer_frames, in_oss_frames;
     UINT32 oss_bufsize_bytes, lcl_offs_frames; /* offs into local_buffer where valid data starts */
 
     BYTE *local_buffer, *tmp_buffer;
@@ -150,9 +149,6 @@ typedef struct _OSSDevice {
 
 static struct list g_devices = LIST_INIT(g_devices);
 
-static const WCHAR drv_keyW[] = {'S','o','f','t','w','a','r','e','\\',
-    'W','i','n','e','\\','D','r','i','v','e','r','s','\\',
-    'w','i','n','e','o','s','s','.','d','r','v',0};
 static const WCHAR drv_key_devicesW[] = {'S','o','f','t','w','a','r','e','\\',
     'W','i','n','e','\\','D','r','i','v','e','r','s','\\',
     'w','i','n','e','o','s','s','.','d','r','v','\\','d','e','v','i','c','e','s',0};
@@ -1207,7 +1203,7 @@ static HRESULT WINAPI AudioClient_GetStreamLatency(IAudioClient *iface,
 
     /* pretend we process audio in Period chunks, so max latency includes
      * the period time.  Some native machines add .6666ms in shared mode. */
-    *latency = This->period_us * 10 + 6666;
+    *latency = (REFERENCE_TIME)This->period_us * 10 + 6666;
 
     LeaveCriticalSection(&This->lock);
 
@@ -1396,19 +1392,10 @@ static void silence_buffer(ACImpl *This, BYTE *buffer, UINT32 frames)
 static void oss_write_data(ACImpl *This)
 {
     ssize_t written_bytes;
-    UINT32 written_frames, in_oss_frames, write_limit, max_period;
-    size_t to_write_frames, to_write_bytes;
+    UINT32 written_frames, in_oss_frames, write_limit, max_period, write_offs_frames, new_frames;
+    SIZE_T to_write_frames, to_write_bytes, advanced;
     audio_buf_info bi;
-    BYTE *buf =
-        This->local_buffer + (This->lcl_offs_frames * This->fmt->nBlockAlign);
-
-    if(This->held_frames == 0)
-        return;
-
-    if(This->lcl_offs_frames + This->held_frames > This->bufsize_frames)
-        to_write_frames = This->bufsize_frames - This->lcl_offs_frames;
-    else
-        to_write_frames = This->held_frames;
+    BYTE *buf;
 
     if(ioctl(This->fd, SNDCTL_DSP_GETOSPACE, &bi) < 0){
         WARN("GETOSPACE failed: %d (%s)\n", errno, strerror(errno));
@@ -1422,19 +1409,56 @@ static void oss_write_data(ACImpl *This)
                 bi.bytes, This->oss_bufsize_bytes);
         This->oss_bufsize_bytes = bi.bytes;
         in_oss_frames = 0;
-    }else if(This->oss_bufsize_bytes - bi.bytes <= bi.fragsize)
-        in_oss_frames = 0;
-    else
+    }else
         in_oss_frames = (This->oss_bufsize_bytes - bi.bytes) / This->fmt->nBlockAlign;
 
+    if(in_oss_frames > This->in_oss_frames){
+        TRACE("Capping reported frames from %u to %u\n",
+                in_oss_frames, This->in_oss_frames);
+        in_oss_frames = This->in_oss_frames;
+    }
+
     write_limit = 0;
     while(write_limit + in_oss_frames < max_period * 3)
         write_limit += max_period;
     if(write_limit == 0)
         return;
 
+    /*        vvvvvv - in_oss_frames
+     * [--xxxxxxxxxx]
+     *       [xxxxxxxxxx--]
+     *        ^^^^^^^^^^ - held_frames
+     *        ^ - lcl_offs_frames
+     */
+    advanced = This->in_oss_frames - in_oss_frames;
+    if(advanced > This->held_frames)
+        advanced = This->held_frames;
+    This->lcl_offs_frames += advanced;
+    This->lcl_offs_frames %= This->bufsize_frames;
+    This->held_frames -= advanced;
+    This->in_oss_frames = in_oss_frames;
+    TRACE("advanced by %lu, lcl_offs: %u, held: %u, in_oss: %u\n",
+            advanced, This->lcl_offs_frames, This->held_frames, This->in_oss_frames);
+
+
+    if(This->held_frames == This->in_oss_frames)
+        return;
+
+    write_offs_frames = (This->lcl_offs_frames + This->in_oss_frames) % This->bufsize_frames;
+    new_frames = This->held_frames - This->in_oss_frames;
+
+    if(write_offs_frames + new_frames > This->bufsize_frames)
+        to_write_frames = This->bufsize_frames - write_offs_frames;
+    else
+        to_write_frames = new_frames;
+
     to_write_frames = min(to_write_frames, write_limit);
     to_write_bytes = to_write_frames * This->fmt->nBlockAlign;
+    TRACE("going to write %lu frames from %u (%lu of %u)\n", to_write_frames,
+            write_offs_frames, to_write_frames + write_offs_frames,
+            This->bufsize_frames);
+
+    buf = This->local_buffer + write_offs_frames * This->fmt->nBlockAlign;
 
     if(This->session->mute)
         silence_buffer(This, buf, to_write_frames);
@@ -1447,54 +1471,42 @@ static void oss_write_data(ACImpl *This)
     }
     written_frames = written_bytes / This->fmt->nBlockAlign;
 
-    This->lcl_offs_frames += written_frames;
-    This->lcl_offs_frames %= This->bufsize_frames;
-    This->held_frames -= written_frames;
+    This->in_oss_frames += written_frames;
 
     if(written_frames < to_write_frames){
         /* OSS buffer probably full */
         return;
     }
 
-    if(This->held_frames && written_frames < write_limit){
+    if(new_frames > written_frames && written_frames < write_limit){
         /* wrapped and have some data back at the start to write */
 
-        to_write_frames = min(write_limit - written_frames, This->held_frames);
+        to_write_frames = min(write_limit - written_frames, new_frames - written_frames);
         to_write_bytes = to_write_frames * This->fmt->nBlockAlign;
 
         if(This->session->mute)
             silence_buffer(This, This->local_buffer, to_write_frames);
 
+        TRACE("wrapping to write %lu frames from beginning\n", to_write_frames);
+
         written_bytes = write(This->fd, This->local_buffer, to_write_bytes);
         if(written_bytes < 0){
             WARN("write failed: %d (%s)\n", errno, strerror(errno));
             return;
         }
         written_frames = written_bytes / This->fmt->nBlockAlign;
-
-        This->lcl_offs_frames += written_frames;
-        This->lcl_offs_frames %= This->bufsize_frames;
-        This->held_frames -= written_frames;
+        This->in_oss_frames += written_frames;
     }
 }
 
 static void oss_read_data(ACImpl *This)
 {
     UINT64 pos, readable;
-    audio_buf_info bi;
     ssize_t nread;
 
-    if(ioctl(This->fd, SNDCTL_DSP_GETISPACE, &bi) < 0){
-        WARN("GETISPACE failed: %d (%s)\n", errno, strerror(errno));
-        return;
-    }
-
     pos = (This->held_frames + This->lcl_offs_frames) % This->bufsize_frames;
     readable = (This->bufsize_frames - pos) * This->fmt->nBlockAlign;
 
-    if(bi.bytes < readable)
-        readable = bi.bytes;
-
     nread = read(This->fd, This->local_buffer + pos * This->fmt->nBlockAlign,
             readable);
     if(nread < 0){
@@ -1587,6 +1599,7 @@ static HRESULT WINAPI AudioClient_Stop(IAudioClient *iface)
     }
 
     This->playing = FALSE;
+    This->in_oss_frames = 0;
 
     LeaveCriticalSection(&This->lock);
 
@@ -1624,6 +1637,7 @@ static HRESULT WINAPI AudioClient_Reset(IAudioClient *iface)
     }
     This->held_frames = 0;
     This->lcl_offs_frames = 0;
+    This->in_oss_frames = 0;
 
     LeaveCriticalSection(&This->lock);
 
@@ -2159,7 +2173,6 @@ static HRESULT WINAPI AudioClock_GetPosition(IAudioClock *iface, UINT64 *pos,
         UINT64 *qpctime)
 {
     ACImpl *This = impl_from_IAudioClock(iface);
-    int delay;
 
     TRACE("(%p)->(%p, %p)\n", This, pos, qpctime);
 
@@ -2169,18 +2182,9 @@ static HRESULT WINAPI AudioClock_GetPosition(IAudioClock *iface, UINT64 *pos,
     EnterCriticalSection(&This->lock);
 
     if(This->dataflow == eRender){
-        if(!This->playing || !This->held_frames ||
-                ioctl(This->fd, SNDCTL_DSP_GETODELAY, &delay) < 0)
-            delay = 0;
-        else
-            delay /= This->fmt->nBlockAlign;
-        if(This->held_frames + delay >= This->written_frames)
+        *pos = This->written_frames - This->held_frames;
+        if(*pos < This->last_pos_frames)
             *pos = This->last_pos_frames;
-        else{
-            *pos = This->written_frames - This->held_frames - delay;
-            if(*pos < This->last_pos_frames)
-                *pos = This->last_pos_frames;
-        }
     }else if(This->dataflow == eCapture){
         audio_buf_info bi;
         UINT32 held;
@@ -2200,6 +2204,7 @@ static HRESULT WINAPI AudioClock_GetPosition(IAudioClock *iface, UINT64 *pos,
 
     This->last_pos_frames = *pos;
 
+    TRACE("returning: %s\n", wine_dbgstr_longlong(*pos));
     if(This->share == AUDCLNT_SHAREMODE_SHARED)
         *pos *= This->fmt->nBlockAlign;
 
