@@ -452,7 +452,7 @@ void reset_clipping_window(void)
  */
 LRESULT clip_cursor_notify( HWND hwnd, HWND new_clip_hwnd )
 {
-    struct x11drv_thread_data *data = x11drv_thread_data();
+    struct x11drv_thread_data *data = x11drv_init_thread_data();
 
     if (hwnd == GetDesktopWindow())  /* change the clip window stored in the desktop process */
     {
@@ -520,6 +520,21 @@ BOOL clip_fullscreen_window( HWND hwnd, BOOL reset )
     return grab_clipping_window( &rect );
 }
 
+
+/***********************************************************************
+ *		is_old_motion_event
+ */
+static BOOL is_old_motion_event( unsigned long serial )
+{
+    struct x11drv_thread_data *thread_data = x11drv_thread_data();
+
+    if (!thread_data->warp_serial) return FALSE;
+    if ((long)(serial - thread_data->warp_serial) < 0) return TRUE;
+    thread_data->warp_serial = 0;  /* we caught up now */
+    return FALSE;
+}
+
+
 /***********************************************************************
  *		send_mouse_input
  *
@@ -587,7 +602,7 @@ static void send_mouse_input( HWND hwnd, Window window, unsigned int state, INPU
 
     /* update the wine server Z-order */
 
-    if (window != x11drv_thread_data()->grab_window &&
+    if (hwnd != x11drv_thread_data()->grab_hwnd &&
         /* ignore event if a button is pressed, since the mouse is then grabbed too */
         !(state & (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask|Button6Mask|Button7Mask)))
     {
@@ -1576,20 +1591,11 @@ void X11DRV_MotionNotify( HWND hwnd, XEvent *xev )
     input.u.mi.time        = EVENT_x11_time_to_win32_time( event->time );
     input.u.mi.dwExtraInfo = 0;
 
-    if (!hwnd)
+    if (!hwnd && is_old_motion_event( event->serial ))
     {
-        struct x11drv_thread_data *thread_data = x11drv_thread_data();
-        if (thread_data->warp_serial)
-        {
-            if ((long)(event->serial - thread_data->warp_serial) < 0)
-            {
-                TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, event->serial );
-                return;
-            }
-            thread_data->warp_serial = 0;  /* we caught up now */
-        }
+        TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, event->serial );
+        return;
     }
-
     send_mouse_input( hwnd, event->window, event->state, &input );
 }
 
@@ -1605,7 +1611,7 @@ void X11DRV_EnterNotify( HWND hwnd, XEvent *xev )
     TRACE( "hwnd %p/%lx pos %d,%d detail %d\n", hwnd, event->window, event->x, event->y, event->detail );
 
     if (event->detail == NotifyVirtual) return;
-    if (event->window == x11drv_thread_data()->grab_window) return;
+    if (hwnd == x11drv_thread_data()->grab_hwnd) return;
 
     /* simulate a mouse motion event */
     input.u.mi.dx          = event->x;
@@ -1615,6 +1621,11 @@ void X11DRV_EnterNotify( HWND hwnd, XEvent *xev )
     input.u.mi.time        = EVENT_x11_time_to_win32_time( event->time );
     input.u.mi.dwExtraInfo = 0;
 
+    if (is_old_motion_event( event->serial ))
+    {
+        TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, event->serial );
+        return;
+    }
     send_mouse_input( hwnd, event->window, event->state, &input );
 }
 
@@ -1677,14 +1688,10 @@ static void X11DRV_RawMotion( XGenericEventCookie *xev )
         break;
     }
 
-    if (broken_rawevents && thread_data->warp_serial)
+    if (broken_rawevents && is_old_motion_event( xev->serial ))
     {
-        if ((long)(xev->serial - thread_data->warp_serial) < 0)
-        {
-            TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, xev->serial );
-            return;
-        }
-        thread_data->warp_serial = 0;  /* we caught up now */
+        TRACE( "pos %d,%d old serial %lu, ignoring\n", input.u.mi.dx, input.u.mi.dy, xev->serial );
+        return;
     }
 
     TRACE( "pos %d,%d (event %f,%f)\n", input.u.mi.dx, input.u.mi.dy, dx, dy );
