@@ -565,7 +565,7 @@ BOOL CDECL X11DRV_LoadTabletInfo(HWND hwnddefault)
         WTI_CURSORS_INFO cursor;
 
         TRACE("Device %i:  [id %d|name %s|type %s|num_classes %d|use %d]\n",
-                loop, (int) devices[loop].id, devices[loop].name, device_type ? device_type : "",
+                loop, (int) devices[loop].id, devices[loop].name, debugstr_a(device_type),
                 devices[loop].num_classes, devices[loop].use );
 
         switch (devices[loop].use)
@@ -620,7 +620,7 @@ BOOL CDECL X11DRV_LoadTabletInfo(HWND hwnddefault)
             if (! is_tablet_cursor(target->name, device_type))
             {
                 WARN("Skipping device %d [name %s|type %s]; not apparently a tablet cursor type device.  If this is wrong, please report it to wine-devel@winehq.org\n",
-                     loop, devices[loop].name, device_type ? device_type : "");
+                     loop, devices[loop].name, debugstr_a(device_type));
                 break;
             }
 
