@@ -721,10 +721,8 @@ static void test_recordWAVE(HWND hwnd)
     /* A few ME machines pass all tests except set format tag pcm! */
     err = mciSendStringA("record x to 2000 wait", NULL, 0, hwnd);
     ok(err || !ok_pcm,"can record yet set wave format pcm returned %s\n", dbg_mcierr(ok_pcm));
-    if(!ndevs) todo_wine /* with sound disabled */
-    ok(ndevs>0 ? !err : err==MCIERR_WAVE_INPUTSUNSUITABLE,"mci record to 2000 returned %s\n", dbg_mcierr(err));
-    else
-    ok(ndevs>0 ? !err : err==MCIERR_WAVE_INPUTSUNSUITABLE,"mci record to 2000 returned %s\n", dbg_mcierr(err));
+    todo_wine_if (!ndevs) /* with sound disabled */
+        ok(ndevs > 0 ? !err : err == MCIERR_WAVE_INPUTSUNSUITABLE, "mci record to 2000 returned %s\n", dbg_mcierr(err));
     if(err) {
         if (err==MCIERR_WAVE_INPUTSUNSUITABLE)
              skip("Please install audio driver. Everything is skipped.\n");
@@ -1165,10 +1163,8 @@ static void test_AutoOpenWAVE(HWND hwnd)
     test_notification(hwnd, "-prior to auto-open-", 0);
 
     err = mciSendStringA("play tempfile.wav notify", buf, sizeof(buf), hwnd);
-    if(ok_saved==MCIERR_FILE_NOT_FOUND) todo_wine /* same as above */
-    ok(err==MCIERR_NOTIFY_ON_AUTO_OPEN,"mci auto-open play notify returned %s\n", dbg_mcierr(err));
-    else
-    ok(err==MCIERR_NOTIFY_ON_AUTO_OPEN,"mci auto-open play notify returned %s\n", dbg_mcierr(err));
+    todo_wine_if (ok_saved == MCIERR_FILE_NOT_FOUND) /* same as above */
+        ok(err==MCIERR_NOTIFY_ON_AUTO_OPEN,"mci auto-open play notify returned %s\n", dbg_mcierr(err));
 
     if(err) /* FIXME: don't open twice yet, it confuses Wine. */
     err = mciSendStringA("play tempfile.wav", buf, sizeof(buf), hwnd);
@@ -1295,6 +1291,106 @@ static void test_AutoOpenWAVE(HWND hwnd)
     ok(0xDEADF00D==intbuf[0] && 0xABADCAFE==intbuf[2],"DWORD buffer corruption\n");
 }
 
+static void test_playWaveTypeMpegvideo(void)
+{
+    MCIERROR err;
+    MCIDEVICEID wDeviceID;
+    MCI_PLAY_PARMS play_parm;
+    MCI_STATUS_PARMS status_parm;
+    char buf[1024];
+    memset(buf, 0, sizeof(buf));
+
+    err = mciSendStringA("open tempfile.wav type MPEGVideo alias mysound", NULL, 0, NULL);
+    ok(err==ok_saved,"mci open tempfile.wav type MPEGVideo returned %s\n", dbg_mcierr(err));
+    if(err) {
+        skip("Cannot open tempfile.wav type MPEGVideo for playing (%s), skipping\n", dbg_mcierr(err));
+        return;
+    }
+
+    wDeviceID = mciGetDeviceIDA("mysound");
+    ok(wDeviceID == 1, "mciGetDeviceIDA mysound returned %u, expected 1\n", wDeviceID);
+
+    err = mciSendCommandA(wDeviceID, MCI_PLAY, 0, (DWORD_PTR)&play_parm);
+    ok(!err,"mciCommand play returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("status mysound mode", buf, sizeof(buf), NULL);
+    ok(!err,"mci status mode returned %s\n", dbg_mcierr(err));
+    ok(!strcmp(buf,"playing"), "mci status mode: %s\n", buf);
+
+    status_parm.dwItem = MCI_STATUS_MODE;
+    err = mciSendCommandA(wDeviceID, MCI_STATUS,
+                          MCI_STATUS_ITEM,
+                          (DWORD_PTR)&status_parm);
+    ok(!err,"mciCommand status mode returned %s\n", dbg_mcierr(err));
+    ok(status_parm.dwReturn == MCI_MODE_PLAY,
+       "mciCommand status mode: %u\n", (DWORD)status_parm.dwReturn);
+
+    err = mciSendStringA("setaudio mysound volume to 1000", NULL, 0, NULL);
+    ok(!err,"mci setaudio volume to 1000 returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("status mysound mode", buf, sizeof(buf), NULL);
+    ok(!err,"mci status mode returned %s\n", dbg_mcierr(err));
+    ok(!strcmp(buf,"playing"), "mci status mode: %s\n", buf);
+
+    err = mciSendStringA("setaudio mysound volume to 1001", NULL, 0, NULL);
+    ok(err==MCIERR_OUTOFRANGE,"mci setaudio volume to 1001 returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("status mysound mode", buf, sizeof(buf), NULL);
+    ok(!err,"mci status mode returned %s\n", dbg_mcierr(err));
+    ok(!strcmp(buf,"playing"), "mci status mode: %s\n", buf);
+
+    err = mciSendStringA("close mysound", NULL, 0, NULL);
+    ok(!err,"mci close returned %s\n", dbg_mcierr(err));
+}
+
+static void test_asyncWaveTypeMpegvideo(HWND hwnd)
+{
+    MCIDEVICEID wDeviceID;
+    int err;
+    char buf[1024];
+    memset(buf, 0, sizeof(buf));
+
+    err = mciSendStringA("open tempfile.wav alias mysound notify type mpegvideo", buf, sizeof(buf), hwnd);
+    ok(err==ok_saved,"mci open tempfile.wav returned %s\n", dbg_mcierr(err));
+    if(err) {
+        skip("Cannot open tempfile.wav for playing (%s), skipping\n", dbg_mcierr(err));
+        return;
+    }
+    ok(!strcmp(buf,"1"), "mci open deviceId: %s, expected 1\n", buf);
+    wDeviceID = atoi(buf);
+    ok(wDeviceID,"mci open DeviceID: %d\n", wDeviceID);
+    test_notification(hwnd,"open alias notify",MCI_NOTIFY_SUCCESSFUL);
+
+    err = mciSendStringA("play mysound notify", NULL, 0, hwnd);
+    ok(!err,"mci play returned %s\n", dbg_mcierr(err));
+
+    Sleep(500); /* milliseconds */
+
+    err = mciSendStringA("pause mysound wait", NULL, 0, hwnd);
+    ok(!err,"mci pause wait returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("status mysound mode notify", buf, sizeof(buf), hwnd);
+    ok(!err,"mci status mode returned %s\n", dbg_mcierr(err));
+    if(!err) ok(!strcmp(buf,"paused"), "mci status mode: %s\n", buf);
+    test_notification(hwnd,"play (superseded)",MCI_NOTIFY_SUPERSEDED);
+    test_notification(hwnd,"status",MCI_NOTIFY_SUCCESSFUL);
+
+    err = mciSendStringA("seek mysound to start wait", NULL, 0, NULL);
+    ok(!err,"mci seek to start wait returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("set mysound time format milliseconds", NULL, 0, NULL);
+    ok(!err,"mci time format milliseconds returned %s\n", dbg_mcierr(err));
+
+    err = mciSendStringA("play mysound to 1500 notify", NULL, 0, hwnd);
+    ok(!err,"mci play returned %s\n", dbg_mcierr(err));
+    Sleep(200);
+    test_notification(hwnd,"play",0);
+
+    err = mciSendStringA("close mysound wait", NULL, 0, NULL);
+    ok(!err,"mci close wait returned %s\n", dbg_mcierr(err));
+    test_notification(hwnd,"play (aborted by close)",MCI_NOTIFY_ABORTED);
+}
+
 START_TEST(mci)
 {
     char curdir[MAX_PATH], tmpdir[MAX_PATH];
@@ -1314,6 +1410,8 @@ START_TEST(mci)
         test_playWAVE(hwnd);
         test_asyncWAVE(hwnd);
         test_AutoOpenWAVE(hwnd);
+        test_playWaveTypeMpegvideo();
+        test_asyncWaveTypeMpegvideo(hwnd);
     }else
         skip("No output devices available, skipping all output tests\n");
     /* Win9X hangs when exiting with something still open. */
