@@ -249,7 +249,7 @@ static void test_simple_streaming(IXAudio2 *xa)
     if(xaudio27){
         IXAPO *xapo;
 
-        hr = CoCreateInstance(&CLSID_AudioVolumeMeter, NULL,
+        hr = CoCreateInstance(&CLSID_AudioVolumeMeter27, NULL,
                 CLSCTX_INPROC_SERVER, &IID_IUnknown, (void**)&vumeter);
         ok(hr == S_OK, "CoCreateInstance(AudioVolumeMeter) failed: %08x\n", hr);
 
@@ -766,6 +766,13 @@ static void test_looping(IXAudio2 *xa)
     ok(played - running_total == 22050 + 44100 * 2, "Got wrong samples played: %u\n", played - running_total);
     ok(nloopends == (played - running_total) / 88200 + 1, "Got wrong OnLoopEnd calls: %u\n", nloopends);
     running_total = played;
+
+    if(xaudio27)
+        IXAudio27SourceVoice_DestroyVoice((IXAudio27SourceVoice*)src);
+    else
+        IXAudio2SourceVoice_DestroyVoice(src);
+    IXAudio2MasteringVoice_DestroyVoice(master);
+    HeapFree(GetProcessHeap(), 0, (void*)buf.pAudioData);
 }
 
 static UINT32 test_DeviceDetails(IXAudio27 *xa)
@@ -827,7 +834,7 @@ static void test_xapo_creation_legacy(const char *module, unsigned int version)
         &CLSID_AudioVolumeMeter24,
         &CLSID_AudioVolumeMeter25,
         &CLSID_AudioVolumeMeter26,
-        &CLSID_AudioVolumeMeter
+        &CLSID_AudioVolumeMeter27
     };
 
     static const GUID *ar_clsids[] = {
@@ -838,7 +845,7 @@ static void test_xapo_creation_legacy(const char *module, unsigned int version)
         &CLSID_AudioReverb24,
         &CLSID_AudioReverb25,
         &CLSID_AudioReverb26,
-        &CLSID_AudioReverb
+        &CLSID_AudioReverb27
     };
 
     xapofxdll = LoadLibraryA(module);
@@ -857,9 +864,7 @@ static void test_xapo_creation_legacy(const char *module, unsigned int version)
     if(pCreateFX){
         for(i = 0; i < sizeof(const_clsids) / sizeof(*const_clsids); ++i){
             hr = pCreateFX(const_clsids[i].clsid, &fx_unk);
-            if(const_clsids[i].todo)
-                todo_wine ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
-            else
+            todo_wine_if(const_clsids[i].todo)
                 ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
             if(SUCCEEDED(hr)){
                 IXAPO *xapo;
@@ -945,9 +950,7 @@ static void test_xapo_creation_modern(const char *module)
     if(pCreateFX){
         for(i = 0; i < sizeof(const_clsids) / sizeof(*const_clsids); ++i){
             hr = pCreateFX(const_clsids[i].clsid, &fx_unk, NULL, 0);
-            if(const_clsids[i].todo)
-                todo_wine ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
-            else
+            todo_wine_if(const_clsids[i].todo)
                 ok(hr == S_OK, "%s: CreateFX(%s) failed: %08x\n", module, wine_dbgstr_guid(const_clsids[i].clsid), hr);
             if(SUCCEEDED(hr)){
                 IXAPO *xapo;
@@ -966,7 +969,7 @@ static void test_xapo_creation_modern(const char *module)
         }
 
         /* test legacy CLSID */
-        hr = pCreateFX(&CLSID_AudioVolumeMeter, &fx_unk, NULL, 0);
+        hr = pCreateFX(&CLSID_AudioVolumeMeter27, &fx_unk, NULL, 0);
         ok(hr == S_OK, "%s: CreateFX(CLSID_AudioVolumeMeter) failed: %08x\n", module, hr);
         if(SUCCEEDED(hr)){
             IXAPO *xapo;
@@ -1053,7 +1056,7 @@ START_TEST(xaudio2)
     test_xapo_creation();
 
     /* XAudio 2.7 (Jun 2010 DirectX) */
-    hr = CoCreateInstance(&CLSID_XAudio2, NULL, CLSCTX_INPROC_SERVER,
+    hr = CoCreateInstance(&CLSID_XAudio27, NULL, CLSCTX_INPROC_SERVER,
             &IID_IXAudio27, (void**)&xa27);
     if(hr == S_OK){
         xaudio27 = TRUE;
@@ -1080,6 +1083,9 @@ START_TEST(xaudio2)
         hr = pXAudio2Create(&xa, 0, XAUDIO2_DEFAULT_PROCESSOR);
         ok(hr == S_OK, "XAudio2Create failed: %08x\n", hr);
 
+        hr = IXAudio2_QueryInterface(xa, &IID_IXAudio27, (void**)&xa27);
+        ok(hr == E_NOINTERFACE, "XA28 object should support IXAudio27, gave: %08x\n", hr);
+
         has_devices = check_has_devices(xa);
         if(has_devices){
             test_simple_streaming(xa);
