@@ -23,6 +23,7 @@
 #define NONAMELESSUNION
 #define COBJMACROS
 
+#include "initguid.h"
 #include "xaudio_private.h"
 #include "xapofx.h"
 
@@ -30,13 +31,28 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(xaudio2);
 
+#ifdef XAPOFX1_VER
+BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD reason, void *pReserved)
+{
+    TRACE("(%p, %d, %p)\n", hinstDLL, reason, pReserved);
+
+    switch (reason)
+    {
+    case DLL_WINE_PREATTACH:
+        return FALSE;  /* prefer native version */
+    case DLL_PROCESS_ATTACH:
+        DisableThreadLibraryCalls( hinstDLL );
+        break;
+    }
+    return TRUE;
+}
+#endif /* XAPOFX1_VER */
+
 typedef struct _VUMeterImpl {
     IXAPO IXAPO_iface;
     IXAPOParameters IXAPOParameters_iface;
 
     LONG ref;
-
-    DWORD version;
 } VUMeterImpl;
 
 static VUMeterImpl *VUMeterImpl_from_IXAPO(IXAPO *iface)
@@ -240,8 +256,6 @@ typedef struct _ReverbImpl {
     IXAPOParameters IXAPOParameters_iface;
 
     LONG ref;
-
-    DWORD version;
 } ReverbImpl;
 
 static ReverbImpl *ReverbImpl_from_IXAPO(IXAPO *iface)
@@ -443,8 +457,6 @@ typedef struct _EQImpl {
     IXAPOParameters IXAPOParameters_iface;
 
     LONG ref;
-
-    DWORD version;
 } EQImpl;
 
 static EQImpl *EQImpl_from_IXAPO(IXAPO *iface)
@@ -644,11 +656,10 @@ static const IXAPOParametersVtbl EQXAPOParameters_Vtbl = {
 struct xapo_cf {
     IClassFactory IClassFactory_iface;
     LONG ref;
-    DWORD version;
     const CLSID *class;
 };
 
-struct xapo_cf *xapo_impl_from_IClassFactory(IClassFactory *iface)
+static struct xapo_cf *xapo_impl_from_IClassFactory(IClassFactory *iface)
 {
     return CONTAINING_RECORD(iface, struct xapo_cf, IClassFactory_iface);
 }
@@ -699,7 +710,7 @@ static HRESULT WINAPI xapocf_CreateInstance(IClassFactory *iface, IUnknown *pOut
     if(pOuter)
         return CLASS_E_NOAGGREGATION;
 
-    if(IsEqualGUID(This->class, &CLSID_AudioVolumeMeter)){
+    if(IsEqualGUID(This->class, &CLSID_AudioVolumeMeter27)){
         VUMeterImpl *object;
 
         object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
@@ -708,14 +719,13 @@ static HRESULT WINAPI xapocf_CreateInstance(IClassFactory *iface, IUnknown *pOut
 
         object->IXAPO_iface.lpVtbl = &VUMXAPO_Vtbl;
         object->IXAPOParameters_iface.lpVtbl = &VUMXAPOParameters_Vtbl;
-        object->version = This->version;
 
         hr = IXAPO_QueryInterface(&object->IXAPO_iface, riid, ppobj);
         if(FAILED(hr)){
             HeapFree(GetProcessHeap(), 0, object);
             return hr;
         }
-    }else if(IsEqualGUID(This->class, &CLSID_AudioReverb)){
+    }else if(IsEqualGUID(This->class, &CLSID_FXReverb)){
         ReverbImpl *object;
 
         object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
@@ -724,7 +734,6 @@ static HRESULT WINAPI xapocf_CreateInstance(IClassFactory *iface, IUnknown *pOut
 
         object->IXAPO_iface.lpVtbl = &RVBXAPO_Vtbl;
         object->IXAPOParameters_iface.lpVtbl = &RVBXAPOParameters_Vtbl;
-        object->version = This->version;
 
         hr = IXAPO_QueryInterface(&object->IXAPO_iface, riid, ppobj);
         if(FAILED(hr)){
@@ -740,7 +749,6 @@ static HRESULT WINAPI xapocf_CreateInstance(IClassFactory *iface, IUnknown *pOut
 
         object->IXAPO_iface.lpVtbl = &EQXAPO_Vtbl;
         object->IXAPOParameters_iface.lpVtbl = &EQXAPOParameters_Vtbl;
-        object->version = This->version;
 
         hr = IXAPO_QueryInterface(&object->IXAPO_iface, riid, ppobj);
         if(FAILED(hr)){
@@ -748,6 +756,7 @@ static HRESULT WINAPI xapocf_CreateInstance(IClassFactory *iface, IUnknown *pOut
             return hr;
         }
     }else
+        /* TODO FXECHO, FXMasteringLimiter, */
         return E_INVALIDARG;
 
     return S_OK;
@@ -768,12 +777,135 @@ static const IClassFactoryVtbl xapo_Vtbl =
     xapocf_LockServer
 };
 
-IClassFactory *make_xapo_factory(REFCLSID clsid, DWORD version)
+IClassFactory *make_xapo_factory(REFCLSID clsid)
 {
     struct xapo_cf *ret = HeapAlloc(GetProcessHeap(), 0, sizeof(struct xapo_cf));
     ret->IClassFactory_iface.lpVtbl = &xapo_Vtbl;
-    ret->version = version;
     ret->class = clsid;
     ret->ref = 0;
     return &ret->IClassFactory_iface;
 }
+
+#if XAUDIO2_VER >= 8
+HRESULT WINAPI CreateAudioVolumeMeter(IUnknown **out)
+{
+    IClassFactory *cf;
+    HRESULT hr;
+
+    cf = make_xapo_factory(&CLSID_AudioVolumeMeter27);
+
+    hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (void**)out);
+
+    IClassFactory_Release(cf);
+
+    return hr;
+}
+
+HRESULT WINAPI CreateAudioReverb(IUnknown **out)
+{
+    IClassFactory *cf;
+    HRESULT hr;
+
+    cf = make_xapo_factory(&CLSID_FXReverb);
+
+    hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (void**)out);
+
+    IClassFactory_Release(cf);
+
+    return hr;
+}
+
+HRESULT CDECL CreateFX(REFCLSID clsid, IUnknown **out, void *initdata, UINT32 initdata_bytes)
+{
+    HRESULT hr;
+    IUnknown *obj;
+    const GUID *class = NULL;
+    IClassFactory *cf;
+
+    *out = NULL;
+
+    if(IsEqualGUID(clsid, &CLSID_FXReverb27) ||
+            IsEqualGUID(clsid, &CLSID_FXReverb))
+        class = &CLSID_FXReverb;
+    else if(IsEqualGUID(clsid, &CLSID_FXEQ27) ||
+            IsEqualGUID(clsid, &CLSID_FXEQ))
+        class = &CLSID_FXEQ;
+
+    if(class){
+        cf = make_xapo_factory(class);
+
+        hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (void**)&obj);
+        IClassFactory_Release(cf);
+        if(FAILED(hr))
+            return hr;
+    }else{
+        hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, &IID_IUnknown, (void**)&obj);
+        if(FAILED(hr)){
+            WARN("CoCreateInstance failed: %08x\n", hr);
+            return hr;
+        }
+    }
+
+    if(initdata && initdata_bytes > 0){
+        IXAPO *xapo;
+
+        hr = IUnknown_QueryInterface(obj, &IID_IXAPO, (void**)&xapo);
+        if(SUCCEEDED(hr)){
+            hr = IXAPO_Initialize(xapo, initdata, initdata_bytes);
+
+            IXAPO_Release(xapo);
+
+            if(FAILED(hr)){
+                WARN("Initialize failed: %08x\n", hr);
+                IUnknown_Release(obj);
+                return hr;
+            }
+        }
+    }
+
+    *out = obj;
+
+    return S_OK;
+}
+#endif /* XAUDIO2_VER >= 8 */
+
+#ifdef XAPOFX1_VER
+HRESULT CDECL CreateFX(REFCLSID clsid, IUnknown **out)
+{
+    HRESULT hr;
+    IUnknown *obj;
+    const GUID *class = NULL;
+    IClassFactory *cf;
+
+    TRACE("%s %p\n", debugstr_guid(clsid), out);
+
+    *out = NULL;
+
+    if(IsEqualGUID(clsid, &CLSID_FXReverb27) ||
+            IsEqualGUID(clsid, &CLSID_FXReverb))
+        class = &CLSID_FXReverb;
+    else if(IsEqualGUID(clsid, &CLSID_FXEQ27) ||
+            IsEqualGUID(clsid, &CLSID_FXEQ))
+        class = &CLSID_FXEQ;
+    /* TODO FXECHO, FXMasteringLimiter, */
+
+    if(class){
+        cf = make_xapo_factory(class);
+
+        hr = IClassFactory_CreateInstance(cf, NULL, &IID_IUnknown, (void**)&obj);
+        IClassFactory_Release(cf);
+        if(FAILED(hr))
+            return hr;
+    }else{
+        hr = CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, &IID_IUnknown, (void**)&obj);
+        if(FAILED(hr)){
+            WARN("CoCreateInstance failed: %08x\n", hr);
+            return hr;
+        }
+    }
+
+    *out = obj;
+
+    return S_OK;
+}
+#endif /* XAPOFX1_VER */
