@@ -22,8 +22,6 @@
 #define NONAMELESSUNION
 #define COBJMACROS
 
-#include "initguid.h"
-
 #include "xaudio_private.h"
 
 #include "ole2.h"
@@ -34,14 +32,22 @@
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(xaudio2);
+WINE_DECLARE_DEBUG_CHANNEL(winediag);
 
 static ALCdevice *(ALC_APIENTRY *palcLoopbackOpenDeviceSOFT)(const ALCchar*);
 static void (ALC_APIENTRY *palcRenderSamplesSOFT)(ALCdevice*, ALCvoid*, ALCsizei);
 
 static HINSTANCE instance;
 
-#define COMPAT_E_INVALID_CALL(v) (v == 20) ? E_INVALIDARG : XAUDIO2_E_INVALID_CALL
-#define COMPAT_E_DEVICE_INVALIDATED(v) (v == 20) ? XAUDIO20_E_DEVICE_INVALIDATED : XAUDIO2_E_DEVICE_INVALIDATED
+#define IN_AL_PERIODS 4
+
+#if XAUDIO2_VER == 0
+#define COMPAT_E_INVALID_CALL E_INVALIDARG
+#define COMPAT_E_DEVICE_INVALIDATED XAUDIO20_E_DEVICE_INVALIDATED
+#else
+#define COMPAT_E_INVALID_CALL XAUDIO2_E_INVALID_CALL
+#define COMPAT_E_DEVICE_INVALIDATED XAUDIO2_E_DEVICE_INVALIDATED
+#endif
 
 static void dump_fmt(const WAVEFORMATEX *fmt)
 {
@@ -449,6 +455,7 @@ static ALenum get_al_format(const WAVEFORMATEX *fmt)
             case 8:
                 return AL_FORMAT_71CHN8;
             }
+            break;
         case 16:
             switch(fmt->nChannels){
             case 1:
@@ -464,6 +471,7 @@ static ALenum get_al_format(const WAVEFORMATEX *fmt)
             case 8:
                 return AL_FORMAT_71CHN16;
             }
+            break;
         }
     }else if(fmt->wFormatTag == WAVE_FORMAT_IEEE_FLOAT ||
             (fmt->wFormatTag == WAVE_FORMAT_EXTENSIBLE &&
@@ -506,7 +514,7 @@ static HRESULT WINAPI XA2SRC_SubmitSourceBuffer(IXAudio2SourceVoice *iface,
     if(This->nbufs >= XAUDIO2_MAX_QUEUED_BUFFERS){
         TRACE("Too many buffers queued!\n");
         LeaveCriticalSection(&This->lock);
-        return COMPAT_E_INVALID_CALL(This->xa2->version);
+        return COMPAT_E_INVALID_CALL;
     }
 
     buf_idx = (This->first_buf + This->nbufs) % XAUDIO2_MAX_QUEUED_BUFFERS;
@@ -517,10 +525,10 @@ static HRESULT WINAPI XA2SRC_SubmitSourceBuffer(IXAudio2SourceVoice *iface,
      * but pBuffer itself may be reused immediately */
     memcpy(&buf->xa2buffer, pBuffer, sizeof(*pBuffer));
 
-    if(This->xa2->version == 20){
-        if(buf->xa2buffer.LoopCount == XAUDIO20_LOOP_INFINITE)
-            buf->xa2buffer.LoopCount = XAUDIO2_LOOP_INFINITE;
-    }
+#if XAUDIO2_VER == 0
+    if(buf->xa2buffer.LoopCount == XAUDIO20_LOOP_INFINITE)
+        buf->xa2buffer.LoopCount = XAUDIO2_LOOP_INFINITE;
+#endif
 
     /* convert samples offsets to bytes */
     if(This->fmt->wFormatTag == WAVE_FORMAT_ADPCM){
@@ -551,29 +559,29 @@ static HRESULT WINAPI XA2SRC_SubmitSourceBuffer(IXAudio2SourceVoice *iface,
         if(buf->xa2buffer.LoopBegin >= buf->play_end_bytes){
             /* this actually crashes on native xaudio 2.7 */
             LeaveCriticalSection(&This->lock);
-            return COMPAT_E_INVALID_CALL(This->xa2->version);
+            return COMPAT_E_INVALID_CALL;
         }
 
         buf->loop_end_bytes = buf->xa2buffer.LoopBegin + buf->xa2buffer.LoopLength;
 
         /* xaudio 2.7 allows some invalid looping setups, but later versions
          * return an error */
-        if(This->xa2->version > 27){
-            if(buf->loop_end_bytes > buf->play_end_bytes){
-                LeaveCriticalSection(&This->lock);
-                return COMPAT_E_INVALID_CALL(This->xa2->version);
-            }
-
-            if(buf->loop_end_bytes <= buf->xa2buffer.PlayBegin){
-                LeaveCriticalSection(&This->lock);
-                return COMPAT_E_INVALID_CALL(This->xa2->version);
-            }
-        }else{
-            if(buf->loop_end_bytes <= buf->xa2buffer.PlayBegin){
-                buf->xa2buffer.LoopCount = 0;
-                buf->loop_end_bytes = buf->play_end_bytes;
-            }
+#if XAUDIO2_VER > 7
+        if(buf->loop_end_bytes > buf->play_end_bytes){
+            LeaveCriticalSection(&This->lock);
+            return COMPAT_E_INVALID_CALL;
         }
+
+        if(buf->loop_end_bytes <= buf->xa2buffer.PlayBegin){
+            LeaveCriticalSection(&This->lock);
+            return COMPAT_E_INVALID_CALL;
+        }
+#else
+        if(buf->loop_end_bytes <= buf->xa2buffer.PlayBegin){
+            buf->xa2buffer.LoopCount = 0;
+            buf->loop_end_bytes = buf->play_end_bytes;
+        }
+#endif
     }else{
         buf->xa2buffer.LoopLength = buf->xa2buffer.PlayLength;
         buf->xa2buffer.LoopBegin = buf->xa2buffer.PlayBegin;
@@ -743,7 +751,7 @@ static HRESULT WINAPI XA2SRC_SetSourceSampleRate(
 
     if(This->nbufs){
         LeaveCriticalSection(&This->lock);
-        return COMPAT_E_INVALID_CALL(This->xa2->version);
+        return COMPAT_E_INVALID_CALL;
     }
 
     This->fmt->nSamplesPerSec = NewSourceSampleRate;
@@ -1187,16 +1195,20 @@ static HRESULT WINAPI IXAudio2Impl_QueryInterface(IXAudio2 *iface, REFIID riid,
     TRACE("(%p)->(%s, %p)\n", This, debugstr_guid(riid), ppvObject);
 
     if(IsEqualGUID(riid, &IID_IUnknown) ||
+            IsEqualGUID(riid, &IID_IXAudio28) ||
             IsEqualGUID(riid, &IID_IXAudio2))
         *ppvObject = &This->IXAudio2_iface;
     else if(IsEqualGUID(riid, &IID_IXAudio27)){
         /* all xaudio versions before 28 share an IID */
-        if(This->version == 20)
-            *ppvObject = &This->IXAudio20_iface;
-        else if(This->version == 21 || This->version == 22)
-            *ppvObject = &This->IXAudio22_iface;
-        else
-            *ppvObject = &This->IXAudio27_iface;
+#if XAUDIO2_VER == 0
+        *ppvObject = &This->IXAudio20_iface;
+#elif XAUDIO2_VER <= 2
+        *ppvObject = &This->IXAudio22_iface;
+#elif XAUDIO2_VER <= 7
+        *ppvObject = &This->IXAudio27_iface;
+#else
+        *ppvObject = NULL;
+#endif
     }else
         *ppvObject = NULL;
 
@@ -1240,12 +1252,14 @@ static ULONG WINAPI IXAudio2Impl_Release(IXAudio2 *iface)
         LIST_FOR_EACH_ENTRY_SAFE(src, src2, &This->source_voices, XA2SourceImpl, entry){
             HeapFree(GetProcessHeap(), 0, src->sends);
             IXAudio2SourceVoice_DestroyVoice(&src->IXAudio2SourceVoice_iface);
+            src->lock.DebugInfo->Spare[0] = 0;
             DeleteCriticalSection(&src->lock);
             HeapFree(GetProcessHeap(), 0, src);
         }
 
         LIST_FOR_EACH_ENTRY_SAFE(sub, sub2, &This->submix_voices, XA2SubmixImpl, entry){
             IXAudio2SubmixVoice_DestroyVoice(&sub->IXAudio2SubmixVoice_iface);
+            sub->lock.DebugInfo->Spare[0] = 0;
             DeleteCriticalSection(&sub->lock);
             HeapFree(GetProcessHeap(), 0, sub);
         }
@@ -1261,6 +1275,7 @@ static ULONG WINAPI IXAudio2Impl_Release(IXAudio2 *iface)
 
         CloseHandle(This->mmevt);
 
+        This->lock.DebugInfo->Spare[0] = 0;
         DeleteCriticalSection(&This->lock);
 
         HeapFree(GetProcessHeap(), 0, This);
@@ -1355,8 +1370,10 @@ static HRESULT WINAPI IXAudio2Impl_CreateSourceVoice(IXAudio2 *iface,
     EnterCriticalSection(&This->lock);
 
     LIST_FOR_EACH_ENTRY(src, &This->source_voices, XA2SourceImpl, entry){
+        EnterCriticalSection(&src->lock);
         if(!src->in_use)
             break;
+        LeaveCriticalSection(&src->lock);
     }
 
     if(&src->entry == &This->source_voices){
@@ -1368,15 +1385,22 @@ static HRESULT WINAPI IXAudio2Impl_CreateSourceVoice(IXAudio2 *iface,
 
         list_add_head(&This->source_voices, &src->entry);
 
-        src->IXAudio20SourceVoice_iface.lpVtbl = &XAudio20SourceVoice_Vtbl;
-        src->IXAudio23SourceVoice_iface.lpVtbl = &XAudio23SourceVoice_Vtbl;
-        src->IXAudio27SourceVoice_iface.lpVtbl = &XAudio27SourceVoice_Vtbl;
         src->IXAudio2SourceVoice_iface.lpVtbl = &XAudio2SourceVoice_Vtbl;
 
+#if XAUDIO2_VER == 0
+        src->IXAudio20SourceVoice_iface.lpVtbl = &XAudio20SourceVoice_Vtbl;
+#elif XAUDIO2_VER <= 3
+        src->IXAudio23SourceVoice_iface.lpVtbl = &XAudio23SourceVoice_Vtbl;
+#elif XAUDIO2_VER <= 7
+        src->IXAudio27SourceVoice_iface.lpVtbl = &XAudio27SourceVoice_Vtbl;
+#endif
+
         InitializeCriticalSection(&src->lock);
         src->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": XA2SourceImpl.lock");
 
         src->xa2 = This;
+
+        EnterCriticalSection(&src->lock);
     }
 
     src->in_use = TRUE;
@@ -1389,6 +1413,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateSourceVoice(IXAudio2 *iface,
     src->al_fmt = get_al_format(pSourceFormat);
     if(!src->al_fmt){
         src->in_use = FALSE;
+        LeaveCriticalSection(&src->lock);
         WARN("OpenAL can't convert this format!\n");
         return AUDCLNT_E_UNSUPPORTED_FORMAT;
     }
@@ -1399,23 +1424,38 @@ static HRESULT WINAPI IXAudio2Impl_CreateSourceVoice(IXAudio2 *iface,
 
     hr = XA2SRC_SetOutputVoices(&src->IXAudio2SourceVoice_iface, pSendList);
     if(FAILED(hr)){
+        HeapFree(GetProcessHeap(), 0, src->fmt);
         src->in_use = FALSE;
+        LeaveCriticalSection(&src->lock);
         return hr;
     }
 
     alGenSources(1, &src->al_src);
+    if(!src->al_src){
+        static int once = 0;
+        if(!once++)
+            ERR_(winediag)("OpenAL ran out of sources, consider increasing its source limit.\n");
+        HeapFree(GetProcessHeap(), 0, src->fmt);
+        src->in_use = FALSE;
+        LeaveCriticalSection(&src->lock);
+        return E_OUTOFMEMORY;
+    }
+
     alGenBuffers(XAUDIO2_MAX_QUEUED_BUFFERS, src->al_bufs);
 
     alSourcePlay(src->al_src);
 
-    if(This->version == 20)
-        *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio20SourceVoice_iface;
-    else if(This->version <= 23)
-        *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio23SourceVoice_iface;
-    else if(This->version <= 27)
-        *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio27SourceVoice_iface;
-    else
-        *ppSourceVoice = &src->IXAudio2SourceVoice_iface;
+    LeaveCriticalSection(&src->lock);
+
+#if XAUDIO2_VER == 0
+    *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio20SourceVoice_iface;
+#elif XAUDIO2_VER <= 3
+    *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio23SourceVoice_iface;
+#elif XAUDIO2_VER <= 7
+    *ppSourceVoice = (IXAudio2SourceVoice*)&src->IXAudio27SourceVoice_iface;
+#else
+    *ppSourceVoice = &src->IXAudio2SourceVoice_iface;
+#endif
 
     TRACE("Created source voice: %p\n", src);
 
@@ -1438,8 +1478,10 @@ static HRESULT WINAPI IXAudio2Impl_CreateSubmixVoice(IXAudio2 *iface,
     EnterCriticalSection(&This->lock);
 
     LIST_FOR_EACH_ENTRY(sub, &This->submix_voices, XA2SubmixImpl, entry){
+        EnterCriticalSection(&sub->lock);
         if(!sub->in_use)
             break;
+        LeaveCriticalSection(&sub->lock);
     }
 
     if(&sub->entry == &This->submix_voices){
@@ -1451,24 +1493,32 @@ static HRESULT WINAPI IXAudio2Impl_CreateSubmixVoice(IXAudio2 *iface,
 
         list_add_head(&This->submix_voices, &sub->entry);
 
-        sub->IXAudio20SubmixVoice_iface.lpVtbl = &XAudio20SubmixVoice_Vtbl;
-        sub->IXAudio23SubmixVoice_iface.lpVtbl = &XAudio23SubmixVoice_Vtbl;
         sub->IXAudio2SubmixVoice_iface.lpVtbl = &XAudio2SubmixVoice_Vtbl;
 
+#if XAUDIO2_VER == 0
+        sub->IXAudio20SubmixVoice_iface.lpVtbl = &XAudio20SubmixVoice_Vtbl;
+#elif XAUDIO2_VER <= 3
+        sub->IXAudio23SubmixVoice_iface.lpVtbl = &XAudio23SubmixVoice_Vtbl;
+#endif
+
         InitializeCriticalSection(&sub->lock);
         sub->lock.DebugInfo->Spare[0] = (DWORD_PTR)(__FILE__ ": XA2SubmixImpl.lock");
+
+        EnterCriticalSection(&sub->lock);
     }
 
     sub->in_use = TRUE;
 
     LeaveCriticalSection(&This->lock);
+    LeaveCriticalSection(&sub->lock);
 
-    if(This->version == 20)
-        *ppSubmixVoice = (IXAudio2SubmixVoice*)&sub->IXAudio20SubmixVoice_iface;
-    else if(This->version <= 23)
-        *ppSubmixVoice = (IXAudio2SubmixVoice*)&sub->IXAudio23SubmixVoice_iface;
-    else
-        *ppSubmixVoice = &sub->IXAudio2SubmixVoice_iface;
+#if XAUDIO2_VER == 0
+    *ppSubmixVoice = (IXAudio2SubmixVoice*)&sub->IXAudio20SubmixVoice_iface;
+#elif XAUDIO2_VER <= 3
+    *ppSubmixVoice = (IXAudio2SubmixVoice*)&sub->IXAudio23SubmixVoice_iface;
+#else
+    *ppSubmixVoice = &sub->IXAudio2SubmixVoice_iface;
+#endif
 
     TRACE("Created submix voice: %p\n", sub);
 
@@ -1525,13 +1575,13 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     /* there can only be one Mastering Voice, so just build it into XA2 */
     if(This->aclient){
         LeaveCriticalSection(&This->lock);
-        return COMPAT_E_INVALID_CALL(This->version);
+        return COMPAT_E_INVALID_CALL;
     }
 
     if(!deviceId){
         if(This->ndevs == 0){
             LeaveCriticalSection(&This->lock);
-            return ERROR_NOT_FOUND;
+            return E_NOTFOUND;
         }
         deviceId = This->devids[0];
     }
@@ -1539,7 +1589,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     hr = IMMDeviceEnumerator_GetDevice(This->devenum, deviceId, &dev);
     if(FAILED(hr)){
         WARN("GetDevice failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1548,7 +1598,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     if(FAILED(hr)){
         WARN("Activate(IAudioClient) failed: %08x\n", hr);
         IMMDevice_Release(dev);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1557,13 +1607,13 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     hr = IAudioClient_GetMixFormat(This->aclient, &fmt);
     if(FAILED(hr)){
         WARN("GetMixFormat failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
     if(sizeof(WAVEFORMATEX) + fmt->cbSize > sizeof(WAVEFORMATEXTENSIBLE)){
         FIXME("Mix format doesn't fit into WAVEFORMATEXTENSIBLE!\n");
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1587,7 +1637,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     if(hr == S_FALSE){
         if(sizeof(WAVEFORMATEX) + fmt->cbSize > sizeof(WAVEFORMATEXTENSIBLE)){
             FIXME("Mix format doesn't fit into WAVEFORMATEXTENSIBLE!\n");
-            hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+            hr = COMPAT_E_DEVICE_INVALIDATED;
             goto exit;
         }
         memcpy(&This->fmt, fmt, sizeof(WAVEFORMATEX) + fmt->cbSize);
@@ -1598,7 +1648,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     hr = IAudioClient_GetDevicePeriod(This->aclient, &period, NULL);
     if(FAILED(hr)){
         WARN("GetDevicePeriod failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1610,7 +1660,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
             0, &This->fmt.Format, NULL);
     if(FAILED(hr)){
         WARN("Initialize failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1619,7 +1669,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     hr = IAudioClient_SetEventHandle(This->aclient, This->mmevt);
     if(FAILED(hr)){
         WARN("Initialize failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1627,7 +1677,7 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
             (void**)&This->render);
     if(FAILED(hr)){
         WARN("GetService(IAudioRenderClient) failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1665,27 +1715,27 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
 
     if(!attrs[5]){
         WARN("OpenAL can't output samples in this format\n");
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
     This->al_device = palcLoopbackOpenDeviceSOFT(NULL);
     if(!This->al_device){
         WARN("alcLoopbackOpenDeviceSOFT failed\n");
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
     This->al_ctx = alcCreateContext(This->al_device, attrs);
     if(!This->al_ctx){
         WARN("alcCreateContext failed\n");
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
     if(alcMakeContextCurrent(This->al_ctx) == ALC_FALSE){
         WARN("alcMakeContextCurrent failed\n");
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
@@ -1693,16 +1743,17 @@ static HRESULT WINAPI IXAudio2Impl_CreateMasteringVoice(IXAudio2 *iface,
     if (FAILED(hr))
     {
         WARN("Start(IAudioClient) failed: %08x\n", hr);
-        hr = COMPAT_E_DEVICE_INVALIDATED(This->version);
+        hr = COMPAT_E_DEVICE_INVALIDATED;
         goto exit;
     }
 
-    if(This->version <= 20)
-        *ppMasteringVoice = (IXAudio2MasteringVoice*)&This->IXAudio20MasteringVoice_iface;
-    else if(This->version <= 23)
-        *ppMasteringVoice = (IXAudio2MasteringVoice*)&This->IXAudio23MasteringVoice_iface;
-    else
-        *ppMasteringVoice = &This->IXAudio2MasteringVoice_iface;
+#if XAUDIO2_VER == 0
+    *ppMasteringVoice = (IXAudio2MasteringVoice*)&This->IXAudio20MasteringVoice_iface;
+#elif XAUDIO2_VER <= 3
+    *ppMasteringVoice = (IXAudio2MasteringVoice*)&This->IXAudio23MasteringVoice_iface;
+#else
+    *ppMasteringVoice = &This->IXAudio2MasteringVoice_iface;
+#endif
 
 exit:
     if(FAILED(hr)){
@@ -1804,7 +1855,6 @@ static const IXAudio2Vtbl XAudio2_Vtbl =
 struct xaudio2_cf {
     IClassFactory IClassFactory_iface;
     LONG ref;
-    DWORD version;
 };
 
 static struct xaudio2_cf *impl_from_IClassFactory(IClassFactory *iface)
@@ -1936,18 +1986,22 @@ static HRESULT WINAPI XAudio2CF_CreateInstance(IClassFactory *iface, IUnknown *p
     if(!object)
         return E_OUTOFMEMORY;
 
-    object->IXAudio20_iface.lpVtbl = &XAudio20_Vtbl;
-    object->IXAudio22_iface.lpVtbl = &XAudio22_Vtbl;
-    object->IXAudio27_iface.lpVtbl = &XAudio27_Vtbl;
     object->IXAudio2_iface.lpVtbl = &XAudio2_Vtbl;
-    object->IXAudio20MasteringVoice_iface.lpVtbl = &XAudio20MasteringVoice_Vtbl;
-    object->IXAudio23MasteringVoice_iface.lpVtbl = &XAudio23MasteringVoice_Vtbl;
     object->IXAudio2MasteringVoice_iface.lpVtbl = &XAudio2MasteringVoice_Vtbl;
 
-    if(IsEqualGUID(riid, &IID_IXAudio27))
-        object->version = This->version;
-    else /* only xaudio 2.8 has a different IID */
-        object->version = 28;
+#if XAUDIO2_VER == 0
+    object->IXAudio20_iface.lpVtbl = &XAudio20_Vtbl;
+#elif XAUDIO2_VER <= 2
+    object->IXAudio22_iface.lpVtbl = &XAudio22_Vtbl;
+#elif XAUDIO2_VER <= 7
+    object->IXAudio27_iface.lpVtbl = &XAudio27_Vtbl;
+#endif
+
+#if XAUDIO2_VER == 0
+    object->IXAudio20MasteringVoice_iface.lpVtbl = &XAudio20MasteringVoice_Vtbl;
+#elif XAUDIO2_VER <= 3
+    object->IXAudio23MasteringVoice_iface.lpVtbl = &XAudio23MasteringVoice_Vtbl;
+#endif
 
     list_init(&object->source_voices);
     list_init(&object->submix_voices);
@@ -1973,7 +2027,7 @@ static HRESULT WINAPI XAudio2CF_CreateInstance(IClassFactory *iface, IUnknown *p
 
     IXAudio2_StartEngine(&object->IXAudio2_iface);
 
-    TRACE("Created XAudio version %u: %p\n", object->version, object);
+    TRACE("Created XAudio version %u: %p\n", 20 + XAUDIO2_VER, object);
 
     return hr;
 }
@@ -1993,11 +2047,10 @@ static const IClassFactoryVtbl XAudio2CF_Vtbl =
     XAudio2CF_LockServer
 };
 
-static IClassFactory *make_xaudio2_factory(DWORD version)
+static IClassFactory *make_xaudio2_factory(void)
 {
     struct xaudio2_cf *ret = HeapAlloc(GetProcessHeap(), 0, sizeof(struct xaudio2_cf));
     ret->IClassFactory_iface.lpVtbl = &XAudio2CF_Vtbl;
-    ret->version = version;
     ret->ref = 0;
     return &ret->IClassFactory_iface;
 }
@@ -2008,88 +2061,35 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
 
     TRACE("(%s, %s, %p)\n", debugstr_guid(rclsid), debugstr_guid(riid), ppv);
 
-    if(IsEqualGUID(rclsid, &CLSID_XAudio20)){
-        factory = make_xaudio2_factory(20);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio21)){
-        factory = make_xaudio2_factory(21);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio22)){
-        factory = make_xaudio2_factory(22);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio23)){
-        factory = make_xaudio2_factory(23);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio24)){
-        factory = make_xaudio2_factory(24);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio25)){
-        factory = make_xaudio2_factory(25);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio26)){
-        factory = make_xaudio2_factory(26);
-    }else if(IsEqualGUID(rclsid, &CLSID_XAudio2)){
-        factory = make_xaudio2_factory(27);
+    if(IsEqualGUID(rclsid, &CLSID_XAudio20) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio21) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio22) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio23) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio24) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio25) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio26) ||
+            IsEqualGUID(rclsid, &CLSID_XAudio27)){
+        factory = make_xaudio2_factory();
 
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter20)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 20);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter21)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 21);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter22)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 22);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter23)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 23);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter24)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 24);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter25)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 25);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter26)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 26);
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter)){
-        factory = make_xapo_factory(&CLSID_AudioVolumeMeter, 27);
+    }else if(IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter20) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter21) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter22) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter23) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter24) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter25) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter26) ||
+                IsEqualGUID(rclsid, &CLSID_AudioVolumeMeter27)){
+        factory = make_xapo_factory(&CLSID_AudioVolumeMeter27);
 
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb20)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 20);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb21) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb10)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 21);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb22) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb11)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 22);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb23) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb12)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 23);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb24) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb13)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 24);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb25)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 25);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb26) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb14)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 26);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb) ||
-            IsEqualGUID(rclsid, &CLSID_WINE_FXReverb15)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 27);
-
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXReverb28)){
-        factory = make_xapo_factory(&CLSID_AudioReverb, 28);
-
-
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ10)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 21);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ11)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 22);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ12)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 23);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ13)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 24);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ14)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 26);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ15)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 27);
-    }else if(IsEqualGUID(rclsid, &CLSID_WINE_FXEQ28)){
-        factory = make_xapo_factory(&CLSID_FXEQ, 28);
+    }else if(IsEqualGUID(rclsid, &CLSID_AudioReverb20) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb21) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb22) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb23) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb24) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb25) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb26) ||
+                IsEqualGUID(rclsid, &CLSID_AudioReverb27)){
+        factory = make_xapo_factory(&CLSID_FXReverb);
     }
 
     if(!factory) return CLASS_E_CLASSNOTAVAILABLE;
@@ -2097,6 +2097,41 @@ HRESULT WINAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void **ppv)
     return IClassFactory_QueryInterface(factory, riid, ppv);
 }
 
+HRESULT xaudio2_initialize(IXAudio2Impl *This, UINT32 flags, XAUDIO2_PROCESSOR proc)
+{
+    if(flags)
+        FIXME("Unimplemented flags: 0x%x\n", flags);
+    return S_OK;
+}
+
+#if XAUDIO2_VER >= 8
+HRESULT WINAPI XAudio2Create(IXAudio2 **ppxa2, UINT32 flags, XAUDIO2_PROCESSOR proc)
+{
+    HRESULT hr;
+    IXAudio2 *xa2;
+    IClassFactory *cf;
+
+    TRACE("%p 0x%x 0x%x\n", ppxa2, flags, proc);
+
+    cf = make_xaudio2_factory();
+
+    hr = IClassFactory_CreateInstance(cf, NULL, &IID_IXAudio2, (void**)&xa2);
+    IClassFactory_Release(cf);
+    if(FAILED(hr))
+        return hr;
+
+    hr = xaudio2_initialize(impl_from_IXAudio2(xa2), flags, proc);
+    if(FAILED(hr)){
+        IXAudio2_Release(xa2);
+        return hr;
+    }
+
+    *ppxa2 = xa2;
+
+    return S_OK;
+}
+#endif /* XAUDIO2_VER >= 8 */
+
 /* returns TRUE if there is more data available in the buffer, FALSE if the
  * buffer's data has all been queued */
 static BOOL xa2buffer_queue_period(XA2SourceImpl *src, XA2Buffer *buf, ALuint al_buf)
@@ -2128,6 +2163,30 @@ static BOOL xa2buffer_queue_period(XA2SourceImpl *src, XA2Buffer *buf, ALuint al
     return buf->offs_bytes < buf->cur_end_bytes;
 }
 
+#if XAUDIO2_VER > 0
+static UINT32 get_underrun_warning(XA2SourceImpl *src)
+{
+    UINT32 period_bytes = src->xa2->period_frames * src->submit_blocksize;
+    UINT32 total = 0, i;
+
+    for(i = 0; i < src->nbufs && total < IN_AL_PERIODS * period_bytes; ++i){
+        XA2Buffer *buf = &src->buffers[(src->first_buf + i) % XAUDIO2_MAX_QUEUED_BUFFERS];
+        total += buf->cur_end_bytes - buf->offs_bytes;
+        if(buf->xa2buffer.LoopCount == XAUDIO2_LOOP_INFINITE)
+            return 0;
+        if(buf->xa2buffer.LoopCount > 0){
+            total += (buf->loop_end_bytes - buf->xa2buffer.LoopBegin) * (buf->xa2buffer.LoopCount - buf->looped);
+            total += buf->play_end_bytes - buf->loop_end_bytes;
+        }
+    }
+
+    if(total >= IN_AL_PERIODS * period_bytes)
+        return 0;
+
+    return ((IN_AL_PERIODS * period_bytes - total) / period_bytes + 1) * period_bytes;
+}
+#endif
+
 /* Looping:
  *
  * The looped section of a buffer is a subset of the play area which is looped
@@ -2143,7 +2202,7 @@ static BOOL xa2buffer_queue_period(XA2SourceImpl *src, XA2Buffer *buf, ALuint al
  *
  * In the simple case, playback will start at PlayBegin. At LoopEnd, playback
  * will move to LoopBegin and repeat that loop LoopCount times. Then, playback
- * will cease at LoopEnd.
+ * will cease at PlayEnd.
  *
  * If PlayLength is zero, then PlayEnd is the end of the buffer.
  *
@@ -2203,9 +2262,9 @@ static void update_source_state(XA2SourceImpl *src)
 
     alGetSourcei(src->al_src, AL_BYTE_OFFSET, &bufpos);
 
-    /* maintain 4 periods in AL */
+    /* maintain IN_AL_PERIODS periods in AL */
     while(src->cur_buf != (src->first_buf + src->nbufs) % XAUDIO2_MAX_QUEUED_BUFFERS &&
-            src->in_al_bytes - bufpos < 4 * src->xa2->period_frames * src->submit_blocksize){
+            src->in_al_bytes - bufpos < IN_AL_PERIODS * src->xa2->period_frames * src->submit_blocksize){
         TRACE("%p: going to queue a period from buffer %u\n", src, src->cur_buf);
 
         /* starting from an empty buffer */
@@ -2280,11 +2339,15 @@ static void do_engine_tick(IXAudio2Impl *This)
         }
 
         if(src->cb){
-            if(This->version == 20)
-                IXAudio20VoiceCallback_OnVoiceProcessingPassStart((IXAudio20VoiceCallback*)src->cb);
-            else
-                /* TODO: detect incoming underrun and inform callback */
-                IXAudio2VoiceCallback_OnVoiceProcessingPassStart(src->cb, 0);
+#if XAUDIO2_VER == 0
+            IXAudio20VoiceCallback_OnVoiceProcessingPassStart((IXAudio20VoiceCallback*)src->cb);
+#else
+            UINT32 underrun;
+            underrun = get_underrun_warning(src);
+            if(underrun > 0)
+                TRACE("Calling OnVoiceProcessingPassStart with BytesRequired: %u\n", underrun);
+            IXAudio2VoiceCallback_OnVoiceProcessingPassStart(src->cb, underrun);
+#endif
         }
 
         update_source_state(src);
