@@ -160,6 +160,16 @@ const WCHAR *get_encoding_name(xml_encoding encoding)
     return xml_encoding_map[encoding].name;
 }
 
+xml_encoding get_encoding_from_codepage(UINT codepage)
+{
+    int i;
+    for (i = 0; i < sizeof(xml_encoding_map)/sizeof(xml_encoding_map[0]); i++)
+    {
+        if (xml_encoding_map[i].cp == codepage) return xml_encoding_map[i].enc;
+    }
+    return XmlEncoding_Unknown;
+}
+
 typedef struct
 {
     char *data;
@@ -233,6 +243,7 @@ typedef struct
     XmlReaderResumeState resumestate;
     XmlNodeType nodetype;
     DtdProcessing dtdmode;
+    IXmlResolver *resolver;
     UINT line, pos;           /* reader position in XML stream */
     struct list attrs; /* attributes list for current node */
     struct attribute *attr; /* current attribute */
@@ -2462,6 +2473,7 @@ static ULONG WINAPI xmlreader_Release(IXmlReader *iface)
     {
         IMalloc *imalloc = This->imalloc;
         if (This->input) IUnknown_Release(&This->input->IXmlReaderInput_iface);
+        if (This->resolver) IXmlResolver_Release(This->resolver);
         reader_clear_attrs(This);
         reader_clear_elements(This);
         reader_free_strvalues(This);
@@ -2546,6 +2558,11 @@ static HRESULT WINAPI xmlreader_GetProperty(IXmlReader* iface, UINT property, LO
 
     switch (property)
     {
+        case XmlReaderProperty_XmlResolver:
+            *value = (LONG_PTR)This->resolver;
+            if (This->resolver)
+                IXmlResolver_AddRef(This->resolver);
+            break;
         case XmlReaderProperty_DtdProcessing:
             *value = This->dtdmode;
             break;
@@ -2564,10 +2581,17 @@ static HRESULT WINAPI xmlreader_SetProperty(IXmlReader* iface, UINT property, LO
 {
     xmlreader *This = impl_from_IXmlReader(iface);
 
-    TRACE("(%p)->(%s %lu)\n", This, debugstr_reader_prop(property), value);
+    TRACE("(%p)->(%s 0x%lx)\n", This, debugstr_reader_prop(property), value);
 
     switch (property)
     {
+        case XmlReaderProperty_XmlResolver:
+            if (This->resolver)
+                IXmlResolver_Release(This->resolver);
+            This->resolver = (IXmlResolver*)value;
+            if (This->resolver)
+                IXmlResolver_AddRef(This->resolver);
+            break;
         case XmlReaderProperty_DtdProcessing:
             if (value < 0 || value > _DtdProcessing_Last) return E_INVALIDARG;
             This->dtdmode = value;
@@ -2845,7 +2869,7 @@ static HRESULT WINAPI xmlreader_GetDepth(IXmlReader* iface, UINT *depth)
 static BOOL WINAPI xmlreader_IsEOF(IXmlReader* iface)
 {
     FIXME("(%p): stub\n", iface);
-    return E_NOTIMPL;
+    return FALSE;
 }
 
 static const struct IXmlReaderVtbl xmlreader_vtbl =
@@ -2964,6 +2988,7 @@ HRESULT WINAPI CreateXmlReader(REFIID riid, void **obj, IMalloc *imalloc)
     reader->instate = XmlReadInState_Initial;
     reader->resumestate = XmlReadResumeState_Initial;
     reader->dtdmode = DtdProcessing_Prohibit;
+    reader->resolver = NULL;
     reader->line  = reader->pos = 0;
     reader->imalloc = imalloc;
     if (imalloc) IMalloc_AddRef(imalloc);
