@@ -96,15 +96,9 @@ static void ok_pos_(IXmlReader *reader, int line, int pos, int line_broken,
         broken_state = broken((line_broken == -1 ? line : line_broken) == l &&
                               (pos_broken == -1 ? pos : pos_broken) == p);
 
-    if (todo)
-        todo_wine
+    todo_wine_if (todo)
         ok_(__FILE__, _line_)((l == line && pos == p) || broken_state,
                             "Expected (%d,%d), got (%d,%d)\n", line, pos, l, p);
-    else
-    {
-        ok_(__FILE__, _line_)((l == line && pos == p) || broken_state,
-                            "Expected (%d,%d), got (%d,%d)\n", line, pos, l, p);
-    }
 }
 #define ok_pos(reader, l, p, l_brk, p_brk, todo) ok_pos_(reader, l, p, l_brk, p_brk, todo, __LINE__)
 
@@ -147,12 +141,8 @@ static void ok_iids_(const input_iids_t *iids, const IID **expected, const IID *
 
     while (expected[i++]) size++;
 
-    if (todo) {
-        todo_wine
-            ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
-    }
-    else
-       ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
+    todo_wine_if (todo)
+        ok_(__FILE__, line)(iids->count == size, "Sequence size mismatch (%d), got (%d)\n", size, iids->count);
 
     if (iids->count != size) return;
 
@@ -247,13 +237,7 @@ static void test_read_state_(IXmlReader *reader, XmlReadState expected,
     else
         broken_state = broken(exp_broken == state);
 
-    if (todo)
-    {
-    todo_wine
-        ok_(__FILE__, line)(state == expected || broken_state, "Expected (%s), got (%s)\n",
-                                   state_to_str(expected), state_to_str(state));
-    }
-    else
+    todo_wine_if (todo)
         ok_(__FILE__, line)(state == expected || broken_state, "Expected (%s), got (%s)\n",
                                    state_to_str(expected), state_to_str(state));
 }
@@ -402,8 +386,51 @@ static BOOL init_pointers(void)
     return TRUE;
 }
 
+static HRESULT WINAPI resolver_QI(IXmlResolver *iface, REFIID riid, void **obj)
+{
+    ok(0, "unexpected call, riid %s\n", wine_dbgstr_guid(riid));
+
+    if (IsEqualIID(riid, &IID_IXmlResolver) || IsEqualIID(riid, &IID_IUnknown))
+    {
+        *obj = iface;
+        IXmlResolver_AddRef(iface);
+        return S_OK;
+    }
+
+    *obj = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG WINAPI resolver_AddRef(IXmlResolver *iface)
+{
+    return 2;
+}
+
+static ULONG WINAPI resolver_Release(IXmlResolver *iface)
+{
+    return 1;
+}
+
+static HRESULT WINAPI resolver_ResolveUri(IXmlResolver *iface, const WCHAR *base_uri,
+    const WCHAR *public_id, const WCHAR *system_id, IUnknown **input)
+{
+    ok(0, "unexpected call\n");
+    return E_NOTIMPL;
+}
+
+static const IXmlResolverVtbl resolvervtbl =
+{
+    resolver_QI,
+    resolver_AddRef,
+    resolver_Release,
+    resolver_ResolveUri
+};
+
+static IXmlResolver testresolver = { &resolvervtbl };
+
 static void test_reader_create(void)
 {
+    IXmlResolver *resolver;
     HRESULT hr;
     IXmlReader *reader;
     IUnknown *input;
@@ -427,6 +454,26 @@ static void test_reader_create(void)
     ok(hr == S_FALSE, "got %08x\n", hr);
     ok(nodetype == XmlNodeType_None, "got %d\n", nodetype);
 
+    resolver = (void*)0xdeadbeef;
+    hr = IXmlReader_GetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR*)&resolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(resolver == NULL, "got %p\n", resolver);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, 0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR)&testresolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    resolver = NULL;
+    hr = IXmlReader_GetProperty(reader, XmlReaderProperty_XmlResolver, (LONG_PTR*)&resolver);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+    ok(resolver == &testresolver, "got %p\n", resolver);
+    IXmlResolver_Release(resolver);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_XmlResolver, 0);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
     dtd = 2;
     hr = IXmlReader_GetProperty(reader, XmlReaderProperty_DtdProcessing, (LONG_PTR*)&dtd);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
@@ -1431,6 +1478,7 @@ static void test_readvaluechunk(void)
     ok(value[0] == 'c', "got %s\n", wine_dbgstr_w(value));
 
     IXmlReader_Release(reader);
+    IStream_Release(stream);
 }
 
 static struct test_entry cdata_tests[] = {
@@ -1514,14 +1562,7 @@ static void test_read_cdata(void)
             ok(hr == S_OK, "got 0x%08x\n", hr);
 
             str_exp = a2w(test->value);
-            if (test->todo)
-            {
-            todo_wine {
-                ok(len == strlen(test->value), "got %u\n", len);
-                ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
-            }
-            }
-            else
+            todo_wine_if (test->todo)
             {
                 ok(len == strlen(test->value), "got %u\n", len);
                 ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
@@ -1614,14 +1655,7 @@ static void test_read_text(void)
             ok(hr == S_OK, "got 0x%08x\n", hr);
 
             str_exp = a2w(test->value);
-            if (test->todo)
-            {
-            todo_wine {
-                ok(len == strlen(test->value), "got %u\n", len);
-                ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
-            }
-            }
-            else
+            todo_wine_if (test->todo)
             {
                 ok(len == strlen(test->value), "got %u\n", len);
                 ok(!lstrcmpW(str, str_exp), "got %s\n", wine_dbgstr_w(str));
