@@ -29,6 +29,10 @@
 #include <float.h>
 #include <ddraw.h>
 
+#ifdef __i386__
+#include <pshpack4.h>
+#endif
+
 #define D3DVALP(val, prec)      ((float)(val))
 #define D3DVAL(val)             ((float)(val))
 #define D3DDivide(a, b)         (float)((double) (a) / (double) (b))
@@ -1356,4 +1360,8 @@ typedef enum _D3DTEXTURETRANSFORMFLAGS {
 #define D3DFVF_TEXCOORDSIZE4(CoordIndex) (D3DFVF_TEXTUREFORMAT4 << (CoordIndex*2 + 16))
 #define D3DFVF_TEXCOORDSIZE1(CoordIndex) (D3DFVF_TEXTUREFORMAT1 << (CoordIndex*2 + 16))
 
+#ifdef __i386__
+#include <poppack.h>
+#endif
+
 #endif
