@@ -463,6 +463,10 @@ UINT WINAPI MsiEnumClientsA(LPCSTR, DWORD, LPSTR);
 UINT WINAPI MsiEnumClientsW(LPCWSTR, DWORD, LPWSTR);
 #define     MsiEnumClients WINELIB_NAME_AW(MsiEnumClients)
 
+UINT WINAPI MsiEnumClientsExA(LPCSTR, LPCSTR, DWORD, DWORD, CHAR*, MSIINSTALLCONTEXT*, LPSTR, LPDWORD);
+UINT WINAPI MsiEnumClientsExW(LPCWSTR, LPCWSTR, DWORD, DWORD, WCHAR*, MSIINSTALLCONTEXT*, LPWSTR, LPDWORD);
+#define     MsiEnumClientsEx WINELIB_NAME_AW(MsiEnumClientsEx)
+
 UINT WINAPI MsiOpenPackageA(LPCSTR, MSIHANDLE*);
 UINT WINAPI MsiOpenPackageW(LPCWSTR, MSIHANDLE*);
 #define     MsiOpenPackage WINELIB_NAME_AW(MsiOpenPackage)
@@ -583,6 +587,10 @@ USERINFOSTATE WINAPI MsiGetUserInfoA(LPCSTR, LPSTR, LPDWORD, LPSTR, LPDWORD, LPS
 USERINFOSTATE WINAPI MsiGetUserInfoW(LPCWSTR, LPWSTR, LPDWORD, LPWSTR, LPDWORD, LPWSTR, LPDWORD);
 #define MsiGetUserInfo WINELIB_NAME_AW(MsiGetUserInfo)
 
+UINT WINAPI MsiProvidedComponentA(LPCSTR, LPCSTR, LPCSTR, DWORD, LPSTR, LPDWORD);
+UINT WINAPI MsiProvideComponentW(LPCWSTR, LPCWSTR, LPCWSTR, DWORD, LPWSTR, LPDWORD);
+#define MsiProvideComponent WINELIB_NAME_AW(MsiProvideComponent)
+
 UINT WINAPI MsiCollectUserInfoA(LPCSTR);
 UINT WINAPI MsiCollectUserInfoW(LPCWSTR);
 #define MsiCollectUserInfo WINELIB_NAME_AW(MsiCollectUserInfo)
@@ -695,6 +703,7 @@ UINT WINAPI MsiBeginTransactionA(LPCSTR, DWORD, MSIHANDLE *, HANDLE *);
 UINT WINAPI MsiBeginTransactionW(LPCWSTR, DWORD, MSIHANDLE *, HANDLE *);
 #define     MsiBeginTransaction WINELIB_NAME_AW(MsiBeginTransaction)
 
+UINT WINAPI MsiJoinTransaction(MSIHANDLE, DWORD, HANDLE *);
 UINT WINAPI MsiEndTransaction(DWORD);
 
 /* Non Unicode */
