@@ -290,6 +290,55 @@ typedef enum {
     WS_WRITE_NILLABLE_POINTER = 4
 } WS_WRITE_OPTION;
 
+typedef struct _WS_BOOL_DESCRIPTION {
+    BOOL value;
+} WS_BOOL_DESCRIPTION;
+
+typedef struct _WS_INT8_DESCRIPTION {
+    char minValue;
+    char maxValue;
+} WS_INT8_DESCRIPTION;
+
+typedef struct _WS_INT16_DESCRIPTION {
+    short minValue;
+    short maxValue;
+} WS_INT16_DESCRIPTION;
+
+typedef struct _WS_INT32_DESCRIPTION {
+    int minValue;
+    int maxValue;
+} WS_INT32_DESCRIPTION;
+
+typedef struct _WS_INT64_DESCRIPTION {
+    __int64 minValue;
+    __int64 maxValue;
+} WS_INT64_DESCRIPTION;
+
+typedef struct _WS_UINT8_DESCRIPTION {
+    BYTE minValue;
+    BYTE maxValue;
+} WS_UINT8_DESCRIPTION;
+
+typedef struct _WS_UINT16_DESCRIPTION {
+    USHORT minValue;
+    USHORT maxValue;
+} WS_UINT16_DESCRIPTION;
+
+typedef struct _WS_UINT32_DESCRIPTION {
+    ULONG minValue;
+    ULONG maxValue;
+} WS_UINT32_DESCRIPTION;
+
+typedef struct _WS_UINT64_DESCRIPTION {
+    unsigned __int64 minValue;
+    unsigned __int64 maxValue;
+} WS_UINT64_DESCRIPTION;
+
+typedef struct _WS_WSZ_DESCRIPTION {
+    ULONG minCharCount;
+    ULONG maxCharCount;
+} WS_WSZ_DESCRIPTION;
+
 typedef enum {
     WS_TYPE_ATTRIBUTE_FIELD_MAPPING,
     WS_ATTRIBUTE_FIELD_MAPPING,
@@ -367,6 +416,13 @@ typedef struct _WS_ATTRIBUTE_DESCRIPTION {
     void *typeDescription;
 } WS_ATTRIBUTE_DESCRIPTION;
 
+typedef struct _WS_ELEMENT_DESCRIPTION {
+    WS_XML_STRING *elementLocalName;
+    WS_XML_STRING *elementNs;
+    WS_TYPE type;
+    void *typeDescription;
+} WS_ELEMENT_DESCRIPTION;
+
 typedef struct _WS_STRING {
     ULONG length;
     WCHAR *chars;
@@ -387,6 +443,21 @@ typedef struct _WS_XML_NODE {
     WS_XML_NODE_TYPE nodeType;
 } WS_XML_NODE;
 
+typedef enum {
+    WS_MOVE_TO_ROOT_ELEMENT,
+    WS_MOVE_TO_NEXT_ELEMENT,
+    WS_MOVE_TO_PREVIOUS_ELEMENT,
+    WS_MOVE_TO_CHILD_ELEMENT,
+    WS_MOVE_TO_END_ELEMENT,
+    WS_MOVE_TO_PARENT_ELEMENT,
+    WS_MOVE_TO_NEXT_NODE,
+    WS_MOVE_TO_PREVIOUS_NODE,
+    WS_MOVE_TO_FIRST_NODE,
+    WS_MOVE_TO_BOF,
+    WS_MOVE_TO_EOF,
+    WS_MOVE_TO_CHILD_NODE
+} WS_MOVE_TO;
+
 typedef enum {
     WS_XML_TEXT_TYPE_UTF8      = 1,
     WS_XML_TEXT_TYPE_UTF16     = 2,
@@ -415,6 +486,25 @@ typedef struct _WS_XML_UTF8_TEXT {
     WS_XML_STRING value;
 } WS_XML_UTF8_TEXT;
 
+typedef enum {
+    WS_BOOL_VALUE_TYPE,
+    WS_INT8_VALUE_TYPE,
+    WS_INT16_VALUE_TYPE,
+    WS_INT32_VALUE_TYPE,
+    WS_INT64_VALUE_TYPE,
+    WS_UINT8_VALUE_TYPE,
+    WS_UINT16_VALUE_TYPE,
+    WS_UINT32_VALUE_TYPE,
+    WS_UINT64_VALUE_TYPE,
+    WS_FLOAT_VALUE_TYPE,
+    WS_DOUBLE_VALUE_TYPE,
+    WS_DECIMAL_VALUE_TYPE,
+    WS_DATETIME_VALUE_TYPE,
+    WS_TIMESPAN_VALUE_TYPE,
+    WS_GUID_VALUE_TYPE,
+    WS_DURATION_VALUE_TYPE
+} WS_VALUE_TYPE;
+
 typedef struct _WS_XML_ATTRIBUTE {
     BYTE singleQuote;
     BYTE isXmlNs;
@@ -449,6 +539,7 @@ typedef struct _WS_XML_NODE_POSITION {
     void *node;
 } WS_XML_NODE_POSITION;
 
+HRESULT WINAPI WsAlloc(WS_HEAP*, SIZE_T, void**, WS_ERROR*);
 HRESULT WINAPI WsCreateError(const WS_ERROR_PROPERTY*, ULONG, WS_ERROR**);
 HRESULT WINAPI WsCreateHeap(SIZE_T, SIZE_T, const WS_HEAP_PROPERTY*, ULONG, WS_HEAP**, WS_ERROR*);
 HRESULT WINAPI WsCreateReader(const WS_XML_READER_PROPERTY*, ULONG, WS_XML_READER**, WS_ERROR*);
@@ -465,14 +556,21 @@ void WINAPI WsFreeWriter(WS_XML_WRITER*);
 HRESULT WINAPI WsGetErrorProperty(WS_ERROR*, WS_ERROR_PROPERTY_ID, void*, ULONG);
 HRESULT WINAPI WsGetErrorString(WS_ERROR*, ULONG, WS_STRING*);
 HRESULT WINAPI WsGetHeapProperty(WS_HEAP*, WS_HEAP_PROPERTY_ID, void*, ULONG, WS_ERROR*);
+HRESULT WINAPI WsGetNamespaceFromPrefix(WS_XML_READER*, const WS_XML_STRING*, BOOL,
+                                        const WS_XML_STRING**, WS_ERROR*);
+HRESULT WINAPI WsGetPrefixFromNamespace(WS_XML_WRITER*, const WS_XML_STRING*, BOOL,
+                                        const WS_XML_STRING**, WS_ERROR*);
 HRESULT WINAPI WsGetReaderNode(WS_XML_READER*, const WS_XML_NODE**, WS_ERROR*);
 HRESULT WINAPI WsGetReaderPosition(WS_XML_READER*, WS_XML_NODE_POSITION*, WS_ERROR*);
 HRESULT WINAPI WsGetReaderProperty(WS_XML_READER*, WS_XML_READER_PROPERTY_ID, void*, ULONG, WS_ERROR*);
 HRESULT WINAPI WsGetWriterProperty(WS_XML_WRITER*, WS_XML_WRITER_PROPERTY_ID, void*, ULONG, WS_ERROR*);
 HRESULT WINAPI WsGetXmlAttribute(WS_XML_READER*, const WS_XML_STRING*, WS_HEAP*, WCHAR**,
                                  ULONG*, WS_ERROR*);
+HRESULT WINAPI WsMoveReader(WS_XML_READER*, WS_MOVE_TO, BOOL*, WS_ERROR*);
+HRESULT WINAPI WsMoveWriter(WS_XML_WRITER*, WS_MOVE_TO, BOOL*, WS_ERROR*);
 HRESULT WINAPI WsReadAttribute(WS_XML_READER*, const WS_ATTRIBUTE_DESCRIPTION*, WS_READ_OPTION,
                                WS_HEAP*, void*, ULONG, WS_ERROR*);
+HRESULT WINAPI WsReadEndAttribute(WS_XML_READER*, WS_ERROR*);
 HRESULT WINAPI WsReadEndElement(WS_XML_READER*, WS_ERROR*);
 HRESULT WINAPI WsReadNode(WS_XML_READER*, WS_ERROR*);
 HRESULT WINAPI WsReadStartAttribute(WS_XML_READER*, ULONG, WS_ERROR*);
@@ -490,14 +588,31 @@ HRESULT WINAPI WsSetOutput(WS_XML_WRITER*, const WS_XML_WRITER_ENCODING*, const
                            const WS_XML_WRITER_PROPERTY*, ULONG, WS_ERROR*);
 HRESULT WINAPI WsSetOutputToBuffer(WS_XML_WRITER*, WS_XML_BUFFER*, const WS_XML_WRITER_PROPERTY*,
                                    ULONG, WS_ERROR*);
+HRESULT WINAPI WsSkipNode(WS_XML_READER*, WS_ERROR*);
+HRESULT WINAPI WsWriteAttribute(WS_XML_WRITER*, const WS_ATTRIBUTE_DESCRIPTION*, WS_WRITE_OPTION,
+                                const void*, ULONG, WS_ERROR*);
+HRESULT WINAPI WsWriteElement(WS_XML_WRITER*, const WS_ELEMENT_DESCRIPTION*, WS_WRITE_OPTION,
+                              const void*, ULONG, WS_ERROR*);
 HRESULT WINAPI WsWriteEndAttribute(WS_XML_WRITER*, WS_ERROR*);
+HRESULT WINAPI WsWriteEndCData(WS_XML_WRITER*, WS_ERROR*);
 HRESULT WINAPI WsWriteEndElement(WS_XML_WRITER*, WS_ERROR*);
+HRESULT WINAPI WsWriteEndStartElement(WS_XML_WRITER*, WS_ERROR*);
 HRESULT WINAPI WsWriteStartAttribute(WS_XML_WRITER*, const WS_XML_STRING*, const WS_XML_STRING*,
                                      const WS_XML_STRING*, BOOL, WS_ERROR*);
+HRESULT WINAPI WsWriteStartCData(WS_XML_WRITER*, WS_ERROR*);
 HRESULT WINAPI WsWriteStartElement(WS_XML_WRITER*, const WS_XML_STRING*, const WS_XML_STRING*,
                                    const WS_XML_STRING*, WS_ERROR*);
+HRESULT WINAPI WsWriteText(WS_XML_WRITER*, const WS_XML_TEXT*, WS_ERROR*);
 HRESULT WINAPI WsWriteType(WS_XML_WRITER*, WS_TYPE_MAPPING, WS_TYPE, const void*, WS_WRITE_OPTION,
                            const void*, ULONG, WS_ERROR*);
+HRESULT WINAPI WsWriteValue(WS_XML_WRITER*, WS_VALUE_TYPE, const void*, ULONG, WS_ERROR*);
+HRESULT WINAPI WsWriteXmlBuffer(WS_XML_WRITER*, WS_XML_BUFFER*, WS_ERROR*);
+HRESULT WINAPI WsWriteXmlBufferToBytes(WS_XML_WRITER*, WS_XML_BUFFER*, const WS_XML_WRITER_ENCODING*,
+                                       const WS_XML_WRITER_PROPERTY*, ULONG, WS_HEAP*, void**,
+                                       ULONG*, WS_ERROR*);
+HRESULT WINAPI WsWriteXmlnsAttribute(WS_XML_WRITER*, const WS_XML_STRING*, const WS_XML_STRING*,
+                                     BOOL, WS_ERROR*);
+HRESULT WINAPI WsXmlStringEquals(const WS_XML_STRING*, const WS_XML_STRING*, WS_ERROR*);
 
 #define WS_S_ASYNC                          0x003d0000
 #define WS_S_END                            0x003d0001
