@@ -21,10 +21,11 @@
 #ifndef __WINE_EXPLORER_PRIVATE_H
 #define __WINE_EXPLORER_PRIVATE_H
 
-extern void manage_desktop( WCHAR *arg );
-extern void initialize_systray( HMODULE graphics_driver, BOOL using_root, BOOL enable_shell );
-extern void initialize_appbar(void);
-extern void do_startmenu( HWND owner );
-extern LRESULT menu_wndproc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
+extern void manage_desktop( WCHAR *arg ) DECLSPEC_HIDDEN;
+extern void initialize_systray( HMODULE graphics_driver, BOOL using_root, BOOL enable_shell ) DECLSPEC_HIDDEN;
+extern void initialize_appbar(void) DECLSPEC_HIDDEN;
+extern void cleanup_systray_window( HWND hwnd ) DECLSPEC_HIDDEN;
+extern void do_startmenu( HWND owner ) DECLSPEC_HIDDEN;
+extern LRESULT menu_wndproc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) DECLSPEC_HIDDEN;
 
 #endif  /* __WINE_EXPLORER_PRIVATE_H */
