@@ -19,6 +19,7 @@
  */
 
 #define WIN32_LEAN_AND_MEAN
+#define NONAMELESSUNION
 
 #include <stdarg.h>
 #include <windows.h>
@@ -113,14 +114,14 @@ static void run_after_timeout(void (*func)(struct service_entry*), struct servic
         return;
     }
 
-    service->ref_count++;
+    InterlockedIncrement(&service->ref_count);
     elem->func = func;
     elem->service_entry = service;
 
     GetSystemTimeAsFileTime(&elem->time);
     time.u.LowPart = elem->time.dwLowDateTime;
     time.u.HighPart = elem->time.dwHighDateTime;
-    time.QuadPart += timeout*10000000;
+    time.QuadPart += (ULONGLONG)timeout * 10000;
     elem->time.dwLowDateTime = time.u.LowPart;
     elem->time.dwHighDateTime = time.u.HighPart;
 
@@ -266,14 +267,13 @@ DWORD __cdecl svcctl_GetServiceDisplayNameW(
     if ((err = validate_scm_handle(hSCManager, 0, &manager)) != ERROR_SUCCESS)
         return err;
 
-    scmdatabase_lock_shared(manager->db);
+    scmdatabase_lock(manager->db);
 
     entry = scmdatabase_find_service(manager->db, lpServiceName);
     if (entry != NULL)
     {
         LPCWSTR name;
         int len;
-        service_lock_shared(entry);
         name = get_display_name(entry);
         len = strlenW(name);
         if (len <= *cchBufSize)
@@ -284,7 +284,6 @@ DWORD __cdecl svcctl_GetServiceDisplayNameW(
         else
             err = ERROR_INSUFFICIENT_BUFFER;
         *cchBufSize = len;
-        service_unlock(entry);
     }
     else
         err = ERROR_SERVICE_DOES_NOT_EXIST;
@@ -312,13 +311,12 @@ DWORD __cdecl svcctl_GetServiceKeyNameW(
     if ((err = validate_scm_handle(hSCManager, 0, &manager)) != ERROR_SUCCESS)
         return err;
 
-    scmdatabase_lock_shared(manager->db);
+    scmdatabase_lock(manager->db);
 
     entry = scmdatabase_find_service_by_displayname(manager->db, lpServiceDisplayName);
     if (entry != NULL)
     {
         int len;
-        service_lock_shared(entry);
         len = strlenW(entry->name);
         if (len <= *cchBufSize)
         {
@@ -328,7 +326,6 @@ DWORD __cdecl svcctl_GetServiceKeyNameW(
         else
             err = ERROR_INSUFFICIENT_BUFFER;
         *cchBufSize = len;
-        service_unlock(entry);
     }
     else
         err = ERROR_SERVICE_DOES_NOT_EXIST;
@@ -351,12 +348,13 @@ static DWORD create_handle_for_service(struct service_entry *entry, DWORD dwDesi
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
     }
 
+    if (dwDesiredAccess & MAXIMUM_ALLOWED)
+        dwDesiredAccess |= SERVICE_ALL_ACCESS;
+
     service->hdr.type = SC_HTYPE_SERVICE;
     service->hdr.access = dwDesiredAccess;
     RtlMapGenericMask(&service->hdr.access, &g_svc_generic);
     service->service_entry = entry;
-    if (dwDesiredAccess & MAXIMUM_ALLOWED)
-        dwDesiredAccess |= SERVICE_ALL_ACCESS;
 
     *phService = &service->hdr;
     return ERROR_SUCCESS;
@@ -379,7 +377,7 @@ DWORD __cdecl svcctl_OpenServiceW(
     if (!validate_service_name(lpServiceName))
         return ERROR_INVALID_NAME;
 
-    scmdatabase_lock_shared(manager->db);
+    scmdatabase_lock(manager->db);
     entry = scmdatabase_find_service(manager->db, lpServiceName);
     if (entry != NULL)
         InterlockedIncrement(&entry->ref_count);
@@ -463,7 +461,7 @@ static DWORD parse_dependencies(const WCHAR *dependencies, struct service_entry
     return ERROR_SUCCESS;
 }
 
-DWORD __cdecl svcctl_CreateServiceW(
+static DWORD create_serviceW(
     SC_RPC_HANDLE hSCManager,
     LPCWSTR lpServiceName,
     LPCWSTR lpDisplayName,
@@ -479,7 +477,8 @@ DWORD __cdecl svcctl_CreateServiceW(
     LPCWSTR lpServiceStartName,
     const BYTE *lpPassword,
     DWORD dwPasswordSize,
-    SC_RPC_HANDLE *phService)
+    SC_RPC_HANDLE *phService,
+    BOOL is_wow64)
 {
     struct service_entry *entry, *found;
     struct sc_manager_handle *manager;
@@ -508,7 +507,7 @@ DWORD __cdecl svcctl_CreateServiceW(
         return err;
     }
 
-    entry->ref_count = 1;
+    entry->is_wow64 = is_wow64;
     entry->config.dwServiceType = entry->status.dwServiceType = dwServiceType;
     entry->config.dwStartType = dwStartType;
     entry->config.dwErrorControl = dwErrorControl;
@@ -531,13 +530,11 @@ DWORD __cdecl svcctl_CreateServiceW(
         return ERROR_INVALID_PARAMETER;
     }
 
-    scmdatabase_lock_exclusive(manager->db);
+    scmdatabase_lock(manager->db);
 
     if ((found = scmdatabase_find_service(manager->db, lpServiceName)))
     {
-        service_lock_exclusive(found);
         err = is_marked_for_delete(found) ? ERROR_SERVICE_MARKED_FOR_DELETE : ERROR_SERVICE_EXISTS;
-        service_unlock(found);
         scmdatabase_unlock(manager->db);
         free_service_entry(entry);
         return err;
@@ -562,6 +559,30 @@ DWORD __cdecl svcctl_CreateServiceW(
     return create_handle_for_service(entry, dwDesiredAccess, phService);
 }
 
+DWORD __cdecl svcctl_CreateServiceW(
+    SC_RPC_HANDLE hSCManager,
+    LPCWSTR lpServiceName,
+    LPCWSTR lpDisplayName,
+    DWORD dwDesiredAccess,
+    DWORD dwServiceType,
+    DWORD dwStartType,
+    DWORD dwErrorControl,
+    LPCWSTR lpBinaryPathName,
+    LPCWSTR lpLoadOrderGroup,
+    DWORD *lpdwTagId,
+    const BYTE *lpDependencies,
+    DWORD dwDependenciesSize,
+    LPCWSTR lpServiceStartName,
+    const BYTE *lpPassword,
+    DWORD dwPasswordSize,
+    SC_RPC_HANDLE *phService)
+{
+    WINE_TRACE("(%s, %s, 0x%x, %s)\n", wine_dbgstr_w(lpServiceName), wine_dbgstr_w(lpDisplayName), dwDesiredAccess, wine_dbgstr_w(lpBinaryPathName));
+    return create_serviceW(hSCManager, lpServiceName, lpDisplayName, dwDesiredAccess, dwServiceType, dwStartType,
+        dwErrorControl, lpBinaryPathName, lpLoadOrderGroup, lpdwTagId, lpDependencies, dwDependenciesSize, lpServiceStartName,
+        lpPassword, dwPasswordSize, phService, FALSE);
+}
+
 DWORD __cdecl svcctl_DeleteService(
     SC_RPC_HANDLE hService)
 {
@@ -571,7 +592,7 @@ DWORD __cdecl svcctl_DeleteService(
     if ((err = validate_service_handle(hService, DELETE, &service)) != ERROR_SUCCESS)
         return err;
 
-    service_lock_exclusive(service->service_entry);
+    service_lock(service->service_entry);
 
     if (!is_marked_for_delete(service->service_entry))
         err = mark_for_delete(service->service_entry);
@@ -585,7 +606,9 @@ DWORD __cdecl svcctl_DeleteService(
 
 DWORD __cdecl svcctl_QueryServiceConfigW(
         SC_RPC_HANDLE hService,
-        QUERY_SERVICE_CONFIGW *config)
+        QUERY_SERVICE_CONFIGW *config,
+        DWORD buf_size,
+        DWORD *needed_size)
 {
     struct sc_service_handle *service;
     DWORD err;
@@ -595,7 +618,7 @@ DWORD __cdecl svcctl_QueryServiceConfigW(
     if ((err = validate_service_handle(hService, SERVICE_QUERY_CONFIG, &service)) != 0)
         return err;
 
-    service_lock_shared(service->service_entry);
+    service_lock(service->service_entry);
     config->dwServiceType = service->service_entry->config.dwServiceType;
     config->dwStartType = service->service_entry->config.dwStartType;
     config->dwErrorControl = service->service_entry->config.dwErrorControl;
@@ -638,7 +661,7 @@ DWORD __cdecl svcctl_ChangeServiceConfigW(
         return ERROR_INVALID_PARAMETER;
 
     /* first check if the new configuration is correct */
-    service_lock_exclusive(service->service_entry);
+    service_lock(service->service_entry);
 
     if (is_marked_for_delete(service->service_entry))
     {
@@ -735,7 +758,7 @@ DWORD __cdecl svcctl_SetServiceStatus(
     if ((err = validate_service_handle(hServiceStatus, SERVICE_SET_STATUS, &service)) != 0)
         return err;
 
-    service_lock_exclusive(service->service_entry);
+    service_lock(service->service_entry);
     /* FIXME: be a bit more discriminant about what parts of the status we set
      * and check that fields are valid */
     service->service_entry->status.dwServiceType = lpServiceStatus->dwServiceType;
@@ -749,13 +772,13 @@ DWORD __cdecl svcctl_SetServiceStatus(
 
     if (lpServiceStatus->dwCurrentState == SERVICE_STOPPED)
         run_after_timeout(service_terminate, service->service_entry, service_kill_timeout);
-    else if (service->service_entry->status_changed_event)
-        SetEvent(service->service_entry->status_changed_event);
+    else if (service->service_entry->process->status_changed_event)
+        SetEvent(service->service_entry->process->status_changed_event);
 
     return ERROR_SUCCESS;
 }
 
-DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, DWORD level, SERVICE_CONFIG2W *config )
+DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, SC_RPC_CONFIG_INFOW config )
 {
     struct sc_service_handle *service;
     DWORD err;
@@ -763,20 +786,20 @@ DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
     if ((err = validate_service_handle(hService, SERVICE_CHANGE_CONFIG, &service)) != 0)
         return err;
 
-    switch (level)
+    switch (config.dwInfoLevel)
     {
     case SERVICE_CONFIG_DESCRIPTION:
         {
             WCHAR *descr = NULL;
 
-            if (config->descr.lpDescription[0])
+            if (config.u.descr->lpDescription[0])
             {
-                if (!(descr = strdupW( config->descr.lpDescription )))
+                if (!(descr = strdupW( config.u.descr->lpDescription )))
                     return ERROR_NOT_ENOUGH_MEMORY;
             }
 
             WINE_TRACE( "changing service %p descr to %s\n", service, wine_dbgstr_w(descr) );
-            service_lock_exclusive( service->service_entry );
+            service_lock( service->service_entry );
             HeapFree( GetProcessHeap(), 0, service->service_entry->description );
             service->service_entry->description = descr;
             save_service_config( service->service_entry );
@@ -785,20 +808,20 @@ DWORD __cdecl svcctl_ChangeServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
         break;
     case SERVICE_CONFIG_FAILURE_ACTIONS:
         WINE_FIXME( "SERVICE_CONFIG_FAILURE_ACTIONS not implemented: period %u msg %s cmd %s\n",
-                    config->actions.dwResetPeriod,
-                    wine_dbgstr_w(config->actions.lpRebootMsg),
-                    wine_dbgstr_w(config->actions.lpCommand) );
+                    config.u.actions->dwResetPeriod,
+                    wine_dbgstr_w(config.u.actions->lpRebootMsg),
+                    wine_dbgstr_w(config.u.actions->lpCommand) );
         break;
     case SERVICE_CONFIG_PRESHUTDOWN_INFO:
         WINE_TRACE( "changing service %p preshutdown timeout to %d\n",
-                service, config->preshutdown.dwPreshutdownTimeout );
-        service_lock_exclusive( service->service_entry );
-        service->service_entry->preshutdown_timeout = config->preshutdown.dwPreshutdownTimeout;
+                service, config.u.preshutdown->dwPreshutdownTimeout );
+        service_lock( service->service_entry );
+        service->service_entry->preshutdown_timeout = config.u.preshutdown->dwPreshutdownTimeout;
         save_service_config( service->service_entry );
         service_unlock( service->service_entry );
         break;
     default:
-        WINE_FIXME("level %u not implemented\n", level);
+        WINE_FIXME("level %u not implemented\n", config.dwInfoLevel);
         err = ERROR_INVALID_LEVEL;
         break;
     }
@@ -822,7 +845,7 @@ DWORD __cdecl svcctl_QueryServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
         {
             SERVICE_DESCRIPTIONW *descr = (SERVICE_DESCRIPTIONW *)buffer;
 
-            service_lock_shared(service->service_entry);
+            service_lock(service->service_entry);
             *needed = sizeof(*descr);
             if (service->service_entry->description)
                 *needed += (strlenW(service->service_entry->description) + 1) * sizeof(WCHAR);
@@ -842,7 +865,7 @@ DWORD __cdecl svcctl_QueryServiceConfig2W( SC_RPC_HANDLE hService, DWORD level,
         break;
 
     case SERVICE_CONFIG_PRESHUTDOWN_INFO:
-        service_lock_shared(service->service_entry);
+        service_lock(service->service_entry);
 
         *needed = sizeof(SERVICE_PRESHUTDOWN_INFO);
         if (size >= *needed)
@@ -892,7 +915,7 @@ DWORD __cdecl svcctl_QueryServiceStatusEx(
         return ERROR_INSUFFICIENT_BUFFER;
     }
 
-    service_lock_shared(service->service_entry);
+    service_lock(service->service_entry);
 
     pSvcStatusData->dwServiceType = service->service_entry->status.dwServiceType;
     pSvcStatusData->dwCurrentState = service->service_entry->status.dwCurrentState;
@@ -960,27 +983,26 @@ static BOOL service_accepts_control(const struct service_entry *service, DWORD d
 }
 
 /******************************************************************************
- * service_send_command
+ * process_send_command
  */
-BOOL service_send_command( struct service_entry *service, HANDLE pipe,
-                           const void *data, DWORD size, DWORD *result )
+BOOL process_send_command(struct process_entry *process, const void *data, DWORD size, DWORD *result)
 {
     OVERLAPPED overlapped;
     DWORD count, ret;
     BOOL r;
 
-    overlapped.hEvent = service->overlapped_event;
-    r = WriteFile(pipe, data, size, &count, &overlapped);
+    overlapped.hEvent = process->overlapped_event;
+    r = WriteFile(process->control_pipe, data, size, &count, &overlapped);
     if (!r && GetLastError() == ERROR_IO_PENDING)
     {
-        ret = WaitForSingleObject( service->overlapped_event, service_pipe_timeout );
+        ret = WaitForSingleObject(process->overlapped_event, service_pipe_timeout);
         if (ret == WAIT_TIMEOUT)
         {
             WINE_ERR("sending command timed out\n");
             *result = ERROR_SERVICE_REQUEST_TIMEOUT;
             return FALSE;
         }
-        r = GetOverlappedResult( pipe, &overlapped, &count, FALSE );
+        r = GetOverlappedResult(process->control_pipe, &overlapped, &count, FALSE);
     }
     if (!r || count != size)
     {
@@ -988,17 +1010,17 @@ BOOL service_send_command( struct service_entry *service, HANDLE pipe,
         *result  = (!r ? GetLastError() : ERROR_WRITE_FAULT);
         return FALSE;
     }
-    r = ReadFile(pipe, result, sizeof *result, &count, &overlapped);
+    r = ReadFile(process->control_pipe, result, sizeof *result, &count, &overlapped);
     if (!r && GetLastError() == ERROR_IO_PENDING)
     {
-        ret = WaitForSingleObject( service->overlapped_event, service_pipe_timeout );
+        ret = WaitForSingleObject(process->overlapped_event, service_pipe_timeout);
         if (ret == WAIT_TIMEOUT)
         {
             WINE_ERR("receiving command result timed out\n");
             *result = ERROR_SERVICE_REQUEST_TIMEOUT;
             return FALSE;
         }
-        r = GetOverlappedResult( pipe, &overlapped, &count, FALSE );
+        r = GetOverlappedResult(process->control_pipe, &overlapped, &count, FALSE);
     }
     if (!r || count != sizeof *result)
     {
@@ -1013,25 +1035,25 @@ BOOL service_send_command( struct service_entry *service, HANDLE pipe,
 }
 
 /******************************************************************************
- * service_send_control
+ * process_send_control
  */
-static BOOL service_send_control(struct service_entry *service, HANDLE pipe, DWORD dwControl, DWORD *result)
+static BOOL process_send_control(struct process_entry *process, const WCHAR *name, DWORD dwControl, DWORD *result)
 {
     service_start_info *ssi;
     DWORD len;
     BOOL r;
 
     /* calculate how much space we need to send the startup info */
-    len = strlenW(service->name) + 1;
+    len = strlenW(name) + 1;
 
     ssi = HeapAlloc(GetProcessHeap(),0,FIELD_OFFSET(service_start_info, data[len]));
     ssi->cmd = WINESERV_SENDCONTROL;
     ssi->control = dwControl;
     ssi->total_size = FIELD_OFFSET(service_start_info, data[len]);
-    ssi->name_size = strlenW(service->name) + 1;
-    strcpyW( ssi->data, service->name );
+    ssi->name_size = strlenW(name) + 1;
+    strcpyW(ssi->data, name);
 
-    r = service_send_command( service, pipe, ssi, ssi->total_size, result );
+    r = process_send_command(process, ssi, ssi->total_size, result);
     HeapFree( GetProcessHeap(), 0, ssi );
     return r;
 }
@@ -1097,7 +1119,7 @@ DWORD __cdecl svcctl_ControlService(
     if ((result = validate_service_handle(hService, access_required, &service)) != 0)
         return result;
 
-    service_lock_exclusive(service->service_entry);
+    service_lock(service->service_entry);
 
     result = ERROR_SUCCESS;
     switch (service->service_entry->status.dwCurrentState)
@@ -1114,7 +1136,8 @@ DWORD __cdecl svcctl_ControlService(
         break;
     }
 
-    if (result==ERROR_SUCCESS && !service->service_entry->control_mutex) {
+    if (result == ERROR_SUCCESS && service->service_entry->force_shutdown)
+    {
         result = ERROR_SERVICE_CANNOT_ACCEPT_CTRL;
         service_terminate(service->service_entry);
     }
@@ -1141,46 +1164,35 @@ DWORD __cdecl svcctl_ControlService(
         return ERROR_INVALID_SERVICE_CONTROL;
     }
 
-    /* prevent races by caching control_mutex and clearing it on
-     * stop instead of outside the services lock */
-    control_mutex = service->service_entry->control_mutex;
+    /* Remember that we tried to shutdown this service. When the service is
+     * still running on the second invocation, it will be forcefully killed. */
     if (dwControl == SERVICE_CONTROL_STOP)
-        service->service_entry->control_mutex = NULL;
+        service->service_entry->force_shutdown = TRUE;
 
+    control_mutex = service->service_entry->process->control_mutex;
     service_unlock(service->service_entry);
 
     ret = WaitForSingleObject(control_mutex, 30000);
-    if (ret == WAIT_OBJECT_0)
-    {
-        service_send_control(service->service_entry, service->service_entry->control_pipe,
-                dwControl, &result);
-
-        if (lpServiceStatus)
-        {
-            service_lock_shared(service->service_entry);
-            lpServiceStatus->dwServiceType = service->service_entry->status.dwServiceType;
-            lpServiceStatus->dwCurrentState = service->service_entry->status.dwCurrentState;
-            lpServiceStatus->dwControlsAccepted = service->service_entry->status.dwControlsAccepted;
-            lpServiceStatus->dwWin32ExitCode = service->service_entry->status.dwWin32ExitCode;
-            lpServiceStatus->dwServiceSpecificExitCode = service->service_entry->status.dwServiceSpecificExitCode;
-            lpServiceStatus->dwCheckPoint = service->service_entry->status.dwCheckPoint;
-            lpServiceStatus->dwWaitHint = service->service_entry->status.dwWaitHint;
-            service_unlock(service->service_entry);
-        }
-
-        if (dwControl == SERVICE_CONTROL_STOP)
-            CloseHandle(control_mutex);
-        else
-            ReleaseMutex(control_mutex);
-
-        return result;
-    }
-    else
-    {
-        if (dwControl == SERVICE_CONTROL_STOP)
-            CloseHandle(control_mutex);
+    if (ret != WAIT_OBJECT_0)
         return ERROR_SERVICE_REQUEST_TIMEOUT;
+
+    process_send_control(service->service_entry->process, service->service_entry->name, dwControl, &result);
+
+    if (lpServiceStatus)
+    {
+        service_lock(service->service_entry);
+        lpServiceStatus->dwServiceType = service->service_entry->status.dwServiceType;
+        lpServiceStatus->dwCurrentState = service->service_entry->status.dwCurrentState;
+        lpServiceStatus->dwControlsAccepted = service->service_entry->status.dwControlsAccepted;
+        lpServiceStatus->dwWin32ExitCode = service->service_entry->status.dwWin32ExitCode;
+        lpServiceStatus->dwServiceSpecificExitCode = service->service_entry->status.dwServiceSpecificExitCode;
+        lpServiceStatus->dwCheckPoint = service->service_entry->status.dwCheckPoint;
+        lpServiceStatus->dwWaitHint = service->service_entry->status.dwWaitHint;
+        service_unlock(service->service_entry);
     }
+
+    ReleaseMutex(control_mutex);
+    return result;
 }
 
 DWORD __cdecl svcctl_CloseServiceHandle(
@@ -1276,7 +1288,8 @@ DWORD __cdecl svcctl_EnumServicesStatusW(
     BYTE *buffer,
     DWORD size,
     LPDWORD needed,
-    LPDWORD returned)
+    LPDWORD returned,
+    LPDWORD resume)
 {
     DWORD err, sz, total_size, num_services;
     DWORD_PTR offset;
@@ -1284,7 +1297,7 @@ DWORD __cdecl svcctl_EnumServicesStatusW(
     struct service_entry *service;
     ENUM_SERVICE_STATUSW *s;
 
-    WINE_TRACE("(%p, 0x%x, 0x%x, %p, %u, %p, %p)\n", hmngr, type, state, buffer, size, needed, returned);
+    WINE_TRACE("(%p, 0x%x, 0x%x, %p, %u, %p, %p, %p)\n", hmngr, type, state, buffer, size, needed, returned, resume);
 
     if (!type || !state)
         return ERROR_INVALID_PARAMETER;
@@ -1292,7 +1305,10 @@ DWORD __cdecl svcctl_EnumServicesStatusW(
     if ((err = validate_scm_handle(hmngr, SC_MANAGER_ENUMERATE_SERVICE, &manager)) != ERROR_SUCCESS)
         return err;
 
-    scmdatabase_lock_exclusive(manager->db);
+    if (resume)
+        WINE_FIXME("resume index not supported\n");
+
+    scmdatabase_lock(manager->db);
 
     total_size = num_services = 0;
     LIST_FOR_EACH_ENTRY(service, &manager->db->services, struct service_entry, entry)
@@ -1363,14 +1379,32 @@ static BOOL match_group(const WCHAR *g1, const WCHAR *g2)
     return FALSE;
 }
 
+DWORD __cdecl svcctl_EnumServicesStatusExA(
+    SC_RPC_HANDLE scmanager,
+    SC_ENUM_TYPE info_level,
+    DWORD service_type,
+    DWORD service_state,
+    BYTE *buffer,
+    DWORD buf_size,
+    DWORD *needed_size,
+    DWORD *services_count,
+    DWORD *resume_index,
+    LPCSTR groupname)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
 DWORD __cdecl svcctl_EnumServicesStatusExW(
     SC_RPC_HANDLE hmngr,
+    SC_ENUM_TYPE info_level,
     DWORD type,
     DWORD state,
     BYTE *buffer,
     DWORD size,
     LPDWORD needed,
     LPDWORD returned,
+    DWORD *resume_handle,
     LPCWSTR group)
 {
     DWORD err, sz, total_size, num_services;
@@ -1382,13 +1416,16 @@ DWORD __cdecl svcctl_EnumServicesStatusExW(
     WINE_TRACE("(%p, 0x%x, 0x%x, %p, %u, %p, %p, %s)\n", hmngr, type, state, buffer, size,
                needed, returned, wine_dbgstr_w(group));
 
+    if (resume_handle)
+        FIXME("resume handle not supported\n");
+
     if (!type || !state)
         return ERROR_INVALID_PARAMETER;
 
     if ((err = validate_scm_handle(hmngr, SC_MANAGER_ENUMERATE_SERVICE, &manager)) != ERROR_SUCCESS)
         return err;
 
-    scmdatabase_lock_exclusive(manager->db);
+    scmdatabase_lock(manager->db);
 
     if (group && !find_service_by_group(manager->db, group))
     {
@@ -1448,26 +1485,178 @@ DWORD __cdecl svcctl_EnumServicesStatusExW(
     return ERROR_SUCCESS;
 }
 
-DWORD __cdecl svcctl_QueryServiceObjectSecurity(void)
+DWORD __cdecl svcctl_unknown43(void)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_SetServiceObjectSecurity(void)
+DWORD __cdecl svcctl_CreateServiceWOW64A(
+    SC_RPC_HANDLE scmanager,
+    LPCSTR servicename,
+    LPCSTR displayname,
+    DWORD accessmask,
+    DWORD service_type,
+    DWORD start_type,
+    DWORD error_control,
+    LPCSTR imagepath,
+    LPCSTR loadordergroup,
+    DWORD *tagid,
+    const BYTE *dependencies,
+    DWORD depend_size,
+    LPCSTR start_name,
+    const BYTE *password,
+    DWORD password_size,
+    SC_RPC_HANDLE *service)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_QueryServiceStatus(void)
+DWORD __cdecl svcctl_CreateServiceWOW64W(
+    SC_RPC_HANDLE scmanager,
+    LPCWSTR servicename,
+    LPCWSTR displayname,
+    DWORD accessmask,
+    DWORD service_type,
+    DWORD start_type,
+    DWORD error_control,
+    LPCWSTR imagepath,
+    LPCWSTR loadordergroup,
+    DWORD *tagid,
+    const BYTE *dependencies,
+    DWORD depend_size,
+    LPCWSTR start_name,
+    const BYTE *password,
+    DWORD password_size,
+    SC_RPC_HANDLE *service)
+{
+    WINE_TRACE("(%s, %s, 0x%x, %s)\n", wine_dbgstr_w(servicename), wine_dbgstr_w(displayname), accessmask, wine_dbgstr_w(imagepath));
+    return create_serviceW(scmanager, servicename, displayname, accessmask, service_type, start_type, error_control, imagepath,
+        loadordergroup, tagid, dependencies, depend_size, start_name, password, password_size, service, TRUE);
+}
+
+DWORD __cdecl svcctl_unknown46(void)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
+DWORD __cdecl svcctl_NotifyServiceStatusChange(
+    SC_RPC_HANDLE service,
+    SC_RPC_NOTIFY_PARAMS params,
+    GUID *clientprocessguid,
+    GUID *scmprocessguid,
+    BOOL *createremotequeue,
+    SC_NOTIFY_RPC_HANDLE *notify)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
 
-DWORD __cdecl svcctl_NotifyBootConfigStatus(void)
+DWORD __cdecl svcctl_GetNotifyResults(
+    SC_NOTIFY_RPC_HANDLE notify,
+    SC_RPC_NOTIFY_PARAMS_LIST **params)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_CloseNotifyHandle(
+    SC_NOTIFY_RPC_HANDLE *notify,
+    BOOL *apc_fired)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_ControlServiceExA(
+    SC_RPC_HANDLE service,
+    DWORD control,
+    DWORD info_level,
+    SC_RPC_SERVICE_CONTROL_IN_PARAMSA *in_params,
+    SC_RPC_SERVICE_CONTROL_OUT_PARAMSA *out_params)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_ControlServiceExW(
+    SC_RPC_HANDLE service,
+    DWORD control,
+    DWORD info_level,
+    SC_RPC_SERVICE_CONTROL_IN_PARAMSW *in_params,
+    SC_RPC_SERVICE_CONTROL_OUT_PARAMSW *out_params)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_unknown52(void)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_unknown53(void)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_unknown54(void)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_unknown55(void)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_QueryServiceConfigEx(
+    SC_RPC_HANDLE service,
+    DWORD info_level,
+    SC_RPC_CONFIG_INFOW *info)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_QueryServiceObjectSecurity(
+    SC_RPC_HANDLE service,
+    SECURITY_INFORMATION info,
+    BYTE *descriptor,
+    DWORD buf_size,
+    DWORD *needed_size)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_SetServiceObjectSecurity(
+    SC_RPC_HANDLE service,
+    SECURITY_INFORMATION info,
+    BYTE *descriptor,
+    DWORD buf_size)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_QueryServiceStatus(
+    SC_RPC_HANDLE service,
+    SERVICE_STATUS *status)
+{
+    WINE_FIXME("\n");
+    return ERROR_CALL_NOT_IMPLEMENTED;
+}
+
+DWORD __cdecl svcctl_NotifyBootConfigStatus(
+    SVCCTL_HANDLEW machinename,
+    DWORD boot_acceptable)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
@@ -1479,14 +1668,23 @@ DWORD __cdecl svcctl_SCSetServiceBitsW(void)
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-
-DWORD __cdecl svcctl_EnumDependentServicesW(void)
+DWORD __cdecl svcctl_EnumDependentServicesW(
+    SC_RPC_HANDLE service,
+    DWORD state,
+    BYTE *services,
+    DWORD buf_size,
+    DWORD *needed_size,
+    DWORD *services_ret)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_QueryServiceLockStatusW(void)
+DWORD __cdecl svcctl_QueryServiceLockStatusW(
+    SC_RPC_HANDLE scmanager,
+    QUERY_SERVICE_LOCK_STATUSW *status,
+    DWORD buf_size,
+    DWORD *needed_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
@@ -1498,67 +1696,137 @@ DWORD __cdecl svcctl_SCSetServiceBitsA(void)
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_ChangeServiceConfigA(void)
+DWORD __cdecl svcctl_ChangeServiceConfigA(
+    SC_RPC_HANDLE service,
+    DWORD service_type,
+    DWORD start_type,
+    DWORD error_control,
+    LPSTR binarypath,
+    LPSTR loadordergroup,
+    DWORD *tagid,
+    BYTE *dependencies,
+    DWORD depend_size,
+    LPSTR startname,
+    BYTE *password,
+    DWORD password_size,
+    LPSTR displayname)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_CreateServiceA(void)
+DWORD __cdecl svcctl_CreateServiceA(
+    SC_RPC_HANDLE scmanager,
+    LPCSTR servicename,
+    LPCSTR displayname,
+    DWORD desiredaccess,
+    DWORD service_type,
+    DWORD start_type,
+    DWORD error_control,
+    LPCSTR binarypath,
+    LPCSTR loadordergroup,
+    DWORD *tagid,
+    const BYTE *dependencies,
+    DWORD depend_size,
+    LPCSTR startname,
+    const BYTE *password,
+    DWORD password_size,
+    SC_RPC_HANDLE *service)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_EnumDependentServicesA(void)
+DWORD __cdecl svcctl_EnumDependentServicesA(
+    SC_RPC_HANDLE service,
+    DWORD state,
+    BYTE *services,
+    DWORD buf_size,
+    DWORD *needed_size,
+    DWORD *services_ret)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_EnumServicesStatusA(void)
+DWORD __cdecl svcctl_EnumServicesStatusA(
+    SC_RPC_HANDLE hmngr,
+    DWORD type,
+    DWORD state,
+    BYTE *buffer,
+    DWORD size,
+    DWORD *needed,
+    DWORD *returned,
+    DWORD *resume)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_OpenSCManagerA(void)
+DWORD __cdecl svcctl_OpenSCManagerA(
+    MACHINE_HANDLEA MachineName,
+    LPCSTR DatabaseName,
+    DWORD dwAccessMask,
+    SC_RPC_HANDLE *handle)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_OpenServiceA(void)
+DWORD __cdecl svcctl_OpenServiceA(
+    SC_RPC_HANDLE hSCManager,
+    LPCSTR lpServiceName,
+    DWORD dwDesiredAccess,
+    SC_RPC_HANDLE *phService)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_QueryServiceConfigA(void)
+DWORD __cdecl svcctl_QueryServiceConfigA(
+    SC_RPC_HANDLE hService,
+    QUERY_SERVICE_CONFIGA *config,
+    DWORD buf_size,
+    DWORD *needed_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_QueryServiceLockStatusA(void)
+DWORD __cdecl svcctl_QueryServiceLockStatusA(
+    SC_RPC_HANDLE scmanager,
+    QUERY_SERVICE_LOCK_STATUSA *status,
+    DWORD buf_size,
+    DWORD *needed_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_StartServiceA(void)
+DWORD __cdecl svcctl_StartServiceA(
+    SC_RPC_HANDLE service,
+    DWORD argc,
+    LPCSTR *args)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_GetServiceDisplayNameA(void)
+DWORD __cdecl svcctl_GetServiceDisplayNameA(
+    SC_RPC_HANDLE hSCManager,
+    LPCSTR servicename,
+    CHAR buffer[],
+    DWORD *buf_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_GetServiceKeyNameA(void)
+DWORD __cdecl svcctl_GetServiceKeyNameA(
+    SC_RPC_HANDLE hSCManager,
+    LPCSTR servicename,
+    CHAR buffer[],
+    DWORD *buf_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
@@ -1570,25 +1838,40 @@ DWORD __cdecl svcctl_GetCurrentGroupStateW(void)
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_EnumServiceGroupW(void)
+DWORD __cdecl svcctl_EnumServiceGroupW(
+    SC_RPC_HANDLE scmanager,
+    DWORD service_type,
+    DWORD service_state,
+    BYTE *buffer,
+    DWORD buf_size,
+    DWORD *needed_size,
+    DWORD *returned_size,
+    DWORD *resume_index,
+    LPCWSTR groupname)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_ChangeServiceConfig2A(void)
+DWORD __cdecl svcctl_ChangeServiceConfig2A(
+    SC_RPC_HANDLE service,
+    SC_RPC_CONFIG_INFOA info)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-DWORD __cdecl svcctl_QueryServiceConfig2A(void)
+DWORD __cdecl svcctl_QueryServiceConfig2A(
+    SC_RPC_HANDLE service,
+    DWORD info_level,
+    BYTE *buffer,
+    DWORD buf_size,
+    DWORD *needed_size)
 {
     WINE_FIXME("\n");
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-
 DWORD RPC_Init(void)
 {
     WCHAR transport[] = SVCCTL_TRANSPORT;
@@ -1619,7 +1902,7 @@ DWORD events_loop(void)
 {
     struct timeout_queue_elem *iter, *iter_safe;
     DWORD err;
-    HANDLE wait_handles[2];
+    HANDLE wait_handles[MAXIMUM_WAIT_OBJECTS];
     DWORD timeout = INFINITE;
 
     wait_handles[0] = __wine_make_process_system();
@@ -1632,13 +1915,29 @@ DWORD events_loop(void)
 
     do
     {
-        err = WaitForMultipleObjects(2, wait_handles, FALSE, timeout);
+        DWORD num_handles = 2;
+
+        /* monitor tracked process handles for process end */
+        EnterCriticalSection(&timeout_queue_cs);
+        LIST_FOR_EACH_ENTRY(iter, &timeout_queue, struct timeout_queue_elem, entry)
+        {
+            if(num_handles == MAXIMUM_WAIT_OBJECTS){
+                WINE_TRACE("Exceeded maximum wait object count\n");
+                break;
+            }
+            wait_handles[num_handles] = iter->service_entry->process->process;
+            num_handles++;
+        }
+        LeaveCriticalSection(&timeout_queue_cs);
+
+        err = WaitForMultipleObjects(num_handles, wait_handles, FALSE, timeout);
         WINE_TRACE("Wait returned %d\n", err);
 
-        if(err==WAIT_OBJECT_0+1 || err==WAIT_TIMEOUT)
+        if(err > WAIT_OBJECT_0 || err == WAIT_TIMEOUT)
         {
             FILETIME cur_time;
             ULARGE_INTEGER time;
+            DWORD idx = 0;
 
             GetSystemTimeAsFileTime(&cur_time);
             time.u.LowPart = cur_time.dwLowDateTime;
@@ -1648,7 +1947,8 @@ DWORD events_loop(void)
             timeout = INFINITE;
             LIST_FOR_EACH_ENTRY_SAFE(iter, iter_safe, &timeout_queue, struct timeout_queue_elem, entry)
             {
-                if(CompareFileTime(&cur_time, &iter->time) >= 0)
+                if(CompareFileTime(&cur_time, &iter->time) >= 0 ||
+                        (err > WAIT_OBJECT_0 + 1 && idx == err - WAIT_OBJECT_0 - 2))
                 {
                     LeaveCriticalSection(&timeout_queue_cs);
                     iter->func(iter->service_entry);
@@ -1669,6 +1969,7 @@ DWORD events_loop(void)
                     if(time_diff.QuadPart < timeout)
                         timeout = time_diff.QuadPart;
                 }
+                idx++;
             }
             LeaveCriticalSection(&timeout_queue_cs);
 
@@ -1701,6 +2002,10 @@ void __RPC_USER SC_RPC_HANDLE_rundown(SC_RPC_HANDLE handle)
     SC_RPC_HANDLE_destroy(handle);
 }
 
+void __RPC_USER SC_NOTIFY_RPC_HANDLE_rundown(SC_NOTIFY_RPC_HANDLE handle)
+{
+}
+
 void  __RPC_FAR * __RPC_USER MIDL_user_allocate(SIZE_T len)
 {
     return HeapAlloc(GetProcessHeap(), 0, len);
