@@ -68,8 +68,31 @@ static const WCHAR SZ_DESCRIPTION[]       = {'D','e','s','c','r','i','p','t','i'
 static const WCHAR SZ_PRESHUTDOWN[]       = {'P','r','e','s','h','u','t','d','o','w','n','T','i','m','e','o','u','t',0};
 static const WCHAR SZ_WOW64[]             = {'W','O','W','6','4',0};
 
+static DWORD process_create(struct process_entry **entry)
+{
+    *entry = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(**entry));
+    if (!*entry)
+        return ERROR_NOT_ENOUGH_SERVER_MEMORY;
+    (*entry)->ref_count = 1;
+    (*entry)->control_pipe = INVALID_HANDLE_VALUE;
+    /* all other fields are zero */
+    return ERROR_SUCCESS;
+}
+
+static void free_process_entry(struct process_entry *entry)
+{
+    CloseHandle(entry->process);
+    CloseHandle(entry->control_mutex);
+    CloseHandle(entry->control_pipe);
+    CloseHandle(entry->overlapped_event);
+    CloseHandle(entry->status_changed_event);
+    HeapFree(GetProcessHeap(), 0, entry);
+}
+
 DWORD service_create(LPCWSTR name, struct service_entry **entry)
 {
+    DWORD err;
+
     *entry = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(**entry));
     if (!*entry)
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
@@ -79,7 +102,13 @@ DWORD service_create(LPCWSTR name, struct service_entry **entry)
         HeapFree(GetProcessHeap(), 0, *entry);
         return ERROR_NOT_ENOUGH_SERVER_MEMORY;
     }
-    (*entry)->control_pipe = INVALID_HANDLE_VALUE;
+    if ((err = process_create(&(*entry)->process)) != ERROR_SUCCESS)
+    {
+        HeapFree(GetProcessHeap(), 0, (*entry)->name);
+        HeapFree(GetProcessHeap(), 0, *entry);
+        return err;
+    }
+    (*entry)->ref_count = 1;
     (*entry)->status.dwCurrentState = SERVICE_STOPPED;
     (*entry)->status.dwWin32ExitCode = ERROR_SERVICE_NEVER_STARTED;
     (*entry)->preshutdown_timeout = default_preshutdown_timeout;
@@ -98,11 +127,7 @@ void free_service_entry(struct service_entry *entry)
     HeapFree(GetProcessHeap(), 0, entry->description);
     HeapFree(GetProcessHeap(), 0, entry->dependOnServices);
     HeapFree(GetProcessHeap(), 0, entry->dependOnGroups);
-    CloseHandle(entry->process);
-    CloseHandle(entry->control_mutex);
-    CloseHandle(entry->control_pipe);
-    CloseHandle(entry->overlapped_event);
-    CloseHandle(entry->status_changed_event);
+    release_process(entry->process);
     HeapFree(GetProcessHeap(), 0, entry);
 }
 
@@ -281,7 +306,7 @@ static void scmdatabase_autostart_services(struct scmdatabase *db)
     if (!services_list)
         return;
 
-    scmdatabase_lock_shared(db);
+    scmdatabase_lock(db);
 
     LIST_FOR_EACH_ENTRY(service, &db->services, struct service_entry, entry)
     {
@@ -299,7 +324,7 @@ static void scmdatabase_autostart_services(struct scmdatabase *db)
                 services_list = slist_new;
             }
             services_list[i] = service;
-            service->ref_count++;
+            InterlockedIncrement(&service->ref_count);
             i++;
         }
     }
@@ -326,19 +351,20 @@ static void scmdatabase_wait_terminate(struct scmdatabase *db)
     struct service_entry *service;
     BOOL run = TRUE;
 
-    scmdatabase_lock_shared(db);
+    scmdatabase_lock(db);
     while(run)
     {
         run = FALSE;
         LIST_FOR_EACH_ENTRY(service, &db->services, struct service_entry, entry)
         {
-            if(service->process)
+            struct process_entry *process = service->process;
+            if (process->process)
             {
                 scmdatabase_unlock(db);
-                WaitForSingleObject(service->process, INFINITE);
-                scmdatabase_lock_shared(db);
-                CloseHandle(service->process);
-                service->process = NULL;
+                WaitForSingleObject(process->process, INFINITE);
+                scmdatabase_lock(db);
+                CloseHandle(process->process);
+                process->process = NULL;
                 run = TRUE;
                 break;
             }
@@ -430,17 +456,23 @@ struct service_entry *scmdatabase_find_service_by_displayname(struct scmdatabase
     return NULL;
 }
 
+void release_process(struct process_entry *process)
+{
+    if (InterlockedDecrement(&process->ref_count) == 0)
+        free_process_entry(process);
+}
+
 void release_service(struct service_entry *service)
 {
+    struct scmdatabase *db = service->db;
+
+    scmdatabase_lock(db);
     if (InterlockedDecrement(&service->ref_count) == 0 && is_marked_for_delete(service))
     {
-        scmdatabase_lock_exclusive(service->db);
-        service_lock_exclusive(service);
-        scmdatabase_remove_service(service->db, service);
-        service_unlock(service);
-        scmdatabase_unlock(service->db);
+        scmdatabase_remove_service(db, service);
         free_service_entry(service);
     }
+    scmdatabase_unlock(db);
 }
 
 static DWORD scmdatabase_create(struct scmdatabase **db)
@@ -533,6 +565,7 @@ static DWORD scmdatabase_load_services(struct scmdatabase *db)
         entry->db = db;
 
         list_add_tail(&db->services, &entry->entry);
+        release_service(entry);
     }
     return ERROR_SUCCESS;
 }
@@ -549,12 +582,7 @@ void scmdatabase_unlock_startup(struct scmdatabase *db)
     InterlockedCompareExchange(&db->service_start_lock, FALSE, TRUE);
 }
 
-void scmdatabase_lock_shared(struct scmdatabase *db)
-{
-    EnterCriticalSection(&db->cs);
-}
-
-void scmdatabase_lock_exclusive(struct scmdatabase *db)
+void scmdatabase_lock(struct scmdatabase *db)
 {
     EnterCriticalSection(&db->cs);
 }
@@ -564,12 +592,7 @@ void scmdatabase_unlock(struct scmdatabase *db)
     LeaveCriticalSection(&db->cs);
 }
 
-void service_lock_shared(struct service_entry *service)
-{
-    EnterCriticalSection(&service->db->cs);
-}
-
-void service_lock_exclusive(struct service_entry *service)
+void service_lock(struct service_entry *service)
 {
     EnterCriticalSection(&service->db->cs);
 }
@@ -631,7 +654,8 @@ static DWORD get_service_binary_path(const struct service_entry *service_entry,
 
     ExpandEnvironmentStringsW(service_entry->config.lpBinaryPathName, *path, size);
 
-    if (service_entry->config.dwServiceType == SERVICE_KERNEL_DRIVER)
+    if (service_entry->config.dwServiceType == SERVICE_KERNEL_DRIVER ||
+        service_entry->config.dwServiceType == SERVICE_FILE_SYSTEM_DRIVER)
     {
         static const WCHAR winedeviceW[] = {'\\','w','i','n','e','d','e','v','i','c','e','.','e','x','e',' ',0};
         WCHAR system_dir[MAX_PATH];
@@ -690,7 +714,7 @@ static DWORD get_service_binary_path(const struct service_entry *service_entry,
     return ERROR_SUCCESS;
 }
 
-static DWORD service_start_process(struct service_entry *service_entry, HANDLE *process)
+static DWORD service_start_process(struct service_entry *service_entry)
 {
     PROCESS_INFORMATION pi;
     STARTUPINFOW si;
@@ -698,7 +722,7 @@ static DWORD service_start_process(struct service_entry *service_entry, HANDLE *
     DWORD err;
     BOOL r;
 
-    service_lock_exclusive(service_entry);
+    service_lock(service_entry);
 
     if (!env)
     {
@@ -733,51 +757,44 @@ static DWORD service_start_process(struct service_entry *service_entry, HANDLE *
     HeapFree(GetProcessHeap(),0,path);
     if (!r)
     {
-        service_lock_exclusive(service_entry);
+        service_lock(service_entry);
         service_entry->status.dwCurrentState = SERVICE_STOPPED;
         service_unlock(service_entry);
         return GetLastError();
     }
 
     service_entry->status.dwProcessId = pi.dwProcessId;
-    service_entry->process = pi.hProcess;
-    *process = pi.hProcess;
+    service_entry->process->process = pi.hProcess;
     CloseHandle( pi.hThread );
 
     return ERROR_SUCCESS;
 }
 
-static DWORD service_wait_for_startup(struct service_entry *service_entry, HANDLE process_handle)
+static DWORD process_wait_for_startup(struct process_entry *process)
 {
-    HANDLE handles[2] = { service_entry->status_changed_event, process_handle };
-    DWORD state, ret;
-
-    WINE_TRACE("%p\n", service_entry);
+    HANDLE handles[2] = { process->status_changed_event, process->process };
+    DWORD ret;
 
     ret = WaitForMultipleObjects( 2, handles, FALSE, service_pipe_timeout );
-    if (ret != WAIT_OBJECT_0)
-        return ERROR_SERVICE_REQUEST_TIMEOUT;
-    service_lock_shared(service_entry);
-    state = service_entry->status.dwCurrentState;
-    service_unlock(service_entry);
-    if (state == SERVICE_START_PENDING)
-    {
-        WINE_TRACE("Service state changed to SERVICE_START_PENDING\n");
-        return ERROR_SUCCESS;
-    }
-    else if (state == SERVICE_RUNNING)
-    {
-        WINE_TRACE("Service started successfully\n");
-        return ERROR_SUCCESS;
-    }
+    return (ret == WAIT_OBJECT_0) ? ERROR_SUCCESS : ERROR_SERVICE_REQUEST_TIMEOUT;
+}
 
-    return ERROR_SERVICE_REQUEST_TIMEOUT;
+static DWORD service_is_running(struct service_entry *service)
+{
+    DWORD state;
+
+    service_lock(service);
+    state = service->status.dwCurrentState;
+    service_unlock(service);
+
+    return (state == SERVICE_START_PENDING || state == SERVICE_RUNNING) ?
+           ERROR_SUCCESS : ERROR_SERVICE_REQUEST_TIMEOUT;
 }
 
 /******************************************************************************
- * service_send_start_message
+ * process_send_start_message
  */
-static BOOL service_send_start_message(struct service_entry *service, HANDLE process_handle,
+static BOOL process_send_start_message(struct process_entry *process, const WCHAR *name,
                                        LPCWSTR *argv, DWORD argc)
 {
     OVERLAPPED overlapped;
@@ -786,21 +803,21 @@ static BOOL service_send_start_message(struct service_entry *service, HANDLE pro
     LPWSTR p;
     BOOL r;
 
-    WINE_TRACE("%s %p %d\n", wine_dbgstr_w(service->name), argv, argc);
+    WINE_TRACE("%p %s %p %d\n", process, wine_dbgstr_w(name), argv, argc);
 
-    overlapped.hEvent = service->overlapped_event;
-    if (!ConnectNamedPipe(service->control_pipe, &overlapped))
+    overlapped.hEvent = process->overlapped_event;
+    if (!ConnectNamedPipe(process->control_pipe, &overlapped))
     {
         if (GetLastError() == ERROR_IO_PENDING)
         {
             HANDLE handles[2];
-            handles[0] = service->overlapped_event;
-            handles[1] = process_handle;
+            handles[0] = process->overlapped_event;
+            handles[1] = process->process;
             if (WaitForMultipleObjects( 2, handles, FALSE, service_pipe_timeout ) != WAIT_OBJECT_0)
-                CancelIo( service->control_pipe );
+                CancelIo(process->control_pipe);
             if (!HasOverlappedIoCompleted( &overlapped ))
             {
-                WINE_ERR( "service %s failed to start\n", wine_dbgstr_w( service->name ));
+                WINE_ERR("service %s failed to start\n", wine_dbgstr_w(name));
                 return FALSE;
             }
         }
@@ -812,7 +829,7 @@ static BOOL service_send_start_message(struct service_entry *service, HANDLE pro
     }
 
     /* calculate how much space do we need to send the startup info */
-    len = strlenW(service->name) + 1;
+    len = strlenW(name) + 1;
     for (i=0; i<argc; i++)
         len += strlenW(argv[i])+1;
     len++;
@@ -821,8 +838,8 @@ static BOOL service_send_start_message(struct service_entry *service, HANDLE pro
     ssi->cmd = WINESERV_STARTINFO;
     ssi->control = 0;
     ssi->total_size = FIELD_OFFSET(service_start_info, data[len]);
-    ssi->name_size = strlenW(service->name) + 1;
-    strcpyW( ssi->data, service->name );
+    ssi->name_size = strlenW(name) + 1;
+    strcpyW(ssi->data, name);
 
     /* copy service args into a single buffer*/
     p = &ssi->data[ssi->name_size];
@@ -833,7 +850,7 @@ static BOOL service_send_start_message(struct service_entry *service, HANDLE pro
     }
     *p=0;
 
-    r = service_send_command( service, service->control_pipe, ssi, ssi->total_size, &result );
+    r = process_send_command( process, ssi, ssi->total_size, &result );
     if (r && result)
     {
         SetLastError(result);
@@ -847,33 +864,34 @@ static BOOL service_send_start_message(struct service_entry *service, HANDLE pro
 
 DWORD service_start(struct service_entry *service, DWORD service_argc, LPCWSTR *service_argv)
 {
+    struct process_entry *process = service->process;
     DWORD err;
     LPWSTR name;
-    HANDLE process_handle = NULL;
 
     err = scmdatabase_lock_startup(service->db);
     if (err != ERROR_SUCCESS)
         return err;
 
-    if (WaitForSingleObject(service->process, 0) == WAIT_TIMEOUT)
+    if (WaitForSingleObject(process->process, 0) == WAIT_TIMEOUT)
     {
         scmdatabase_unlock_startup(service->db);
         return ERROR_SERVICE_ALREADY_RUNNING;
     }
 
-    CloseHandle(service->control_pipe);
-    service->control_mutex = CreateMutexW(NULL, TRUE, NULL);
+    CloseHandle(process->control_pipe);
+    process->control_mutex = CreateMutexW(NULL, TRUE, NULL);
+    service->force_shutdown = FALSE;
 
-    if (!service->status_changed_event)
-        service->status_changed_event = CreateEventW(NULL, FALSE, FALSE, NULL);
-    if (!service->overlapped_event)
-        service->overlapped_event = CreateEventW(NULL, TRUE, FALSE, NULL);
+    if (!process->status_changed_event)
+        process->status_changed_event = CreateEventW(NULL, FALSE, FALSE, NULL);
+    if (!process->overlapped_event)
+        process->overlapped_event = CreateEventW(NULL, TRUE, FALSE, NULL);
 
     name = service_get_pipe_name();
-    service->control_pipe = CreateNamedPipeW(name, PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
+    process->control_pipe = CreateNamedPipeW(name, PIPE_ACCESS_DUPLEX | FILE_FLAG_OVERLAPPED,
                   PIPE_TYPE_BYTE|PIPE_WAIT, 1, 256, 256, 10000, NULL );
     HeapFree(GetProcessHeap(), 0, name);
-    if (service->control_pipe==INVALID_HANDLE_VALUE)
+    if (process->control_pipe == INVALID_HANDLE_VALUE)
     {
         WINE_ERR("failed to create pipe for %s, error = %d\n",
             wine_dbgstr_w(service->name), GetLastError());
@@ -881,19 +899,22 @@ DWORD service_start(struct service_entry *service, DWORD service_argc, LPCWSTR *
     }
     else
     {
-        err = service_start_process(service, &process_handle);
+        err = service_start_process(service);
         if (err == ERROR_SUCCESS)
         {
-            if (!service_send_start_message(service, process_handle, service_argv, service_argc))
+            if (!process_send_start_message(process, service->name, service_argv, service_argc))
                 err = ERROR_SERVICE_REQUEST_TIMEOUT;
         }
 
         if (err == ERROR_SUCCESS)
-            err = service_wait_for_startup(service, process_handle);
+            err = process_wait_for_startup(process);
+
+        if (err == ERROR_SUCCESS)
+            err = service_is_running(service);
     }
 
     if (err == ERROR_SUCCESS)
-        ReleaseMutex(service->control_mutex);
+        ReleaseMutex(process->control_mutex);
     else
         service_terminate(service);
     scmdatabase_unlock_startup(service->db);
@@ -905,16 +926,18 @@ DWORD service_start(struct service_entry *service, DWORD service_argc, LPCWSTR *
 
 void service_terminate(struct service_entry *service)
 {
-    service_lock_exclusive(service);
-    TerminateProcess(service->process, 0);
-    CloseHandle(service->process);
-    service->process = NULL;
-    CloseHandle(service->status_changed_event);
-    service->status_changed_event = NULL;
-    CloseHandle(service->control_mutex);
-    service->control_mutex = NULL;
-    CloseHandle(service->control_pipe);
-    service->control_pipe = INVALID_HANDLE_VALUE;
+    struct process_entry *process = service->process;
+
+    service_lock(service);
+    TerminateProcess(process->process, 0);
+    CloseHandle(process->process);
+    process->process = NULL;
+    CloseHandle(process->status_changed_event);
+    process->status_changed_event = NULL;
+    CloseHandle(process->control_mutex);
+    process->control_mutex = NULL;
+    CloseHandle(process->control_pipe);
+    process->control_pipe = INVALID_HANDLE_VALUE;
 
     service->status.dwProcessId = 0;
     service->status.dwCurrentState = SERVICE_STOPPED;
