@@ -148,13 +148,14 @@ static struct security_descriptor *dir_get_sd( struct object *obj );
 static int dir_set_sd( struct object *obj, const struct security_descriptor *sd,
                        unsigned int set_info );
 static void dir_dump( struct object *obj, int verbose );
+static struct object_type *dir_get_type( struct object *obj );
 static void dir_destroy( struct object *obj );
 
 static const struct object_ops dir_ops =
 {
     sizeof(struct dir),       /* size */
     dir_dump,                 /* dump */
-    no_get_type,              /* get_type */
+    dir_get_type,             /* get_type */
     add_queue,                /* add_queue */
     remove_queue,             /* remove_queue */
     default_fd_signaled,      /* signaled */
@@ -165,6 +166,8 @@ static const struct object_ops dir_ops =
     dir_get_sd,               /* get_sd */
     dir_set_sd,               /* set_sd */
     no_lookup_name,           /* lookup_name */
+    no_link_name,             /* link_name */
+    NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
     fd_close_handle,          /* close_handle */
     dir_destroy               /* destroy */
@@ -250,6 +253,13 @@ static void dir_dump( struct object *obj, int verbose )
     fprintf( stderr, "Dirfile fd=%p filter=%08x\n", dir->fd, dir->filter );
 }
 
+static struct object_type *dir_get_type( struct object *obj )
+{
+    static const WCHAR name[] = {'F','i','l','e'};
+    static const struct unicode_str str = { name, sizeof(name) };
+    return get_object_type( &str );
+}
+
 /* enter here directly from SIGIO signal handler */
 void do_change_notify( int unix_fd )
 {
