@@ -64,6 +64,8 @@ static const struct object_ops object_type_ops =
     default_get_sd,               /* get_sd */
     default_set_sd,               /* set_sd */
     no_lookup_name,               /* lookup_name */
+    directory_link_name,          /* link_name */
+    default_unlink_name,          /* unlink_name */
     no_open_file,                 /* open_file */
     no_close_handle,              /* close_handle */
     no_destroy                    /* destroy */
@@ -97,6 +99,8 @@ static const struct object_ops directory_ops =
     default_get_sd,               /* get_sd */
     default_set_sd,               /* set_sd */
     directory_lookup_name,        /* lookup_name */
+    directory_link_name,          /* link_name */
+    default_unlink_name,          /* unlink_name */
     no_open_file,                 /* open_file */
     no_close_handle,              /* close_handle */
     directory_destroy             /* destroy */
@@ -108,11 +112,7 @@ static struct directory *dir_objtype;
 
 static void object_type_dump( struct object *obj, int verbose )
 {
-    assert( obj->ops == &object_type_ops );
-
-    fputs( "Object type ", stderr );
-    dump_object_name( obj );
-    fputc( '\n', stderr );
+    fputs( "Object type\n", stderr );
 }
 
 static struct object_type *object_type_get_type( struct object *obj )
@@ -124,11 +124,7 @@ static struct object_type *object_type_get_type( struct object *obj )
 
 static void directory_dump( struct object *obj, int verbose )
 {
-    assert( obj->ops == &directory_ops );
-
-    fputs( "Directory ", stderr );
-    dump_object_name( obj );
-    fputc( '\n', stderr );
+    fputs( "Directory\n", stderr );
 }
 
 static struct object_type *directory_get_type( struct object *obj )
@@ -148,6 +144,8 @@ static struct object *directory_lookup_name( struct object *obj, struct unicode_
 
     assert( obj->ops == &directory_ops );
 
+    if (!name) return NULL;  /* open the directory itself */
+
     if (!(p = memchrW( name->str, '\\', name->len / sizeof(WCHAR) )))
         /* Last element in the path name */
         tmp.len = name->len;
@@ -169,18 +167,30 @@ static struct object *directory_lookup_name( struct object *obj, struct unicode_
         return found;
     }
 
-    if (name->str)
+    if (name->str)  /* not the last element */
     {
         if (tmp.len == 0) /* Double backslash */
             set_error( STATUS_OBJECT_NAME_INVALID );
         else if (p)  /* Path still has backslashes */
             set_error( STATUS_OBJECT_PATH_NOT_FOUND );
-        else
-            clear_error();
     }
     return NULL;
 }
 
+int directory_link_name( struct object *obj, struct object_name *name, struct object *parent )
+{
+    struct directory *dir = (struct directory *)parent;
+
+    if (parent->ops != &directory_ops)
+    {
+        set_error( STATUS_OBJECT_TYPE_MISMATCH );
+        return 0;
+    }
+    namespace_add( dir->entries, name );
+    name->parent = grab_object( parent );
+    return 1;
+}
+
 static void directory_destroy( struct object *obj )
 {
     struct directory *dir = (struct directory *)obj;
@@ -188,157 +198,33 @@ static void directory_destroy( struct object *obj )
     free( dir->entries );
 }
 
-static struct directory *create_directory( struct directory *root, const struct unicode_str *name,
-                                           unsigned int attr, unsigned int hash_size )
+static struct directory *create_directory( struct object *root, const struct unicode_str *name,
+                                           unsigned int attr, unsigned int hash_size,
+                                           const struct security_descriptor *sd )
 {
     struct directory *dir;
 
-    if ((dir = create_named_object_dir( root, name, attr, &directory_ops )) &&
+    if ((dir = create_named_object( root, &directory_ops, name, attr, sd )) &&
         get_error() != STATUS_OBJECT_NAME_EXISTS)
     {
         if (!(dir->entries = create_namespace( hash_size )))
         {
             release_object( dir );
-            dir = NULL;
+            return NULL;
         }
     }
     return dir;
 }
 
-struct directory *get_directory_obj( struct process *process, obj_handle_t handle, unsigned int access )
+struct object *get_root_directory(void)
 {
-    return (struct directory *)get_handle_obj( process, handle, access, &directory_ops );
+    return grab_object( root_directory );
 }
 
-/******************************************************************************
- * Find an object by its name in a given root object
- *
- * PARAMS
- *  root      [I] directory to start search from or NULL to start from \\
- *  name      [I] object name to search for
- *  attr      [I] OBJECT_ATTRIBUTES.Attributes
- *  name_left [O] [optional] leftover name if object is not found
- *
- * RETURNS
- *  NULL:      If params are invalid
- *  Found:     If object with exact name is found returns that object
- *             (name_left->len == 0). Object's refcount is incremented
- *  Not found: The last matched parent. (name_left->len > 0)
- *             Parent's refcount is incremented.
- */
-struct object *find_object_dir( struct directory *root, const struct unicode_str *name,
-                                unsigned int attr, struct unicode_str *name_left )
+/* return a directory object for creating/opening some object; no access rights are required */
+struct object *get_directory_obj( struct process *process, obj_handle_t handle )
 {
-    struct object *obj, *parent;
-    struct unicode_str name_tmp;
-
-    if (name) name_tmp = *name;
-    else name_tmp.len = 0;
-
-    /* Arguments check:
-     * - Either rootdir or name have to be specified
-     * - If root is specified path shouldn't start with backslash */
-    if (root)
-    {
-        if (name_tmp.len && name_tmp.str[0] == '\\')
-        {
-            set_error( STATUS_OBJECT_PATH_SYNTAX_BAD );
-            return NULL;
-        }
-        parent = grab_object( root );
-    }
-    else
-    {
-        if (!name_tmp.len || name_tmp.str[0] != '\\')
-        {
-            set_error( STATUS_OBJECT_PATH_SYNTAX_BAD );
-            return NULL;
-        }
-        parent = grab_object( &root_directory->obj );
-        /* skip leading backslash */
-        name_tmp.str++;
-        name_tmp.len -= sizeof(WCHAR);
-    }
-
-    /* Special case for opening RootDirectory */
-    if (!name_tmp.len) goto done;
-
-    while ((obj = parent->ops->lookup_name( parent, &name_tmp, attr )))
-    {
-        /* move to the next element */
-        release_object ( parent );
-        parent = obj;
-    }
-    if (get_error())
-    {
-        release_object( parent );
-        return NULL;
-    }
-
-    done:
-    if (name_left) *name_left = name_tmp;
-    return parent;
-}
-
-/* create a named (if name is present) or unnamed object. */
-void *create_named_object_dir( struct directory *root, const struct unicode_str *name,
-                               unsigned int attributes, const struct object_ops *ops )
-{
-    struct object *obj, *new_obj = NULL;
-    struct unicode_str new_name;
-
-    if (!name || !name->len) return alloc_object( ops );
-
-    if (!(obj = find_object_dir( root, name, attributes, &new_name ))) return NULL;
-    if (!new_name.len)
-    {
-        if (attributes & OBJ_OPENIF && obj->ops == ops)
-            set_error( STATUS_OBJECT_NAME_EXISTS );
-        else
-        {
-            release_object( obj );
-            obj = NULL;
-            if (attributes & OBJ_OPENIF)
-                set_error( STATUS_OBJECT_TYPE_MISMATCH );
-            else
-                set_error( STATUS_OBJECT_NAME_COLLISION );
-        }
-        return obj;
-    }
-
-    /* ATM we can't insert objects into anything else but directories */
-    if (obj->ops != &directory_ops)
-        set_error( STATUS_OBJECT_TYPE_MISMATCH );
-    else
-    {
-        struct directory *dir = (struct directory *)obj;
-        if ((new_obj = create_object( dir->entries, ops, &new_name, &dir->obj )))
-            clear_error();
-    }
-
-    release_object( obj );
-    return new_obj;
-}
-
-/* open a new handle to an existing object */
-void *open_object_dir( struct directory *root, const struct unicode_str *name,
-                       unsigned int attr, const struct object_ops *ops )
-{
-    struct unicode_str name_left;
-    struct object *obj;
-
-    if ((obj = find_object_dir( root, name, attr, &name_left )))
-    {
-        if (name_left.len) /* not fully parsed */
-            set_error( STATUS_OBJECT_NAME_NOT_FOUND );
-        else if (ops && obj->ops != ops)
-            set_error( STATUS_OBJECT_TYPE_MISMATCH );
-        else
-            return obj;
-
-        release_object( obj );
-    }
-    return NULL;
+    return get_handle_obj( process, handle, 0, &directory_ops );
 }
 
 /* retrieve an object type, creating it if needed */
@@ -346,13 +232,13 @@ struct object_type *get_object_type( const struct unicode_str *name )
 {
     struct object_type *type;
 
-    if ((type = open_object_dir( dir_objtype, name, 0, &object_type_ops )))
-        return type;
-
-    if ((type = create_named_object_dir( dir_objtype, name, 0, &object_type_ops )))
+    if ((type = create_named_object( &dir_objtype->obj, &object_type_ops, name, OBJ_OPENIF, NULL )))
     {
-        grab_object( type );
-        make_object_static( &type->obj );
+        if (get_error() != STATUS_OBJECT_NAME_EXISTS)
+        {
+            grab_object( type );
+            make_object_static( &type->obj );
+        }
         clear_error();
     }
     return type;
@@ -373,6 +259,8 @@ void init_directories(void)
     static const WCHAR dir_objtypeW[] = {'O','b','j','e','c','t','T','y','p','e','s'};
     static const WCHAR dir_sessionsW[] = {'S','e','s','s','i','o','n','s'};
     static const WCHAR dir_kernelW[] = {'K','e','r','n','e','l','O','b','j','e','c','t','s'};
+    static const WCHAR dir_windowsW[] = {'W','i','n','d','o','w','s'};
+    static const WCHAR dir_winstationsW[] = {'W','i','n','d','o','w','S','t','a','t','i','o','n','s'};
     static const struct unicode_str dir_global_str = {dir_globalW, sizeof(dir_globalW)};
     static const struct unicode_str dir_driver_str = {dir_driverW, sizeof(dir_driverW)};
     static const struct unicode_str dir_device_str = {dir_deviceW, sizeof(dir_deviceW)};
@@ -383,6 +271,8 @@ void init_directories(void)
     static const struct unicode_str dir_objtype_str = {dir_objtypeW, sizeof(dir_objtypeW)};
     static const struct unicode_str dir_sessions_str = {dir_sessionsW, sizeof(dir_sessionsW)};
     static const struct unicode_str dir_kernel_str = {dir_kernelW, sizeof(dir_kernelW)};
+    static const struct unicode_str dir_windows_str = {dir_windowsW, sizeof(dir_windowsW)};
+    static const struct unicode_str dir_winstations_str = {dir_winstationsW, sizeof(dir_winstationsW)};
 
     /* symlinks */
     static const WCHAR link_dosdevW[] = {'D','o','s','D','e','v','i','c','e','s'};
@@ -431,40 +321,43 @@ void init_directories(void)
     };
     static const struct unicode_str keyed_event_crit_sect_str = {keyed_event_crit_sectW, sizeof(keyed_event_crit_sectW)};
 
-    struct directory *dir_driver, *dir_device, *dir_global, *dir_basenamed, *dir_sessions, *dir_kernel;
+    struct directory *dir_driver, *dir_device, *dir_global, *dir_basenamed, *dir_sessions, *dir_kernel, *dir_windows, *dir_winstation;
     struct symlink *link_dosdev, *link_global1, *link_global2, *link_local, *link_nul, *link_pipe, *link_mailslot, *link_0, *link_session;
     struct keyed_event *keyed_event;
     unsigned int i;
 
-    root_directory = create_directory( NULL, NULL, 0, HASH_SIZE );
-    dir_driver     = create_directory( root_directory, &dir_driver_str, 0, HASH_SIZE );
-    dir_device     = create_directory( root_directory, &dir_device_str, 0, HASH_SIZE );
-    dir_objtype    = create_directory( root_directory, &dir_objtype_str, 0, HASH_SIZE );
-    dir_sessions   = create_directory( root_directory, &dir_sessions_str, 0, HASH_SIZE );
-    dir_kernel     = create_directory( root_directory, &dir_kernel_str, 0, HASH_SIZE );
+    root_directory = create_directory( NULL, NULL, 0, HASH_SIZE, NULL );
+    dir_driver     = create_directory( &root_directory->obj, &dir_driver_str, 0, HASH_SIZE, NULL );
+    dir_device     = create_directory( &root_directory->obj, &dir_device_str, 0, HASH_SIZE, NULL );
+    dir_objtype    = create_directory( &root_directory->obj, &dir_objtype_str, 0, HASH_SIZE, NULL );
+    dir_sessions   = create_directory( &root_directory->obj, &dir_sessions_str, 0, HASH_SIZE, NULL );
+    dir_kernel     = create_directory( &root_directory->obj, &dir_kernel_str, 0, HASH_SIZE, NULL );
+    dir_windows    = create_directory( &root_directory->obj, &dir_windows_str, 0, HASH_SIZE, NULL );
+    dir_winstation = create_directory( &dir_windows->obj, &dir_winstations_str, 0, HASH_SIZE, NULL );
     make_object_static( &root_directory->obj );
     make_object_static( &dir_driver->obj );
     make_object_static( &dir_objtype->obj );
+    make_object_static( &dir_winstation->obj );
 
-    dir_global     = create_directory( NULL, &dir_global_str, 0, HASH_SIZE );
+    dir_global     = create_directory( NULL, &dir_global_str, 0, HASH_SIZE, NULL );
     /* use a larger hash table for this one since it can contain a lot of objects */
-    dir_basenamed  = create_directory( NULL, &dir_basenamed_str, 0, 37 );
+    dir_basenamed  = create_directory( NULL, &dir_basenamed_str, 0, 37, NULL );
 
     /* devices */
-    create_named_pipe_device( dir_device, &named_pipe_str );
-    create_mailslot_device( dir_device, &mailslot_str );
-    create_unix_device( dir_device, &null_str, "/dev/null" );
+    create_named_pipe_device( &dir_device->obj, &named_pipe_str );
+    create_mailslot_device( &dir_device->obj, &mailslot_str );
+    create_unix_device( &dir_device->obj, &null_str, "/dev/null" );
 
     /* symlinks */
-    link_dosdev    = create_symlink( root_directory, &link_dosdev_str, 0, &dir_global_str );
-    link_global1   = create_symlink( dir_global, &link_global_str, 0, &dir_global_str );
-    link_global2   = create_symlink( dir_basenamed, &link_global_str, 0, &dir_basenamed_str );
-    link_local     = create_symlink( dir_basenamed, &link_local_str, 0, &dir_basenamed_str );
-    link_nul       = create_symlink( dir_global, &link_nul_str, 0, &dir_null_str );
-    link_pipe      = create_symlink( dir_global, &link_pipe_str, 0, &dir_named_pipe_str );
-    link_mailslot  = create_symlink( dir_global, &link_mailslot_str, 0, &dir_mailslot_str );
-    link_0         = create_symlink( dir_sessions, &link_0_str, 0, &dir_basenamed_str );
-    link_session   = create_symlink( dir_basenamed, &link_session_str, 0, &link_sessions_str );
+    link_dosdev    = create_symlink( &root_directory->obj, &link_dosdev_str, 0, &dir_global_str, NULL );
+    link_global1   = create_symlink( &dir_global->obj, &link_global_str, 0, &dir_global_str, NULL );
+    link_global2   = create_symlink( &dir_basenamed->obj, &link_global_str, 0, &dir_basenamed_str, NULL );
+    link_local     = create_symlink( &dir_basenamed->obj, &link_local_str, 0, &dir_basenamed_str, NULL );
+    link_nul       = create_symlink( &dir_global->obj, &link_nul_str, 0, &dir_null_str, NULL );
+    link_pipe      = create_symlink( &dir_global->obj, &link_pipe_str, 0, &dir_named_pipe_str, NULL );
+    link_mailslot  = create_symlink( &dir_global->obj, &link_mailslot_str, 0, &dir_mailslot_str, NULL );
+    link_0         = create_symlink( &dir_sessions->obj, &link_0_str, 0, &dir_basenamed_str, NULL );
+    link_session   = create_symlink( &dir_basenamed->obj, &link_session_str, 0, &link_sessions_str, NULL );
     make_object_static( (struct object *)link_dosdev );
     make_object_static( (struct object *)link_global1 );
     make_object_static( (struct object *)link_global2 );
@@ -478,10 +371,10 @@ void init_directories(void)
     /* events */
     for (i = 0; i < sizeof(kernel_events)/sizeof(kernel_events[0]); i++)
     {
-        struct event *event = create_event( dir_kernel, &kernel_events[i], 0, 1, 0, NULL );
+        struct event *event = create_event( &dir_kernel->obj, &kernel_events[i], 0, 1, 0, NULL );
         make_object_static( (struct object *)event );
     }
-    keyed_event = create_keyed_event( dir_kernel, &keyed_event_crit_sect_str, 0, NULL );
+    keyed_event = create_keyed_event( &dir_kernel->obj, &keyed_event_crit_sect_str, 0, NULL );
     make_object_static( (struct object *)keyed_event );
 
     /* the objects hold references so we can release these directories */
@@ -490,22 +383,23 @@ void init_directories(void)
     release_object( dir_basenamed );
     release_object( dir_sessions );
     release_object( dir_kernel );
+    release_object( dir_windows );
 }
 
 /* create a directory object */
 DECL_HANDLER(create_directory)
 {
     struct unicode_str name;
-    struct directory *dir, *root = NULL;
+    struct object *root;
+    struct directory *dir;
+    const struct security_descriptor *sd;
+    const struct object_attributes *objattr = get_req_object_attributes( &sd, &name, &root );
 
-    reply->handle = 0;
-    get_req_unicode_str( &name );
-    if (req->rootdir && !(root = get_directory_obj( current->process, req->rootdir, 0 )))
-        return;
+    if (!objattr) return;
 
-    if ((dir = create_directory( root, &name, req->attributes, HASH_SIZE )))
+    if ((dir = create_directory( root, &name, objattr->attributes, HASH_SIZE, sd )))
     {
-        reply->handle = alloc_handle( current->process, dir, req->access, req->attributes );
+        reply->handle = alloc_handle( current->process, dir, req->access, objattr->attributes );
         release_object( dir );
     }
 
@@ -515,26 +409,17 @@ DECL_HANDLER(create_directory)
 /* open a directory object */
 DECL_HANDLER(open_directory)
 {
-    struct unicode_str name;
-    struct directory *dir, *root = NULL;
+    struct unicode_str name = get_req_unicode_str();
 
-    get_req_unicode_str( &name );
-    if (req->rootdir && !(root = get_directory_obj( current->process, req->rootdir, 0 )))
-        return;
-
-    if ((dir = open_object_dir( root, &name, req->attributes, &directory_ops )))
-    {
-        reply->handle = alloc_handle( current->process, &dir->obj, req->access, req->attributes );
-        release_object( dir );
-    }
-
-    if (root) release_object( root );
+    reply->handle = open_object( current->process, req->rootdir, req->access,
+                                 &directory_ops, &name, req->attributes );
 }
 
 /* get a directory entry by index */
 DECL_HANDLER(get_directory_entry)
 {
-    struct directory *dir = get_directory_obj( current->process, req->handle, DIRECTORY_QUERY );
+    struct directory *dir = (struct directory *)get_handle_obj( current->process, req->handle,
+                                                                DIRECTORY_QUERY, &directory_ops );
     if (dir)
     {
         struct object *obj = find_object_index( dir->entries, req->index );
