@@ -209,6 +209,8 @@ static const struct object_ops fd_ops =
     default_get_sd,           /* get_sd */
     default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
+    no_link_name,             /* link_name */
+    NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
     fd_destroy                /* destroy */
@@ -246,6 +248,8 @@ static const struct object_ops device_ops =
     default_get_sd,           /* get_sd */
     default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
+    no_link_name,             /* link_name */
+    NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
     device_destroy            /* destroy */
@@ -282,6 +286,8 @@ static const struct object_ops inode_ops =
     default_get_sd,           /* get_sd */
     default_set_sd,           /* set_sd */
     no_lookup_name,           /* lookup_name */
+    no_link_name,             /* link_name */
+    NULL,                     /* unlink_name */
     no_open_file,             /* open_file */
     no_close_handle,          /* close_handle */
     inode_destroy             /* destroy */
@@ -320,6 +326,8 @@ static const struct object_ops file_lock_ops =
     default_get_sd,             /* get_sd */
     default_set_sd,             /* set_sd */
     no_lookup_name,             /* lookup_name */
+    no_link_name,               /* link_name */
+    NULL,                       /* unlink_name */
     no_open_file,               /* open_file */
     no_close_handle,            /* close_handle */
     no_destroy                  /* destroy */
@@ -566,8 +574,8 @@ static inline void set_fd_epoll_events( struct fd *fd, int user, int events )
 
     if (kqueue_fd == -1) return;
 
-    EV_SET( &ev[0], fd->unix_fd, EVFILT_READ, 0, NOTE_LOWAT, 1, (void *)user );
-    EV_SET( &ev[1], fd->unix_fd, EVFILT_WRITE, 0, NOTE_LOWAT, 1, (void *)user );
+    EV_SET( &ev[0], fd->unix_fd, EVFILT_READ, 0, NOTE_LOWAT, 1, (void *)(long)user );
+    EV_SET( &ev[1], fd->unix_fd, EVFILT_WRITE, 0, NOTE_LOWAT, 1, (void *)(long)user );
 
     if (events == -1)  /* stop waiting on this fd completely */
     {
@@ -1952,7 +1960,7 @@ void set_fd_signaled( struct fd *fd, int signaled )
     if (signaled) wake_up( fd->user, 0 );
 }
 
-/* set or clear the fd signaled state */
+/* check if fd is signaled */
 int is_fd_signaled( struct fd *fd )
 {
     return fd->signaled;
@@ -2384,29 +2392,14 @@ DECL_HANDLER(flush)
 /* open a file object */
 DECL_HANDLER(open_file_object)
 {
-    struct unicode_str name;
-    struct directory *root = NULL;
-    struct object *obj, *result;
+    struct unicode_str name = get_req_unicode_str();
+    struct object *obj, *result, *root = NULL;
 
-    get_req_unicode_str( &name );
-    if (req->rootdir && !(root = get_directory_obj( current->process, req->rootdir, 0 )))
-    {
-        if (get_error() != STATUS_OBJECT_TYPE_MISMATCH) return;
-        if (!(obj = (struct object *)get_file_obj( current->process, req->rootdir, 0 ))) return;
-        if (name.len)
-        {
-            release_object( obj );
-            set_error( STATUS_OBJECT_PATH_NOT_FOUND );
-            return;
-        }
-        clear_error();
-    }
-    else
-    {
-        obj = open_object_dir( root, &name, req->attributes, NULL );
-        if (root) release_object( root );
-        if (!obj) return;
-    }
+    if (req->rootdir && !(root = get_handle_obj( current->process, req->rootdir, 0, NULL ))) return;
+
+    obj = open_named_object( root, NULL, &name, req->attributes );
+    if (root) release_object( root );
+    if (!obj) return;
 
     if ((result = obj->ops->open_file( obj, req->access, req->sharing, req->options )))
     {
