@@ -130,6 +130,8 @@ static const struct object_ops handle_table_ops =
     default_get_sd,                  /* get_sd */
     default_set_sd,                  /* set_sd */
     no_lookup_name,                  /* lookup_name */
+    no_link_name,                    /* link_name */
+    NULL,                            /* unlink_name */
     no_open_file,                    /* open_file */
     no_close_handle,                 /* close_handle */
     handle_table_destroy             /* destroy */
@@ -153,6 +155,7 @@ static void handle_table_dump( struct object *obj, int verbose )
         if (!entry->ptr) continue;
         fprintf( stderr, "    %04x: %p %08x ",
                  index_to_handle(i), entry->ptr, entry->access );
+        dump_object_name( entry->ptr );
         entry->ptr->ops->dump( entry->ptr, 0 );
     }
 }
@@ -402,6 +405,12 @@ static inline struct object *get_magic_handle( obj_handle_t handle )
 {
     switch(handle)
     {
+        case 0xfffffffa:  /* current thread impersonation token pseudo-handle */
+            return (struct object *)thread_get_impersonation_token( current );
+        case 0xfffffffb:  /* current thread token pseudo-handle */
+            return (struct object *)current->token;
+        case 0xfffffffc:  /* current process token pseudo-handle */
+            return (struct object *)current->process->token;
         case 0xfffffffe:  /* current thread pseudo-handle */
             return &current->obj;
         case 0x7fffffff:  /* current process pseudo-handle */
@@ -570,21 +579,34 @@ obj_handle_t duplicate_handle( struct process *src, obj_handle_t src_handle, str
 }
 
 /* open a new handle to an existing object */
-obj_handle_t open_object( const struct namespace *namespace, const struct unicode_str *name,
-                          const struct object_ops *ops, unsigned int access, unsigned int attr )
+obj_handle_t open_object( struct process *process, obj_handle_t parent, unsigned int access,
+                          const struct object_ops *ops, const struct unicode_str *name,
+                          unsigned int attributes )
 {
     obj_handle_t handle = 0;
-    struct object *obj = find_object( namespace, name, attr );
-    if (obj)
+    struct object *obj, *root = NULL;
+
+    if (name->len >= 65534)
     {
-        if (ops && obj->ops != ops)
-            set_error( STATUS_OBJECT_TYPE_MISMATCH );
-        else
-            handle = alloc_handle( current->process, obj, access, attr );
+        set_error( STATUS_OBJECT_NAME_INVALID );
+        return 0;
+    }
+
+    if (parent)
+    {
+        if (name->len)
+            root = get_directory_obj( process, parent );
+        else  /* opening the object itself can work for non-directories too */
+            root = get_handle_obj( process, parent, 0, NULL );
+        if (!root) return 0;
+    }
+
+    if ((obj = open_named_object( root, ops, name, attributes )))
+    {
+        handle = alloc_handle( process, obj, access, attributes );
         release_object( obj );
     }
-    else
-        set_error( STATUS_OBJECT_NAME_NOT_FOUND );
+    if (root) release_object( root );
     return handle;
 }
 
