@@ -1884,7 +1884,7 @@ void destroy_window( struct window *win )
 DECL_HANDLER(create_window)
 {
     struct window *win, *parent = NULL, *owner = NULL;
-    struct unicode_str cls_name;
+    struct unicode_str cls_name = get_req_unicode_str();
     atom_t atom;
 
     reply->handle = 0;
@@ -1901,10 +1901,10 @@ DECL_HANDLER(create_window)
             return;
         }
         else /* owner must be a top-level window */
-            while (!is_desktop_window(owner->parent)) owner = owner->parent;
+            while ((owner->style & (WS_POPUP|WS_CHILD)) == WS_CHILD && !is_desktop_window(owner->parent))
+                owner = owner->parent;
     }
 
-    get_req_unicode_str( &cls_name );
     atom = cls_name.len ? find_global_atom( NULL, &cls_name ) : req->atom;
 
     if (!(win = create_window( parent, owner, atom, req->instance ))) return;
@@ -1953,10 +1953,14 @@ DECL_HANDLER(destroy_window)
 DECL_HANDLER(get_desktop_window)
 {
     struct desktop *desktop = get_thread_desktop( current, 0 );
+    int force;
 
     if (!desktop) return;
 
-    if (!desktop->top_window && req->force)  /* create it */
+    /* if winstation is invisible, then avoid roundtrip */
+    force = req->force || !(desktop->winstation->flags & WSF_VISIBLE);
+
+    if (!desktop->top_window && force)  /* create it */
     {
         if ((desktop->top_window = create_window( NULL, NULL, DESKTOP_ATOM, 0 )))
         {
@@ -1965,7 +1969,7 @@ DECL_HANDLER(get_desktop_window)
         }
     }
 
-    if (!desktop->msg_window && req->force)  /* create it */
+    if (!desktop->msg_window && force)  /* create it */
     {
         static const WCHAR messageW[] = {'M','e','s','s','a','g','e'};
         static const struct unicode_str name = { messageW, sizeof(messageW) };
@@ -2114,11 +2118,10 @@ DECL_HANDLER(get_window_children)
     unsigned int total;
     user_handle_t *data;
     data_size_t len;
-    struct unicode_str cls_name;
+    struct unicode_str cls_name = get_req_unicode_str();
     atom_t atom = req->atom;
     struct desktop *desktop = NULL;
 
-    get_req_unicode_str( &cls_name );
     if (cls_name.len && !(atom = find_global_atom( NULL, &cls_name ))) return;
 
     if (req->desktop)
@@ -2669,12 +2672,11 @@ DECL_HANDLER(redraw_window)
 /* set a window property */
 DECL_HANDLER(set_window_property)
 {
-    struct unicode_str name;
+    struct unicode_str name = get_req_unicode_str();
     struct window *win = get_window( req->window );
 
     if (!win) return;
 
-    get_req_unicode_str( &name );
     if (name.len)
     {
         atom_t atom = add_global_atom( NULL, &name );
@@ -2691,10 +2693,9 @@ DECL_HANDLER(set_window_property)
 /* remove a window property */
 DECL_HANDLER(remove_window_property)
 {
-    struct unicode_str name;
+    struct unicode_str name = get_req_unicode_str();
     struct window *win = get_window( req->window );
 
-    get_req_unicode_str( &name );
     if (win)
     {
         atom_t atom = name.len ? find_global_atom( NULL, &name ) : req->atom;
@@ -2706,10 +2707,9 @@ DECL_HANDLER(remove_window_property)
 /* get a window property */
 DECL_HANDLER(get_window_property)
 {
-    struct unicode_str name;
+    struct unicode_str name = get_req_unicode_str();
     struct window *win = get_window( req->window );
 
-    get_req_unicode_str( &name );
     if (win)
     {
         atom_t atom = name.len ? find_global_atom( NULL, &name ) : req->atom;
