@@ -1270,6 +1270,23 @@ void output_stubs( DLLSPEC *spec )
             if (exp_name) output( "\t.long .L%s_string-2b\n", name );
             else output( "\t.long %u\n", odp->ordinal );
             break;
+        case CPU_ARM64:
+            output( "\tadr x0, 2f\n" );
+            output( "\tldur x0, [x0, #0]\n" );
+            output( "\tadr x1, 2f+8\n" );
+            output( "\tldur x1, [x1, #0]\n" );
+            output( "\tadr x2, 1f\n" );
+            output( "\tldur x2, [x2, #0]\n" );
+            output( "\tblr x2\n" );
+            output( "1:\t.quad %s\n", asm_name("__wine_spec_unimplemented_stub") );
+            output( "2:\t.quad %s\n", asm_name("__wine_spec_file_name") );
+            if (exp_name)
+            {
+                output( "\t.quad .L%s_string\n", name );
+                count++;
+            }
+            else output( "\t.quad %u\n", odp->ordinal );
+            break;
         default:
             assert(0);
         }
@@ -1312,6 +1329,7 @@ void output_import_lib( DLLSPEC *spec, char **argv )
 {
     struct strarray *args;
     char *def_file;
+    const char *as_flags, *m_flag;
 
     if (target_platform != PLATFORM_WINDOWS)
         fatal_error( "Unix-style import libraries not supported yet\n" );
@@ -1325,7 +1343,23 @@ void output_import_lib( DLLSPEC *spec, char **argv )
     output_file = NULL;
 
     args = find_tool( "dlltool", NULL );
+    switch (target_cpu)
+    {
+        case CPU_x86:
+            m_flag = "i386";
+            as_flags = "--as-flags=--32";
+            break;
+        case CPU_x86_64:
+            m_flag = "i386:x86-64";
+            as_flags = "--as-flags=--64";
+            break;
+        default:
+            m_flag = NULL;
+            break;
+    }
     strarray_add( args, "-k", "-l", output_file_name, "-d", def_file, NULL );
+    if (m_flag)
+        strarray_add( args, "-m", m_flag, as_flags, NULL );
     spawn( args );
     strarray_free( args );
 
