@@ -64,6 +64,9 @@ static const struct
     { "x86_64",  CPU_x86_64 },
     { "powerpc", CPU_POWERPC },
     { "arm",     CPU_ARM },
+    { "armv5",   CPU_ARM },
+    { "armv6",   CPU_ARM },
+    { "armv7",   CPU_ARM },
     { "arm64",   CPU_ARM64 },
     { "aarch64", CPU_ARM64 },
 };
@@ -388,6 +391,7 @@ struct strarray *get_as_command(void)
         if (force_pointer_size)
             strarray_add_one( args, (force_pointer_size == 8) ? "-m64" : "-m32" );
         if (cpu_option) strarray_add_one( args, strmake("-mcpu=%s", cpu_option) );
+        if (arch_option) strarray_add_one( args, strmake("-march=%s", arch_option) );
         return args;
     }
 
